@@ -11,13 +11,6 @@
  * check_and_add_state.c - the various possible implementations of the function
  * fc_solve_check_and_add_state().
  */
-#include "dll_thunk.h"
-
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-
-#include "instance.h"
 
 #include "move_stack_compact_alloc.h"
 
@@ -42,13 +35,14 @@ static inline void fc_solve_hash_rehash(fc_solve_hash_t *const hash)
 
     const int new_size_bitmask = new_size - 1;
 
+    var_AUTO(entries, hash->entries);
     fc_solve_hash_symlink_t *const new_entries =
         calloc(new_size, sizeof(new_entries[0]));
 
     /* Copy the items to the new hash while not allocating them again */
     for (int i = 0; i < old_size; i++)
     {
-        fc_solve_hash_symlink_item_t *item = hash->entries[i].first_item;
+        fcs_hash_item_t *item = entries[i].first_item;
         /* traverse the chain item by item */
         while (item != NULL)
         {
@@ -57,7 +51,7 @@ static inline void fc_solve_hash_rehash(fc_solve_hash_t *const hash)
 
             /* Store the next item in the linked list in a safe place,
                so we can retrieve it after the assignment */
-            fc_solve_hash_symlink_item_t *const next_item = item->next;
+            fcs_hash_item_t *const next_item = item->next;
             /* It is placed in front of the first element in the chain,
                so it should link to it */
             item->next = new_entries[place].first_item;
@@ -94,19 +88,14 @@ static inline void *fc_solve_hash_insert(
     ,
     const fc_solve_hash_value_t secondary_hash_value
 #endif
-    )
+)
 {
 #if defined(FCS_INLINED_HASH_COMPARISON) && defined(INDIRECT_STACK_STATES)
     const_SLOT(hash_type, hash);
 #endif
-/* Get the index of the appropriate chain in the hash table */
-#define PLACE() (hash_value & (hash->size_bitmask))
-
-    typeof(hash->entries[0]) *const list = (hash->entries + PLACE());
-
-#undef PLACE
-
-    fc_solve_hash_symlink_item_t **item_placeholder;
+    typeof(hash->entries[0]) *const list =
+        (hash->entries + (hash_value & (hash->size_bitmask)));
+    fcs_hash_item_t **item_placeholder;
     /* If first_item is non-existent */
     if (list->first_item == NULL)
     {
@@ -116,17 +105,15 @@ static inline void *fc_solve_hash_insert(
     else
     {
         /* Initialize item to the chain's first_item */
-        fc_solve_hash_symlink_item_t *item = list->first_item;
-        fc_solve_hash_symlink_item_t *last_item = NULL;
+        fcs_hash_item_t *item = list->first_item;
+        fcs_hash_item_t *last_item = NULL;
 
-/*
- * MY_HASH_COMPARE_PROTO() returns -1/0/+1 depending on the compared
- * states order. We need to negate it for the desired condition of equality.
- * */
+// MY_HASH_COMPARE_PROTO() returns -1/0/+1 depending on the compared
+// states order. We need to negate it for the desired condition of equality.
 #define MY_HASH_COMPARE() (!MY_HASH_COMPARE_PROTO())
 
 /* Define MY_HASH_COMPARE_PROTO() */
-#if defined(FCS_RCS_STATES)
+#ifdef FCS_RCS_STATES
 
 #define MY_HASH_COMPARE_PROTO()                                                \
     (fc_solve_state_compare(key_id,                                            \
@@ -182,9 +169,9 @@ static inline void *fc_solve_hash_insert(
 
 #define ITEM_ALLOC() fcs_compact_alloc_ptr(&(hash->allocator), sizeof(*item))
 #ifdef FCS_WITHOUT_TRIM_MAX_STORED_STATES
-    fc_solve_hash_symlink_item_t *const item = ITEM_ALLOC();
+    fcs_hash_item_t *const item = ITEM_ALLOC();
 #else
-    fc_solve_hash_symlink_item_t *item;
+    fcs_hash_item_t *item;
 
     if ((item = hash->list_of_vacant_items))
     {
@@ -215,12 +202,16 @@ static inline void *fc_solve_hash_insert(
 
 #endif
 
+#if ((defined(INDIRECT_STACK_STATES) &&                                        \
+         (FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH)) ||            \
+     (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH))
+
 typedef unsigned long ul;
 typedef unsigned char ub1;
 
 static inline ul perl_hash_function(register const ub1 *s_ptr, /* the key */
     register const ul len /* the length of the key */
-    )
+)
 {
     register ul hash_value_int = 0;
     register const ub1 *const s_end = s_ptr + len;
@@ -233,6 +224,7 @@ static inline ul perl_hash_function(register const ub1 *s_ptr, /* the key */
 
     return hash_value_int;
 }
+#endif
 
 #ifdef INDIRECT_STACK_STATES
 
@@ -254,19 +246,14 @@ static inline void fc_solve_cache_stacks(
 #else
     fc_solve_instance_t *const instance = hard_thread->instance;
 #endif
-
-#ifndef HARD_CODED_NUM_STACKS
-    SET_GAME_PARAMS();
-#endif
+    STACKS__SET_PARAMS();
     register fcs_state_t *const new_state_key = new_state->key;
     register fcs_state_extra_info_t *const new_state_info = new_state->val;
-
     fcs_compact_allocator_t *const stacks_allocator =
         &(HT_FIELD(hard_thread, allocator));
-
     fcs_cards_column_t *current_stack = new_state_key->columns;
 
-    for (int i = 0; i < LOCAL_STACKS_NUM; i++, current_stack++)
+    for (int i = 0; i < LOCAL_STACKS_NUM; ++i, ++current_stack)
     {
         /*
          * If the stack is not a copy - it is already cached so skip
@@ -277,7 +264,7 @@ static inline void fc_solve_cache_stacks(
             continue;
         }
 
-        fcs_cards_column_t column = fcs_state_get_col(*(new_state_key), i);
+        var_AUTO(column, fcs_state_get_col(*(new_state_key), i));
         const int col_len = (fcs_col_len(column) + 1);
 
         char *const new_ptr =
@@ -319,7 +306,7 @@ static inline void fc_solve_cache_stacks(
                 ,
             hash_value_int
 #endif
-            );
+        );
         REPLACE_WITH_CACHED(cached_stack);
 
 #elif (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GOOGLE_DENSE_HASH)
@@ -379,11 +366,7 @@ static inline void fc_solve_cache_stacks(
 }
 
 #else /* #ifdef INDIRECT_STACK_STATES */
-
-#define fc_solve_cache_stacks(hard_thread, new_state_key)                      \
-    {                                                                          \
-    }
-
+#define fc_solve_cache_stacks(hard_thread, new_state_key)
 #endif
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_HASH)
@@ -438,14 +421,35 @@ static inline void upon_new_state(fc_solve_instance_t *const instance,
 #endif
     }
 
+#ifndef FCS_DISABLE_NUM_STORED_STATES
 #ifndef FCS_WITHOUT_TRIM_MAX_STORED_STATES
-    instance->active_num_states_in_collection++;
+    ++instance->active_num_states_in_collection;
+#endif
+    ++instance->num_states_in_collection;
 #endif
-    instance->num_states_in_collection++;
-
-    return;
 }
 
+#define ON_STATE_NEW() upon_new_state(instance, hard_thread, new_state->val);
+static inline fcs_bool_t handle_existing_void(
+    fc_solve_instance_t *const instance,
+    fc_solve_hard_thread_t *const hard_thread, fcs_kv_state_t *const new_state,
+    fcs_kv_state_t *const existing_state_raw, void *const existing_void)
+{
+    if (existing_void)
+    {
+        FCS_STATE_collectible_to_kv(existing_state_raw, existing_void);
+        return FALSE;
+    }
+    else
+    {
+        ON_STATE_NEW();
+        return TRUE;
+    }
+}
+#define HANDLE_existing_void(existing_void)                                    \
+    handle_existing_void(                                                      \
+        instance, hard_thread, new_state, existing_state_raw, (existing_void))
+
 fcs_bool_t fc_solve_check_and_add_state(
     fc_solve_hard_thread_t *const hard_thread, fcs_kv_state_t *const new_state,
     fcs_kv_state_t *const existing_state_raw)
@@ -457,8 +461,6 @@ fcs_bool_t fc_solve_check_and_add_state(
  * */
 #define new_state_key (new_state->key)
 
-#define ON_STATE_NEW() upon_new_state(instance, hard_thread, new_state->val);
-
 #ifdef FCS_SINGLE_HARD_THREAD
 #define instance hard_thread
 #else
@@ -468,13 +470,13 @@ fcs_bool_t fc_solve_check_and_add_state(
     fc_solve_canonize_state(new_state_key PASS_FREECELLS(INSTANCE_FREECELLS_NUM)
             PASS_STACKS(INSTANCE_STACKS_NUM));
 
-/*
-    The objective of this part of the code is:
-    1. To check if new_state_key / new_state_val is already in the prev_states
-       collection.
-    2. If not, to add it and to set check to true.
-    3. If so, to set check to false.
-  */
+    /*
+        The objective of this part of the code is:
+        1. To check if new_state_key / new_state_val is already in the
+       prev_states collection.
+        2. If not, to add it and to set check to true.
+        3. If so, to set check to false.
+      */
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH)
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
@@ -497,97 +499,51 @@ fcs_bool_t fc_solve_check_and_add_state(
         hash_value_int &= (~(1 << ((sizeof(hash_value_int) << 3) - 1)));
     }
 #endif
-    {
-        void *const existing_void = fc_solve_hash_insert(&(instance->hash),
 #ifdef FCS_RCS_STATES
-            new_state->val, new_state->key,
+#define A new_state->val, new_state->key
 #else
-            FCS_STATE_kv_to_collectible(new_state),
+#define A FCS_STATE_kv_to_collectible(new_state)
 #endif
-            perl_hash_function((ub1 *)(new_state_key), sizeof(*(new_state_key)))
 #ifdef FCS_ENABLE_SECONDARY_HASH_VALUE
-                ,
-            hash_value_int
+#define B , hash_value_int
+#else
+#define B
 #endif
-            );
-        if (existing_void)
-        {
-            FCS_STATE_collectible_to_kv(existing_state_raw, existing_void);
-            return FALSE;
-        }
-        else
-        {
-            ON_STATE_NEW();
-            return TRUE;
-        }
-    }
+    return HANDLE_existing_void(fc_solve_hash_insert(&(instance->hash), A,
+        perl_hash_function((ub1 *)(new_state_key), sizeof(*(new_state_key)))
+            B));
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GOOGLE_DENSE_HASH)
+    void *existing_void;
+    if (!fc_solve_states_google_hash_insert(instance->hash,
+            FCS_STATE_kv_to_collectible(new_state), &(existing_void)))
     {
-        void *existing_void;
-
-        /*  TODO : check if this condition should be negated. */
-        if (fc_solve_states_google_hash_insert(instance->hash,
-                FCS_STATE_kv_to_collectible(new_state), &(existing_void)))
-        {
-            FCS_STATE_collectible_to_kv(existing_state_raw, existing_void);
-            return FALSE;
-        }
-        else
-        {
-            ON_STATE_NEW();
-            return TRUE;
-        }
+        existing_void = NULL;
     }
+    return HANDLE_existing_void(existing_void);
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBREDBLACK_TREE)
-    existing_state_val =
-        (fcs_state_extra_info_t *)rbsearch(new_state_val, instance->tree);
-    is_state_new = ((*existing_state_val) == new_state_val);
-
+    const_AUTO(new_state_void, FCS_STATE_kv_to_collectible(new_state));
+    const void *existing_void = rbsearch(new_state_void, instance->tree);
+    return HANDLE_existing_void(
+        existing_void == new_state_void ? NULL : existing_void);
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_KAZ_TREE)
 #ifdef FCS_RCS_STATES
     instance->tree_new_state_key = new_state->key;
     instance->tree_new_state = new_state->val;
 #endif
-
-    {
-        void *existing_void;
-        if ((existing_void = fc_solve_kaz_tree_alloc_insert(instance->tree,
+    return HANDLE_existing_void(fc_solve_kaz_tree_alloc_insert(instance->tree,
 #ifdef FCS_RCS_STATES
-                 new_state_val
+        new_state_val
 #else
-                 FCS_STATE_kv_to_collectible(new_state)
+        FCS_STATE_kv_to_collectible(new_state)
 #endif
-                 )) == NULL)
-        {
-            ON_STATE_NEW();
-            return TRUE;
-        }
-        else
-        {
-            FCS_STATE_collectible_to_kv(existing_state_raw, existing_void);
-            return FALSE;
-        }
-    }
+        ));
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBAVL2_TREE)
 #ifdef FCS_RCS_STATES
     instance->tree_new_state_key = new_state->key;
     instance->tree_new_state = new_state->val;
 #endif
-
-    {
-        void *existing_void;
-        if ((existing_void = fcs_libavl2_states_tree_insert(instance->tree,
-                 FCS_STATE_kv_to_collectible(new_state))) == NULL)
-        {
-            ON_STATE_NEW();
-            return TRUE;
-        }
-        else
-        {
-            FCS_STATE_collectible_to_kv(existing_state_raw, existing_void);
-            return FALSE;
-        }
-    }
+    return HANDLE_existing_void(fcs_libavl2_states_tree_insert(
+        instance->tree, FCS_STATE_kv_to_collectible(new_state)));
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_TREE)
     existing_state_val = g_tree_lookup(instance->tree, (gpointer)new_state_key);
     if ((is_state_new = (existing_state_val == NULL)))
@@ -610,49 +566,37 @@ fcs_bool_t fc_solve_check_and_add_state(
         g_hash_table_insert(
             instance->hash, (gpointer)new_state_key, (gpointer)new_state_val
 
-            );
+        );
     }
 
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_DB_FILE)
+    DBT key, value;
+    key.data = new_state;
+    key.size = sizeof(*new_state);
+    if ((is_state_new =
+                (instance->db->get(instance->db, NULL, &key, &value, 0) == 0)))
     {
-        DBT key, value;
-        key.data = new_state;
-        key.size = sizeof(*new_state);
-        if ((is_state_new = (instance->db->get(
-                                 instance->db, NULL, &key, &value, 0) == 0)))
-        {
-            /* The new state was not found. Let's insert it.
-             * The value must be the same as the key, so g_tree_lookup()
-             * will return it. */
+        /* The new state was not found. Let's insert it.
+         * The value must be the same as the key, so g_tree_lookup()
+         * will return it. */
 
-            value.data = key.data;
-            value.size = key.size;
-            instance->db->put(instance->db, NULL, &key, &value, 0);
-        }
+        value.data = key.data;
+        value.size = key.size;
+        instance->db->put(instance->db, NULL, &key, &value, 0);
     }
 #elif (FCS_STATE_STORAGE == FCS_STATE_STORAGE_JUDY)
+    PWord_t *PValue;
+
+    JHSI(PValue, instance->judy_array, new_state_key, sizeof(*new_state_key));
+
+    /* later_todo : Handle out-of-memory. */
+    const_AUTO(val, *PValue);
+    if (val == 0)
     {
-        PWord_t *PValue;
-
-        JHSI(PValue, instance->judy_array, new_state_key,
-            sizeof(*new_state_key));
-
-        /* later_todo : Handle out-of-memory. */
-        if (*PValue == 0)
-        {
-            /* A new state. */
-            *PValue = (PWord_t)(FCS_STATE_kv_to_collectible(new_state));
-            ON_STATE_NEW();
-            return TRUE;
-        }
-        else
-        {
-            /* Already exists. */
-            FCS_STATE_collectible_to_kv(
-                existing_state_raw, (fcs_collectible_state_t *)(*PValue));
-            return FALSE;
-        }
+        /* A new state. */
+        *PValue = (PWord_t)(FCS_STATE_kv_to_collectible(new_state));
     }
+    return HANDLE_existing_void(val);
 #else
 #error Unknown FCS_STATE_STORAGE. Please define it to a valid value.
 #endif
