@@ -12,27 +12,19 @@
  * routines. Useful for more easily configuring a Freecell Solver instance.
  */
 
-#include "dll_thunk.h"
-
 #include "config.h"
 
-#ifdef HAVE_VASPRINTF
-#define _GNU_SOURCE
-#else
+#ifndef HAVE_VASPRINTF
 #include "asprintf.h"
 #endif
 
-#include <string.h>
-#include <stdlib.h>
 #include <stdarg.h>
-#include <stdio.h>
 #include <ctype.h>
 #ifdef _WIN32
 #include <windows.h>
 #endif
 
 #include "set_weights.h"
-#include "fcs_user.h"
 #include "fcs_cl.h"
 #include "split_cmd_line.h"
 
@@ -63,31 +55,23 @@ static inline fcs_bool_t read_preset(const char *const preset_name,
             "%s/.freecell-solver/presetrc", home_dir);
     }
     const char *global_presetrc;
-    char *env_var_presetrc = getenv("FREECELL_SOLVER_PRESETRC");
-
 #ifdef _WIN32
-
     char windows_exe_dir[MAX_PATH] = "";
     char windows_global_presetc[MAX_PATH + 100] = "";
-
-    /* Will contain exe path */
     const HMODULE hModule = GetModuleHandle(NULL);
     if (hModule != NULL)
     {
-        /* When passing NULL to GetModuleHandle, it returns handle of exe itself
-         */
+        // When passing NULL to GetModuleHandle, it returns handle of exe itself
         GetModuleFileName(hModule, windows_exe_dir, (sizeof(windows_exe_dir)));
+        /* Remove the basename and keep only the dirname. */
+        char *s = windows_exe_dir + strlen(windows_exe_dir) - 1;
+        while ((s > windows_exe_dir) && (!((*s == '/') || (*s == '\\'))))
         {
-            /* Remove the basename and keep only the dirname. */
-            char *s = windows_exe_dir + strlen(windows_exe_dir) - 1;
-            while ((s > windows_exe_dir) && (!((*s == '/') || (*s == '\\'))))
-            {
-                s--;
-            }
-            if (s > windows_exe_dir)
-            {
-                *s = '\0';
-            }
+            s--;
+        }
+        if (s > windows_exe_dir)
+        {
+            *s = '\0';
         }
         sprintf(windows_global_presetc, "%s/../share/freecell-solver/presetrc",
             windows_exe_dir);
@@ -97,17 +81,17 @@ static inline fcs_bool_t read_preset(const char *const preset_name,
     {
         global_presetrc = NULL;
     }
-
 #else
     global_presetrc = (FREECELL_SOLVER_PKG_DATA_DIR "/presetrc");
 #endif
     const char *const presetrc_pathes[4] = {
-        (const char *)env_var_presetrc,
+        (const char *)getenv("FREECELL_SOLVER_PRESETRC"),
         (home_dir ? ((const char *)home_dir_presetrc) : NULL),
-        (const char *)global_presetrc, user_preset_dir,
+        (const char *)global_presetrc,
+        user_preset_dir,
     };
     fcs_bool_t read_next_preset = FALSE;
-    for (size_t path_idx = 0; path_idx < COUNT(presetrc_pathes); path_idx++)
+    for (size_t path_idx = 0; path_idx < COUNT(presetrc_pathes); ++path_idx)
     {
         const char *const path = presetrc_pathes[path_idx];
         if (path == NULL)
@@ -115,7 +99,7 @@ static inline fcs_bool_t read_preset(const char *const preset_name,
             continue;
         }
         FILE *f = fopen(path, "rt");
-        if (f == NULL)
+        if (!f)
         {
             continue;
         }
@@ -172,6 +156,7 @@ static inline fcs_bool_t read_preset(const char *const preset_name,
     return TRUE;
 }
 
+#ifdef FCS_WITH_ERROR_STRS
 static inline char *calc_errstr_s(const char *const format, ...)
 {
     va_list my_va_list;
@@ -187,18 +172,30 @@ static inline char *calc_errstr_s(const char *const format, ...)
     return errstr;
 }
 
-#ifdef FCS_WITH_ERROR_STRS
 #define ASSIGN_ERR_STR(error_string, format, ...)                              \
     *(error_string) = calc_errstr_s(format, __VA_ARGS__)
 #define ASSIGN_ERR_STR_AND_FREE(fcs_user_errstr, error_string, format, ...)    \
-    {                                                                          \
-        ASSIGN_ERR_STR(error_string, format, __VA_ARGS__);                     \
-        free(fcs_user_errstr);                                                 \
-    }
+    ASSIGN_ERR_STR(error_string, format, __VA_ARGS__);                         \
+    free(fcs_user_errstr)
 #else
 #define ASSIGN_ERR_STR(error_string, format, ...)
 #define ASSIGN_ERR_STR_AND_FREE(fcs_user_errstr, error_string, format, ...)
 #endif
+#define PROCESS_OPT_ARG()                                                      \
+    if ((++arg) == arg_argc)                                                   \
+    {                                                                          \
+        *last_arg = arg - 1 - &(argv[0]);                                      \
+        return FCS_CMD_LINE_PARAM_WITH_NO_ARG;                                 \
+    }
+#define RET_ERROR_IN_ARG()                                                     \
+    *last_arg = arg - &(argv[0]);                                              \
+    return FCS_CMD_LINE_ERROR_IN_ARG
+#define RET_ERR_STR(...)                                                       \
+    ASSIGN_ERR_STR(__VA_ARGS__);                                               \
+    RET_ERROR_IN_ARG()
+#define RET_ERR_STR_AND_FREE(...)                                              \
+    ASSIGN_ERR_STR_AND_FREE(__VA_ARGS__);                                      \
+    RET_ERROR_IN_ARG()
 
 DLLEXPORT int freecell_solver_user_cmd_line_read_cmd_line_preset(
     void *const instance, const char *const preset_name,
@@ -244,21 +241,11 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
     int *last_arg, const int file_nesting_count,
     freecell_solver_str_t opened_files_dir)
 {
-#if 0
-    fprintf(stderr, "Eclibri: <instance=%p argc=%d argv=%p start_arg=%d "
-                    "known_parameters=%p error_string=%p last_arg=%p "
-                    "file_nesting_count=%d opened_files_dir=%p>\n",
-        instance, argc, argv, start_arg, known_parameters, error_string,
-        last_arg, file_nesting_count, opened_files_dir);
-    fflush(stderr);
-#endif
 #ifdef FCS_WITH_ERROR_STRS
     char *fcs_user_errstr;
     *error_string = NULL;
 #endif
-
     const freecell_solver_str_t *const arg_argc = &(argv[argc]);
-
     freecell_solver_str_t *arg = &(argv[start_arg]);
     for (; arg < arg_argc; arg++)
     {
@@ -300,102 +287,73 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             *last_arg = arg - &(argv[0]);
             return FCS_CMD_LINE_UNRECOGNIZED_OPTION;
 
-#define PROCESS_OPT_ARG()                                                      \
-    {                                                                          \
-        if ((++arg) == arg_argc)                                               \
-        {                                                                      \
-            *last_arg = arg - 1 - &(argv[0]);                                  \
-            return FCS_CMD_LINE_PARAM_WITH_NO_ARG;                             \
-        }                                                                      \
-    }
-
-#define RET_ERROR_IN_ARG()                                                     \
-    *last_arg = arg - &(argv[0]);                                              \
-    return FCS_CMD_LINE_ERROR_IN_ARG;
-
         case FCS_OPT_MAX_DEPTH: /* STRINGS=-md|--max-depth; */
-        {
             PROCESS_OPT_ARG();
-        }
-        break;
+            break;
 
         case FCS_OPT_MAX_ITERS: /* STRINGS=-mi|--max-iters; */
-        {
             PROCESS_OPT_ARG();
-
+#ifndef FCS_WITHOUT_MAX_NUM_STATES
             freecell_solver_user_limit_iterations_long(instance, atol((*arg)));
-        }
-        break;
+#endif
+            break;
 
         case FCS_OPT_TESTS_ORDER: /* STRINGS=-to|--tests-order; */
-        {
             PROCESS_OPT_ARG();
 
-            if (freecell_solver_user_set_tests_order(
-                    instance, (*arg)FCS__PASS_ERR_STR(&fcs_user_errstr)) != 0)
+            if (freecell_solver_user_set_depth_tests_order(instance, 0,
+                    (*arg)FCS__PASS_ERR_STR(&fcs_user_errstr)) != 0)
             {
-                ASSIGN_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
+                RET_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
                     "Error in tests' order!\n%s\n", fcs_user_errstr);
-                RET_ERROR_IN_ARG();
             }
-        }
-        break;
+            break;
 
         case FCS_OPT_FREECELLS_NUM: /* STRINGS=--freecells-num; */
-        {
             PROCESS_OPT_ARG();
 #ifndef HARD_CODED_NUM_FREECELLS
-            if (freecell_solver_user_set_num_freecells(
-                    instance, atoi((*arg))) != 0)
+            if (freecell_solver_user_set_num_freecells(instance, atoi(*arg)) !=
+                0)
             {
-                ASSIGN_ERR_STR(error_string,
+                RET_ERR_STR(error_string,
                     "Error! The freecells\' number "
                     "exceeds the maximum of %i.\n"
                     "Recompile the program if you wish to have more.\n",
                     freecell_solver_user_get_max_num_freecells());
-                RET_ERROR_IN_ARG();
             }
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_STACKS_NUM: /* STRINGS=--stacks-num; */
-        {
             PROCESS_OPT_ARG();
 #ifndef HARD_CODED_NUM_STACKS
-            if (freecell_solver_user_set_num_stacks(instance, atoi((*arg))) !=
-                0)
+            if (freecell_solver_user_set_num_stacks(instance, atoi(*arg)) != 0)
             {
-                ASSIGN_ERR_STR(error_string,
+                RET_ERR_STR(error_string,
                     "Error! The stacks\' number "
                     "exceeds the maximum of %i.\n"
                     "Recompile the program if you wish to have more.\n",
                     freecell_solver_user_get_max_num_stacks());
-                RET_ERROR_IN_ARG();
             }
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_DECKS_NUM: /* STRINGS=--decks-num; */
-        {
             PROCESS_OPT_ARG();
 #ifndef HARD_CODED_NUM_DECKS
-            if (freecell_solver_user_set_num_decks(instance, atoi((*arg))) != 0)
+            if (freecell_solver_user_set_num_decks(instance, atoi(*arg)) != 0)
             {
-                ASSIGN_ERR_STR(error_string,
+                RET_ERR_STR(error_string,
                     "Error! The decks\' number "
                     "exceeds the maximum of %i.\n"
                     "Recopmile the program if you wish to have more.\n",
                     freecell_solver_user_get_max_num_decks());
-                RET_ERROR_IN_ARG();
             }
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_SEQUENCES_ARE_BUILT_BY: /* STRINGS=--sequences-are-built-by;
-                                                */
+                                              */
         {
             PROCESS_OPT_ARG();
 #ifndef FCS_FREECELL_ONLY
@@ -410,19 +368,16 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
         break;
 
         case FCS_OPT_SEQUENCE_MOVE: /* STRINGS=--sequence-move; */
-        {
             PROCESS_OPT_ARG();
 #ifndef FCS_FREECELL_ONLY
             freecell_solver_user_set_sequence_move(
                 instance, !strcmp((*arg), "unlimited"));
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_EMPTY_STACKS_FILLED_BY: /* STRINGS=--empty-stacks-filled-by;
-                                                */
+                                              */
         {
-
             PROCESS_OPT_ARG();
 #ifndef FCS_FREECELL_ONLY
             const char *const s = (*arg);
@@ -438,7 +393,6 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
         case FCS_OPT_GAME: /* STRINGS=--game|--preset|-g; */
         {
             PROCESS_OPT_ARG();
-
 #ifndef FCS_FREECELL_ONLY
             const char *const string_arg = (*arg);
             switch (freecell_solver_user_apply_preset(instance, string_arg))
@@ -447,35 +401,31 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                 break;
 
             case FCS_PRESET_CODE_NOT_FOUND:
-                ASSIGN_ERR_STR(
+                RET_ERR_STR(
                     error_string, "Unknown game \"%s\"!\n\n", string_arg);
-                RET_ERROR_IN_ARG();
 
             case FCS_PRESET_CODE_FREECELLS_EXCEED_MAX:
-                ASSIGN_ERR_STR(error_string,
+                RET_ERR_STR(error_string,
                     "The game \"%s\" exceeds the maximal number "
                     "of freecells in the program.\n"
                     "Modify the file \"config.h\" and recompile, "
                     "if you wish to solve one of its boards.\n",
                     string_arg);
-                RET_ERROR_IN_ARG();
 
             case FCS_PRESET_CODE_STACKS_EXCEED_MAX:
-                ASSIGN_ERR_STR(error_string,
+                RET_ERR_STR(error_string,
                     "The game \"%s\" exceeds the maximal number "
                     "of stacks in the program.\n"
                     "Modify the file \"config.h\" and recompile, "
                     "if you wish to solve one of its boards.\n",
                     string_arg);
-                RET_ERROR_IN_ARG();
 
             default:
-                ASSIGN_ERR_STR(error_string,
+                RET_ERR_STR(error_string,
                     "The game \"%s\" exceeds the limits of the program.\n"
                     "Modify the file \"config.h\" and recompile, if you wish "
                     "to solve one of its boards.\n",
                     string_arg);
-                RET_ERROR_IN_ARG();
             }
 #endif
         }
@@ -484,9 +434,7 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
         case FCS_OPT_METHOD: /* STRINGS=-me|--method; */
         {
             int int_method;
-
             PROCESS_OPT_ARG();
-
             const_AUTO(s, (*arg));
             if (!strcmp(s, "soft-dfs") || !strcmp(s, "dfs"))
             {
@@ -512,10 +460,8 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
 #endif
             else
             {
-                ASSIGN_ERR_STR(
+                RET_ERR_STR(
                     error_string, "Unknown solving method \"%s\".\n", s);
-
-                RET_ERROR_IN_ARG();
             }
 
             freecell_solver_user_set_solving_method(instance, int_method);
@@ -525,19 +471,14 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
         case FCS_OPT_BEFS_WEIGHTS: /* STRINGS=-asw|--a-star-weights; */
         {
             PROCESS_OPT_ARG();
+            double befs_weights[FCS_NUM_BEFS_WEIGHTS];
+            const char *const s = *arg;
 
+            fc_solve_set_weights(s, strchr(s, '\0'), befs_weights);
+            for (int i = 0; i < FCS_NUM_BEFS_WEIGHTS; i++)
             {
-                double befs_weights[FCS_NUM_BEFS_WEIGHTS];
-
-                const char *const s = *arg;
-
-                fc_solve_set_weights(s, strchr(s, '\0'), befs_weights);
-
-                for (int i = 0; i < FCS_NUM_BEFS_WEIGHTS; i++)
-                {
-                    freecell_solver_user_set_a_star_weight(
-                        instance, i, befs_weights[i]);
-                }
+                freecell_solver_user_set_a_star_weight(
+                    instance, i, befs_weights[i]);
             }
         }
         break;
@@ -549,69 +490,63 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             break;
 
         case FCS_OPT_SEED: /* STRINGS=-seed; */
-        {
             PROCESS_OPT_ARG();
-
-            freecell_solver_user_set_random_seed(instance, atoi((*arg)));
-        }
-        break;
+            freecell_solver_user_set_random_seed(instance, atoi(*arg));
+            break;
 
         case FCS_OPT_MAX_STORED_STATES: /* STRINGS=-mss|--max-stored-states; */
-        {
             PROCESS_OPT_ARG();
 
+#ifndef FCS_DISABLE_NUM_STORED_STATES
             freecell_solver_user_limit_num_states_in_collection_long(
                 instance, atol((*arg)));
-        }
-        break;
+#endif
+            break;
 
         case FCS_OPT_TRIM_MAX_STORED_STATES: /* STRINGS=-tmss|--trim-max-stored-states;
-                                                */
-        {
+                                              */
             PROCESS_OPT_ARG();
-
+#ifndef FCS_DISABLE_NUM_STORED_STATES
 #ifndef FCS_WITHOUT_TRIM_MAX_STORED_STATES
             freecell_solver_set_stored_states_trimming_limit(
                 instance, atol((*arg)));
 #endif
-        }
-        break;
+#endif
+            break;
 
         case FCS_OPT_NEXT_INSTANCE: /* STRINGS=-ni|--next-instance; */
-        {
+#ifdef FCS_WITH_NI
             freecell_solver_user_next_instance(instance);
-        }
-        break;
+#endif
+            break;
 
         case FCS_OPT_NEXT_FLARE: /* STRINGS=-nf|--next-flare; */
 #ifdef FCS_WITH_FLARES
-        {
             freecell_solver_user_next_flare(instance);
-        }
 #endif
-        break;
+            break;
 
         case FCS_OPT_NEXT_SOFT_THREAD: /* STRINGS=-nst|--next-soft-thread; */
         case FCS_OPT_NEXT_HARD_THREAD: /* STRINGS=-nht|--next-hard-thread; */
-        {
-            if ((opt == FCS_OPT_NEXT_SOFT_THREAD)
+            if (
+#ifdef FCS_SINGLE_HARD_THREAD
+                freecell_solver_user_next_soft_thread(instance)
+#else
+                (opt == FCS_OPT_NEXT_SOFT_THREAD)
                     ? freecell_solver_user_next_soft_thread(instance)
-                    : freecell_solver_user_next_hard_thread(instance))
+                    : freecell_solver_user_next_hard_thread(instance)
+#endif
+            )
             {
-                ASSIGN_ERR_STR(error_string, "%s",
+                RET_ERR_STR(error_string, "%s",
                     "The maximal number of soft threads has been exceeded\n");
-                RET_ERROR_IN_ARG();
             }
-        }
-        break;
+            break;
 
         case FCS_OPT_SOFT_THREAD_STEP: /* STRINGS=-step|--soft-thread-step; */
-        {
             PROCESS_OPT_ARG();
-
-            freecell_solver_user_set_soft_thread_step(instance, atoi((*arg)));
-        }
-        break;
+            freecell_solver_user_set_soft_thread_step(instance, atoi(*arg));
+            break;
 
         case FCS_OPT_REPARENT_STATES: /* STRINGS=--reparent-states; */
 #ifndef FCS_HARD_CODE_REPARENT_STATES_AS_FALSE
@@ -626,65 +561,50 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             break;
 
         case FCS_OPT_ST_NAME: /* STRINGS=--st-name; */
-        {
             PROCESS_OPT_ARG();
-
             freecell_solver_user_set_soft_thread_name(instance, (*arg));
-        }
-        break;
+            break;
 
         case FCS_OPT_FLARE_NAME: /* STRINGS=--flare-name; */
-        {
             PROCESS_OPT_ARG();
 #ifdef FCS_WITH_FLARES
             freecell_solver_user_set_flare_name(instance, (*arg));
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_PRELUDE: /* STRINGS=--prelude; */
-        {
             PROCESS_OPT_ARG();
-
             freecell_solver_user_set_hard_thread_prelude(instance, (*arg));
-        }
-        break;
+            break;
 
         case FCS_OPT_FLARES_PLAN: /* STRINGS=--flares-plan; */
-        {
             PROCESS_OPT_ARG();
 #ifdef FCS_WITH_FLARES
             freecell_solver_user_set_flares_plan(instance, (*arg));
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_FLARES_ITERS_FACTOR: /* STRINGS=-fif|--flares-iters-factor;
-                                             */
-        {
+                                           */
             PROCESS_OPT_ARG();
 #ifdef FCS_WITH_FLARES
             freecell_solver_user_set_flares_iters_factor(instance, atof(*arg));
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_OPTIMIZATION_TESTS_ORDER: /* STRINGS=-opt-to|--optimization-tests-order;
-                                                  */
-        {
+                                                */
             PROCESS_OPT_ARG();
 #ifdef FCS_WITH_MOVES
             if (freecell_solver_user_set_optimization_scan_tests_order(
                     instance, (*arg)FCS__PASS_ERR_STR(&fcs_user_errstr)) != 0)
             {
-                ASSIGN_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
+                RET_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
                     "Error in the optimization scan's tests' order!\n%s\n",
                     fcs_user_errstr);
-                RET_ERROR_IN_ARG();
             }
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_SCANS_SYNERGY: /* STRINGS=--scans-synergy; */
         {
@@ -702,10 +622,8 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             }
             else
             {
-                ASSIGN_ERR_STR(error_string,
+                RET_ERR_STR(error_string,
                     "Unknown scans' synergy type \"%s\"!\n", (*arg));
-
-                RET_ERROR_IN_ARG();
             }
 
             freecell_solver_user_set_scans_synergy(instance, value);
@@ -714,32 +632,26 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
         break;
 
         case FCS_OPT_RESET: /* STRINGS=--reset; */
-        {
+#ifndef FCS_BREAK_BACKWARD_COMPAT_1
             freecell_solver_user_reset(instance);
-        }
-        break;
+#endif
+            break;
 
         case FCS_OPT_READ_FROM_FILE: /* STRINGS=--read-from-file; */
-        {
             PROCESS_OPT_ARG();
 
-            if (file_nesting_count == 0)
+            if (file_nesting_count != 0)
             {
-                /* do nothing */
-            }
-            else
-            {
-                freecell_solver_str_t s;
                 FILE *f;
                 int num_file_args_to_skip = 0;
-                s = (*arg);
+                freecell_solver_str_t s = (*arg);
                 while (isdigit(*s))
                 {
                     s++;
                 }
                 if (*s == ',')
                 {
-                    num_file_args_to_skip = atoi((*arg));
+                    num_file_args_to_skip = atoi(*arg);
                     s++;
                 }
 
@@ -759,29 +671,26 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                 }
 
                 /* Try to open from the local path */
-                if (f == NULL)
+                if (!f)
                 {
                     f = fopen(s, "rt");
                 }
 
                 /* If we still could not open it return an error */
-                if (f == NULL)
+                if (!f)
                 {
-                    ASSIGN_ERR_STR(error_string,
+                    RET_ERR_STR(error_string,
                         "Could not open file \"%s\"!\nQuitting.\n", s);
-                    RET_ERROR_IN_ARG();
                 }
                 fseek(f, 0, SEEK_END);
                 const long file_len = ftell(f);
                 char *buffer = SMALLOC(buffer, file_len + 1);
                 if (buffer == NULL)
                 {
-                    ASSIGN_ERR_STR(error_string, "%s",
+                    fclose(f);
+                    RET_ERR_STR(error_string, "%s",
                         "Could not allocate enough memory "
                         "to parse the file. Quitting.\n");
-                    fclose(f);
-
-                    RET_ERROR_IN_ARG();
                 }
                 fseek(f, 0, SEEK_SET);
                 buffer[fread(buffer, 1, file_len, f)] = '\0';
@@ -799,9 +708,10 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                                     *)(args_man.argv + num_file_args_to_skip),
                             0, known_parameters, callback,
                             callback_context FCS__PASS_ERR_STR(error_string),
-                            last_arg, ((file_nesting_count < 0)
-                                              ? file_nesting_count
-                                              : (file_nesting_count - 1)),
+                            last_arg,
+                            ((file_nesting_count < 0)
+                                    ? file_nesting_count
+                                    : (file_nesting_count - 1)),
                             opened_files_dir);
 
                     if (!((ret == FCS_CMD_LINE_UNRECOGNIZED_OPTION) ||
@@ -815,8 +725,7 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
                 }
                 fc_solve_args_man_free(&args_man);
             }
-        }
-        break;
+            break;
 
         case FCS_OPT_LOAD_CONFIG: /* STRINGS=-l|--load-config; */
         {
@@ -830,9 +739,8 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
             switch (ret)
             {
             case FCS_CMD_LINE_ERROR_IN_ARG:
-                ASSIGN_ERR_STR(error_string,
+                RET_ERR_STR(error_string,
                     "Unable to load the \"%s\" configuration!\n", (*arg));
-                RET_ERROR_IN_ARG();
 
             case FCS_CMD_LINE_UNRECOGNIZED_OPTION:
             case FCS_CMD_LINE_OK:
@@ -846,110 +754,90 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
 
         case FCS_OPT_DEPTH_TESTS_ORDER: /* STRINGS=-dto|--depth-tests-order; */
         case FCS_OPT_DEPTH_TESTS_ORDER_2: /* STRINGS=-dto2|--depth-tests-order2;
-                                             */
+                                           */
         {
             PROCESS_OPT_ARG();
-
+            int min_depth = 0;
+            const char *s = (*arg);
+            while (isdigit(*s))
             {
-                int min_depth = 0;
+                s++;
+            }
+            if (*s == ',')
+            {
+                min_depth = atoi(*arg);
+                s++;
+            }
+            else
+            {
+                s = (*arg);
+            }
 
-                const char *s = (*arg);
-                while (isdigit(*s))
-                {
-                    s++;
-                }
-                if (*s == ',')
-                {
-                    min_depth = atoi((*arg));
-                    s++;
-                }
-                else
-                {
-                    s = (*arg);
-                }
-
-                /* Note: passing (*arg) instead of s to the
-                 * set_depth_tests_order is likely wrong, but when doing
-                 * it right, the tests fail.
-                 * */
-                if (freecell_solver_user_set_depth_tests_order(instance,
-                        min_depth,
-                        ((opt == FCS_OPT_DEPTH_TESTS_ORDER_2) ? s : (*arg))
-                            FCS__PASS_ERR_STR(&fcs_user_errstr)))
-                {
-                    ASSIGN_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
-                        "Error in depth tests' order!\n%s\n", fcs_user_errstr);
-                    RET_ERROR_IN_ARG();
-                }
+            /* Note: passing (*arg) instead of s to the
+             * set_depth_tests_order is likely wrong, but when doing
+             * it right, the tests fail.
+             * */
+            if (freecell_solver_user_set_depth_tests_order(instance, min_depth,
+                    ((opt == FCS_OPT_DEPTH_TESTS_ORDER_2) ? s : (*arg))
+                        FCS__PASS_ERR_STR(&fcs_user_errstr)))
+            {
+                RET_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
+                    "Error in depth tests' order!\n%s\n", fcs_user_errstr);
             }
         }
         break;
 
         case FCS_OPT_SET_PRUNING: /* STRINGS=-sp|--set-pruning; */
-        {
             PROCESS_OPT_ARG();
+#ifndef FCS_ENABLE_PRUNE__R_TF__UNCOND
             if (freecell_solver_user_set_pruning(
                     instance, (*arg)FCS__PASS_ERR_STR(&fcs_user_errstr)) != 0)
             {
-                ASSIGN_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
+                RET_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
                     "Error in the optimization scan's pruning!\n%s\n",
                     fcs_user_errstr);
-                RET_ERROR_IN_ARG();
             }
-        }
-        break;
+#endif
+            break;
 
         case FCS_OPT_CACHE_LIMIT: /* STRINGS=--cache-limit; */
-        {
             PROCESS_OPT_ARG();
-
 #ifndef FCS_BREAK_BACKWARD_COMPAT_1
 #ifdef FCS_RCS_STATES
             freecell_solver_user_set_cache_limit(instance, atol(*arg));
 #endif
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_FLARES_CHOICE: /* STRINGS=--flares-choice; */
-        {
             PROCESS_OPT_ARG();
-
 #ifdef FCS_WITH_FLARES
 #ifndef FCS_WITHOUT_FC_PRO_MOVES_COUNT
             if (freecell_solver_user_set_flares_choice(instance, (*arg)) != 0)
             {
-                ASSIGN_ERR_STR(error_string,
+                RET_ERR_STR(error_string,
                     "Unknown flares choice argument '%s'.\n", (*arg));
-                RET_ERROR_IN_ARG();
             }
 #endif
 #endif
-        }
-        break;
+            break;
 
         case FCS_OPT_PATSOLVE_X_PARAM: /* STRINGS=--patsolve-x-param; */
         {
             PROCESS_OPT_ARG();
-
 #ifndef FCS_DISABLE_PATSOLVE
+            int position, x_param_val;
+            if (sscanf((*arg), "%d,%d", &position, &x_param_val) != 2)
             {
-                int position, x_param_val;
-                if (sscanf((*arg), "%d,%d", &position, &x_param_val) != 2)
-                {
-                    ASSIGN_ERR_STR(error_string, "%s",
-                        "Wrong format for --patsolve-x-param");
-                    RET_ERROR_IN_ARG();
-                }
-                if (freecell_solver_user_set_patsolve_x_param(instance,
-                        position,
-                        x_param_val FCS__PASS_ERR_STR(&fcs_user_errstr)) != 0)
-                {
-                    ASSIGN_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
-                        "Error in patsolve X param setting!\n%s\n",
-                        fcs_user_errstr);
-                    RET_ERROR_IN_ARG();
-                }
+                RET_ERR_STR(
+                    error_string, "%s", "Wrong format for --patsolve-x-param");
+            }
+            if (freecell_solver_user_set_patsolve_x_param(instance, position,
+                    x_param_val FCS__PASS_ERR_STR(&fcs_user_errstr)) != 0)
+            {
+                RET_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
+                    "Error in patsolve X param setting!\n%s\n",
+                    fcs_user_errstr);
             }
 #endif
         }
@@ -958,26 +846,20 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
         case FCS_OPT_PATSOLVE_Y_PARAM: /* STRINGS=--patsolve-y-param; */
         {
             PROCESS_OPT_ARG();
-
 #ifndef FCS_DISABLE_PATSOLVE
+            int position;
+            double y_param_val;
+            if (sscanf((*arg), "%d,%lf", &position, &y_param_val) != 2)
             {
-                int position;
-                double y_param_val;
-                if (sscanf((*arg), "%d,%lf", &position, &y_param_val) != 2)
-                {
-                    ASSIGN_ERR_STR(error_string, "%s",
-                        "Wrong format for --patsolve-y-param");
-                    RET_ERROR_IN_ARG();
-                }
-                if (freecell_solver_user_set_patsolve_y_param(instance,
-                        position,
-                        y_param_val FCS__PASS_ERR_STR(&fcs_user_errstr)) != 0)
-                {
-                    ASSIGN_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
-                        "Error in patsolve Y param setting!\n%s\n",
-                        fcs_user_errstr);
-                    RET_ERROR_IN_ARG();
-                }
+                RET_ERR_STR(
+                    error_string, "%s", "Wrong format for --patsolve-y-param");
+            }
+            if (freecell_solver_user_set_patsolve_y_param(instance, position,
+                    y_param_val FCS__PASS_ERR_STR(&fcs_user_errstr)) != 0)
+            {
+                RET_ERR_STR_AND_FREE(fcs_user_errstr, error_string,
+                    "Error in patsolve Y param setting!\n%s\n",
+                    fcs_user_errstr);
             }
 #endif
         }
@@ -998,7 +880,7 @@ DLLEXPORT int freecell_solver_user_cmd_line_parse_args(void *instance, int argc,
     freecell_solver_str_t *known_parameters,
     freecell_solver_user_cmd_line_known_commands_callback_t callback,
     void *const callback_context FCS__PASS_ERR_STR(char **error_string),
-    int *last_arg)
+    int *const last_arg)
 {
     return freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
         instance, argc, argv, start_arg, known_parameters, callback,
