@@ -16,9 +16,6 @@
 extern "C" {
 #endif
 
-#include "config.h"
-
-#include "state.h"
 #include "meta_alloc.h"
 #include "fcs_enums.h"
 
@@ -28,8 +25,8 @@ extern "C" {
 #include "dbm_calc_derived_iface.h"
 #include "dbm_lru_cache.h"
 
-static int fc_solve_compare_lru_cache_keys(
-    const void *void_a, const void *void_b, void *context GCC_UNUSED)
+static int fc_solve_compare_lru_cache_keys(const void *const void_a,
+    const void *const void_b, void *context GCC_UNUSED)
 {
 #define GET_PARAM(p) ((((const fcs_cache_key_info_t *)(p))->key))
     return memcmp(
@@ -37,33 +34,27 @@ static int fc_solve_compare_lru_cache_keys(
 #undef GET_PARAM
 }
 
+static inline void cache_destroy_key(fcs_cache_key_info_t *cache_key)
+{
+    for (; cache_key; cache_key = RECYCLE_BIN_NEXT(cache_key))
+    {
+        free(cache_key->moves_to_key);
+        cache_key->moves_to_key = NULL;
+    }
+}
+
 static inline void cache_destroy(fcs_lru_cache_t *cache)
 {
-    {
-#define NUM_CHAINS_TO_RELEASE 2
-        fcs_cache_key_info_t *const to_release[NUM_CHAINS_TO_RELEASE] = {
-            cache->recycle_bin, cache->lowest_pri};
-
-        for (size_t i = 0; i < COUNT(to_release); i++)
-        {
-            fcs_cache_key_info_t *cache_key;
-
-            for (cache_key = to_release[i]; cache_key;
-                 cache_key = RECYCLE_BIN_NEXT(cache_key))
-            {
-                free(cache_key->moves_to_key);
-                cache_key->moves_to_key = NULL;
-            }
-        }
-#undef NUM_CHAINS_TO_RELEASE
-    }
+    cache_destroy_key(cache->recycle_bin);
+    cache_destroy_key(cache->lowest_pri);
     fc_solve_kaz_tree_destroy(cache->kaz_tree);
     fc_solve_compact_allocator_finish(
         &(cache->states_values_to_keys_allocator));
 }
 
-static inline void cache_init(fcs_lru_cache_t *cache,
-    long max_num_elements_in_cache, fcs_meta_compact_allocator_t *meta_alloc)
+static inline void cache_init(fcs_lru_cache_t *const cache,
+    const long max_num_elements_in_cache,
+    fcs_meta_compact_allocator_t *const meta_alloc)
 {
 #if (FCS_RCS_CACHE_STORAGE == FCS_RCS_CACHE_STORAGE_JUDY)
     cache->states_values_to_keys_map = ((Pvoid_t)NULL);
@@ -128,9 +119,7 @@ static inline fcs_cache_key_info_t *cache_insert(fcs_lru_cache_t *cache,
     const fcs_fcc_move_t final_move)
 {
     fcs_cache_key_info_t *cache_key;
-    dict_t *kaz_tree;
-
-    kaz_tree = cache->kaz_tree;
+    var_AUTO(kaz_tree, cache->kaz_tree);
 
     if (cache->count_elements_in_cache >= cache->max_num_elements_in_cache)
     {
@@ -145,15 +134,14 @@ static inline fcs_cache_key_info_t *cache_insert(fcs_lru_cache_t *cache,
         cache_key = (fcs_cache_key_info_t *)fcs_compact_alloc_ptr(
             &(cache->states_values_to_keys_allocator), sizeof(*cache_key));
         cache_key->moves_to_key = NULL;
-        cache->count_elements_in_cache++;
+        ++cache->count_elements_in_cache;
     }
 
     cache_key->key = *key;
     if (moves_to_parent)
     {
-        size_t len;
         fcs_fcc_move_t *moves;
-        len = strlen((const char *)moves_to_parent);
+        const_AUTO(len, strlen((const char *)moves_to_parent));
         cache_key->moves_to_key = moves =
             SREALLOC(cache_key->moves_to_key, len + 1 + 1);
         memcpy(moves, moves_to_parent, len);
