@@ -17,14 +17,8 @@
 extern "C" {
 #endif
 
-#include <stdlib.h>
-
-#include "config.h"
-
-#include "state.h"
 #include "meta_alloc.h"
 #include "fcs_enums.h"
-
 #include "fcs_dllexport.h"
 #include "dbm_common.h"
 #include "delta_states.h"
@@ -45,6 +39,23 @@ typedef struct fcs_derived_state_struct
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
 } fcs_derived_state_t;
 
+static inline void fcs_derived_state_list__recycle(
+    fcs_derived_state_t **const p_recycle_bin,
+    fcs_derived_state_t **const p_list)
+{
+    var_AUTO(list, *p_list);
+    var_AUTO(bin, *p_recycle_bin);
+    while (list)
+    {
+        var_AUTO(list_next, list->next);
+        list->next = bin;
+        bin = list;
+        list = list_next;
+    }
+    *p_recycle_bin = bin;
+    *p_list = NULL;
+}
+
 #define MAKE_MOVE(src, dest) ((fcs_fcc_move_t)((src) | ((dest) << 4)))
 #define COL2MOVE(idx) (idx)
 #define FREECELL2MOVE(idx) (idx + 8)
@@ -103,8 +114,6 @@ static inline void fc_solve_add_to_irrev_moves_bitmask(
         ((((*by_rank_ptr) >> (suit_times_two)) & ((1 << 2) - 1)) + count);
     *by_rank_ptr &= (~((((unsigned char)0x3) << (suit_times_two))));
     *by_rank_ptr |= (new_count << (suit_times_two));
-
-    return;
 }
 
 #define COMMIT_NEW_STATE_WITH_COUNT(src, dest, count, moved_card)              \
@@ -134,32 +143,31 @@ static inline int calc_foundation_to_put_card_on(
     const fcs_card_t card)
 {
 #ifndef FCS_FREECELL_ONLY
-    /* needed by the macros. */
     const int sequences_are_built_by = CALC_SEQUENCES_ARE_BUILT_BY();
 #endif
-    for (int deck = 0; deck < INSTANCE_DECKS_NUM; deck++)
+    const int_fast32_t rank = fcs_card_rank(card);
+    const int_fast32_t suit = fcs_card_suit(card);
+    for (uint_fast32_t deck = 0; deck < INSTANCE_DECKS_NUM; ++deck)
     {
-        if (fcs_foundation_value(*ptr_state,
-                (deck << 2) + fcs_card_suit(card)) == fcs_card_rank(card) - 1)
+        if (fcs_foundation_value(*ptr_state, (deck << 2) + suit) == rank - 1)
         {
-            int other_deck_idx;
+            uint_fast32_t other_deck_idx;
 
             for (other_deck_idx = 0; other_deck_idx < (INSTANCE_DECKS_NUM << 2);
-                 other_deck_idx++)
+                 ++other_deck_idx)
             {
                 if (fcs_foundation_value(*ptr_state, other_deck_idx) <
-                    fcs_card_rank(card) - 2 -
+                    rank - 2 -
                         (FCS__SEQS_ARE_BUILT_BY_RANK()
                                 ? 0
-                                : ((other_deck_idx & 0x1) ==
-                                      (fcs_card_suit(card) & 0x1))))
+                                : ((other_deck_idx & 0x1) == (suit & 0x1))))
                 {
                     break;
                 }
             }
             if (other_deck_idx == (INSTANCE_DECKS_NUM << 2))
             {
-                return (deck << 2) + fcs_card_suit(card);
+                return (deck << 2) + suit;
             }
         }
     }
@@ -173,10 +181,9 @@ typedef struct
 } fcs_fcc_moves_seq_allocator_t;
 
 static inline fcs_fcc_moves_list_item_t *fc_solve_fcc_alloc_moves_list_item(
-    fcs_fcc_moves_seq_allocator_t *allocator)
+    fcs_fcc_moves_seq_allocator_t *const allocator)
 {
     fcs_fcc_moves_list_item_t *new_item;
-
     if (allocator->recycle_bin)
     {
         allocator->recycle_bin = (new_item = allocator->recycle_bin)->next;
@@ -187,13 +194,14 @@ static inline fcs_fcc_moves_list_item_t *fc_solve_fcc_alloc_moves_list_item(
             allocator->allocator, sizeof(*new_item));
     }
     new_item->next = NULL;
-
     return new_item;
 }
 
 #define FROM_COL_IS_REVERSIBLE_MOVE()                                          \
-    ((cards_num <= 1) ? TRUE : fcs_is_parent_card(card,                        \
-                                   fcs_col_get_card(col, cards_num - 2)))
+    ((cards_num <= 1)                                                          \
+            ? TRUE                                                             \
+            : fcs_is_parent_card(card, fcs_col_get_card(col, cards_num - 2)))
+#define COUNT_NON_REV(is_reversible) ((is_reversible) ? 1 : 2)
 
 /* Returns the number of amortized irreversible moves performed. */
 static inline int horne_prune(const fcs_dbm_variant_type_t local_variant,
@@ -205,45 +213,46 @@ static inline int horne_prune(const fcs_dbm_variant_type_t local_variant,
     fcs_fcc_move_t additional_moves[RANK_KING * 4 * DECKS_NUM];
     int count_moves_so_far = 0;
     int count_additional_irrev_moves = 0;
-
 #ifndef FCS_FREECELL_ONLY
     const int sequences_are_built_by = CALC_SEQUENCES_ARE_BUILT_BY();
 #endif
 
 #define the_state (init_state_kv_ptr->s)
-    int num_cards_moved;
+    uint_fast32_t num_cards_moved;
     do
     {
         num_cards_moved = 0;
         for (int stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)
         {
-            fcs_cards_column_t col = fcs_state_get_col(the_state, stack_idx);
+            var_AUTO(col, fcs_state_get_col(the_state, stack_idx));
             const int cards_num = fcs_col_len(col);
-            if (cards_num)
+            if (!cards_num)
             {
-                /* Get the top card in the stack */
-                const fcs_card_t card = fcs_col_get_card(col, cards_num - 1);
-                const int dest_foundation = calc_foundation_to_put_card_on(
-                    local_variant, &the_state, card);
-                if (dest_foundation >= 0)
+                continue;
+            }
+            /* Get the top card in the stack */
+            const fcs_card_t card = fcs_col_get_card(col, cards_num - 1);
+            const int dest_foundation =
+                calc_foundation_to_put_card_on(local_variant, &the_state, card);
+            if (dest_foundation >= 0)
+            {
+                const_AUTO(is_reversible, FROM_COL_IS_REVERSIBLE_MOVE());
+                if (!is_reversible)
                 {
-                    if (!FROM_COL_IS_REVERSIBLE_MOVE())
-                    {
-                        count_additional_irrev_moves++;
-                    }
-                    /* We can safely move it. */
-                    num_cards_moved++;
-                    fc_solve_add_to_irrev_moves_bitmask(
-                        which_irreversible_moves_bitmask, card,
-                        ((!FROM_COL_IS_REVERSIBLE_MOVE()) ? 2 : 1));
-
-                    fcs_col_pop_top(col);
-
-                    fcs_increment_foundation(the_state, dest_foundation);
-
-                    additional_moves[count_moves_so_far++] = MAKE_MOVE(
-                        COL2MOVE(stack_idx), FOUND2MOVE(dest_foundation));
+                    count_additional_irrev_moves++;
                 }
+                /* We can safely move it. */
+                num_cards_moved++;
+                fc_solve_add_to_irrev_moves_bitmask(
+                    which_irreversible_moves_bitmask, card,
+                    COUNT_NON_REV(is_reversible));
+
+                fcs_col_pop_top(col);
+
+                fcs_increment_foundation(the_state, dest_foundation);
+
+                additional_moves[count_moves_so_far++] =
+                    MAKE_MOVE(COL2MOVE(stack_idx), FOUND2MOVE(dest_foundation));
             }
         }
 
@@ -330,6 +339,15 @@ static inline int horne_prune__simple(
     return horne_prune(local_variant, init_state_kv_ptr, &no_use, NULL, NULL);
 }
 
+static inline fcs_bool_t card_cannot_be_placed(const fcs_state_t *const s,
+    const uint16_t ds, const fcs_card_t card, const int sequences_are_built_by)
+{
+    const_AUTO(col, fcs_state_get_col(*s, ds));
+    const_AUTO(col_len, fcs_col_len(col));
+    return ((col_len == 0) ||
+            (!fcs_is_parent_card(card, fcs_col_get_card(col, col_len - 1))));
+}
+
 #define the_state (init_state_kv_ptr->s)
 static inline fcs_bool_t is_state_solved(
     const fcs_dbm_variant_type_t local_variant,
@@ -355,17 +373,8 @@ static inline fcs_bool_t instance_solver_thread_calc_derived_states(
     const fcs_bool_t perform_horne_prune)
 {
     fcs_derived_state_t *ptr_new_state;
-    int stack_idx, cards_num, ds;
-    fcs_cards_column_t col, dest_col;
-    fcs_card_t dest_card;
-    int deck, suit;
     int empty_stack_idx = -1;
-
-#ifndef FCS_FREECELL_ONLY
-    /* needed by the macros. */
     const int sequences_are_built_by = CALC_SEQUENCES_ARE_BUILT_BY();
-#endif
-
 #define new_state (ptr_new_state->state.s)
     if (is_state_solved(local_variant, init_state_kv_ptr))
     {
@@ -373,68 +382,64 @@ static inline fcs_bool_t instance_solver_thread_calc_derived_states(
     }
 
     /* Move top stack cards to foundations. */
-    for (stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)
+    for (int stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)
     {
-        col = fcs_state_get_col(the_state, stack_idx);
-        cards_num = fcs_col_len(col);
-        if (cards_num)
-        {
-            /* Get the top card in the stack */
-            const_AUTO(card, fcs_col_get_card(col, cards_num - 1));
-            suit = fcs_card_suit(card);
-            for (deck = 0; deck < INSTANCE_DECKS_NUM; deck++)
-            {
-
-                if (fcs_foundation_value(the_state, deck * 4 + suit) ==
-                    fcs_card_rank(card) - 1)
-                {
-                    /* We can put it there */
-                    BEGIN_NEW_STATE()
-
-                    fcs_state_pop_col_top(&new_state, stack_idx);
-                    fcs_increment_foundation(new_state, deck * 4 + suit);
-
-                    COMMIT_NEW_STATE_WITH_COUNT(COL2MOVE(stack_idx),
-                        FOUND2MOVE(suit),
-                        (FROM_COL_IS_REVERSIBLE_MOVE() ? 1 : 2), card)
-                }
-            }
-        }
-        else
+        const_AUTO(col, fcs_state_get_col(the_state, stack_idx));
+        const_AUTO(cards_num, fcs_col_len(col));
+        if (cards_num == 0)
         {
             empty_stack_idx = stack_idx;
+            continue;
+        }
+        /* Get the top card in the stack */
+        const_AUTO(card, fcs_col_get_card(col, cards_num - 1));
+        const_AUTO(suit, fcs_card_suit(card));
+        for (int deck = 0; deck < INSTANCE_DECKS_NUM; deck++)
+        {
+            if (fcs_foundation_value(the_state, deck * 4 + suit) !=
+                fcs_card_rank(card) - 1)
+            {
+                continue;
+            }
+            /* We can put it there */
+            BEGIN_NEW_STATE()
+
+            fcs_state_pop_col_top(&new_state, stack_idx);
+            fcs_increment_foundation(new_state, deck * 4 + suit);
+
+            COMMIT_NEW_STATE_WITH_COUNT(COL2MOVE(stack_idx), FOUND2MOVE(suit),
+                COUNT_NON_REV(FROM_COL_IS_REVERSIBLE_MOVE()), card)
+            break;
         }
     }
 
     int empty_fc_idx = -1;
-#define fc_idx stack_idx
     /* Move freecell cards to foundations. */
-    for (fc_idx = 0; fc_idx < LOCAL_FREECELLS_NUM; fc_idx++)
+    for (int fc_idx = 0; fc_idx < LOCAL_FREECELLS_NUM; fc_idx++)
     {
         const_AUTO(card, fcs_freecell_card(the_state, fc_idx));
-        suit = fcs_card_suit(card);
-        if (fcs_card_is_valid(card))
-        {
-            for (deck = 0; deck < INSTANCE_DECKS_NUM; deck++)
-            {
-                if (fcs_foundation_value(the_state, deck * 4 + suit) ==
-                    fcs_card_rank(card) - 1)
-                {
-                    BEGIN_NEW_STATE()
-
-                    /* We can put it there */
-                    fcs_empty_freecell(new_state, fc_idx);
-
-                    fcs_increment_foundation(new_state, deck * 4 + suit);
-
-                    COMMIT_NEW_STATE(
-                        FREECELL2MOVE(fc_idx), FOUND2MOVE(suit), FALSE, card)
-                }
-            }
-        }
-        else
+        if (fcs_card_is_empty(card))
         {
             empty_fc_idx = fc_idx;
+            continue;
+        }
+        const_AUTO(suit, fcs_card_suit(card));
+        for (int deck = 0; deck < INSTANCE_DECKS_NUM; deck++)
+        {
+            if (fcs_foundation_value(the_state, deck * 4 + suit) !=
+                fcs_card_rank(card) - 1)
+            {
+                continue;
+            }
+            BEGIN_NEW_STATE()
+
+            /* We can put it there */
+            fcs_empty_freecell(new_state, fc_idx);
+            fcs_increment_foundation(new_state, deck * 4 + suit);
+
+            COMMIT_NEW_STATE(
+                FREECELL2MOVE(fc_idx), FOUND2MOVE(suit), FALSE, card)
+            break;
         }
     }
 
@@ -442,86 +447,56 @@ static inline fcs_bool_t instance_solver_thread_calc_derived_states(
         ((local_variant == FCS_DBM_VARIANT_BAKERS_DOZEN) ? 1 : 0);
 
     /* Move stack card on top of a parent */
-    for (stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)
+    for (int stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)
     {
-        col = fcs_state_get_col(the_state, stack_idx);
-        cards_num = fcs_col_len(col);
-        if (cards_num > cards_num_min_limit)
+        const_AUTO(col, fcs_state_get_col(the_state, stack_idx));
+        const_AUTO(cards_num, fcs_col_len(col));
+        if (cards_num <= cards_num_min_limit)
         {
-            const_AUTO(card, fcs_col_get_card(col, cards_num - 1));
+            continue;
+        }
+        const_AUTO(card, fcs_col_get_card(col, cards_num - 1));
 
-            for (ds = 0; ds < LOCAL_STACKS_NUM; ds++)
+        for (int ds = 0; ds < LOCAL_STACKS_NUM; ++ds)
+        {
+            if (ds == stack_idx || card_cannot_be_placed(&the_state, ds, card,
+                                       sequences_are_built_by))
             {
-                if (ds == stack_idx)
-                {
-                    continue;
-                }
-
-                dest_col = fcs_state_get_col(the_state, ds);
-
-                if (fcs_col_len(dest_col) > 0)
-                {
-                    dest_card =
-                        fcs_col_get_card(dest_col, fcs_col_len(dest_col) - 1);
-                    if (fcs_is_parent_card(card, dest_card))
-                    {
-                        /* Let's move it */
-                        BEGIN_NEW_STATE()
-
-                        {
-                            fcs_cards_column_t new_src_col;
-                            fcs_cards_column_t new_dest_col;
-
-                            new_src_col =
-                                fcs_state_get_col(new_state, stack_idx);
-                            new_dest_col = fcs_state_get_col(new_state, ds);
-
-                            fcs_col_pop_top(new_src_col);
-                            fcs_col_push_card(new_dest_col, card);
-                        }
-
-                        COMMIT_NEW_STATE(COL2MOVE(stack_idx), COL2MOVE(ds),
-                            FROM_COL_IS_REVERSIBLE_MOVE(), card)
-                    }
-                }
+                continue;
             }
+            /* Let's move it */
+            BEGIN_NEW_STATE()
+
+            fcs_state_pop_col_top(&new_state, stack_idx);
+            fcs_state_push(&new_state, ds, card);
+
+            COMMIT_NEW_STATE(COL2MOVE(stack_idx), COL2MOVE(ds),
+                FROM_COL_IS_REVERSIBLE_MOVE(), card)
         }
     }
 
     /* Move freecell card on top of a parent */
-    for (fc_idx = 0; fc_idx < LOCAL_FREECELLS_NUM; fc_idx++)
+    for (int fc_idx = 0; fc_idx < LOCAL_FREECELLS_NUM; fc_idx++)
     {
         const_AUTO(card, fcs_freecell_card(the_state, fc_idx));
-        if (fcs_card_is_valid(card))
+        if (!fcs_card_is_valid(card))
         {
-            for (ds = 0; ds < LOCAL_STACKS_NUM; ds++)
+            continue;
+        }
+        for (int ds = 0; ds < LOCAL_STACKS_NUM; ++ds)
+        {
+            if (card_cannot_be_placed(
+                    &the_state, ds, card, sequences_are_built_by))
             {
-                dest_col = fcs_state_get_col(the_state, ds);
-
-                if (fcs_col_len(dest_col) > 0)
-                {
-                    dest_card =
-                        fcs_col_get_card(dest_col, fcs_col_len(dest_col) - 1);
-                    if (fcs_is_parent_card(card, dest_card))
-                    {
-                        /* Let's move it */
-                        BEGIN_NEW_STATE()
-
-                        {
-                            fcs_cards_column_t new_dest_col;
-
-                            new_dest_col = fcs_state_get_col(new_state, ds);
-
-                            fcs_col_push_card(new_dest_col, card);
-
-                            fcs_empty_freecell(new_state, fc_idx);
-                        }
-
-                        COMMIT_NEW_STATE(
-                            FREECELL2MOVE(fc_idx), COL2MOVE(ds), TRUE, card)
-                    }
-                }
+                continue;
             }
+            /* Let's move it */
+            BEGIN_NEW_STATE()
+
+            fcs_state_push(&new_state, ds, card);
+            fcs_empty_freecell(new_state, fc_idx);
+
+            COMMIT_NEW_STATE(FREECELL2MOVE(fc_idx), COL2MOVE(ds), TRUE, card)
         }
     }
 
@@ -529,40 +504,31 @@ static inline fcs_bool_t instance_solver_thread_calc_derived_states(
         (empty_stack_idx >= 0))
     {
         /* Stack Card to Empty Stack */
-        for (stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)
+        for (int stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)
         {
-            col = fcs_state_get_col(the_state, stack_idx);
-            cards_num = fcs_col_len(col);
+            const_AUTO(col, fcs_state_get_col(the_state, stack_idx));
+            const_AUTO(cards_num, fcs_col_len(col));
             /* Bug fix: if there's only one card in a column, there's no
              * point moving it to a new empty column.
              * */
-            if (cards_num > 1)
+            if (cards_num <= 1)
             {
-                const_AUTO(card, fcs_col_get_card(col, cards_num - 1));
-                /* Let's move it */
-                {
-                    BEGIN_NEW_STATE()
-
-                    {
-                        fcs_cards_column_t new_src_col;
-
-                        new_src_col = fcs_state_get_col(new_state, stack_idx);
-
-                        fcs_col_pop_top(new_src_col);
-
-                        fcs_cards_column_t empty_stack_col =
-                            fcs_state_get_col(new_state, empty_stack_idx);
-                        fcs_col_push_card(empty_stack_col, card);
-                    }
-                    COMMIT_NEW_STATE(COL2MOVE(stack_idx),
-                        COL2MOVE(empty_stack_idx),
-                        FROM_COL_IS_REVERSIBLE_MOVE(), card)
-                }
+                continue;
             }
+
+            const_AUTO(card, fcs_col_get_card(col, cards_num - 1));
+            /* Let's move it */
+            BEGIN_NEW_STATE()
+
+            fcs_state_pop_col_top(&new_state, stack_idx);
+            fcs_state_push(&new_state, empty_stack_idx, card);
+
+            COMMIT_NEW_STATE(COL2MOVE(stack_idx), COL2MOVE(empty_stack_idx),
+                FROM_COL_IS_REVERSIBLE_MOVE(), card)
         }
 
         /* Freecell card -> Empty Stack. */
-        for (fc_idx = 0; fc_idx < LOCAL_FREECELLS_NUM; fc_idx++)
+        for (int fc_idx = 0; fc_idx < LOCAL_FREECELLS_NUM; fc_idx++)
         {
             const_AUTO(card, fcs_freecell_card(the_state, fc_idx));
             if (fcs_card_is_empty(card))
@@ -570,79 +536,58 @@ static inline fcs_bool_t instance_solver_thread_calc_derived_states(
                 continue;
             }
 
-            {
-                BEGIN_NEW_STATE()
+            BEGIN_NEW_STATE()
 
-                {
-                    fcs_cards_column_t new_dest_col;
-                    new_dest_col =
-                        fcs_state_get_col(new_state, empty_stack_idx);
-                    fcs_col_push_card(new_dest_col, card);
-                    fcs_empty_freecell(new_state, fc_idx);
-                }
+            fcs_state_push(&new_state, empty_stack_idx, card);
+            fcs_empty_freecell(new_state, fc_idx);
 
-                COMMIT_NEW_STATE(FREECELL2MOVE(fc_idx),
-                    COL2MOVE(empty_stack_idx), TRUE, card);
-            }
+            COMMIT_NEW_STATE(
+                FREECELL2MOVE(fc_idx), COL2MOVE(empty_stack_idx), TRUE, card);
         }
     }
 
     if (empty_fc_idx >= 0)
     {
         /* Stack Card to Empty Freecell */
-        for (stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)
+        for (int stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)
         {
-            col = fcs_state_get_col(the_state, stack_idx);
-            cards_num = fcs_col_len(col);
-            if (cards_num > cards_num_min_limit)
+            const_AUTO(col, fcs_state_get_col(the_state, stack_idx));
+            const_AUTO(cards_num, fcs_col_len(col));
+            if (cards_num <= cards_num_min_limit)
             {
-                const_AUTO(card, fcs_col_get_card(col, cards_num - 1));
-                /* Let's move it */
-                {
-                    BEGIN_NEW_STATE()
-
-                    {
-                        fcs_cards_column_t new_src_col;
-
-                        new_src_col = fcs_state_get_col(new_state, stack_idx);
-
-                        fcs_col_pop_top(new_src_col);
-
-                        fcs_put_card_in_freecell(new_state, empty_fc_idx, card);
-                    }
-                    COMMIT_NEW_STATE(COL2MOVE(stack_idx),
-                        FREECELL2MOVE(empty_fc_idx),
-                        FROM_COL_IS_REVERSIBLE_MOVE(), card)
-                }
+                continue;
             }
+            const_AUTO(card, fcs_col_get_card(col, cards_num - 1));
+            /* Let's move it */
+            BEGIN_NEW_STATE()
+
+            fcs_state_pop_col_top(&new_state, stack_idx);
+            fcs_put_card_in_freecell(new_state, empty_fc_idx, card);
+
+            COMMIT_NEW_STATE(COL2MOVE(stack_idx), FREECELL2MOVE(empty_fc_idx),
+                FROM_COL_IS_REVERSIBLE_MOVE(), card)
         }
     }
-#undef fc_idx
-
     /* Perform Horne's Prune on all the states,
      * or just set their num irreversible moves counts.
      * */
+    for (var_AUTO(derived_iter, *derived_list); derived_iter;
+         derived_iter = derived_iter->next)
     {
-        fcs_derived_state_t *derived_iter;
-
-        for (derived_iter = (*derived_list); derived_iter;)
-        {
-            derived_iter->num_non_reversible_moves_including_prune =
-                (derived_iter->core_irreversible_moves_count +
-                    (perform_horne_prune
-                            ? horne_prune(local_variant, &(derived_iter->state),
-                                  &(derived_iter
-                                          ->which_irreversible_moves_bitmask),
-                                  NULL, NULL)
-                            : 0));
-            derived_iter = derived_iter->next;
-        }
+        derived_iter->num_non_reversible_moves_including_prune =
+            (derived_iter->core_irreversible_moves_count +
+                (perform_horne_prune
+                        ? horne_prune(local_variant, &(derived_iter->state),
+                              &(derived_iter->which_irreversible_moves_bitmask),
+                              NULL, NULL)
+                        : 0));
     }
 
     return FALSE;
 }
 
 #undef FROM_COL_IS_REVERSIBLE_MOVE
+#undef COUNT_NON_REV
 #undef the_state
 #undef new_state
 
