@@ -12,10 +12,6 @@
  * This is compiled into a shared library and then loaded by Inline::C,
  * ctypes, etc.
  */
-#include "dll_thunk.h"
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
 #include <assert.h>
 
 #include "config.h"
@@ -28,18 +24,15 @@
  * The char * returned is malloc()ed and should be free()ed.
  */
 DLLEXPORT int fc_solve_user_INTERNAL_calc_derived_states_wrapper(
-    fcs_dbm_variant_type_t local_variant, const char *init_state_str_proto,
-    int *const num_out_derived_states,
+    const fcs_dbm_variant_type_t local_variant,
+    const char *init_state_str_proto, int *const num_out_derived_states,
     fcs_derived_state_debug_t **out_derived_states,
     const fcs_bool_t perform_horne_prune)
 {
     fcs_state_keyval_pair_t init_state;
     fcs_encoded_state_buffer_t enc_state;
-    fcs_state_locs_struct_t locs;
     fcs_derived_state_t *derived_list = NULL;
     fcs_derived_state_t *derived_list_recycle_bin = NULL;
-    fcs_compact_allocator_t allocator;
-    fcs_meta_compact_allocator_t meta_alloc;
     size_t states_count = 0;
     fcs_derived_state_t *iter;
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
@@ -54,11 +47,13 @@ DLLEXPORT int fc_solve_user_INTERNAL_calc_derived_states_wrapper(
         ,
         FCS_SEQ_BUILT_BY_ALTERNATE_COLOR
 #endif
-        );
+    );
 
     fcs_init_and_encode_state(&delta, local_variant, &(init_state), &enc_state);
 
+    fcs_meta_compact_allocator_t meta_alloc;
     fc_solve_meta_compact_allocator_init(&meta_alloc);
+    fcs_compact_allocator_t allocator;
     fc_solve_compact_allocator_init(&allocator, &meta_alloc);
 
     instance_solver_thread_calc_derived_states(local_variant, &init_state, NULL,
@@ -79,6 +74,7 @@ DLLEXPORT int fc_solve_user_INTERNAL_calc_derived_states_wrapper(
         SMALLOC(debug_ret, states_count);
     *(out_derived_states) = debug_ret;
 
+    fcs_state_locs_struct_t locs;
     fc_solve_init_locs(&locs);
 
     iter = derived_list;
@@ -97,15 +93,13 @@ DLLEXPORT int fc_solve_user_INTERNAL_calc_derived_states_wrapper(
         FCS__RENDER_STATE(debug_ret[idx].state_string, &(iter->state.s), &locs);
         /* TODO : Put something meaningful there by passing it to the function.
          */
-        idx++;
+        ++idx;
         iter = iter->next;
     }
 
     assert(idx == states_count);
-
     fc_solve_compact_allocator_finish(&allocator);
     fc_solve_meta_compact_allocator_finish(&meta_alloc);
-
     fc_solve_delta_stater_release(&delta);
 
     return 0;
@@ -127,21 +121,18 @@ DLLEXPORT void fc_solve_user_INTERNAL_free_derived_states(
  * The char * returned is malloc()ed and should be free()ed.
  */
 DLLEXPORT int fc_solve_user_INTERNAL_perform_horne_prune(
-    fcs_dbm_variant_type_t local_variant, const char *init_state_str_proto,
-    char **ret_state_s)
+    const fcs_dbm_variant_type_t local_variant,
+    const char *init_state_str_proto, char **ret_state_s)
 {
     fcs_state_keyval_pair_t init_state;
-    fcs_state_locs_struct_t locs;
-    int prune_ret;
-
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
-
+    fcs_state_locs_struct_t locs;
     fc_solve_init_locs(&locs);
 
     fc_solve_initial_user_state_to_c(init_state_str_proto, &init_state,
         FREECELLS_NUM, STACKS_NUM, DECKS_NUM, indirect_stacks_buffer);
 
-    prune_ret = horne_prune__simple(local_variant, &init_state);
+    const_AUTO(prune_ret, horne_prune__simple(local_variant, &init_state));
     *ret_state_s = SMALLOC(*ret_state_s, 1000);
     FCS__RENDER_STATE(*ret_state_s, &(init_state.s), &locs);
 
