@@ -14,6 +14,7 @@
 
 #include "dbm_move_to_string.h"
 #include "render_state.h"
+#include "try_param.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -68,10 +69,7 @@ static inline void pre_cache_insert(fcs_pre_cache_t *pre_cache,
 static inline fcs_bool_t pre_cache_does_key_exist(
     fcs_pre_cache_t *pre_cache, fcs_encoded_state_buffer_t *key)
 {
-    fcs_pre_cache_key_val_pair_t to_check;
-
-    to_check.key = *key;
-
+    fcs_pre_cache_key_val_pair_t to_check = {.key = *key};
     return (
         fc_solve_kaz_tree_lookup_value(pre_cache->kaz_tree, &to_check) != NULL);
 }
@@ -80,25 +78,19 @@ static inline void cache_populate_from_pre_cache(
     fcs_lru_cache_t *const cache, fcs_pre_cache_t *const pre_cache)
 {
 #ifdef FCS_DBM_USE_LIBAVL
-    dict_t *kaz_tree;
     struct rb_traverser trav;
-    dict_key_t item;
-
-    kaz_tree = pre_cache->kaz_tree;
+    var_AUTO(kaz_tree, pre_cache->kaz_tree);
     rb_t_init(&trav, kaz_tree);
 
-    for (item = rb_t_first(&trav, kaz_tree); item; item = rb_t_next(&trav))
+    for (dict_key_t item = rb_t_first(&trav, kaz_tree); item;
+         item = rb_t_next(&trav))
     {
         cache_insert(cache, &(((fcs_pre_cache_key_val_pair_t *)(item))->key),
             NULL, '\0');
     }
 #else
-    dnode_t *node;
-    dict_t *kaz_tree;
-
-    kaz_tree = pre_cache->kaz_tree;
-
-    for (node = fc_solve_kaz_tree_first(kaz_tree); node;
+    var_AUTO(kaz_tree, pre_cache->kaz_tree);
+    for (dnode_t *node = fc_solve_kaz_tree_first(kaz_tree); node;
          node = fc_solve_kaz_tree_next(kaz_tree, node))
     {
         cache_insert(
@@ -141,31 +133,37 @@ static inline void pre_cache_offload_and_reset(fcs_pre_cache_t *const pre_cache,
 
 #endif
 
-#endif /* FCS_DBM_WITHOUT_CACHES */
+#endif // FCS_DBM_WITHOUT_CACHES
 
 typedef struct fcs_dbm_solver_thread_struct fcs_dbm_solver_thread_t;
 
+typedef struct
+{
+    fcs_dbm_solver_thread_t *thread;
+} thread_arg_t;
+
 static inline void instance_check_key(fcs_dbm_solver_thread_t *const thread,
     fcs_dbm_solver_instance_t *const instance, const int key_depth,
     fcs_encoded_state_buffer_t *const key, fcs_dbm_record_t *const parent,
     const unsigned char move,
     const fcs_which_moves_bitmask_t *const which_irreversible_moves_bitmask
-#ifdef FCS_DBM_CACHE_ONLY
+#ifndef FCS_DBM_WITHOUT_CACHES
     ,
     const fcs_fcc_move_t *moves_to_parent
 #endif
-    );
+);
 
 static inline fcs_bool_t instance_check_multiple_keys(
-    fcs_dbm_solver_thread_t *thread, fcs_dbm_solver_instance_t *instance,
+    fcs_dbm_solver_thread_t *const thread,
+    fcs_dbm_solver_instance_t *const instance,
     fcs_dbm__cache_store__common_t *const cache_store,
     fcs_meta_compact_allocator_t *const meta_alloc, fcs_derived_state_t **lists,
     size_t batch_size
-#ifdef FCS_DBM_CACHE_ONLY
+#ifndef FCS_DBM_WITHOUT_CACHES
     ,
     const fcs_fcc_move_t *moves_to_parent
 #endif
-    )
+)
 {
     /* Small optimization in case the list is empty. */
     if (batch_size == 1 && !lists[0])
@@ -180,11 +178,11 @@ static inline fcs_bool_t instance_check_multiple_keys(
             instance_check_key(thread, instance, CHECK_KEY_CALC_DEPTH(),
                 &(list->key), list->parent, list->move,
                 &(list->which_irreversible_moves_bitmask)
-#ifdef FCS_DBM_CACHE_ONLY
+#ifndef FCS_DBM_WITHOUT_CACHES
                     ,
                 moves_to_parent
 #endif
-                );
+            );
 #ifndef FCS_DBM_WITHOUT_CACHES
 #ifndef FCS_DBM_CACHE_ONLY
             if (cache_store->pre_cache.count_elements >=
@@ -218,9 +216,7 @@ static inline fcs_bool_t instance_check_multiple_keys(
 
 static void instance_print_stats(fcs_dbm_solver_instance_t *const instance)
 {
-    fcs_portable_time_t mytime;
-    FCS_GET_TIME(mytime);
-
+    const_AUTO(mytime, fcs_get_time());
     FILE *const out_fh = instance->common.out_fh;
     fprintf(out_fh,
         ("Reached %ld ; States-in-collection: %ld ; Time: " FCS_T_FMT "\n"
@@ -237,19 +233,19 @@ static void instance_print_stats(fcs_dbm_solver_instance_t *const instance)
 #ifdef DEBUG_FOO
 
 static inline void instance_debug_out_state(
-    fcs_dbm_solver_instance_t *instance, fcs_encoded_state_buffer_t *enc_state)
+    fcs_dbm_solver_instance_t *const instance,
+    fcs_encoded_state_buffer_t *enc_state)
 {
     fcs_state_keyval_pair_t state;
     fcs_state_locs_struct_t locs;
-    DECLARE_IND_BUF_T(indirect_stacks_buffer)
-
     fc_solve_init_locs(&locs);
+    DECLARE_IND_BUF_T(indirect_stacks_buffer)
     const_AUTO(local_variant, instance->common.variant);
     /* Handle item. */
     fc_solve_delta_stater_decode_into_state(
         &global_delta_stater, enc_state->s, &state, indirect_stacks_buffer);
 
-    char state_str[1000];
+    fcs_render_state_str_t state_str;
     FCS__RENDER_STATE(state_str, &(state.s), &locs);
     fprintf(instance->common.out_fh, "Found State:\n<<<\n%s>>>\n", state_str);
     fflush(instance->common.out_fh);
@@ -287,8 +283,6 @@ static void calc_trace(fcs_dbm_record_t *const ptr_initial_record,
 #undef GROW_BY
     *ptr_trace_num = trace_num;
     *ptr_trace = trace;
-
-    return;
 }
 
 static inline void mark_and_sweep_old_states(
@@ -372,8 +366,8 @@ static inline void mark_and_sweep_old_states(
 
 static inline void instance_increment(fcs_dbm_solver_instance_t *const instance)
 {
-    instance->common.count_of_items_in_queue--;
-    instance->common.queue_num_extracted_and_processed++;
+    --instance->common.count_of_items_in_queue;
+    ++instance->common.queue_num_extracted_and_processed;
     if (++instance->common.count_num_processed % 100000 == 0)
     {
         instance_print_stats(instance);
@@ -405,20 +399,6 @@ static inline void fcs_dbm__cache_store__init(
 #endif
 }
 
-static const char *try_argv_param(
-    const int argc, char **const argv, int *const arg, const char *const flag)
-{
-    if (!strcmp(argv[*arg], flag))
-    {
-        if (++(*arg) == argc)
-        {
-            fc_solve_err("%s came without an argument!\n", flag);
-        }
-        return argv[*arg];
-    }
-    return NULL;
-}
-
 typedef struct
 {
     fcs_dbm_variant_type_t local_variant;
@@ -436,20 +416,16 @@ static const fcs_dbm_common_input_t fcs_dbm_common_input_init = {
     .caches_delta = 1000000,
     .num_threads = 2};
 
-#define TRY_PARAM(s) try_argv_param(argc, argv, arg, s)
-#define TRY_P(s) try_argv_param(argc, argv, &arg, s)
 static inline fcs_bool_t fcs_dbm__extract_common_from_argv(const int argc,
     char **const argv, int *const arg, fcs_dbm_common_input_t *const inp)
 {
     const char *param;
     if ((param = TRY_PARAM("--pre-cache-max-count")))
     {
-        const_AUTO(pre_cache_max_count, atol(param));
-        if (pre_cache_max_count < 1000)
+        if ((inp->pre_cache_max_count = atol(param)) < 1000)
         {
             fc_solve_err("--pre-cache-max-count must be at least 1,000.\n");
         }
-        inp->pre_cache_max_count = pre_cache_max_count;
         return TRUE;
     }
     else if ((param = TRY_PARAM("--game")))
@@ -475,12 +451,10 @@ static inline fcs_bool_t fcs_dbm__extract_common_from_argv(const int argc,
     }
     else if ((param = TRY_PARAM("--num-threads")))
     {
-        const_AUTO(num_threads, (size_t)atoi(param));
-        if (num_threads < 1)
+        if ((inp->num_threads = (size_t)atoi(param)) < 1)
         {
             fc_solve_err("--num-threads must be at least 1.\n");
         }
-        inp->num_threads = num_threads;
         return TRUE;
     }
     else if ((param = TRY_PARAM("--iters-delta-limit")))
@@ -490,12 +464,10 @@ static inline fcs_bool_t fcs_dbm__extract_common_from_argv(const int argc,
     }
     else if ((param = TRY_PARAM("--caches-delta")))
     {
-        const_AUTO(caches_delta, atol(param));
-        if (caches_delta < 1000)
+        if ((inp->caches_delta = atol(param)) < 1000)
         {
             fc_solve_err("--caches-delta must be at least 1,000.\n");
         }
-        inp->caches_delta = caches_delta;
         return TRUE;
     }
     else if ((param = TRY_PARAM("--dbm-store-path")))
@@ -520,8 +492,6 @@ static inline fcs_dbm_record_t *cache_store__has_key(
     {
         return NULL;
     }
-#endif
-#ifndef FCS_DBM_CACHE_ONLY
     else if (fc_solve_dbm_store_does_key_exist(cache_store->store, key->s))
     {
         cache_insert(&(cache_store->cache), key, NULL, '\0');
@@ -543,11 +513,10 @@ static inline fcs_cache_key_info_t *cache_store__insert_key(
     const unsigned char move GCC_UNUSED)
 {
 #ifndef FCS_DBM_CACHE_ONLY
-    pre_cache_insert(&(instance->cache_store.pre_cache), key, parent);
+    pre_cache_insert(&(cache_store->pre_cache), key, &(parent->key));
     return NULL;
 #else
-    return cache_insert(
-        &(instance->cache_store.cache), key, moves_to_parent, move);
+    return cache_insert(&(cache_store->cache), key, moves_to_parent, move);
 #endif
 }
 #endif
