@@ -49,9 +49,8 @@ static inline void instance_init(fcs_dbm_solver_instance_t *const instance,
         inp->caches_delta);
 }
 
-static inline void instance_recycle(fcs_dbm_solver_instance_t *instance)
+static inline void instance_recycle(fcs_dbm_solver_instance_t *const instance)
 {
-
     fcs_offloading_queue__destroy(&(instance->queue));
 #ifdef FCS_DBM_USE_OFFLOADING_QUEUE
     fcs_offloading_queue__init(
@@ -67,7 +66,7 @@ static inline void instance_recycle(fcs_dbm_solver_instance_t *instance)
     instance->common.count_of_items_in_queue = 0;
 }
 
-static inline void instance_destroy(fcs_dbm_solver_instance_t *instance)
+static inline void instance_destroy(fcs_dbm_solver_instance_t *const instance)
 {
     fcs_offloading_queue__destroy(&(instance->queue));
     DESTROY_CACHE(instance);
@@ -86,7 +85,7 @@ static inline void instance_check_key(
     ,
     const fcs_fcc_move_t *moves_to_parent
 #endif
-    )
+)
 {
     fcs_dbm_record_t *token;
     if ((token = cache_store__has_key(&instance->cache_store, key, parent)))
@@ -94,18 +93,17 @@ static inline void instance_check_key(
 #ifndef FCS_DBM_WITHOUT_CACHES
         fcs_cache_key_info_t *cache_key = cache_store__insert_key(
             &(instance->cache_store), key, parent, moves_to_parent, move);
-        endif
 #endif
-            /* Now insert it into the queue. */
+        /* Now insert it into the queue. */
 
-            instance->common.count_of_items_in_queue++;
-        instance->common.num_states_in_collection++;
+        ++instance->common.count_of_items_in_queue;
+        ++instance->common.num_states_in_collection;
 
         instance_debug_out_state(instance, &(token->key));
 
         fcs_offloading_queue__insert(
             &(instance->queue), ((fcs_offloading_queue_item_t *)(&token)));
-        instance->common.count_of_items_in_queue++;
+        ++instance->common.count_of_items_in_queue;
     }
 }
 
@@ -115,28 +113,19 @@ struct fcs_dbm_solver_thread_struct
     fc_solve_delta_stater_t delta_stater;
 };
 
-typedef struct
-{
-    fcs_dbm_solver_thread_t *thread;
-} thread_arg_t;
-
 static void *instance_run_solver_thread(void *const void_arg)
 {
     fcs_dbm_queue_item_t physical_item;
     fcs_state_keyval_pair_t state;
-#ifdef DEBUG_OUT
-    fcs_state_locs_struct_t locs;
-#endif
     fcs_dbm_record_t *token = NULL;
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
 
-    thread_arg_t *const arg = (thread_arg_t *)void_arg;
-    fcs_dbm_solver_thread_t *const thread = arg->thread;
+    const_AUTO(thread, ((thread_arg_t *)void_arg)->thread);
     fcs_dbm_solver_instance_t *const instance = thread->instance;
     fc_solve_delta_stater_t *const delta_stater = &(thread->delta_stater);
 
     fcs_dbm_queue_item_t *item = NULL, *prev_item = NULL;
-    int queue_num_extracted_and_processed = 0;
+    long queue_num_extracted_and_processed = 0;
 
     fcs_compact_allocator_t derived_list_allocator;
     fc_solve_compact_allocator_init(
@@ -144,11 +133,11 @@ static void *instance_run_solver_thread(void *const void_arg)
     fcs_derived_state_t *derived_list_recycle_bin = NULL;
     fcs_derived_state_t *derived_list = NULL;
     FILE *const out_fh = instance->common.out_fh;
-
     fcs_dbm_variant_type_t local_variant = instance->common.variant;
 
     TRACE("%s\n", "instance_run_solver_thread start");
 #ifdef DEBUG_OUT
+    fcs_state_locs_struct_t locs;
     fc_solve_init_locs(&locs);
 #endif
     while (1)
@@ -158,7 +147,7 @@ static void *instance_run_solver_thread(void *const void_arg)
 
         if (prev_item)
         {
-            instance->common.queue_num_extracted_and_processed--;
+            --instance->common.queue_num_extracted_and_processed;
         }
 
         if (instance->common.should_terminate == DONT_TERMINATE)
@@ -190,7 +179,9 @@ static void *instance_run_solver_thread(void *const void_arg)
         {
             /* Sleep until more items become available in the
              * queue. */
-            usleep(5000);
+            const struct timespec ts = {.tv_sec = 0, .tv_nsec = 5000000};
+            struct timespec no_use;
+            nanosleep(&ts, &no_use);
         }
         else
         {
@@ -211,7 +202,7 @@ static void *instance_run_solver_thread(void *const void_arg)
             }
 
             /* Encode all the states. */
-            for (fcs_derived_state_t *derived_iter = derived_list; derived_iter;
+            for (var_AUTO(derived_iter, derived_list); derived_iter;
                  derived_iter = derived_iter->next)
             {
                 fcs_init_and_encode_state(delta_stater, local_variant,
@@ -225,17 +216,10 @@ static void *instance_run_solver_thread(void *const void_arg)
                 ,
                 item->moves_to_key
 #endif
-                );
+            );
 
-            /* Now recycle the derived_list */
-            while (derived_list)
-            {
-                fcs_derived_state_t *const derived_list_next =
-                    derived_list->next;
-                derived_list->next = derived_list_recycle_bin;
-                derived_list_recycle_bin = derived_list;
-                derived_list = derived_list_next;
-            }
+            fcs_derived_state_list__recycle(
+                &derived_list_recycle_bin, &derived_list);
             /* End handle item. */
         }
         /* End of main thread loop */
@@ -243,16 +227,14 @@ static void *instance_run_solver_thread(void *const void_arg)
     }
 
     fc_solve_compact_allocator_finish(&(derived_list_allocator));
-
     TRACE("%s\n", "instance_run_solver_thread end");
-
     return NULL;
 }
 
 static fcs_bool_t populate_instance_with_intermediate_input_line(
     fcs_dbm_solver_instance_t *const instance,
     fc_solve_delta_stater_t *const delta,
-    fcs_state_keyval_pair_t *const init_state_ptr, char *line,
+    fcs_state_keyval_pair_t *const init_state_ptr, char *const line,
     const long line_num, fcs_encoded_state_buffer_t *const parent_state_enc)
 {
     fcs_encoded_state_buffer_t final_stack_encoded_state;
@@ -293,7 +275,7 @@ static fcs_bool_t populate_instance_with_intermediate_input_line(
     }
 
     /* Skip the '|'. */
-    s_ptr++;
+    ++s_ptr;
 
     FCS_STATE__DUP_keyval_pair(running_state, *init_state_ptr);
 
@@ -311,7 +293,7 @@ static fcs_bool_t populate_instance_with_intermediate_input_line(
     running_parent = fc_solve_dbm_store_insert_key_value(
         instance->cache_store.store, &(running_key), running_parent, TRUE);
 #endif
-    instance->common.num_states_in_collection++;
+    ++instance->common.num_states_in_collection;
 
     int hex_digits;
     while (sscanf(s_ptr, "%2X,", &hex_digits) == 1)
@@ -331,10 +313,8 @@ static fcs_bool_t populate_instance_with_intermediate_input_line(
         /* Extract the card from the source. */
         if (src < 8)
         {
-            fcs_cards_column_t src_col;
-            src_col = fcs_state_get_col(the_state, src);
-            src_card = fcs_col_get_card(src_col, fcs_col_len(src_col) - 1);
-            fcs_col_pop_top(src_col);
+            var_AUTO(src_col, fcs_state_get_col(the_state, src));
+            fcs_col_pop_card(src_col, src_card);
         }
         else
         {
@@ -355,10 +335,7 @@ static fcs_bool_t populate_instance_with_intermediate_input_line(
         /* Apply src_card to dest. */
         if (dest < 8)
         {
-            fcs_cards_column_t dest_col;
-            dest_col = fcs_state_get_col(the_state, dest);
-
-            fcs_col_push_card(dest_col, src_card);
+            fcs_state_push(&the_state, dest, src_card);
         }
         else
         {
@@ -395,7 +372,7 @@ static fcs_bool_t populate_instance_with_intermediate_input_line(
             return FALSE;
         }
 #endif
-        instance->common.num_states_in_collection++;
+        ++instance->common.num_states_in_collection;
 
         /* We need to do the round-trip from encoding back
          * to decoding, because the order can change after
@@ -417,7 +394,7 @@ static fcs_bool_t populate_instance_with_intermediate_input_line(
     }
     fcs_offloading_queue__insert(
         &(instance->queue), (const fcs_offloading_queue_item_t *)(&token));
-    instance->common.count_of_items_in_queue++;
+    ++instance->common.count_of_items_in_queue;
 
     return TRUE;
 }
@@ -427,7 +404,7 @@ static fcs_bool_t populate_instance_with_intermediate_input_line(
 static void init_thread(fcs_dbm_solver_thread_t *const thread) {}
 static void free_thread(fcs_dbm_solver_thread_t *const thread) {}
 
-static void instance_run_all_threads(fcs_dbm_solver_instance_t *instance,
+static void instance_run_all_threads(fcs_dbm_solver_instance_t *const instance,
     fcs_state_keyval_pair_t *init_state, const size_t num_threads)
 {
     const_AUTO(threads,
@@ -447,8 +424,6 @@ static void instance_run_all_threads(fcs_dbm_solver_instance_t *instance,
     }
 
     dbm__free_threads(instance, num_threads, threads, free_thread);
-
-    return;
 }
 
 /* Returns if the process should terminate. */
@@ -478,9 +453,7 @@ static fcs_bool_t handle_and_destroy_instance_solution(
         if (instance->common.should_terminate == QUEUE_TERMINATE)
         {
             fcs_dbm_queue_item_t physical_item;
-            fcs_dbm_queue_item_t *item;
-
-            item = &physical_item;
+            fcs_dbm_queue_item_t *const item = &physical_item;
 
             while (fcs_offloading_queue__extract(
                 &(instance->queue), (fcs_offloading_queue_item_t *)(&token)))
@@ -498,59 +471,50 @@ static fcs_bool_t handle_and_destroy_instance_solution(
                     fprintf(out_fh, "%.2X", (int)item->key.s[1 + i]);
                 }
 #endif
-
                 fprintf(out_fh, "%s", "|");
                 fflush(out_fh);
 
 #ifdef FCS_DBM_CACHE_ONLY
+                fcs_fcc_move_t *move_ptr = item->moves_to_key;
+                if (move_ptr)
                 {
-                    fcs_fcc_move_t *move_ptr = item->moves_to_key;
-                    if (move_ptr)
+                    while (*(move_ptr))
                     {
-                        while (*(move_ptr))
-                        {
-                            fprintf(out_fh, "%.2X,", *(move_ptr));
-                            move_ptr++;
-                        }
+                        fprintf(out_fh, "%.2X,", *(move_ptr));
+                        move_ptr++;
                     }
                 }
 #else
-                {
-                    int trace_num;
-                    fcs_encoded_state_buffer_t *trace;
-
-                    calc_trace(token, &trace, &trace_num);
+                int trace_num;
+                fcs_encoded_state_buffer_t *trace;
+                calc_trace(token, &trace, &trace_num);
 
 /*
  * We stop at 1 because the deepest state does not contain
  * a move (as it is the ultimate state).
  * */
 #define PENULTIMATE_DEPTH 1
-                    for (int i = trace_num - 1; i >= PENULTIMATE_DEPTH; i--)
-                    {
-                        fprintf(out_fh, "%.2X,",
-                            get_move_from_parent_to_child(
-                                instance, delta, trace[i], trace[i - 1]));
-                    }
-#undef PENULTIMATE_DEPTH
-                    free(trace);
+                for (int i = trace_num - 1; i >= PENULTIMATE_DEPTH; i--)
+                {
+                    fprintf(out_fh, "%.2X,",
+                        get_move_from_parent_to_child(
+                            instance, delta, trace[i], trace[i - 1]));
                 }
+#undef PENULTIMATE_DEPTH
+                free(trace);
 #endif
                 fprintf(out_fh, "\n");
                 fflush(out_fh);
 #ifdef DEBUG_OUT
-                {
-                    fcs_state_keyval_pair_t state;
-                    fcs_state_locs_struct_t locs;
-                    DECLARE_IND_BUF_T(indirect_stacks_buffer)
+                fcs_state_keyval_pair_t state;
+                fcs_state_locs_struct_t locs;
+                fc_solve_init_locs(&locs);
+                DECLARE_IND_BUF_T(indirect_stacks_buffer)
 
-                    fc_solve_init_locs(&locs);
+                fc_solve_delta_stater_decode_into_state(
+                    delta, item->key.s, &state, indirect_stacks_buffer);
 
-                    fc_solve_delta_stater_decode_into_state(
-                        delta, item->key.s, &state, indirect_stacks_buffer);
-
-                    FCS__OUTPUT_STATE(out_fh, "", &(state.s), &locs);
-                }
+                FCS__OUTPUT_STATE(out_fh, "", &(state.s), &locs);
 #endif
             }
         }
@@ -566,7 +530,6 @@ static fcs_bool_t handle_and_destroy_instance_solution(
     }
 
     TRACE("%s\n", "handle_and_destroy_instance_solution end");
-
     instance_destroy(instance);
 
     return ret;
@@ -629,7 +592,7 @@ int main(int argc, char *argv[])
         ,
         FCS_SEQ_BUILT_BY_ALTERNATE_COLOR
 #endif
-        );
+    );
 
     if (intermediate_input_filename)
     {
@@ -662,7 +625,7 @@ int main(int argc, char *argv[])
         fcs_bool_t found_line;
         do
         {
-            line_num++;
+            ++line_num;
             found_line = FALSE;
 #ifdef HAVE_GETLINE
             while (getline(&line, &line_size, intermediate_in_fh) >= 0)
@@ -675,7 +638,7 @@ int main(int argc, char *argv[])
                     found_line = TRUE;
                     break;
                 }
-                line_num++;
+                ++line_num;
             }
 
             if (found_line)
@@ -706,16 +669,18 @@ int main(int argc, char *argv[])
                     else if (limit_instance.common.should_terminate ==
                              MAX_ITERS_TERMINATE)
                     {
-                        fprintf(out_fh, "Reached Max-or-more iterations of %ld "
-                                        "in intermediate-input line No. %ld.\n",
+                        fprintf(out_fh,
+                            "Reached Max-or-more iterations of %ld "
+                            "in intermediate-input line No. %ld.\n",
                             limit_instance.common.max_count_num_processed,
                             line_num);
                     }
                     else if (limit_instance.common.should_terminate ==
                              DONT_TERMINATE)
                     {
-                        fprintf(out_fh, "Pruning due to unsolvability in "
-                                        "intermediate-input line No. %ld\n",
+                        fprintf(out_fh,
+                            "Pruning due to unsolvability in "
+                            "intermediate-input line No. %ld\n",
                             line_num);
                         skip_queue_output = TRUE;
                     }
@@ -788,24 +753,21 @@ int main(int argc, char *argv[])
 
         fcs_offloading_queue__insert(
             &(instance.queue), (const fcs_offloading_queue_item_t *)&token);
-        instance.common.num_states_in_collection++;
-        instance.common.count_of_items_in_queue++;
+        ++instance.common.num_states_in_collection;
+        ++instance.common.count_of_items_in_queue;
 
         instance_run_all_threads(&instance, &init_state, NUM_THREADS());
         handle_and_destroy_instance_solution(&instance, &delta);
     }
 
     fc_solve_delta_stater_release(&delta);
-
     if (out_filename)
     {
         fclose(out_fh);
     }
-
     if (intermediate_in_fh)
     {
         fclose(intermediate_in_fh);
-        intermediate_in_fh = NULL;
     }
 
     return 0;
