@@ -18,26 +18,14 @@
 extern "C" {
 #endif
 
-#include <stdio.h>
-#include <string.h>
 #include <pthread.h>
-#include <unistd.h>
-#include <assert.h>
-#include <limits.h>
 
-#if 0
-#define DEBUG_FOO
-#endif
-
-/*
- * Define FCS_DBM_SINGLE_THREAD to have single thread-per-instance traversal.
- */
+// Define FCS_DBM_SINGLE_THREAD to have single thread-per-instance traversal.
 #if 0
 #define FCS_DBM_SINGLE_THREAD 1
 #endif
 
 #include "config.h"
-
 #undef FCS_RCS_STATES
 
 #include "lock.h"
