@@ -19,8 +19,7 @@ extern "C" {
 #endif
 
 extern char *fc_solve_user_INTERNAL_debondt_delta_states_enc_and_dec(
-    fcs_dbm_variant_type_t local_variant, const char *init_state_str_proto,
-    const char *derived_state_str_proto);
+    fcs_dbm_variant_type_t, const char *, const char *);
 
 #ifdef __cplusplus
 }
