@@ -34,8 +34,8 @@ static inline void fc_solve_delta_stater_init(
     fc_solve_debondt_delta_stater_t *const delta, fcs_state_t *const init_state,
     const size_t num_columns, const int num_freecells)
 {
-    return fc_solve_debondt_delta_stater_init(delta,
-        FCS_DBM_VARIANT_2FC_FREECELL, init_state, num_columns, num_freecells);
+    fc_solve_debondt_delta_stater_init(delta, FCS_DBM_VARIANT_2FC_FREECELL,
+        init_state, num_columns, num_freecells);
 }
 #else
 static inline void fc_solve_delta_stater_init(
@@ -43,17 +43,16 @@ static inline void fc_solve_delta_stater_init(
     const size_t num_columns, const int num_freecells,
     const int sequences_are_built_by)
 {
-    return fc_solve_debondt_delta_stater_init(delta,
-        FCS_DBM_VARIANT_2FC_FREECELL, init_state, num_columns, num_freecells,
-        sequences_are_built_by);
+    fc_solve_debondt_delta_stater_init(delta, FCS_DBM_VARIANT_2FC_FREECELL,
+        init_state, num_columns, num_freecells, sequences_are_built_by);
 }
 #endif
 #define fc_solve_delta_stater_release(a)                                       \
     fc_solve_debondt_delta_stater_release(a)
 #endif
 
-/****************************************************/
-/* compare_enc_states */
+    /****************************************************/
+    /* compare_enc_states */
 
 #ifdef FCS_DEBONDT_DELTA_STATES
 
