@@ -17,16 +17,12 @@
 extern "C" {
 #endif
 
-#include "config.h"
-#include "state.h"
-
 #include "delta_states.h"
 #include "var_base_writer.h"
 #include "var_base_reader.h"
-
 #include "dbm_common.h"
 
-#define RANK_KING 13
+#define CARD_ARRAY_LEN ((RANK_KING + 1) * FCS_NUM_SUITS)
 
 typedef struct
 {
@@ -37,15 +33,14 @@ typedef struct
     size_t num_columns;
     fcs_state_t *init_state, *derived_state;
     int bits_per_orig_cards_in_column;
-    int card_states[4 * RANK_KING];
+    int card_states[CARD_ARRAY_LEN];
     int8_t bakers_dozen_topmost_cards_lookup[((1 << 6) / 8) + 1];
     fcs_var_base_reader_t r;
     fcs_var_base_writer_t w;
 } fc_solve_debondt_delta_stater_t;
 
 extern char *fc_solve_user_INTERNAL_debondt_delta_states_enc_and_dec(
-    fcs_dbm_variant_type_t local_variant, const char *init_state_str_proto,
-    const char *derived_state_str_proto);
+    fcs_dbm_variant_type_t, const char *, const char *);
 
 #ifdef __cplusplus
 }
