@@ -15,12 +15,7 @@
  * This encoding improves upon the original delta_states.c .
  */
 
-#include "dll_thunk.h"
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
 #include "bit_rw.h"
-#include "state.h"
 #include "indirect_buffer.h"
 #include "delta_states_iface.h"
 #include "delta_states.h"
@@ -62,7 +57,7 @@ static void fc_solve_debondt_delta_stater_init(
     ,
     const int sequences_are_built_by
 #endif
-    )
+)
 {
 #ifndef FCS_FREECELL_ONLY
     self->sequences_are_built_by = sequences_are_built_by;
@@ -83,15 +78,14 @@ static void fc_solve_debondt_delta_stater_init(
     {
         for (size_t col_idx = 0; col_idx < self->num_columns; col_idx++)
         {
-            const fcs_cards_column_t col =
-                fcs_state_get_col(*init_state, col_idx);
+            const_AUTO(col, fcs_state_get_col(*init_state, col_idx));
             const int col_len = fcs_col_len(col);
 
             if (col_len > 0)
             {
-                const fcs_card_t top_card = fcs_col_get_card(col, 0);
+                const_AUTO(top_card, fcs_col_get_card(col, 0));
                 self->bakers_dozen_topmost_cards_lookup[top_card >> 3] |=
-                    (1 << top_card & (8 - 1));
+                    (1 << (top_card & (8 - 1)));
             }
         }
     }
@@ -120,12 +114,10 @@ static inline void fc_solve_debondt_delta_stater_set_derived(
     self->derived_state = state;
 }
 
-#define GET_SUIT_BIT(card) (((fcs_card_suit(card)) & 0x2) >> 1)
-
 static inline int wanted_suit_bit_opt(const fcs_card_t parent_card)
 {
-    return GET_SUIT_BIT(parent_card) ? OPT_PARENT_SUIT_MOD_IS_1
-                                     : OPT_PARENT_SUIT_MOD_IS_0;
+    return ((fcs_card_suit(parent_card) & 0x2) ? OPT_PARENT_SUIT_MOD_IS_1
+                                               : OPT_PARENT_SUIT_MOD_IS_0);
 }
 
 static inline int wanted_suit_idx_opt(const fcs_card_t parent_card)
@@ -140,7 +132,7 @@ static inline int calc_child_card_option(
     ,
     const int sequences_are_built_by
 #endif
-    )
+)
 {
     if (IS_BAKERS_DOZEN())
     {
@@ -183,33 +175,31 @@ static void fc_solve_debondt_delta_stater_encode_composite(
 
     fc_solve_debondt_delta_stater__init_card_states(self);
 
+    for (int suit_idx = 0; suit_idx < FCS_NUM_SUITS; suit_idx++)
     {
-        for (int suit_idx = 0; suit_idx < FCS_NUM_SUITS; suit_idx++)
+        const unsigned long rank = fcs_foundation_value(*derived, suit_idx);
+
+        fc_solve_var_base_writer_write(writer, FOUNDATION_BASE, rank);
+
+        const unsigned long max_rank = ((rank < 1) ? 1 : rank);
+
+        for (unsigned long r = 1; r <= max_rank; r++)
         {
-            const unsigned long rank = fcs_foundation_value(*derived, suit_idx);
-
-            fc_solve_var_base_writer_write(writer, FOUNDATION_BASE, rank);
-
-            const unsigned long max_rank = ((rank < 1) ? 1 : rank);
-
-            for (unsigned long r = 1; r <= max_rank; r++)
-            {
-#define STATE_POS(suit_idx, rank) (rank - 1 + suit_idx * RANK_KING)
-#define CARD_POS(card) (STATE_POS((fcs_card_suit(card)), (fcs_card_rank(card))))
-                self->card_states[STATE_POS(suit_idx, r)] = OPT_DONT_CARE;
-            }
+#define CARD_POS(card) ((size_t)(card))
+#define CARD_STATE(card) self->card_states[CARD_POS(card)]
+#define SET_CARD_STATE(card, opt) CARD_STATE(card) = (opt)
+#define RS_STATE(rank, suit_idx) CARD_STATE(fcs_make_card(rank, suit_idx))
+            RS_STATE(r, suit_idx) = OPT_DONT_CARE;
         }
     }
 
+    for (int fc_idx = 0; fc_idx < self->num_freecells; fc_idx++)
     {
-        for (int fc_idx = 0; fc_idx < self->num_freecells; fc_idx++)
-        {
-            const fcs_card_t card = fcs_freecell_card(*derived, fc_idx);
+        const fcs_card_t card = fcs_freecell_card(*derived, fc_idx);
 
-            if (fcs_card_is_valid(card))
-            {
-                self->card_states[CARD_POS(card)] = OPT_FREECELL;
-            }
+        if (fcs_card_is_valid(card))
+        {
+            SET_CARD_STATE(card, OPT_FREECELL);
         }
     }
 
@@ -217,41 +207,34 @@ static void fc_solve_debondt_delta_stater_encode_composite(
     {
         for (size_t col_idx = 0; col_idx < self->num_columns; col_idx++)
         {
-            const fcs_cards_column_t col = fcs_state_get_col(*derived, col_idx);
+            const_AUTO(col, fcs_state_get_col(*derived, col_idx));
             const int col_len = fcs_col_len(col);
 
-            if (col_len > 0)
+            if (!col_len)
             {
-                const fcs_card_t top_card = fcs_col_get_card(col, 0);
+                continue;
+            }
+            const_AUTO(top_card, fcs_col_get_card(col, 0));
+
+            /* Skip Aces which were already set. */
+            if (fcs_card_rank(top_card) != 1)
+            {
+                SET_CARD_STATE(top_card, OPT__BAKERS_DOZEN__ORIG_POS);
+            }
+
+            for (int pos = 1; pos < col_len; pos++)
+            {
+                const fcs_card_t parent_card = fcs_col_get_card(col, pos - 1);
+                const fcs_card_t this_card = fcs_col_get_card(col, pos);
 
                 /* Skip Aces which were already set. */
-                if (fcs_card_rank(top_card) != 1)
+                if (fcs_card_rank(this_card) != 1)
                 {
-                    self->card_states[CARD_POS(top_card)] =
-                        OPT__BAKERS_DOZEN__ORIG_POS;
-                }
-
-                for (int pos = 1; pos < col_len; pos++)
-                {
-                    const fcs_card_t parent_card =
-                        fcs_col_get_card(col, pos - 1);
-                    const fcs_card_t this_card = fcs_col_get_card(col, pos);
-
-                    /* Skip Aces which were already set. */
-                    if (fcs_card_rank(this_card) != 1)
-                    {
-                        if (fcs_card_rank(this_card) + 1 ==
-                            fcs_card_rank(parent_card))
-                        {
-                            self->card_states[CARD_POS(this_card)] =
-                                wanted_suit_idx_opt(parent_card);
-                        }
-                        else
-                        {
-                            self->card_states[CARD_POS(this_card)] =
-                                OPT__BAKERS_DOZEN__ORIG_POS;
-                        }
-                    }
+                    SET_CARD_STATE(
+                        this_card, ((fcs_card_rank(this_card) + 1 ==
+                                        fcs_card_rank(parent_card))
+                                           ? wanted_suit_idx_opt(parent_card)
+                                           : OPT__BAKERS_DOZEN__ORIG_POS));
                 }
             }
         }
@@ -260,39 +243,37 @@ static void fc_solve_debondt_delta_stater_encode_composite(
     {
         for (size_t col_idx = 0; col_idx < self->num_columns; col_idx++)
         {
-            const fcs_cards_column_t col = fcs_state_get_col(*derived, col_idx);
+            const_AUTO(col, fcs_state_get_col(*derived, col_idx));
             const int col_len = fcs_col_len(col);
 
-            if (col_len > 0)
+            if (!col_len)
             {
-                fcs_card_t top_card = fcs_col_get_card(col, 0);
+                continue;
+            }
+            const_AUTO(top_card, fcs_col_get_card(col, 0));
+            if (fcs_card_rank(top_card) != 1)
+            {
+                SET_CARD_STATE(top_card, OPT_TOPMOST);
+            }
 
-                if (fcs_card_rank(top_card) != 1)
+            fcs_card_t parent_card = top_card;
+            for (int child_idx = 1; child_idx < col_len; child_idx++)
+            {
+                const fcs_card_t child_card = fcs_col_get_card(col, child_idx);
+
+                if (fcs_card_rank(child_card) != 1)
                 {
-                    self->card_states[CARD_POS(top_card)] = OPT_TOPMOST;
-                }
-
-                fcs_card_t parent_card = top_card;
-
-                for (int child_idx = 1; child_idx < col_len; child_idx++)
-                {
-                    const fcs_card_t child_card =
-                        fcs_col_get_card(col, child_idx);
-
-                    if (fcs_card_rank(child_card) != 1)
-                    {
-                        self->card_states[CARD_POS(child_card)] =
-                            calc_child_card_option(
-                                local_variant, parent_card, child_card
+                    const int opt = calc_child_card_option(
+                        local_variant, parent_card, child_card
 #ifndef FCS_FREECELL_ONLY
-                                ,
-                                self->sequences_are_built_by
+                        ,
+                        self->sequences_are_built_by
 #endif
-                                );
-                    }
-
-                    parent_card = child_card;
+                    );
+                    SET_CARD_STATE(child_card, opt);
                 }
+
+                parent_card = child_card;
             }
         }
     }
@@ -304,37 +285,33 @@ static void fc_solve_debondt_delta_stater_encode_composite(
      *
      * Skip encoding the aces, and the kings are encoded with less bits.
      */
+    const int top_rank_for_iter = get_top_rank_for_iter(local_variant);
+    for (int rank = 2; rank <= top_rank_for_iter; rank++)
     {
-        const int top_rank_for_iter = get_top_rank_for_iter(local_variant);
-        for (int rank = 2; rank <= top_rank_for_iter; rank++)
+        for (int suit_idx = 0; suit_idx < FCS_NUM_SUITS; suit_idx++)
         {
-            for (int suit_idx = 0; suit_idx < FCS_NUM_SUITS; suit_idx++)
+            const unsigned long opt = RS_STATE(rank, suit_idx);
+            unsigned long base;
+
+            if (IS_BAKERS_DOZEN())
             {
-                const unsigned long opt =
-                    self->card_states[STATE_POS(suit_idx, rank)];
-                unsigned long base;
+                const_AUTO(card, fcs_card2char(fcs_make_card(rank, suit_idx)));
 
-                if (IS_BAKERS_DOZEN())
+                if (self->bakers_dozen_topmost_cards_lookup[card >> 3] &
+                    (1 << (card & (8 - 1))))
                 {
-                    const_AUTO(
-                        card, fcs_card2char(fcs_make_card(rank, suit_idx)));
-
-                    if (self->bakers_dozen_topmost_cards_lookup[card >> 3] &
-                        (1 << (card & (8 - 1))))
-                    {
-                        continue;
-                    }
-                    base = NUM__BAKERS_DOZEN__OPTS;
+                    continue;
                 }
-                else
-                {
-                    base = ((rank == RANK_KING) ? NUM_KING_OPTS : NUM_OPTS);
-                }
-
-                assert(opt < base);
-
-                fc_solve_var_base_writer_write(writer, base, opt);
+                base = NUM__BAKERS_DOZEN__OPTS;
             }
+            else
+            {
+                base = ((rank == RANK_KING) ? NUM_KING_OPTS : NUM_OPTS);
+            }
+
+            assert(opt < base);
+
+            fc_solve_var_base_writer_write(writer, base, opt);
         }
     }
 }
@@ -362,7 +339,7 @@ fc_solve_debondt_delta_stater__fill_column_with_descendent_cards(
             const fcs_card_t candidate_card =
                 fcs_make_card(candidate_rank, suit);
 
-            if (self->card_states[CARD_POS(candidate_card)] == wanted_opt)
+            if (CARD_STATE(candidate_card) == wanted_opt)
             {
                 child_card = candidate_card;
                 break;
@@ -382,26 +359,23 @@ static void fc_solve_debondt_delta_stater_decode(
     const fcs_dbm_variant_type_t local_variant,
     fcs_var_base_reader_t *const reader, fcs_state_t *const ret)
 {
-    unsigned char orig_top_most_cards[4 * RANK_KING];
     fcs_card_t new_top_most_cards[MAX_NUM_STACKS];
 
     fc_solve_debondt_delta_stater__init_card_states(self);
 
+    for (int suit_idx = 0; suit_idx < FCS_NUM_SUITS; suit_idx++)
     {
-        for (int suit_idx = 0; suit_idx < FCS_NUM_SUITS; suit_idx++)
+        const unsigned long foundation_rank =
+            fc_solve_var_base_reader_read(reader, FOUNDATION_BASE);
+
+        for (unsigned long rank = 1; rank <= foundation_rank; ++rank)
         {
-            const unsigned long foundation_rank =
-                fc_solve_var_base_reader_read(reader, FOUNDATION_BASE);
-
-            for (unsigned long rank = 1; rank <= foundation_rank; rank++)
-            {
-                self->card_states[STATE_POS(suit_idx, rank)] =
-                    (IS_BAKERS_DOZEN() ? OPT__BAKERS_DOZEN__IN_FOUNDATION
-                                       : OPT_IN_FOUNDATION);
-            }
-
-            fcs_set_foundation(*ret, suit_idx, foundation_rank);
+            RS_STATE(rank, suit_idx) =
+                (IS_BAKERS_DOZEN() ? OPT__BAKERS_DOZEN__IN_FOUNDATION
+                                   : OPT_IN_FOUNDATION);
         }
+
+        fcs_set_foundation(*ret, suit_idx, foundation_rank);
     }
 
 #define IS_IN_FOUNDATIONS(card)                                                \
@@ -412,20 +386,16 @@ static void fc_solve_debondt_delta_stater_decode(
     const int orig_pos_opt =
         (IS_BAKERS_DOZEN() ? OPT__BAKERS_DOZEN__ORIG_POS : OPT_ORIG_POS);
 
-    const int num_freecells = self->num_freecells;
+    const_SLOT(num_freecells, self);
 
-    memset(orig_top_most_cards, '\0', sizeof(orig_top_most_cards));
+    fcs_bool_t orig_top_most_cards[CARD_ARRAY_LEN] = {FALSE};
+    for (size_t col_idx = 0; col_idx < self->num_columns; ++col_idx)
     {
-        for (size_t col_idx = 0; col_idx < self->num_columns; col_idx++)
-        {
-            const fcs_cards_column_t col =
-                fcs_state_get_col(*init_state, col_idx);
+        const_AUTO(col, fcs_state_get_col(*init_state, col_idx));
 
-            if (fcs_col_len(col))
-            {
-                const fcs_card_t card = fcs_col_get_card(col, 0);
-                orig_top_most_cards[CARD_POS(card)] = 1;
-            }
+        if (fcs_col_len(col))
+        {
+            orig_top_most_cards[CARD_POS(fcs_col_get_card(col, 0))] = TRUE;
         }
     }
 
@@ -434,11 +404,11 @@ static void fc_solve_debondt_delta_stater_decode(
     {
         for (int suit_idx = 0; suit_idx < FCS_NUM_SUITS; suit_idx++)
         {
-            fcs_card_t card = fcs_make_card(RANK_KING, suit_idx);
+            const fcs_card_t card = fcs_make_card(RANK_KING, suit_idx);
 
             if (!IS_IN_FOUNDATIONS(card))
             {
-                self->card_states[CARD_POS(card)] = OPT__BAKERS_DOZEN__ORIG_POS;
+                SET_CARD_STATE(card, OPT__BAKERS_DOZEN__ORIG_POS);
             }
         }
     }
@@ -460,21 +430,19 @@ static void fc_solve_debondt_delta_stater_decode(
                 {
                     if (!IS_IN_FOUNDATIONS(card))
                     {
-                        self->card_states[CARD_POS(card)] =
-                            OPT__BAKERS_DOZEN__ORIG_POS;
+                        SET_CARD_STATE(card, OPT__BAKERS_DOZEN__ORIG_POS);
                     }
                     continue;
                 }
             }
 
-            const int existing_opt =
-                self->card_states[STATE_POS(suit_idx, rank)];
+            const int existing_opt = RS_STATE(rank, suit_idx);
 
             if (rank == 1)
             {
                 if (existing_opt < 0)
                 {
-                    self->card_states[STATE_POS(suit_idx, rank)] = orig_pos_opt;
+                    RS_STATE(rank, suit_idx) = orig_pos_opt;
                 }
             }
             else
@@ -488,7 +456,7 @@ static void fc_solve_debondt_delta_stater_decode(
 
                 if (existing_opt < 0)
                 {
-                    self->card_states[STATE_POS(suit_idx, rank)] = item_opt;
+                    RS_STATE(rank, suit_idx) = item_opt;
 
                     if (!IS_BAKERS_DOZEN())
                     {
@@ -523,14 +491,13 @@ static void fc_solve_debondt_delta_stater_decode(
         int top_opt;
 
         fcs_cards_column_t col = fcs_state_get_col(*ret, col_idx);
-        const fcs_cards_column_t orig_col =
-            fcs_state_get_col(*init_state, col_idx);
+        const_AUTO(orig_col, fcs_state_get_col(*init_state, col_idx));
         const_AUTO(orig_col_len, fcs_col_len(orig_col));
 
         if (orig_col_len)
         {
             top_card = fcs_col_get_card(orig_col, 0);
-            top_opt = self->card_states[CARD_POS(top_card)];
+            top_opt = CARD_STATE(top_card);
         }
         else
         {
@@ -552,28 +519,21 @@ static void fc_solve_debondt_delta_stater_decode(
         }
         else
         {
-            fcs_card_t parent_card;
-            int pos;
-
             fcs_col_push_card(col, top_card);
-
-            parent_card = top_card;
-
-            for (pos = 1; pos < orig_col_len; pos++)
+            var_AUTO(parent_card, top_card);
+            for (int pos = 1; pos < orig_col_len; pos++)
             {
-                fcs_card_t child_card;
-
-                child_card = fcs_col_get_card(orig_col, pos);
+                const fcs_card_t child_card = fcs_col_get_card(orig_col, pos);
 
                 if ((!IS_IN_FOUNDATIONS(child_card)) &&
-                    (self->card_states[CARD_POS(child_card)] ==
-                        calc_child_card_option(
-                            local_variant, parent_card, child_card
+                    (CARD_STATE(child_card) == calc_child_card_option(
+                                                   local_variant, parent_card,
+                                                   child_card
 #ifndef FCS_FREECELL_ONLY
-                            ,
-                            self->sequences_are_built_by
+                                                   ,
+                                                   self->sequences_are_built_by
 #endif
-                            )))
+                                                   )))
                 {
                     fcs_col_push_card(col, child_card);
                     parent_card = child_card;
