@@ -12,13 +12,6 @@
  * states are encoded and decoded based on a compact delta from the initial
  * state.
  */
-#include "dll_thunk.h"
-
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-
-#include "state.h"
 #include "bit_rw.h"
 #include "delta_states_iface.h"
 #include "delta_states.h"
@@ -30,26 +23,18 @@
 static int fc_solve_get_column_orig_num_cards(
     fc_solve_delta_stater_t *const self, const fcs_const_cards_column_t col)
 {
-    int num_cards;
 #ifndef FCS_FREECELL_ONLY
-    int sequences_are_built_by;
+    const_SLOT(sequences_are_built_by, self);
 #endif
-
-#ifndef FCS_FREECELL_ONLY
-    sequences_are_built_by = self->sequences_are_built_by;
-#endif
-    for (num_cards = fcs_col_len(col); num_cards >= 2; num_cards--)
+    for (int num_cards = fcs_col_len(col); num_cards >= 2; --num_cards)
     {
-        fcs_card_t child_card = fcs_col_get_card(col, num_cards - 1);
-        fcs_card_t parent_card = fcs_col_get_card(col, num_cards - 2);
-
-        if (!fcs_is_parent_card(child_card, parent_card))
+        if (!fcs_is_parent_card(fcs_col_get_card(col, num_cards - 1),
+                fcs_col_get_card(col, num_cards - 2)))
         {
-            break;
+            return num_cards;
         }
     }
-
-    return ((num_cards >= 2) ? num_cards : 0);
+    return 0;
 }
 
 static void fc_solve_delta_stater_init(
@@ -59,14 +44,13 @@ static void fc_solve_delta_stater_init(
     ,
     const int sequences_are_built_by
 #endif
-    )
+)
 {
 #ifndef FCS_FREECELL_ONLY
     self->sequences_are_built_by = sequences_are_built_by;
 #endif
     self->num_columns = num_columns;
     self->num_freecells = num_freecells;
-
     self->init_state = init_state;
 
     int max_num_cards = 0;
@@ -124,19 +108,19 @@ static inline void fc_solve_get_column_encoding_composite(
     fc_solve_column_encoding_composite_t *const ret)
 {
     const fcs_state_t *const derived = self->derived_state;
-    fcs_const_cards_column_t col = fcs_state_get_col(*derived, col_idx);
+    const_AUTO(col, fcs_state_get_col(*derived, col_idx));
 
     const_AUTO(num_orig_cards, fc_solve_get_column_orig_num_cards(self, col));
     const int col_len = fcs_col_len(col);
     const int num_derived_cards = col_len - num_orig_cards;
 
-    int num_cards_in_seq = num_derived_cards;
+    int_fast16_t num_cards_in_seq = num_derived_cards;
     fcs_card_t init_card = fc_solve_empty_card;
 
     if ((num_orig_cards == 0) && num_derived_cards)
     {
         init_card = fcs_col_get_card(col, 0);
-        num_cards_in_seq--;
+        --num_cards_in_seq;
     }
 
     /* Prepare the encoding. */
@@ -156,10 +140,8 @@ static inline void fc_solve_get_column_encoding_composite(
     for (int i = col_len - num_cards_in_seq; i < col_len; i++)
     {
 #define GET_SUIT_BIT(card) (((fcs_card_suit(card)) & 0x2) >> 1)
-
         fc_solve_bit_writer_write(
             &bit_w, 1, GET_SUIT_BIT(fcs_col_get_card(col, i)));
-
 #undef GET_SUIT_BIT
     }
 
@@ -175,7 +157,6 @@ static inline void fc_solve_get_column_encoding_composite(
 static inline void fc_solve_get_freecells_encoding(
     fc_solve_delta_stater_t *const self, fc_solve_bit_writer_t *const bit_w)
 {
-
     const fcs_state_t *const derived = self->derived_state;
     const_SLOT(num_freecells, self);
 
@@ -186,10 +167,10 @@ static inline void fc_solve_get_freecells_encoding(
     }
 
     /* Sort the freecells using selection-sort. */
-    for (int i = 0; i < num_freecells; i++)
+    for (int i = 0; i < num_freecells; ++i)
     {
         int min_idx = i;
-        for (int j = i + 1; j < num_freecells; j++)
+        for (int j = i + 1; j < num_freecells; ++j)
         {
             if (fcs_card2char(freecells[j]) < fcs_card2char(freecells[min_idx]))
             {
@@ -197,13 +178,13 @@ static inline void fc_solve_get_freecells_encoding(
             }
         }
         const_AUTO(i_card, freecells[i]);
-        fc_solve_bit_writer_write(
-            bit_w, 6, fcs_card2char((min_idx != i) ? ({
+        fc_solve_bit_writer_write(bit_w, 6,
+            fcs_card2char((min_idx != i) ? ({
                 const_AUTO(min_card, freecells[min_idx]);
                 freecells[min_idx] = i_card;
                 min_card;
             })
-                                                   : i_card));
+                                         : i_card));
     }
 }
 
@@ -245,8 +226,8 @@ static inline void fc_solve_delta__promote_empty_cols(const size_t num_columns,
         cols_indexes[non_orig_idx] = cols_indexes[empty_col_idx];
         cols_indexes[empty_col_idx] = swap_int;
 
-        non_orig_idx++;
-        empty_col_idx--;
+        ++non_orig_idx;
+        --empty_col_idx;
     }
 }
 
@@ -272,45 +253,42 @@ static void fc_solve_delta_stater_encode_composite(
      * */
     fc_solve_delta__promote_empty_cols(num_columns, cols_indexes, cols);
 
-    {
-        int new_non_orig_cols_indexes[MAX_NUM_STACKS];
-        int new_non_orig_cols_indexes_count = 0;
+    int new_non_orig_cols_indexes[MAX_NUM_STACKS];
+    int new_non_orig_cols_indexes_count = 0;
 
-        /* Filter the new_non_orig_cols_indexes */
-        for (int i = 0; i < num_columns; i++)
+    /* Filter the new_non_orig_cols_indexes */
+    for (int i = 0; i < num_columns; i++)
+    {
+        if (cols[cols_indexes[i]].type == COL_TYPE_ENTIRELY_NON_ORIG)
         {
-            if (cols[cols_indexes[i]].type == COL_TYPE_ENTIRELY_NON_ORIG)
-            {
-                new_non_orig_cols_indexes[new_non_orig_cols_indexes_count++] =
-                    cols_indexes[i];
-            }
+            new_non_orig_cols_indexes[new_non_orig_cols_indexes_count++] =
+                cols_indexes[i];
         }
+    }
 
 /* Sort the new_non_orig_cols_indexes_count using insertion-sort. */
 #define COMP_BY(idx)                                                           \
     (fcs_card2char(fcs_col_get_card(fcs_state_get_col((*derived), (idx)), 0)))
 #define ITEM_IDX(idx) (new_non_orig_cols_indexes[idx])
 #define COMP_BY_IDX(idx) (COMP_BY(ITEM_IDX(idx)))
-        for (int b = 1; b < new_non_orig_cols_indexes_count; b++)
+    for (int b = 1; b < new_non_orig_cols_indexes_count; b++)
+    {
+        for (int c = b; (c > 0) && (COMP_BY_IDX(c - 1) > COMP_BY_IDX(c)); c--)
         {
-            for (int c = b; (c > 0) && (COMP_BY_IDX(c - 1) > COMP_BY_IDX(c));
-                 c--)
-            {
-                const_AUTO(swap_int, ITEM_IDX(c));
-                ITEM_IDX(c) = ITEM_IDX(c - 1);
-                ITEM_IDX(c - 1) = swap_int;
-            }
+            const_AUTO(swap_int, ITEM_IDX(c));
+            ITEM_IDX(c) = ITEM_IDX(c - 1);
+            ITEM_IDX(c - 1) = swap_int;
         }
+    }
 #undef COMP_BY_IDX
 #undef ITEM_IDX
 #undef COMP_BY
 #undef b
-        for (int i = 0, sorted_idx = 0; i < num_columns; i++)
+    for (int i = 0, sorted_idx = 0; i < num_columns; i++)
+    {
+        if (cols[cols_indexes[i]].type == COL_TYPE_ENTIRELY_NON_ORIG)
         {
-            if (cols[cols_indexes[i]].type == COL_TYPE_ENTIRELY_NON_ORIG)
-            {
-                cols_indexes[i] = new_non_orig_cols_indexes[sorted_idx++];
-            }
+            cols_indexes[i] = new_non_orig_cols_indexes[sorted_idx++];
         }
     }
 
@@ -334,11 +312,9 @@ static void fc_solve_delta_stater_decode(fc_solve_delta_stater_t *const self,
     fc_solve_bit_reader_t *const bit_r, fcs_state_t *const ret)
 {
 #define PROCESS_CARD(card)                                                     \
+    if (fcs_card_rank(card) < foundations[fcs_card_suit(card)])                \
     {                                                                          \
-        if (fcs_card_rank(card) < foundations[fcs_card_suit(card)])            \
-        {                                                                      \
-            foundations[fcs_card_suit(card)] = fcs_card_rank(card);            \
-        }                                                                      \
+        foundations[fcs_card_suit(card)] = fcs_card_rank(card);                \
     }
 
     const_SLOT(num_freecells, self);
@@ -351,7 +327,7 @@ static void fc_solve_delta_stater_decode(fc_solve_delta_stater_t *const self,
     {
         const fcs_card_t card =
             fcs_char2card(fc_solve_bit_reader_read(bit_r, 6));
-        if (!(card == fc_solve_empty_card))
+        if (fcs_card_is_valid(card))
         {
             PROCESS_CARD(card);
         }
@@ -363,12 +339,10 @@ static void fc_solve_delta_stater_decode(fc_solve_delta_stater_t *const self,
 
     for (size_t col_idx = 0; col_idx < num_columns; col_idx++)
     {
-        const fcs_cards_column_t col = fcs_state_get_col(*ret, col_idx);
+        const_AUTO(col, fcs_state_get_col(*ret, col_idx));
         const int num_orig_cards =
             fc_solve_bit_reader_read(bit_r, bits_per_orig_cards_in_column);
-
-        fcs_const_cards_column_t orig_col =
-            fcs_state_get_col(*init_state, col_idx);
+        const_AUTO(orig_col, fcs_state_get_col(*init_state, col_idx));
 
         for (int i = 0; i < num_orig_cards; i++)
         {
@@ -378,7 +352,7 @@ static void fc_solve_delta_stater_decode(fc_solve_delta_stater_t *const self,
         }
 
         const int num_derived_cards = fc_solve_bit_reader_read(bit_r, 4);
-        int num_cards_in_seq = num_derived_cards;
+        int_fast16_t num_cards_in_seq = num_derived_cards;
 
         if ((num_orig_cards == 0) && num_derived_cards)
         {
@@ -387,14 +361,14 @@ static void fc_solve_delta_stater_decode(fc_solve_delta_stater_t *const self,
             PROCESS_CARD(card);
             fcs_col_push_card(col, card);
 
-            num_cards_in_seq--;
+            --num_cards_in_seq;
         }
 
         if (num_cards_in_seq)
         {
             fcs_card_t last_card = fcs_col_get_card(col, fcs_col_len(col) - 1);
 
-            for (int i = 0; i < num_cards_in_seq; i++)
+            for (int_fast16_t i = 0; i < num_cards_in_seq; i++)
             {
                 const int suit_bit = fc_solve_bit_reader_read(bit_r, 1);
                 const fcs_card_t new_card =
@@ -426,9 +400,7 @@ static inline void fc_solve_delta_stater_decode_into_state_proto(
 {
     fc_solve_bit_reader_t bit_r;
     fc_solve_bit_reader_init(&bit_r, enc_state + 1);
-
     fc_solve_state_init(ret, STACKS_NUM, indirect_stacks_buffer);
-
     fc_solve_delta_stater_decode(delta_stater, &bit_r, &(ret->s));
 }
 
@@ -458,9 +430,10 @@ static inline void fc_solve_delta_stater_encode_into_buffer(
 }
 
 static inline void fcs_init_and_encode_state(
-    fc_solve_delta_stater_t *delta_stater,
-    const fcs_dbm_variant_type_t local_variant, fcs_state_keyval_pair_t *state,
-    fcs_encoded_state_buffer_t *enc_state)
+    fc_solve_delta_stater_t *const delta_stater,
+    const fcs_dbm_variant_type_t local_variant,
+    fcs_state_keyval_pair_t *const state,
+    fcs_encoded_state_buffer_t *const enc_state)
 {
     fcs_init_encoded_state(enc_state);
 
