@@ -70,7 +70,7 @@ static inline void instance_init(fcs_dbm_solver_instance_t *const instance,
     fcs_condvar_init(&(instance->monitor));
 }
 
-static inline void instance_destroy(fcs_dbm_solver_instance_t *instance)
+static inline void instance_destroy(fcs_dbm_solver_instance_t *const instance)
 {
     for (int depth = 0; depth < MAX_FCC_DEPTH; depth++)
     {
@@ -92,14 +92,9 @@ struct fcs_dbm_solver_thread_struct
     fcs_meta_compact_allocator_t thread_meta_alloc;
 };
 
-typedef struct
-{
-    fcs_dbm_solver_thread_t *thread;
-} thread_arg_t;
-
 static void *instance_run_solver_thread(void *const void_arg)
 {
-    fcs_derived_state_t *derived_list_recycle_bin = NULL, *derived_iter;
+    fcs_derived_state_t *derived_list_recycle_bin = NULL;
     fcs_state_keyval_pair_t state;
 #ifdef DEBUG_OUT
     fcs_state_locs_struct_t locs;
@@ -122,7 +117,6 @@ static void *instance_run_solver_thread(void *const void_arg)
     TRACE("%s\n", "instance_run_solver_thread start");
 
     const_AUTO(coll, &(instance->colls_by_depth[instance->curr_depth]));
-    int queue_num_extracted_and_processed = 0;
     fcs_dbm_record_t *tokens[max_batch_size];
     fcs_derived_state_t *derived_lists[max_batch_size];
     while (TRUE)
@@ -161,9 +155,8 @@ static void *instance_run_solver_thread(void *const void_arg)
                     }
                 }
 
-                queue_num_extracted_and_processed =
-                    instance->common.queue_num_extracted_and_processed;
-                should_break = (!queue_num_extracted_and_processed);
+                should_break =
+                    !instance->common.queue_num_extracted_and_processed;
 
                 prev_size = batch_size;
                 if ((!should_break) && (batch_size == 0))
@@ -213,7 +206,7 @@ static void *instance_run_solver_thread(void *const void_arg)
         /* Encode all the states. */
         for (batch_size_t batch_i = 0; batch_i < batch_size; ++batch_i)
         {
-            for (derived_iter = derived_lists[batch_i]; derived_iter;
+            for (var_AUTO(derived_iter, derived_lists[batch_i]); derived_iter;
                  derived_iter = derived_iter->next)
             {
                 fcs_init_and_encode_state(delta_stater, local_variant,
@@ -227,21 +220,12 @@ static void *instance_run_solver_thread(void *const void_arg)
             ,
             item->moves_to_key
 #endif
-            );
+        );
 
-        /* Now recycle the derived_list */
         for (batch_size_t batch_i = 0; batch_i < batch_size; ++batch_i)
         {
-            var_AUTO(list, derived_lists[batch_i]);
-            while (list)
-            {
-#define derived_list_next derived_iter
-                derived_list_next = list->next;
-                list->next = derived_list_recycle_bin;
-                derived_list_recycle_bin = list;
-                list = derived_list_next;
-#undef derived_list_next
-            }
+            fcs_derived_state_list__recycle(
+                &derived_list_recycle_bin, &derived_lists[batch_i]);
         }
         /* End handle item. */
         /* End of main thread loop */
@@ -249,9 +233,7 @@ static void *instance_run_solver_thread(void *const void_arg)
 thread_end:
 
     fc_solve_compact_allocator_finish(&(derived_list_allocator));
-
     TRACE("%s\n", "instance_run_solver_thread end");
-
     return NULL;
 }
 
@@ -268,7 +250,7 @@ static inline void instance_check_key(
     ,
     const fcs_fcc_move_t *moves_to_parent
 #endif
-    )
+)
 {
     const_AUTO(coll, &(instance->colls_by_depth[key_depth]));
     fcs_dbm_record_t *token;
@@ -285,14 +267,14 @@ static inline void instance_check_key(
         fcs_offloading_queue__insert(
             &(coll->queue), (const fcs_offloading_queue_item_t *)(&token));
 
-        instance->common.count_of_items_in_queue++;
-        instance->common.num_states_in_collection++;
+        ++instance->common.count_of_items_in_queue;
+        ++instance->common.num_states_in_collection;
         instance_debug_out_state(instance, &(token->key));
     }
 }
 
-static void instance_run_all_threads(fcs_dbm_solver_instance_t *instance,
-    fcs_state_keyval_pair_t *init_state, size_t num_threads)
+static void instance_run_all_threads(fcs_dbm_solver_instance_t *const instance,
+    fcs_state_keyval_pair_t *const init_state, const size_t num_threads)
 {
     const_AUTO(threads,
         dbm__calc_threads(instance, init_state, num_threads, init_thread));
@@ -308,7 +290,7 @@ static void instance_run_all_threads(fcs_dbm_solver_instance_t *instance,
                 instance->colls_by_depth[instance->curr_depth]
                     .cache_store.store),
             instance->curr_depth);
-        instance->curr_depth++;
+        ++instance->curr_depth;
     }
 
     dbm__free_threads(instance, num_threads, threads, free_thread);
@@ -337,13 +319,9 @@ int main(int argc, char *argv[])
                 fc_solve_err("--batch-size must be at least 1.\n");
             }
         }
-        else if (!strcmp(argv[*arg], "-o"))
+        else if ((param = TRY_PARAM("-o")))
         {
-            if (++(*arg) == argc)
-            {
-                fc_solve_err("-o came without an argument.\n");
-            }
-            out_filename = argv[*arg];
+            out_filename = param;
         }
         else
         {
@@ -374,7 +352,7 @@ int main(int argc, char *argv[])
         ,
         CALC_SEQUENCES_ARE_BUILT_BY()
 #endif
-            );
+    );
 
 #define KEY_PTR() (key_ptr)
     fcs_dbm_solver_instance_t instance;
@@ -399,14 +377,13 @@ int main(int argc, char *argv[])
 
     fcs_offloading_queue__insert(&(instance.colls_by_depth[0].queue),
         (const fcs_offloading_queue_item_t *)(&token));
-    instance.common.num_states_in_collection++;
-    instance.common.count_of_items_in_queue++;
+    ++instance.common.num_states_in_collection;
+    ++instance.common.count_of_items_in_queue;
 
     instance_run_all_threads(&instance, &init_state, NUM_THREADS());
     handle_and_destroy_instance_solution(&instance, &delta);
 
     fc_solve_delta_stater_release(&delta);
-
     if (out_filename)
     {
         fclose(out_fh);
