@@ -18,12 +18,9 @@ extern "C" {
 #endif
 
 #include <assert.h>
-#include "config.h"
-
 #include "state.h"
 #include "meta_alloc.h"
 #include "fcs_enums.h"
-
 #include "dbm_common.h"
 #include "delta_states.h"
 #include "dbm_calc_derived_iface.h"
@@ -140,7 +137,7 @@ static inline void fcc_brfs_add_key_to_tree(
     fc_solve_kaz_tree_alloc_insert(tree, key_to_add);
 }
 
-static void perform_FCC_brfs(fcs_dbm_variant_type_t local_variant,
+static void perform_FCC_brfs(const fcs_dbm_variant_type_t local_variant,
     /* The first state in the game, from which all states are encoded. */
     fcs_state_keyval_pair_t *init_state,
     /* The start state. */
@@ -177,7 +174,7 @@ static void perform_FCC_brfs(fcs_dbm_variant_type_t local_variant,
     /* [Input/Output]: The ${next}_depth_FCCs.DoesExist
      * (for the right depth based on the current depth and pruning.)
      * Of type Map{min_by_sorting => Bool Exists} DoesExist.
-    */
+     */
     dict_t *does_min_by_sorting_exist,
     /* [Input/Output]: The LRU cache.
      * Of type <<LRU_Map{any_state_in_the_FCCs => Bool Exists} Cache>>
@@ -189,22 +186,19 @@ static void perform_FCC_brfs(fcs_dbm_variant_type_t local_variant,
      * */
     long *const out_num_new_positions,
     /* [Input/Output]: the list allocator. */
-    fcs_fcc_moves_seq_allocator_t *moves_list_allocator,
+    fcs_fcc_moves_seq_allocator_t *const moves_list_allocator,
     /* [Input/Output]: The meta allocator - needed to allocate and free
      * the compact allocators. */
     fcs_meta_compact_allocator_t *const meta_alloc)
 {
     void *tree_recycle_bin = NULL;
-    fcs_dbm_queue_item_t *queue_head, *queue_tail, *queue_recycle_bin,
-        *new_item, *extracted_item;
-    fcs_compact_allocator_t queue_allocator, derived_list_allocator;
-    fcs_derived_state_t *derived_list, *derived_list_recycle_bin, *derived_iter,
-        *next_derived_iter;
+    fcs_dbm_queue_item_t *queue_head, *queue_tail, *queue_recycle_bin = NULL,
+                                                   *extracted_item;
+    fcs_derived_state_t *derived_list_recycle_bin = NULL, *next_derived_iter;
     fcs_state_keyval_pair_t state;
     fcs_bool_t running_min_was_assigned = FALSE;
-    fcs_encoded_state_buffer_t running_min;
+    fcs_encoded_state_buffer_t running_min = {{0}};
     long num_new_positions = 0;
-
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
 
 /* Some sanity checks. */
@@ -217,10 +211,10 @@ static void perform_FCC_brfs(fcs_dbm_variant_type_t local_variant,
     assert(does_state_exist_in_any_FCC_cache);
 #endif
 
-    /* Initialize the queue_allocator. */
+    /* Initialize the allocators. */
+    fcs_compact_allocator_t queue_allocator, derived_list_allocator;
     fc_solve_compact_allocator_init(&(queue_allocator), meta_alloc);
     fc_solve_compact_allocator_init(&(derived_list_allocator), meta_alloc);
-    queue_recycle_bin = NULL;
 
     /* TODO : maybe pass delta_stater as an argument  */
     fc_solve_delta_stater_t delta_stater;
@@ -230,22 +224,18 @@ static void perform_FCC_brfs(fcs_dbm_variant_type_t local_variant,
         ,
         FCS_SEQ_BUILT_BY_ALTERNATE_COLOR
 #endif
-        );
+    );
     dict_t *traversed_states =
         fcc_brfs_kaz_tree_create(meta_alloc, &tree_recycle_bin);
 
-    new_item = (fcs_dbm_queue_item_t *)fcs_compact_alloc_ptr(
-        &(queue_allocator), sizeof(*new_item));
-
-    new_item->key = start_state;
-    new_item->next = NULL;
-    new_item->moves_seq.count = 0;
-    new_item->moves_seq.moves_list = NULL;
-
+    fcs_dbm_queue_item_t *new_item =
+        (fcs_dbm_queue_item_t *)fcs_compact_alloc_ptr(
+            &(queue_allocator), sizeof(*new_item));
+    *new_item = (typeof(*new_item)){.key = start_state,
+        .next = NULL,
+        .moves_seq = {.count = 0, .moves_list = NULL}};
     queue_head = queue_tail = new_item;
 
-    derived_list_recycle_bin = NULL;
-
     *out_num_new_positions = num_new_positions;
 
     fc_solve_kaz_tree_alloc_insert(traversed_states, &(new_item->key));
@@ -280,7 +270,7 @@ static void perform_FCC_brfs(fcs_dbm_variant_type_t local_variant,
         }
 
         /* Calculate the derived list. */
-        derived_list = NULL;
+        fcs_derived_state_t *derived_list = NULL;
 
         /* Handle item. */
         fc_solve_delta_stater_decode_into_state(&delta_stater,
@@ -305,7 +295,7 @@ static void perform_FCC_brfs(fcs_dbm_variant_type_t local_variant,
         }
 
         /* Enqueue the new elements to the queue. */
-        for (derived_iter = derived_list; derived_iter;
+        for (var_AUTO(derived_iter, derived_list); derived_iter;
              derived_iter = next_derived_iter)
         {
             const fcs_bool_t is_reversible =
@@ -436,7 +426,7 @@ static fcs_bool_t fc_solve_add_start_point_in_mem(
     const fcs_fcc_moves_list_item_t *start_iter =
         start_state_moves_seq->moves_list;
     const int count_start_state_moves = start_state_moves_seq->count;
-    fcs_fcc_moves_seq_allocator_t *moves_list_allocator =
+    fcs_fcc_moves_seq_allocator_t *const moves_list_allocator =
         context->moves_list_allocator;
     for (; pos_in_moves < count_start_state_moves;)
     {
