@@ -10,13 +10,6 @@
 /*
  * fcc_brfs_test.c - provide utility routines to test fcc_brfs.h
  */
-
-#include "dll_thunk.h"
-
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-
 #include "config.h"
 #undef FCS_RCS_STATES
 
@@ -27,8 +20,9 @@
 #include "render_state.h"
 
 static void fc_solve_state_string_to_enc(
-    const fcs_dbm_variant_type_t local_variant, fc_solve_delta_stater_t *delta,
-    const char *const state_s_proto, fcs_encoded_state_buffer_t *enc_state)
+    const fcs_dbm_variant_type_t local_variant,
+    fc_solve_delta_stater_t *const delta, const char *const state_s_proto,
+    fcs_encoded_state_buffer_t *const enc_state)
 {
     fcs_state_keyval_pair_t state;
     DECLARE_IND_BUF_T(state_indirect_stacks_buffer)
@@ -52,15 +46,9 @@ DLLEXPORT int fc_solve_user_INTERNAL_find_fcc_start_points(
     fcs_state_keyval_pair_t init_state;
     fcs_encoded_state_buffer_t enc_state;
     fcs_state_locs_struct_t locs;
-    dict_t *do_next_fcc_start_points_exist;
-    dict_t *does_min_by_sorting_exist;
-    fcs_lru_cache_t does_state_exist_in_any_FCC_cache;
-    const int max_num_elements_in_cache = 1000;
+    fc_solve_init_locs(&locs);
     fcs_encoded_state_buffer_t min_by_sorting;
-    fcs_fcc_moves_seq_t start_state_moves_seq;
-    add_start_point_context_t add_start_point_context;
     void *tree_recycle_bin = NULL;
-
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
 
     fc_solve_initial_user_state_to_c(init_state_str_proto, &init_state,
@@ -73,7 +61,7 @@ DLLEXPORT int fc_solve_user_INTERNAL_find_fcc_start_points(
         ,
         FCS_SEQ_BUILT_BY_ALTERNATE_COLOR
 #endif
-        );
+    );
 
     fcs_init_and_encode_state(&delta, local_variant, &(init_state), &enc_state);
 
@@ -83,19 +71,19 @@ DLLEXPORT int fc_solve_user_INTERNAL_find_fcc_start_points(
     fc_solve_meta_compact_allocator_init(&meta_alloc);
     fc_solve_compact_allocator_init(
         &(start_points_list.allocator), &meta_alloc);
-    do_next_fcc_start_points_exist =
+    var_AUTO(do_next_fcc_start_points_exist,
+        fcc_brfs_kaz_tree_create(&meta_alloc, &tree_recycle_bin));
+    dict_t *does_min_by_sorting_exist =
         fcc_brfs_kaz_tree_create(&meta_alloc, &tree_recycle_bin);
-    does_min_by_sorting_exist =
-        fcc_brfs_kaz_tree_create(&meta_alloc, &tree_recycle_bin);
-    cache_init(&does_state_exist_in_any_FCC_cache, max_num_elements_in_cache,
-        &meta_alloc);
+    fcs_lru_cache_t does_state_exist_in_any_FCC_cache;
+    cache_init(&does_state_exist_in_any_FCC_cache, 1000, &meta_alloc);
 
     fcs_compact_allocator_t moves_list_compact_alloc;
     fc_solve_compact_allocator_init(&(moves_list_compact_alloc), &(meta_alloc));
     fcs_fcc_moves_seq_allocator_t moves_list_allocator = {
         .recycle_bin = NULL, .allocator = &(moves_list_compact_alloc)};
-    start_state_moves_seq.count = start_state_moves_count;
-    start_state_moves_seq.moves_list = NULL;
+    fcs_fcc_moves_seq_t start_state_moves_seq = {
+        .count = start_state_moves_count, .moves_list = NULL};
     {
         fcs_fcc_moves_list_item_t **moves_iter =
             &(start_state_moves_seq.moves_list);
@@ -115,11 +103,11 @@ DLLEXPORT int fc_solve_user_INTERNAL_find_fcc_start_points(
         }
     }
 
-    add_start_point_context.do_next_fcc_start_points_exist =
-        do_next_fcc_start_points_exist;
-    add_start_point_context.next_start_points_list = &start_points_list;
-    add_start_point_context.moves_list_allocator = &moves_list_allocator;
-
+    add_start_point_context_t add_start_point_context = {
+        .do_next_fcc_start_points_exist = do_next_fcc_start_points_exist,
+        .next_start_points_list = &start_points_list,
+        .moves_list_allocator = &moves_list_allocator,
+    };
     fcs_bool_t is_min_by_sorting_new;
     perform_FCC_brfs(local_variant, &(init_state), enc_state,
         &start_state_moves_seq, fc_solve_add_start_point_in_mem,
@@ -141,8 +129,6 @@ DLLEXPORT int fc_solve_user_INTERNAL_find_fcc_start_points(
 
     ret[states_count].count = 0;
 
-    fc_solve_init_locs(&locs);
-
     iter = start_points_list.list;
     for (size_t i = 0; i < states_count; i++)
     {
@@ -204,14 +190,8 @@ DLLEXPORT int fc_solve_user_INTERNAL_is_fcc_new(
     fcs_state_keyval_pair_t init_state;
     fcs_encoded_state_buffer_t enc_state;
     fcs_encoded_state_buffer_t start_enc_state;
-    dict_t *do_next_fcc_start_points_exist;
-    dict_t *does_min_by_sorting_exist;
-    const int max_num_elements_in_cache = 1000000;
-    fcs_lru_cache_t does_state_exist_in_any_FCC_cache;
     fcs_encoded_state_buffer_t min_by_sorting;
-    add_start_point_context_t add_start_point_context;
     void *tree_recycle_bin = NULL;
-
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
 
     fc_solve_initial_user_state_to_c(init_state_str_proto, &init_state,
@@ -224,7 +204,7 @@ DLLEXPORT int fc_solve_user_INTERNAL_is_fcc_new(
         ,
         FCS_SEQ_BUILT_BY_ALTERNATE_COLOR
 #endif
-        );
+    );
 
     fcs_init_and_encode_state(&delta, local_variant, &(init_state), &enc_state);
 
@@ -237,9 +217,9 @@ DLLEXPORT int fc_solve_user_INTERNAL_is_fcc_new(
         .list = NULL, .recycle_bin = NULL};
     fc_solve_compact_allocator_init(
         &(start_points_list.allocator), &meta_alloc);
-    do_next_fcc_start_points_exist =
-        fcc_brfs_kaz_tree_create(&meta_alloc, &tree_recycle_bin);
-    does_min_by_sorting_exist =
+    var_AUTO(do_next_fcc_start_points_exist,
+        fcc_brfs_kaz_tree_create(&meta_alloc, &tree_recycle_bin));
+    dict_t *does_min_by_sorting_exist =
         fcc_brfs_kaz_tree_create(&meta_alloc, &tree_recycle_bin);
     fcs_compact_allocator_t temp_allocator;
     fc_solve_compact_allocator_init(&(temp_allocator), &meta_alloc);
@@ -251,9 +231,9 @@ DLLEXPORT int fc_solve_user_INTERNAL_is_fcc_new(
 
         for (; *(min_states_iter); min_states_iter++)
         {
-            fcs_encoded_state_buffer_t *min_enc_state;
-            min_enc_state = (fcs_encoded_state_buffer_t *)fcs_compact_alloc_ptr(
-                &(temp_allocator), sizeof(*min_enc_state));
+            fcs_encoded_state_buffer_t *const min_enc_state =
+                (fcs_encoded_state_buffer_t *)fcs_compact_alloc_ptr(
+                    &(temp_allocator), sizeof(*min_enc_state));
 
             fc_solve_state_string_to_enc(
                 local_variant, &delta, *(min_states_iter), min_enc_state);
@@ -263,8 +243,8 @@ DLLEXPORT int fc_solve_user_INTERNAL_is_fcc_new(
         }
     }
 
-    cache_init(&does_state_exist_in_any_FCC_cache, max_num_elements_in_cache,
-        &meta_alloc);
+    fcs_lru_cache_t does_state_exist_in_any_FCC_cache;
+    cache_init(&does_state_exist_in_any_FCC_cache, 1000000, &meta_alloc);
 
     /* Populate does_state_exist_in_any_FCC_cache from states_in_cache */
     {
@@ -272,9 +252,9 @@ DLLEXPORT int fc_solve_user_INTERNAL_is_fcc_new(
 
         for (; *(min_states_iter); min_states_iter++)
         {
-            fcs_encoded_state_buffer_t *min_enc_state;
-            min_enc_state = (fcs_encoded_state_buffer_t *)fcs_compact_alloc_ptr(
-                &(temp_allocator), sizeof(*min_enc_state));
+            fcs_encoded_state_buffer_t *const min_enc_state =
+                (fcs_encoded_state_buffer_t *)fcs_compact_alloc_ptr(
+                    &(temp_allocator), sizeof(*min_enc_state));
 
             fc_solve_state_string_to_enc(
                 local_variant, &delta, *(min_states_iter), min_enc_state);
@@ -285,18 +265,17 @@ DLLEXPORT int fc_solve_user_INTERNAL_is_fcc_new(
     }
 
     fcs_fcc_moves_seq_t init_moves_seq = {.moves_list = NULL, .count = 0};
-    add_start_point_context.do_next_fcc_start_points_exist =
-        do_next_fcc_start_points_exist;
-    add_start_point_context.next_start_points_list = &start_points_list;
-    add_start_point_context.moves_list_allocator = &moves_list_allocator;
-    {
-        long num_new_positions_temp;
-        perform_FCC_brfs(local_variant, &(init_state), start_enc_state,
-            &init_moves_seq, fc_solve_add_start_point_in_mem,
-            &add_start_point_context, out_is_fcc_new, &min_by_sorting,
-            does_min_by_sorting_exist, &does_state_exist_in_any_FCC_cache,
-            &num_new_positions_temp, &moves_list_allocator, &meta_alloc);
-    }
+    add_start_point_context_t add_start_point_context = {
+        .do_next_fcc_start_points_exist = do_next_fcc_start_points_exist,
+        .next_start_points_list = &start_points_list,
+        .moves_list_allocator = &moves_list_allocator,
+    };
+    long num_new_positions_temp;
+    perform_FCC_brfs(local_variant, &(init_state), start_enc_state,
+        &init_moves_seq, fc_solve_add_start_point_in_mem,
+        &add_start_point_context, out_is_fcc_new, &min_by_sorting,
+        does_min_by_sorting_exist, &does_state_exist_in_any_FCC_cache,
+        &num_new_positions_temp, &moves_list_allocator, &meta_alloc);
 
     fc_solve_compact_allocator_finish(&(start_points_list.allocator));
     fc_solve_compact_allocator_finish(&(temp_allocator));
