@@ -7,10 +7,8 @@
  *
  * Copyright (c) 2000 Shlomi Fish
  */
-/*
- * fcs_enums.h - header file for various Freecell Solver Enumertaions. Common
- * to the main program headers and to the library headers.
- */
+// fcs_enums.h - header file for various Freecell Solver Enumerations. Common
+// to the main program headers and to the library headers.
 #pragma once
 
 #ifdef __cplusplus
@@ -38,8 +36,7 @@ enum
     FCS_TALON_KLONDIKE
 };
 
-enum
-{
+typedef enum {
     FCS_STATE_WAS_SOLVED,
     FCS_STATE_IS_NOT_SOLVEABLE,
     FCS_STATE_ALREADY_EXISTS,
@@ -53,7 +50,7 @@ enum
     FCS_STATE_DOES_NOT_EXIST,
     FCS_STATE_OPTIMIZED,
     FCS_STATE_FLARES_PLAN_ERROR
-};
+} fc_solve_solve_process_ret_t;
 
 typedef enum {
     FCS_PRESET_CODE_OK,
@@ -70,12 +67,10 @@ enum
     FC_SOLVE__STANDARD_NOTATION_EXTENDED = 2
 };
 
-#define FCS_METHOD_NONE -1
 #define FCS_METHOD_HARD_DFS 0
 #define FCS_METHOD_SOFT_DFS 1
 #define FCS_METHOD_BFS 2
 #define FCS_METHOD_A_STAR 3
-#define FCS_METHOD_OPTIMIZE 4
 #define FCS_METHOD_RANDOM_DFS 5
 #define FCS_METHOD_PATSOLVE 6
 
