@@ -17,8 +17,6 @@
 extern "C" {
 #endif
 
-#include "config.h"
-
 #include "meta_alloc.h"
 #include "rinutils.h"
 
@@ -50,11 +48,11 @@ struct fc_solve_hash_symlink_item_struct
     struct fc_solve_hash_symlink_item_struct *next;
 };
 
-typedef struct fc_solve_hash_symlink_item_struct fc_solve_hash_symlink_item_t;
+typedef struct fc_solve_hash_symlink_item_struct fcs_hash_item_t;
 
 typedef struct
 {
-    fc_solve_hash_symlink_item_t *first_item;
+    fcs_hash_item_t *first_item;
 } fc_solve_hash_symlink_t;
 
 struct fc_solve_instance_struct;
@@ -76,7 +74,7 @@ typedef struct
 #ifndef FCS_WITHOUT_TRIM_MAX_STORED_STATES
     /* The list of vacant items as freed by the garbage collector. Use
      * if before allocating more. */
-    fc_solve_hash_symlink_item_t *list_of_vacant_items;
+    fcs_hash_item_t *list_of_vacant_items;
 #endif
 /* A comparison function that can be used for comparing two keys
    in the collection */
@@ -125,7 +123,7 @@ static inline void fc_solve_hash_init(
 #else
 #endif
 #endif
-    )
+)
 {
     const typeof(hash->size) initial_hash_size = 2048;
 
@@ -155,14 +153,11 @@ static inline void fc_solve_hash_init(
 #endif
 
     fc_solve_compact_allocator_init(&(hash->allocator), meta_alloc);
-
-    return;
 }
 
 static inline void fc_solve_hash_free(fc_solve_hash_t *const hash)
 {
     fc_solve_compact_allocator_finish(&(hash->allocator));
-
     free(hash->entries);
     hash->entries = NULL;
 }
@@ -173,14 +168,15 @@ static inline void fc_solve_hash_foreach(fc_solve_hash_t *const hash,
     void *const context)
 {
     const_SLOT(size, hash);
+    var_AUTO(entries, hash->entries);
     for (int i = 0; i < size; i++)
     {
-        fc_solve_hash_symlink_item_t **item = &(hash->entries[i].first_item);
+        fcs_hash_item_t **item = &(entries[i].first_item);
         while ((*item) != NULL)
         {
             if (should_delete_ptr((*item)->key, context))
             {
-                fc_solve_hash_symlink_item_t *const next_item = (*item)->next;
+                fcs_hash_item_t *const next_item = (*item)->next;
                 /* Garbage collect (*item). */
                 (*item)->next = hash->list_of_vacant_items;
                 hash->list_of_vacant_items = (*item);
