@@ -11,6 +11,7 @@
  * fcs_user.h - main header file for the Freecell Solver library.
  */
 #pragma once
+#include <stddef.h>
 
 #include "fcs_dllexport.h"
 #include "fcs_enums.h"
@@ -41,10 +42,10 @@ DLLEXPORT extern void freecell_solver_user_limit_iterations_long(
 #ifndef FCS_BREAK_BACKWARD_COMPAT_1
 DLLEXPORT extern void freecell_solver_user_limit_iterations(
     void *user_instance, int max_iters);
-#endif
 
 DLLEXPORT extern int freecell_solver_user_set_tests_order(void *user_instance,
-    const char *tests_order FCS__PASS_ERR_STR(char **error_string));
+    const char *moves_order FCS__PASS_ERR_STR(char **error_string));
+#endif
 
 DLLEXPORT extern int freecell_solver_user_solve_board(
     void *user_instance, const char *state_as_string);
@@ -67,7 +68,7 @@ DLLEXPORT extern char *freecell_solver_user_current_state_as_string(
     ,
     int display_10_as_t
 #endif
-    );
+);
 #endif
 
 DLLEXPORT extern void freecell_solver_user_current_state_stringify(
@@ -82,7 +83,7 @@ DLLEXPORT extern void freecell_solver_user_current_state_stringify(
     ,
     int display_10_as_t
 #endif
-    );
+);
 
 DLLEXPORT extern void freecell_solver_user_free(void *user_instance);
 
@@ -112,20 +113,19 @@ DLLEXPORT extern int freecell_solver_user_set_game(void *user_instance,
 #endif
 
 DLLEXPORT extern void freecell_solver_user_set_solution_optimization(
-    void *const user_instance, const int optimize);
+    void *user_instance, int optimize);
 
 #ifndef FCS_BREAK_BACKWARD_COMPAT_1
 DLLEXPORT extern char *freecell_solver_user_move_to_string(
     fcs_move_t move, int standard_notation);
 
 DLLEXPORT extern char *freecell_solver_user_move_to_string_w_state(
-    void *const user_instance, const fcs_move_t move,
-    const int standard_notation);
+    void *user_instance, fcs_move_t move, int standard_notation);
 #endif
 
 DLLEXPORT extern void freecell_solver_user_stringify_move_w_state(
-    void *const user_instance, char *const output_string, const fcs_move_t move,
-    const int standard_notation);
+    void *user_instance, char *output_string, fcs_move_t move,
+    int standard_notation);
 
 #ifndef FCS_BREAK_BACKWARD_COMPAT_1
 DLLEXPORT extern void freecell_solver_user_limit_depth(
@@ -154,7 +154,7 @@ DLLEXPORT void freecell_solver_user_get_invalid_state_error_into_string(
     ,
     const int print_ts
 #endif
-    );
+);
 #endif
 
 #ifdef FCS_WITH_ERROR_STRS
@@ -165,7 +165,7 @@ DLLEXPORT extern char *freecell_solver_user_get_invalid_state_error_string(
     ,
     int print_ts
 #endif
-    );
+);
 #endif
 #endif
 
@@ -214,7 +214,7 @@ DLLEXPORT extern char *freecell_solver_user_iter_state_as_string(
     ,
     const int display_10_as_t
 #endif
-    );
+);
 #endif
 
 DLLEXPORT extern void freecell_solver_user_iter_state_stringify(
@@ -229,11 +229,12 @@ DLLEXPORT extern void freecell_solver_user_iter_state_stringify(
     ,
     const int display_10_as_t
 #endif
-    );
+);
 
 DLLEXPORT extern void freecell_solver_user_set_random_seed(
     void *const user_instance, const int seed);
 
+#ifndef FCS_DISABLE_NUM_STORED_STATES
 DLLEXPORT fcs_int_limit_t
 freecell_solver_user_get_num_states_in_collection_long(
     void *const user_instance);
@@ -253,6 +254,7 @@ DLLEXPORT extern void freecell_solver_user_limit_num_states_in_collection(
 
 DLLEXPORT extern void freecell_solver_set_stored_states_trimming_limit(
     void *user_instance, long max_num_states);
+#endif
 
 DLLEXPORT extern int freecell_solver_user_next_soft_thread(void *user_instance);
 
@@ -278,13 +280,13 @@ DLLEXPORT extern int freecell_solver_user_set_hard_thread_prelude(
     void *const user_instance, freecell_solver_str_t prelude);
 
 DLLEXPORT extern int freecell_solver_user_set_flares_plan(
-    void *const api_instance, freecell_solver_str_t prelude);
+    void *const api_instance, freecell_solver_str_t plan);
 
 DLLEXPORT extern void freecell_solver_user_recycle(void *const user_instance);
 
 DLLEXPORT extern int freecell_solver_user_set_optimization_scan_tests_order(
     void *const user_instance,
-    const char *const tests_order FCS__PASS_ERR_STR(char **const error_string));
+    const char *const moves_order FCS__PASS_ERR_STR(char **const error_string));
 
 DLLEXPORT extern void freecell_solver_user_set_reparent_states(
     void *const user_instance, const int to_reparent_states);
@@ -306,10 +308,12 @@ DLLEXPORT extern int freecell_solver_user_next_flare(void *const api_instance);
  * This function resets the user_instance, making it lose
  * all the previous command line arguments it encountered
  * */
+#ifndef FCS_BREAK_BACKWARD_COMPAT_1
 DLLEXPORT extern int freecell_solver_user_reset(void *const user_instance);
 
 DLLEXPORT const char *freecell_solver_user_get_lib_version(
     void *const user_instance);
+#endif
 
 DLLEXPORT const char *freecell_solver_user_get_current_soft_thread_name(
     void *const user_instance);
@@ -324,14 +328,16 @@ DLLEXPORT const char *freecell_solver_user_get_last_error_string(
 #endif
 
 /*
- * This sets the tests_order from min_depth onwards.
+ * This sets the moves_order from min_depth onwards.
  * */
 DLLEXPORT extern int freecell_solver_user_set_depth_tests_order(
     void *const user_instance, const int min_depth,
-    const char *const tests_order FCS__PASS_ERR_STR(char **const error_string));
+    const char *const moves_order FCS__PASS_ERR_STR(char **const error_string));
 
+#ifndef FCS_ENABLE_PRUNE__R_TF__UNCOND
 DLLEXPORT extern int freecell_solver_user_set_pruning(void *const user_instance,
     const char *const pruning FCS__PASS_ERR_STR(char **const error_string));
+#endif
 
 DLLEXPORT extern int freecell_solver_user_set_cache_limit(
     void *const user_instance, long limit);
@@ -353,6 +359,15 @@ DLLEXPORT extern int freecell_solver_user_set_patsolve_y_param(
     void *const api_instance, const int position,
     const double y_param_val FCS__PASS_ERR_STR(char **const error_string));
 
+typedef struct
+{
+    size_t scan_idx;
+    size_t quota;
+} fc_solve_prelude_item;
+
+DLLEXPORT extern void fc_solve_user_set_ht_compiled_prelude(
+    void *, size_t, const fc_solve_prelude_item *);
+
 #ifdef __cplusplus
 }
 #endif
