@@ -7,16 +7,15 @@
  *
  * Copyright (c) 2000 Shlomi Fish
  */
-/*
- * game_type_params.h: provide fcs_game_type_params_t
- */
+// game_type_params.h: provide fcs_game_type_params_t
 #pragma once
 
-#include "config.h"
 #include "game_type_limit.h"
 
+#ifndef FCS_FREECELL_ONLY
 typedef struct
 {
+#endif
 /*
  * The number of Freecells, Stacks and Foundations present in the game.
  *
@@ -82,5 +81,16 @@ typedef struct
 #define INSTANCE_EMPTY_STACKS_FILL ((INSTANCE_GAME_FLAGS >> 2) & 0x3)
 
 #endif
-
+#if ((!defined(HARD_CODED_NUM_FREECELLS)) || (!defined(HARD_CODED_NUM_STACKS)))
+#define FC__STACKS__SET_PARAMS() SET_GAME_PARAMS()
+#else
+#define FC__STACKS__SET_PARAMS()
+#endif
+#ifndef HARD_CODED_NUM_STACKS
+#define STACKS__SET_PARAMS() SET_GAME_PARAMS()
+#else
+#define STACKS__SET_PARAMS()
+#endif
+#ifndef FCS_FREECELL_ONLY
 } fcs_game_type_params_t;
+#endif
