@@ -12,17 +12,7 @@
  * languages.
  */
 
-#include "dll_thunk.h"
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include "set_weights.h"
-#include "fcs_user.h"
 #include "fcs_cl.h"
-#include "split_cmd_line.h"
-#include "prefix.h"
-#include "rinutils.h"
-#include "cmd_line_enum.h"
 
 DLLEXPORT void freecell_solver_user_stringify_move_ptr(
     void *const user_instance, char *const output_string,
