@@ -18,11 +18,13 @@
 extern "C" {
 #endif
 
-#include <stdio.h>
-
 #include "rinutils.h"
+#ifndef FCS_USE_PRECOMPILED_CMD_LINE_THEME
 #include "fcs_cl.h"
-
+#else
+#include "fcs_user.h"
+#include "fc_solve__precompiled__theme.h"
+#endif
 enum
 {
     EXIT_AND_RETURN_0 = FCS_CMD_LINE_USER
@@ -34,10 +36,10 @@ static inline void *alloc_instance_and_parse(const int argc, char **const argv,
     void *const callback_context, const fcs_bool_t only_recognized)
 {
     void *const instance = freecell_solver_user_alloc();
-
-#ifdef FCS_WITH_ERROR_STRS
-    char *error_string;
-#endif
+#ifdef FCS_USE_PRECOMPILED_CMD_LINE_THEME
+    my_init_instance(instance);
+#else
+    FCS__DECL_ERR_PTR(error_string);
     switch (freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
         instance, argc, (freecell_solver_str_t *)(void *)argv, (*arg_ptr),
         known_parameters, callback,
@@ -54,8 +56,9 @@ static inline void *alloc_instance_and_parse(const int argc, char **const argv,
         }
         break;
     case FCS_CMD_LINE_PARAM_WITH_NO_ARG:
-        fprintf(stderr, "The command line parameter \"%s\" requires an argument"
-                        " and was not supplied with one.\n",
+        fprintf(stderr,
+            "The command line parameter \"%s\" requires an argument"
+            " and was not supplied with one.\n",
             argv[*arg_ptr]);
         freecell_solver_user_free(instance);
         exit(-1);
@@ -72,15 +75,24 @@ static inline void *alloc_instance_and_parse(const int argc, char **const argv,
         freecell_solver_user_free(instance);
         exit(-1);
     }
-
+#endif
     return instance;
 }
 
 static inline void *simple_alloc_and_parse(
-    const int argc, char **const argv, int *const arg_ptr)
+    const int argc, char **const argv, int arg)
 {
-    return alloc_instance_and_parse(
-        argc, argv, arg_ptr, NULL, NULL, NULL, TRUE);
+    return alloc_instance_and_parse(argc, argv, &arg, NULL, NULL, NULL, TRUE);
+}
+
+static inline void print_flares_plan_error(void *const instance)
+{
+#ifdef FCS_WITH_ERROR_STRS
+    fprintf(stderr, "Flares Plan: %s\n",
+        freecell_solver_user_get_last_error_string(instance));
+#else
+    fprintf(stderr, "%s\n", "Flares Plan Error");
+#endif
 }
 
 #ifdef __cplusplus
