@@ -7,10 +7,8 @@
  *
  * Copyright (c) 2000 Shlomi Fish
  */
-/*
- * instance.h - header file of fc_solve_instance_t / fc_solve_hard_thread_t /
- * fc_solve_soft_thread_t .
- */
+// instance.h - header file of fc_solve_instance_t / fc_solve_hard_thread_t /
+// fc_solve_soft_thread_t .
 #pragma once
 
 #ifdef __cplusplus
@@ -19,11 +17,9 @@ extern "C" {
 
 #include <math.h>
 
-#include "config.h"
-#include "state.h"
 #include "move.h"
 #include "fcs_enums.h"
-
+#include "fcs_user.h"
 #include "rate_state.h"
 #include "indirect_buffer.h"
 #include "rand.h"
@@ -32,21 +28,15 @@ extern "C" {
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBREDBLACK_TREE) ||               \
     (defined(INDIRECT_STACK_STATES) &&                                         \
         (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBREDBLACK_TREE))
-
 #include <redblack.h>
-
 #endif
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_LIBAVL2_TREE)
-
 #include "fcs_libavl2_state_storage.h"
-
 #endif
 
 #if (FCS_STACK_STORAGE == FCS_STACK_STORAGE_LIBAVL2_TREE)
-
 #include "fcs_libavl2_stack_storage.h"
-
 #endif
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_TREE) ||                      \
@@ -54,9 +44,7 @@ extern "C" {
     (defined(INDIRECT_STACK_STATES) &&                                         \
         ((FCS_STACK_STORAGE == FCS_STACK_STORAGE_GLIB_TREE) ||                 \
             (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GLIB_HASH)))
-
 #include <glib.h>
-
 #endif
 
 #if ((defined(FCS_RCS_STATES) &&                                               \
@@ -64,53 +52,32 @@ extern "C" {
      (FCS_STATE_STORAGE == FCS_STATE_STORAGE_JUDY) ||                          \
      (defined(INDIRECT_STACK_STATES) &&                                        \
          (FCS_STACK_STORAGE == FCS_STACK_STORAGE_JUDY)))
-
 #include <Judy.h>
-
 #endif
 
 #if ((FCS_STATE_STORAGE == FCS_STATE_STORAGE_INTERNAL_HASH) ||                 \
      (FCS_STACK_STORAGE == FCS_STACK_STORAGE_INTERNAL_HASH))
-
 #include "fcs_hash.h"
-
 #endif
 
-#if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GOOGLE_DENSE_HASH)
-
+#if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GOOGLE_DENSE_HASH) ||              \
+    (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GOOGLE_DENSE_HASH)
 #include "google_hash.h"
-
-#endif
-
-#if (FCS_STACK_STORAGE == FCS_STACK_STORAGE_GOOGLE_DENSE_HASH)
-
-#include "google_hash.h"
-
 #endif
 
 #if ((FCS_STATE_STORAGE == FCS_STATE_STORAGE_KAZ_TREE) ||                      \
      (defined(FCS_RCS_STATES) &&                                               \
          (FCS_RCS_CACHE_STORAGE == FCS_RCS_CACHE_STORAGE_KAZ_TREE)))
-
 #include "kaz_tree.h"
-
 #endif
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_DB_FILE)
-#include <sys/types.h>
-#include <limits.h>
 #include <db.h>
 #endif
 
 #include "pqueue.h"
-
 #include "meta_alloc.h"
 
-#ifndef FCS_DISABLE_SIMPLE_SIMON
-#define FCS_SS_POS_BY_RANK_WIDTH (13 + 1)
-#define FCS_SS_POS_BY_RANK_LEN (FCS_SS_POS_BY_RANK_WIDTH * 4)
-#endif
-
 /* We need 2 chars per card - one for the column_idx and one
  * for the card_idx.
  *
@@ -121,15 +88,16 @@ extern "C" {
  * */
 #define FCS_POS_BY_RANK_WIDTH (MAX_NUM_DECKS << 3)
 
-/* We don't keep track of kings (rank == 13). */
-#define NUM_POS_BY_RANK_SLOTS 13
-#define FCS_POS_BY_RANK_LEN (NUM_POS_BY_RANK_SLOTS * FCS_POS_BY_RANK_WIDTH)
+// We don't keep track of kings
+#define FCS_POS_BY_RANK_LEN (FCS_RANK_KING * FCS_POS_BY_RANK_WIDTH)
 typedef struct
 {
     char col, height;
 } fcs_pos_by_rank_t;
 
 #ifndef FCS_DISABLE_SIMPLE_SIMON
+#define FCS_SS_POS_BY_RANK_WIDTH (FCS_RANK_KING + 1)
+#define FCS_SS_POS_BY_RANK_LEN (FCS_SS_POS_BY_RANK_WIDTH * 4)
 #define FCS_BOTH__POS_BY_RANK__SIZE                                            \
     (max(FCS_SS_POS_BY_RANK_LEN * sizeof(fcs_pos_by_rank_t),                   \
         FCS_POS_BY_RANK_LEN))
@@ -158,7 +126,7 @@ struct fc_solve_soft_thread_struct;
 struct fc_solve_instance_struct;
 
 typedef void (*fc_solve_solve_for_state_move_func_t)(
-    struct fc_solve_soft_thread_struct *, fcs_kv_state_t *,
+    struct fc_solve_soft_thread_struct *, fcs_kv_state_t,
     fcs_derived_states_list_t *);
 
 #ifdef FCS_SINGLE_HARD_THREAD
@@ -168,9 +136,7 @@ typedef struct fc_solve_hard_thread_struct fc_solve_hard_thread_t;
 #endif
 
 extern fcs_bool_t fc_solve_check_and_add_state(
-    fc_solve_hard_thread_t *const hard_thread,
-    fcs_kv_state_t *const new_state_val,
-    fcs_kv_state_t *const existing_state_val);
+    fc_solve_hard_thread_t *, fcs_kv_state_t *, fcs_kv_state_t *);
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_HASH)
 extern guint fc_solve_hash_function(gconstpointer key);
@@ -198,12 +164,7 @@ extern guint fc_solve_hash_function(gconstpointer key);
     fc_solve_soft_thread_t *const end_soft_thread =                            \
         ht_soft_threads + HT_FIELD(hard_thread, num_soft_threads);             \
     for (; soft_thread < end_soft_thread; soft_thread++)
-
-#define ST_LOOP__WAS_FINISHED() (soft_thread == end_soft_thread)
-
-#define ST_LOOP__GET_INDEX() (soft_thread - ht_soft_threads)
-
-#define TESTS_ORDER_GROW_BY 16
+#define MOVES_GROW_BY 16
 
 typedef struct
 {
@@ -217,10 +178,8 @@ typedef struct
         num_cards_not_on_parents_factor;
 
     double num_cards_out_lookup_table[14];
-    /*
-     * The BeFS weights of the different BeFS tests. Those
-     * weights determine the commulative priority of the state.
-     * */
+    // The BeFS weights of the different BeFS tests. Those
+    // weights determine the commulative priority of the state.
     fcs_default_weights_t befs_weights;
 } fc_solve_state_weighting_t;
 
@@ -228,55 +187,42 @@ typedef enum {
     FCS_NO_SHUFFLING,
     FCS_RAND,
     FCS_WEIGHTING,
-} fcs_tests_group_type_t;
+} fcs_moves_group_kind;
+
+typedef union {
+    fc_solve_solve_for_state_move_func_t f;
+    uint_fast32_t idx;
+} fcs_move_func;
 
 typedef struct
 {
-    size_t num;
-    size_t *order_group_tests;
-    fcs_tests_group_type_t shuffling_type;
+    fcs_move_func *move_funcs;
+    uint_fast32_t num;
+    fcs_moves_group_kind shuffling_type;
     fc_solve_state_weighting_t weighting;
-} fcs_tests_order_group_t;
+} fcs_moves_group;
 
 typedef struct
 {
-    size_t num_groups;
-    fcs_tests_order_group_t *groups;
-} fcs_tests_order_t;
+    uint_fast32_t num;
+    fcs_moves_group *groups;
+} fcs_moves_order;
 
 typedef struct
 {
     ssize_t max_depth;
-    fcs_tests_order_t tests_order;
-} fcs_by_depth_tests_order_t;
-
-typedef struct
-{
-    size_t num;
-    fcs_by_depth_tests_order_t *by_depth_tests;
-} fcs_by_depth_tests_order_array_t;
-
-#define STRUCT_CLEAR_FLAG(instance, flag)                                      \
-    {                                                                          \
-        (instance)->flag = FALSE;                                              \
-    }
-
-#define STRUCT_TURN_ON_FLAG(instance, flag)                                    \
-    {                                                                          \
-        (instance)->flag = TRUE;                                               \
-    }
+    fcs_moves_order moves_order;
+} fcs_by_depth_moves_order;
 
+#define STRUCT_CLEAR_FLAG(instance, flag) (instance)->flag = FALSE
+#define STRUCT_TURN_ON_FLAG(instance, flag) (instance)->flag = TRUE
 #define STRUCT_QUERY_FLAG(instance, flag) ((instance)->flag)
-
-#define STRUCT_SET_FLAG_TO(instance, flag, value)                              \
-    {                                                                          \
-        (instance)->flag = (value);                                            \
-    }
+#define STRUCT_SET_FLAG_TO(instance, flag, value) (instance)->flag = (value)
 
 #ifdef FCS_RCS_STATES
 struct fcs_cache_key_info_struct
 {
-    fcs_collectible_state_t *val_ptr;
+    const fcs_collectible_state_t *val_ptr;
     fcs_state_t key;
     /* lower_pri and higher_pri form a doubly linked list.
      *
@@ -299,9 +245,7 @@ typedef struct
     fcs_compact_allocator_t states_values_to_keys_allocator;
     fcs_int_limit_t count_elements_in_cache, max_num_elements_in_cache;
 
-    fcs_cache_key_info_t *lowest_pri, *highest_pri;
-
-    fcs_cache_key_info_t *recycle_bin;
+    fcs_cache_key_info_t *lowest_pri, *highest_pri, *recycle_bin;
 } fcs_lru_cache_t;
 
 #endif
@@ -310,23 +254,13 @@ typedef struct
 typedef void *fcs_instance_debug_iter_output_context_t;
 
 typedef void (*fcs_instance_debug_iter_output_func_t)(
-    fcs_instance_debug_iter_output_context_t, fcs_int_limit_t iter_num,
-    int depth, void *instance, fcs_kv_state_t *state,
-    fcs_int_limit_t parent_iter_num);
+    fcs_instance_debug_iter_output_context_t, fcs_int_limit_t, int, void *,
+    fcs_kv_state_t *, fcs_int_limit_t);
 #endif
 
 typedef struct fc_solve_soft_thread_struct fc_solve_soft_thread_t;
-
 typedef struct fc_solve_instance_struct fc_solve_instance_t;
 
-/***************************************************/
-
-typedef struct
-{
-    size_t scan_idx;
-    size_t quota;
-} fcs_prelude_item_t;
-
 struct fc_solve_hard_thread_struct
 {
 #ifndef FCS_SINGLE_HARD_THREAD
@@ -351,21 +285,10 @@ struct fc_solve_hard_thread_struct
      * */
     fcs_int_limit_t ht__max_num_checked_states;
 
-    /*
-     * These variables are used to compute the MD5 checksum of a state
-     * that is about to be checked. I decided to make them globals so
-     * they won't have to be re-allocated and freed all the time.
-     *
-     * Notice that it is only used with my internal hash implmentation
-     * as GLib requires a dedicated hash function, which cannot
-     * access the instance.
-     *
-     * */
-
     /*
      * The index for the soft-thread that is currently processed
      * */
-    int st_idx;
+    int_fast32_t st_idx;
     /*
      * This is the mechanism used to allocate memory for stacks, states
      * and move stacks.
@@ -375,7 +298,7 @@ struct fc_solve_hard_thread_struct
 #ifdef FCS_WITH_MOVES
     /*
      * This is a move stack that is used and re-used by the
-     * tests functions of this hard thread
+     * moves functions of this hard thread
      * */
     fcs_move_stack_t reusable_move_stack;
 #endif
@@ -388,10 +311,14 @@ struct fc_solve_hard_thread_struct
 
     size_t prelude_num_items;
     size_t prelude_idx;
-    fcs_prelude_item_t *prelude;
+#ifndef FCS_USE_PRECOMPILED_CMD_LINE_THEME
+    fc_solve_prelude_item *prelude;
+#else
+    const fc_solve_prelude_item *prelude;
+#endif
 
     fcs_bool_t allocated_from_list;
-    int num_soft_threads;
+    int_fast32_t num_soft_threads;
 
     /*
      * A counter that determines how many of the soft threads that belong
@@ -400,7 +327,9 @@ struct fc_solve_hard_thread_struct
      * */
     int num_soft_threads_finished;
 
+#ifndef FCS_USE_PRECOMPILED_CMD_LINE_THEME
     char *prelude_as_string;
+#endif
 };
 
 /********************************************/
@@ -415,8 +344,8 @@ typedef struct
 {
     fcs_collectible_state_t *state;
     fcs_derived_states_list_t derived_states_list;
+    int_fast32_t move_func_list_idx;
     int current_state_index;
-    int tests_list_index;
     int move_func_idx;
     int derived_states_random_indexes_max_size;
     fcs_rating_with_index_t *derived_states_random_indexes;
@@ -425,32 +354,17 @@ typedef struct
     fcs_game_limit_t num_vacant_freecells;
 } fcs_soft_dfs_stack_item_t;
 
-typedef struct
-{
-    fc_solve_solve_for_state_move_func_t *tests;
-    /* num_move_funcs should be int instead of size_t for performance.*/
-    int num_move_funcs;
-    fcs_tests_group_type_t shuffling_type;
-    fc_solve_state_weighting_t weighting;
-} fcs_tests_list_t;
-
-typedef struct
-{
-    int num_lists;
-    fcs_tests_list_t *lists;
-} fcs_tests_list_of_lists;
-
 typedef struct
 {
     ssize_t max_depth;
-    fcs_tests_list_of_lists tests;
-} fcs_tests_by_depth_unit_t;
+    fcs_moves_order move_funcs;
+} fcs_moves_by_depth_unit_t;
 
 typedef struct
 {
     int num_units;
-    fcs_tests_by_depth_unit_t *by_depth_units;
-} fcs_tests_by_depth_array_t;
+    fcs_moves_by_depth_unit_t *by_depth_units;
+} fcs_moves_by_depth_array;
 
 typedef enum {
     FCS_SUPER_METHOD_DFS,
@@ -471,21 +385,16 @@ struct fc_solve_soft_thread_struct
      * */
     int id;
 
-    /*
-     * The tests' order indicates which tests (i.e: kinds of multi-moves) to
-     * do at what order. This is most relevant to DFS and Soft-DFS.
-     *
-     * tests_order_num is the number of tests in the test's order. Notice
-     * that it can be lower than FCS_TESTS_NUM, thus enabling several tests
-     * to be removed completely.
-     * */
-    fcs_by_depth_tests_order_array_t by_depth_tests_order;
+    // The moves' order indicates which move funcs to run.
+    struct
+    {
+        size_t num;
+        fcs_by_depth_moves_order *by_depth_moves;
+    } by_depth_moves_order;
 
     /* The super-method type - can be  */
     fcs_super_method_type_t super_method_type;
 
-    fc_solve_seq_cards_power_type_t initial_cards_under_sequences_value;
-
     struct
     {
         struct
@@ -496,22 +405,17 @@ struct fc_solve_soft_thread_struct
             ssize_t dfs_max_depth;
 
             /*
-             * These are stacks used by the Soft-DFS for various uses.
+             * Soft-DFS uses a stack of fcs_soft_dfs_stack_item_t s.
              *
-             * states_to_check - an array of states to be checked next. Not
+             * derived_states_list - a list of states to be checked next. Not
              * all of them will be checked because it is possible that future
              * states already visited them.
              *
-             * states_to_check_move_stacks - an array of move stacks that
-             * lead to those states.
-             *
-             * num_states_to_check - the size of states_to_check[i]
-             *
-             * current_state_indexes - the index of the last checked state
+             * current_state_index - the index of the last checked state
              * in depth i.
              *
-             * test_indexes - the index of the test that was last
-             * performed. FCS performs each test separately, so
+             * move_func_list_idx - the index of the move list that is
+             * performed. FCS performs each move separately, so
              * states_to_check and friends will not be overpopulated.
              *
              * num_vacant_stacks - the number of unoccpied stacks that
@@ -521,7 +425,6 @@ struct fc_solve_soft_thread_struct
              * num_vacant_freecells - ditto for the freecells.
              *
              * */
-
             fcs_soft_dfs_stack_item_t *soft_dfs_info;
 
             /* The depth of the DFS stacks */
@@ -538,14 +441,14 @@ struct fc_solve_soft_thread_struct
             int rand_seed;
 
             /*
-             * The tests to be performed in a preprocessed form.
+             * The moves to be performed in a preprocessed form.
              * */
-            fcs_tests_by_depth_array_t tests_by_depth_array;
+            fcs_moves_by_depth_array moves_by_depth;
         } soft_dfs;
         struct
         {
             fcs__positions_by_rank_t befs_positions_by_rank;
-            fc_solve_solve_for_state_move_func_t *tests_list, *tests_list_end;
+            fcs_move_func *moves_list, *moves_list_end;
             struct
             {
                 struct
@@ -587,34 +490,28 @@ struct fc_solve_soft_thread_struct
     fcs_bool_t FCS_SOFT_THREAD_IS_FINISHED, FCS_SOFT_THREAD_INITIALIZED,
         FCS_SOFT_THREAD_IS_A_COMPLETE_SCAN;
 
-    /*
-     * The number of vacant stacks in the current state - is read by
-     * the move functions in freecell.c .
-     * */
-    fcs_game_limit_t num_vacant_stacks;
-
-    /*
-     * The number of vacant freecells in the current state - is read
-     * from the test functions in freecell.c .
-     * */
-    fcs_game_limit_t num_vacant_freecells;
+    // The numbers of vacant stacks and freecells in the current state - is
+    // read by the move functions in freecell.c .
+    fcs_game_limit_t num_vacant_stacks, num_vacant_freecells;
 
     /*
      * The number of iterations with which to process this scan
      * */
-    int checked_states_step;
+    fcs_int_limit_t checked_states_step;
 
-    /*
-     * A string that serves as an identification for the user.
-     * */
+#ifndef FCS_USE_PRECOMPILED_CMD_LINE_THEME
+    // A string that serves as an identification for the user.
     char name[FCS_MAX_IDENT_LEN];
+#endif
 
+#ifndef FCS_ENABLE_PRUNE__R_TF__UNCOND
     /*
      * Whether pruning should be done.
      * This variable is temporary - there should be a better pruning
      * abstraction with several optional prunes.
      * */
     fcs_bool_t enable_pruning;
+#endif
 
 #ifndef FCS_DISABLE_PATSOLVE
     /*
@@ -636,13 +533,9 @@ struct fc_solve_soft_thread_struct
 
 struct fc_solve_instance_struct
 {
-    /*
-     * The parameters of the game - see the declaration of
-     * fcs_game_type_params_t .
-     *
-     * */
-    fcs_game_type_params_t game_params;
+// The parameters of the game - see the declaration of fcs_game_type_params_t .
 #ifndef FCS_FREECELL_ONLY
+    fcs_game_type_params_t game_params;
 #ifndef FCS_DISABLE_PATSOLVE
     fcs_card_t game_variant_suit_mask;
     fcs_card_t game_variant_desired_suit_value;
@@ -652,6 +545,7 @@ struct fc_solve_instance_struct
     /* The number of states that were checked by the solving algorithm. */
     fcs_int_limit_t i__num_checked_states;
 
+#ifndef FCS_WITHOUT_MAX_NUM_STATES
     /*
      * Limit for the maximal number of checked states. max_num_checked_states
      * is useful because it can limit the amount of consumed memory (and time).
@@ -661,11 +555,15 @@ struct fc_solve_instance_struct
      *
      * Normally should be used instead.
      * */
-    fcs_int_limit_t effective_max_num_checked_states,
-        effective_max_num_states_in_collection;
+    fcs_int_limit_t effective_max_num_checked_states;
+#endif
+#ifndef FCS_DISABLE_NUM_STORED_STATES
+    fcs_int_limit_t effective_max_num_states_in_collection;
 #ifndef FCS_WITHOUT_TRIM_MAX_STORED_STATES
     fcs_int_limit_t effective_trim_states_in_collection_from;
 #endif
+#endif
+    fc_solve_seq_cards_power_type_t initial_cards_under_sequences_value;
 /*
  * tree is the balanced binary tree that is used to store and index
  * the checked states.
@@ -751,10 +649,12 @@ struct fc_solve_instance_struct
  *
  * It gives a rough estimate of the memory occupied by the instance.
  * */
+#ifndef FCS_DISABLE_NUM_STORED_STATES
 #ifndef FCS_WITHOUT_TRIM_MAX_STORED_STATES
     fcs_int_limit_t active_num_states_in_collection;
 #endif
     fcs_int_limit_t num_states_in_collection;
+#endif
 
 #ifdef FCS_SINGLE_HARD_THREAD
     struct fc_solve_hard_thread_struct hard_thread;
@@ -763,7 +663,7 @@ struct fc_solve_instance_struct
     struct fc_solve_soft_thread_struct optimization_soft_thread;
 #endif
 #else
-    int num_hard_threads;
+    uint_fast32_t num_hard_threads;
     struct fc_solve_hard_thread_struct *hard_threads;
     /*
      * An iterator over the hard threads.
@@ -778,11 +678,9 @@ struct fc_solve_instance_struct
 #endif
 #endif
 
-    /*
-     * This is the master tests order. It is used to initialize all
-     * the new Soft-Threads.
-     * */
-    fcs_tests_order_t instance_tests_order;
+    // The master moves' order. It is used to initialize all the new
+    // Soft-Threads.
+    fcs_moves_order instance_moves_order;
 
     /*
      * A counter that determines how many of the hard threads that belong
@@ -792,10 +690,8 @@ struct fc_solve_instance_struct
     int num_hard_threads_finished;
 
 #ifdef FCS_WITH_MOVES
-    /*
-     * The tests order for the optimization scan as specified by the user.
-     * */
-    fcs_tests_order_t opt_tests_order;
+    // The moves for the optimization scan, as specified by the user.
+    fcs_moves_order opt_moves;
 #endif
 
 #ifdef FCS_RCS_STATES
@@ -833,15 +729,15 @@ struct fc_solve_instance_struct
      * */
     int next_soft_thread_id;
 
-    /* This is a place-holder for the initial state */
-    fcs_state_keyval_pair_t *state_copy_ptr;
+    /* This is the initial state */
+    fcs_state_keyval_pair_t state_copy;
 
+#ifdef FCS_WITH_MOVES
     /* This is the final state that the scan recommends to the
      * interface
      * */
     fcs_collectible_state_t *final_state;
 
-#ifdef FCS_WITH_MOVES
     /*
      * A move stack that contains the moves leading to the solution.
      *
@@ -856,13 +752,14 @@ struct fc_solve_instance_struct
      * */
     fcs_meta_compact_allocator_t *meta_alloc;
 
+#if (defined(FCS_WITH_MOVES) && (!defined(FCS_DISABLE_PATSOLVE)))
     /*
      * The soft_thread that solved the state.
      *
      * Needed to trace the patsolve solutions.
      * */
     fc_solve_soft_thread_t *solving_soft_thread;
-
+#endif
 #ifndef FCS_DISABLE_PATSOLVE
     /*
      * This is intended to be used by the patsolve scan which is
@@ -881,24 +778,15 @@ struct fc_solve_instance_struct
 };
 
 #ifdef FCS_SINGLE_HARD_THREAD
-static inline fc_solve_instance_t *fcs_st_instance(
-    fc_solve_soft_thread_t *const soft_thread)
-{
-    return soft_thread->hard_thread;
-}
 #define HT_FIELD(ht, field) (ht)->hard_thread.field
 #define HT_INSTANCE(hard_thread) (hard_thread)
 #define INST_HT0(instance) ((instance)->hard_thread)
 #else
-static inline fc_solve_instance_t *fcs_st_instance(
-    fc_solve_soft_thread_t *const soft_thread)
-{
-    return soft_thread->hard_thread->instance;
-}
 #define HT_FIELD(hard_thread, field) (hard_thread)->field
 #define HT_INSTANCE(hard_thread) ((hard_thread)->instance)
 #define INST_HT0(instance) ((instance)->hard_threads[0])
 #endif
+#define fcs_st_instance(soft_thread) HT_INSTANCE((soft_thread)->hard_thread)
 
 #define DFS_VAR(soft_thread, var) (soft_thread)->method_specific.soft_dfs.var
 #define BEFS_VAR(soft_thread, var)                                             \
@@ -922,7 +810,7 @@ static inline fc_solve_instance_t *fcs_st_instance(
 #include "pat.h"
 #endif
 
-extern int fc_solve_befs_or_bfs_do_solve(
+extern fc_solve_solve_process_ret_t fc_solve_befs_or_bfs_do_solve(
     fc_solve_soft_thread_t *const soft_thread);
 
 extern void fc_solve_increase_dfs_max_depth(
@@ -947,7 +835,7 @@ static inline int update_col_cards_under_sequences(
 #endif
     const fcs_const_cards_column_t col,
     int d /* One less than cards_num of col. */
-    )
+)
 {
     fcs_card_t this_card = fcs_col_get_card(col, d);
     fcs_card_t prev_card = fcs_col_get_card(col, d - 1);
@@ -961,45 +849,14 @@ static inline int update_col_cards_under_sequences(
     return d;
 }
 
-static inline void fc_solve_soft_thread_update_initial_cards_val(
-    fc_solve_soft_thread_t *const soft_thread)
-{
-    fc_solve_instance_t *const instance = fcs_st_instance(soft_thread);
-#ifndef FCS_FREECELL_ONLY
-    const int sequences_are_built_by =
-        GET_INSTANCE_SEQUENCES_ARE_BUILT_BY(instance);
-#endif
-    /* We cannot use typeof here because clang complains about double
-     * const.
-     * */
-    const fcs_state_t *const s = &(instance->state_copy_ptr->s);
-
-    fc_solve_seq_cards_power_type_t cards_under_sequences = 0;
-    for (int a = 0; a < INSTANCE_STACKS_NUM; a++)
-    {
-        const fcs_const_cards_column_t col = fcs_state_get_col(*s, a);
-        cards_under_sequences +=
-            FCS_SEQS_OVER_RENEGADE_POWER(update_col_cards_under_sequences(
-#ifndef FCS_FREECELL_ONLY
-                sequences_are_built_by,
-#endif
-                col, fcs_col_len(col) - 1));
-    }
-    soft_thread->initial_cards_under_sequences_value = cards_under_sequences;
-
-    return;
-}
-
 extern fcs_collectible_state_t *fc_solve_sfs_raymond_prune(
-    fc_solve_soft_thread_t *const, fcs_kv_state_t *const);
+    fc_solve_soft_thread_t *, fcs_kv_state_t);
 
 #ifdef FCS_RCS_STATES
-fcs_state_t *fc_solve_lookup_state_key_from_val(
-    fc_solve_instance_t *const instance,
-    fcs_collectible_state_t *const orig_ptr_state_val);
+fcs_state_t *fc_solve_lookup_state_key_from_val(fc_solve_instance_t *instance,
+    const fcs_collectible_state_t *orig_ptr_state_val);
 
-extern int fc_solve_compare_lru_cache_keys(
-    const void *const void_a, const void *const void_b, void *const context);
+extern int fc_solve_compare_lru_cache_keys(const void *, const void *, void *);
 
 #endif
 
@@ -1012,27 +869,22 @@ extern void fc_solve_instance__init_hard_thread(
 #endif
     fc_solve_hard_thread_t *const hard_thread);
 
-extern void fc_solve_free_soft_thread_by_depth_test_array(
+extern void fc_solve_free_soft_thread_by_depth_move_array(
     fc_solve_soft_thread_t *const soft_thread);
 
-static inline fcs_tests_order_t tests_order_dup(fcs_tests_order_t *const orig)
+static inline fcs_moves_order moves_order_dup(fcs_moves_order *const orig)
 {
-    fcs_tests_order_t ret;
+    const_SLOT(num, orig);
+    fcs_moves_order ret = (fcs_moves_order){.num = num,
+        .groups = memdup(orig->groups,
+            sizeof(orig->groups[0]) *
+                ((num & (~(MOVES_GROW_BY - 1))) + MOVES_GROW_BY))};
 
-    ret.num_groups = orig->num_groups;
-
-    ret.groups = memdup(
-        orig->groups, sizeof(orig->groups[0]) *
-                          ((ret.num_groups & (~(TESTS_ORDER_GROW_BY - 1))) +
-                              TESTS_ORDER_GROW_BY));
-
-    for (int i = 0; i < ret.num_groups; i++)
+    for (int i = 0; i < num; i++)
     {
-        ret.groups[i].order_group_tests =
-            memdup(ret.groups[i].order_group_tests,
-                sizeof(ret.groups[i].order_group_tests[0]) *
-                    ((ret.groups[i].num & (~(TESTS_ORDER_GROW_BY - 1))) +
-                        TESTS_ORDER_GROW_BY));
+        ret.groups[i].move_funcs = memdup(ret.groups[i].move_funcs,
+            sizeof(ret.groups[i].move_funcs[0]) *
+                ((ret.groups[i].num & (~(MOVES_GROW_BY - 1))) + MOVES_GROW_BY));
     }
 
     return ret;
@@ -1077,22 +929,24 @@ extern void fc_solve_foreach_soft_thread(fc_solve_instance_t *const instance,
     whatever memory was allocated by alloc_instance().
   */
 
-static inline void fc_solve_free_tests_order(fcs_tests_order_t *tests_order)
+static inline void moves_order__free(fcs_moves_order *moves_order)
 {
-    for (size_t group_idx = 0; group_idx < tests_order->num_groups; group_idx++)
+    const_SLOT(groups, moves_order);
+    const_SLOT(num, moves_order);
+    for (size_t group_idx = 0; group_idx < num; ++group_idx)
     {
-        free(tests_order->groups[group_idx].order_group_tests);
+        free(groups[group_idx].move_funcs);
     }
-    free(tests_order->groups);
-    tests_order->groups = NULL;
-    tests_order->num_groups = 0;
+    free(groups);
+    moves_order->groups = NULL;
+    moves_order->num = 0;
 }
 
-/***********************************************************/
+    /***********************************************************/
 
 #define DECLARE_MOVE_FUNCTION(name)                                            \
     extern void name(fc_solve_soft_thread_t *const soft_thread,                \
-        fcs_kv_state_t *const raw_ptr_state_raw,                               \
+        fcs_kv_state_t raw_state_raw,                                          \
         fcs_derived_states_list_t *const derived_states_list)
 
 #ifdef FCS_SINGLE_HARD_THREAD
@@ -1108,6 +962,11 @@ static inline fcs_bool_t fcs_get_calc_real_depth(
 }
 #endif
 
+#ifdef FCS_WITH_MOVES
+extern void fc_solve_trace_solution(fc_solve_instance_t *const instance);
+#endif
+extern void fc_solve_finish_instance(fc_solve_instance_t *const instance);
+
 #ifdef __cplusplus
 }
 #endif
