@@ -12,15 +12,19 @@
  */
 static inline void my_iter_handler_base(const fcs_int_limit_t iter_num,
     const int depth, void *const user_instance,
-    const fc_solve_display_information_context_t *const display_context,
+    const fc_solve_display_information_context_t *const dc,
     const fcs_int_limit_t parent_iter_num)
 {
     printf("Iteration: %li\nDepth: %i\nStored-States: %li\nScan: %s\n",
         (long)iter_num, depth,
+#ifdef FCS_DISABLE_NUM_STORED_STATES
+        1L,
+#else
         (long)freecell_solver_user_get_num_states_in_collection_long(
             user_instance),
+#endif
         freecell_solver_user_get_current_soft_thread_name(user_instance));
-    if (display_context->display_parent_iter_num)
+    if (dc->display_parent_iter_num)
     {
         printf("Parent Iteration: %li\n", (long)parent_iter_num);
     }
