@@ -63,24 +63,20 @@
 #define right dict_right
 #define parent dict_parent
 #define color dict_color
-#define data dict_data
 
 #define nilnode dict_nilnode
-#define nodecount dict_nodecount
-#define maxcount dict_maxcount
 #define compare dict_compare
-#define allocnode dict_allocnode
-#define freenode dict_free_node
 #define context dict_context
-#define dupes dict_dupes
-
-#define dictptr dict_dictptr
 
 #define dict_root(D) ((D)->nilnode.left)
 #define dict_nil(D) (&(D)->nilnode)
-#define DICT_DEPTH_MAX 64
 
 #ifdef NO_FC_SOLVE
+#define DICT_DEPTH_MAX 64
+#define allocnode dict_allocnode
+#define dupes dict_dupes
+#define dictptr dict_dictptr
+#define freenode dict_free_node
 static dnode_t *dnode_alloc(void *context);
 static void dnode_free(dnode_t *node, void *context);
 #endif
@@ -281,9 +277,9 @@ static int verify_dict_has_node(dnode_t *nil, dnode_t *root, dnode_t *node)
 }
 #endif
 
-/*
- * Dynamically allocate and initialize a dictionary object.
- */
+    /*
+     * Dynamically allocate and initialize a dictionary object.
+     */
 
 #ifdef NO_FC_SOLVE
 dict_t *dict_create(dictcount_t maxcount, dict_comp_t comp, void *context)
@@ -375,7 +371,6 @@ static void safe_traverse(dict_t *dict, void (*func)(dnode_t *, void *))
 void fc_solve_kaz_tree_destroy(dict_t *dict)
 {
     assert(dict_isempty(dict));
-
     fc_solve_compact_allocator_finish(&(dict->dict_allocator));
     free(dict);
 }
@@ -390,7 +385,7 @@ void fc_solve_kaz_tree_free_nodes(dict_t *dict)
 /* Removed for fc-solve. */
 #ifdef NO_FC_SOLVE
     safe_traverse(dict, dict->freenode);
-    dict->nodecount = 0;
+    dict->dict_nodecount = 0;
 #endif
     dict->nilnode.left = &dict->nilnode;
     dict->nilnode.right = &dict->nilnode;
@@ -410,9 +405,9 @@ void dict_free(dict_t *dict)
 }
 #endif
 
-/*
- * Initialize a user-supplied dictionary object.
- */
+    /*
+     * Initialize a user-supplied dictionary object.
+     */
 
 #ifdef NO_FC_SOLVE
 dict_t *dict_init(dict_t *dict, dictcount_t maxcount, dict_comp_t comp)
@@ -433,8 +428,8 @@ dict_t *dict_init(dict_t *dict, dict_comp_t comp,
 
     dict->context = NULL;
 #ifdef NO_FC_SOLVE
-    dict->nodecount = 0;
-    dict->maxcount = maxcount;
+    dict->dict_nodecount = 0;
+    dict->dict_maxcount = maxcount;
 #endif
     dict->nilnode.left = &dict->nilnode;
     dict->nilnode.right = &dict->nilnode;
@@ -462,8 +457,8 @@ void dict_init_like(dict_t *dict, const dict_t *orig)
 #endif
     dict->context = orig->context;
 #ifdef NO_FC_SOLVE
-    dict->nodecount = 0;
-    dict->maxcount = orig->maxcount;
+    dict->dict_nodecount = 0;
+    dict->dict_maxcount = orig->dict_maxcount;
 #endif
     dict->nilnode.left = &dict->nilnode;
     dict->nilnode.right = &dict->nilnode;
@@ -489,8 +484,8 @@ extern dict_t *dict_init_alloc(dict_t *dict, dictcount_t maxcount,
     dict->freenode = fr;
 #endif
     dict->context = context;
-    dict->nodecount = 0;
-    dict->maxcount = maxcount;
+    dict->dict_nodecount = 0;
+    dict->dict_maxcount = maxcount;
     dict->nilnode.left = &dict->nilnode;
     dict->nilnode.right = &dict->nilnode;
     dict->nilnode.parent = &dict->nilnode;
@@ -863,7 +858,7 @@ dict_ret_key_t fc_solve_kaz_tree_insert(
     node->right = nil;
 
 #ifdef NO_FC_SOLVE
-    dict->nodecount++;
+    ++dict->dict_nodecount;
 #endif
 
     /* red black adjustments */
@@ -1036,7 +1031,7 @@ static dnode_t *fc_solve_kaz_tree_delete(dict_t *dict, dnode_t *target)
     target->left = NULL;
 
 #ifdef NO_FC_SOLVE
-    dict->nodecount--;
+    --dict->dict_nodecount;
 #endif
 
     assert(verify_bintree(dict));
@@ -1143,7 +1138,7 @@ static inline dnode_t *dnode_init(dnode_t *dnode)
 #endif
 {
 #ifdef NO_FC_SOLVE
-    dnode->data = data;
+    dnode->dict_data = data;
 #endif
     dnode->parent = NULL;
     dnode->left = NULL;
@@ -1151,10 +1146,10 @@ static inline dnode_t *dnode_init(dnode_t *dnode)
     return dnode;
 }
 
-/*
- * Allocate a node using the dictionary's allocator routine, give it
- * the data item.
- */
+    /*
+     * Allocate a node using the dictionary's allocator routine, give it
+     * the data item.
+     */
 
 #ifdef NO_FC_SOLVE
 dict_ret_key_t fc_solve_kaz_tree_alloc_insert(
@@ -1314,11 +1309,14 @@ dnode_t *dict_prev(dict_t *dict, dnode_t *curr)
 #ifdef NO_FC_SOLVE
 void dict_allow_dupes(dict_t *dict) { dict->dupes = 1; }
 
-dictcount_t dict_count(dict_t *dict) { return dict->nodecount; }
+dictcount_t dict_count(dict_t *dict) { return dict->dict_nodecount; }
 
-int dict_isempty(dict_t *dict) { return dict->nodecount == 0; }
+int dict_isempty(dict_t *dict) { return dict->dict_nodecount == 0; }
 
-int dict_isfull(dict_t *dict) { return dict->nodecount == dict->maxcount; }
+int dict_isfull(dict_t *dict)
+{
+    return dict->dict_nodecount == dict->dict_maxcount;
+}
 
 int dict_contains(dict_t *dict, dnode_t *node)
 {
@@ -1336,7 +1334,7 @@ dnode_t *const dnode_create(void *const data)
 {
     dnode_t *dnode = (dnode_t *)SMALLOC1(dnode) if (dnode)
     {
-        dnode->data = data;
+        dnode->dict_data = data;
         dnode->parent = NULL;
         dnode->left = NULL;
         dnode->right = NULL;
@@ -1350,11 +1348,11 @@ void dnode_destroy(dnode_t *dnode)
     free(dnode);
 }
 
-void *dnode_get(dnode_t *dnode) { return dnode->data; }
+void *dnode_get(dnode_t *dnode) { return dnode->dict_data; }
 
 const void *dnode_getkey(dnode_t *dnode) { return dnode->key; }
 
-void dnode_put(dnode_t *dnode, void *data) { dnode->data = data; }
+void dnode_put(dnode_t *dnode, void *data) { dnode->dict_data = data; }
 
 int dnode_is_in_a_dict(dnode_t *dnode)
 {
@@ -1401,10 +1399,10 @@ void dict_load_next(dict_load_t *load, dnode_t *newnode, const void *key)
     dnode_t *nil = &load->nilnode;
 
     assert(!dnode_is_in_a_dict(newnode));
-    assert(dict->nodecount < dict->maxcount);
+    assert(dict->dict_nodecount < dict->dict_maxcount);
 
 #ifndef NDEBUG
-    if (dict->nodecount > 0)
+    if (dict->dict_nodecount > 0)
     {
         if (dict->dupes)
             assert(dict->compare(nil->left->key, key) <= 0);
@@ -1418,7 +1416,7 @@ void dict_load_next(dict_load_t *load, dnode_t *newnode, const void *key)
     nil->right = newnode;
     newnode->left = nil;
 #ifdef NO_FC_SOLVE
-    dict->nodecount++;
+    ++dict->dict_nodecount;
 #endif
 }
 
@@ -1428,7 +1426,7 @@ void dict_load_end(dict_load_t *load)
     dnode_t *tree[DICT_DEPTH_MAX] = {0};
     dnode_t *curr, *dictnil = dict_nil(dict), *loadnil = &load->nilnode, *next;
     dnode_t *complete = 0;
-    dictcount_t fullcount = DICTCOUNT_T_MAX, nodecount = dict->nodecount;
+    dictcount_t fullcount = DICTCOUNT_T_MAX, nodecount = dict->dict_nodecount;
     dictcount_t botrowcount;
     unsigned baselevel = 0, level = 0, i;
 
@@ -1525,7 +1523,7 @@ void dict_merge(dict_t *const dest, dict_t *const source)
         return;
 
 #ifdef NO_FC_SOLVE
-    dest->nodecount = 0;
+    dest->dict_nodecount = 0;
 #endif
     load_begin_internal(&load, dest);
 
