@@ -74,8 +74,6 @@ static inline enum rb_color rb_set_color(struct rb_node *const node, const enum
 }
 #endif
 
-#define NODEPTR_SET_COLOR(p, b) rb_set_color((p), (b))
-
 /* Creates and returns a new table
    with comparison function |compare| using parameter |param|
    and memory allocator |allocator|.
@@ -93,6 +91,7 @@ rb_create (rb_comparison_func *compare, void *param,
   if (tree == NULL)
     return NULL;
 
+#define SET_TREE_AVL_ROOT(tree, val) rb_set_link(&(tree->rb_proto_root), 0, val)
   SET_TREE_AVL_ROOT(tree, NULL);
   tree->rb_compare = compare;
   tree->rb_param = param;
