@@ -7,15 +7,11 @@
  *
  * Copyright (c) 2010 Shlomi Fish
  */
-/*
- * likely.h - the purpose of this file is to define the likely
- * and unlikely macros. See: https://lwn.net/Articles/255364/ .
- */
+// likely.h - the purpose of this file is to define the likely
+// and unlikely macros. See: https://lwn.net/Articles/255364/ .
 #pragma once
 
-#include "config.h"
-
-#if defined(__GNUC__)
+#ifdef __GNUC__
 #define unlikely(expr) __builtin_expect(!!(expr), 0)
 #define likely(expr) __builtin_expect(!!(expr), 1)
 #else
