@@ -12,20 +12,17 @@
  * depth-dependent
  * performance of two scans.
  */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "rinutils.h"
-#include "fcs_user.h"
 #include "fcs_cl.h"
 #include "range_solvers_gen_ms_boards.h"
-#include "help_err.h"
+#include "try_param.h"
+#include "range_solvers.h"
 
+#ifndef FCS_WITHOUT_CMD_LINE_HELP
 static void print_help(void)
 {
     printf("\n%s",
-        "measure-depth-dep-tests-order-performance start end\n"
+        "measure-depth-dep-tests-order-performance start end stop_at\n"
         "   [--args-start] [--args-end] [--output-to]\n"
         "[--scan1-to] [--scan2-to] [--iters-limit]\n"
         "[--max-ver-depth]"
@@ -42,6 +39,7 @@ static void print_help(void)
         "--total-iterations-limit  limit\n"
         "     Limits each board for up to 'limit' iterations.\n");
 }
+#endif
 
 typedef struct
 {
@@ -52,33 +50,39 @@ typedef struct
 
 static const char *known_parameters[] = {NULL};
 
-int main(int argc, char *argv[])
+static void set_tests_order(
+    void *const instance, const int min_depth, const char *const moves_order)
+{
+    FCS__DECL_ERR_PTR(error_string);
+    if (freecell_solver_user_set_depth_tests_order(
+            instance, min_depth, moves_order FCS__PASS_ERR_STR(&error_string)))
+    {
+#ifdef FCS_WITH_ERROR_STRS
+        fprintf(stderr, "Error! Got '%s'!\n", error_string);
+        free(error_string);
+#else
+        fprintf(stderr, "%s\n", "depth-tests-order Error!");
+#endif
+        exit(-1);
+    }
+}
+
+static inline int range_solvers_main(int argc, char *argv[], int arg,
+    const long long start_board, const long long end_board,
+    const long long stop_at)
 {
     /* char buffer[2048]; */
-    int board_num;
-    int start_board, end_board;
-    FILE *output_fh;
     fcs_int_limit_t iters_limit = 100000;
     int max_var_depth_to_check = 100;
-#ifdef FCS_WITH_ERROR_STRS
-    char *error_string;
-#endif
+    FCS__DECL_ERR_PTR(error_string);
     const char *scan1_to = NULL, *scan2_to = NULL;
+    const char *output_filename = NULL;
 
-    char *output_filename = NULL;
-    fcs_state_string_t state_string;
-
-    int arg = 1, start_from_arg = -1, end_args = -1;
-
-    if (argc < 3)
-    {
-        help_err("Not Enough Arguments!\n");
-    }
-    start_board = atoi(argv[arg++]);
-    end_board = atoi(argv[arg++]);
+    int start_from_arg = -1, end_args = -1;
 
     for (; arg < argc; arg++)
     {
+        const char *param;
         if (!strcmp(argv[arg], "--args-start"))
         {
             arg++;
@@ -94,50 +98,25 @@ int main(int argc, char *argv[])
             }
             end_args = arg;
         }
-        else if (!strcmp(argv[arg], "--output-to"))
+        else if ((param = TRY_P("--output-to")))
         {
-            arg++;
-            if (arg == argc)
-            {
-                help_err("--output-to came without an argument!\n");
-            }
-            output_filename = argv[arg];
+            output_filename = param;
         }
-        else if (!strcmp(argv[arg], "--scan1-to"))
+        else if ((param = TRY_P("--scan1-to")))
         {
-            arg++;
-            if (arg == argc)
-            {
-                help_err("--scan1-to came without an argument!\n");
-            }
-            scan1_to = argv[arg];
+            scan1_to = param;
         }
-        else if (!strcmp(argv[arg], "--scan2-to"))
+        else if ((param = TRY_P("--scan2-to")))
         {
-            arg++;
-            if (arg == argc)
-            {
-                help_err("--scan1-to came without an argument!\n");
-            }
-            scan2_to = argv[arg];
+            scan2_to = param;
         }
-        else if (!strcmp(argv[arg], "--iters-limit"))
+        else if ((param = TRY_P("--iters-limit")))
         {
-            arg++;
-            if (arg == argc)
-            {
-                help_err("--iters-limit came without an argument!\n");
-            }
-            iters_limit = (fcs_int_limit_t)atol(argv[arg]);
+            iters_limit = (fcs_int_limit_t)atol(param);
         }
-        else if (!strcmp(argv[arg], "--max-var-depth"))
+        else if ((param = TRY_P("--max-var-depth")))
         {
-            arg++;
-            if (arg == argc)
-            {
-                help_err("--max-var-depth came without an argument!\n");
-            }
-            max_var_depth_to_check = atoi(argv[arg]);
+            max_var_depth_to_check = atoi(param);
         }
         else
         {
@@ -166,13 +145,11 @@ int main(int argc, char *argv[])
     result_t *const results =
         SMALLOC(results, (size_t)(end_board - start_board + 1));
 
-    output_fh = fopen(output_filename, "wt");
-
+    FILE *const output_fh = fopen(output_filename, "wt");
     for (int min_depth_for_scan2 = 0;
          min_depth_for_scan2 < max_var_depth_to_check; min_depth_for_scan2++)
     {
         void *const instance = freecell_solver_user_alloc();
-
         if (start_from_arg >= 0)
         {
             freecell_solver_user_cmd_line_parse_args_with_file_nesting_count(
@@ -180,46 +157,24 @@ int main(int argc, char *argv[])
                 start_from_arg, known_parameters, NULL,
                 NULL FCS__PASS_ERR_STR(&error_string), &arg, -1, NULL);
         }
-
-        if (freecell_solver_user_set_depth_tests_order(
-                instance, 0, scan1_to FCS__PASS_ERR_STR(&error_string)))
-        {
-#ifdef FCS_WITH_ERROR_STRS
-            fprintf(stderr, "Error! Got '%s'!\n", error_string);
-            free(error_string);
-#else
-            fprintf(stderr, "%s\n", "depth-tests-order Error!");
-#endif
-
-            exit(-1);
-        }
-        if (freecell_solver_user_set_depth_tests_order(instance,
-                min_depth_for_scan2, scan2_to FCS__PASS_ERR_STR(&error_string)))
-        {
-#ifdef FCS_WITH_ERROR_STRS
-            fprintf(stderr, "Error! Got '%s'!\n", error_string);
-            free(error_string);
-#else
-            fprintf(stderr, "%s\n", "depth-tests-order Error!");
-#endif
-            exit(-1);
-        }
+        set_tests_order(instance, 0, scan1_to);
+        set_tests_order(instance, min_depth_for_scan2, scan2_to);
 
         result_t *curr_result;
+        int board_num;
         for (board_num = start_board, curr_result = results;
              board_num <= end_board; board_num++, curr_result++)
         {
+            fcs_state_string_t state_string;
             get_board(board_num, state_string);
 
+#ifndef FCS_WITHOUT_MAX_NUM_STATES
             freecell_solver_user_limit_iterations_long(instance, iters_limit);
-
-            FCS_GET_TIME(curr_result->start_time);
-
+#endif
+            curr_result->start_time = fcs_get_time();
             curr_result->verdict =
                 freecell_solver_user_solve_board(instance, state_string);
-
-            FCS_GET_TIME(curr_result->end_time);
-
+            curr_result->end_time = fcs_get_time();
             curr_result->num_iters =
                 freecell_solver_user_get_num_times_long(instance);
 
@@ -246,8 +201,6 @@ int main(int argc, char *argv[])
 
             FPRINTF_TIME("start", start_time);
             FPRINTF_TIME("end", end_time);
-
-#undef FPRINTF_TIME
         }
         fflush(output_fh);
     }
