@@ -7,21 +7,9 @@
  *
  * Copyright (c) 2000 Shlomi Fish
  */
-/*
- * meta_alloc.c - the Freecell Solver compact allocator based on the
- * meta-allocator concept that is used to collect the pages allocated by
- * the standard allocator after it is destroyed and to recycle them.
- *
- * Also see alloc.c.
- */
-
-#include "dll_thunk.h"
-
-#include <stdlib.h>
-#include <stdio.h>
-
-#include "config.h"
-
+// meta_alloc.c - the Freecell Solver compact allocator based on the
+// meta-allocator concept that is used to collect the pages allocated by
+// the standard allocator after it is destroyed and to recycle them.
 #include "meta_alloc.h"
 
 #define ALLOCED_SIZE (FCS_IA_PACK_SIZE * 1024 - (256 + 128))
@@ -32,8 +20,6 @@ void fc_solve_compact_allocator_init(fcs_compact_allocator_t *const allocator,
     allocator->meta = meta_allocator;
 
     fc_solve_compact_allocator_init_helper(allocator);
-
-    return;
 }
 
 #define OLD_LIST_NEXT(ptr) (*((char **)(ptr)))
@@ -41,9 +27,9 @@ void fc_solve_compact_allocator_init(fcs_compact_allocator_t *const allocator,
 static inline char *meta_request_new_buffer(
     fcs_meta_compact_allocator_t *const meta_allocator)
 {
-    if (meta_allocator->recycle_bin)
+    char *const ret = meta_allocator->recycle_bin;
+    if (ret)
     {
-        char *ret = meta_allocator->recycle_bin;
         meta_allocator->recycle_bin = OLD_LIST_NEXT(ret);
         return ret;
     }
@@ -68,14 +54,12 @@ void fc_solve_meta_compact_allocator_finish(
     fcs_meta_compact_allocator_t *const meta_allocator)
 {
     char *iter, *iter_next;
-
     for (iter = meta_allocator->recycle_bin,
         iter_next = iter ? OLD_LIST_NEXT(iter) : NULL;
          iter_next; iter = iter_next, iter_next = OLD_LIST_NEXT(iter))
     {
         free(iter);
     }
-
     free(iter);
     meta_allocator->recycle_bin = NULL;
 }
@@ -84,17 +68,16 @@ void fc_solve_compact_allocator_finish(fcs_compact_allocator_t *const allocator)
 {
     char *iter, *iter_next;
     fcs_meta_compact_allocator_t *const meta = allocator->meta;
-
-    /* Enqueue all the allocated buffers in the meta allocator for re-use.
-     * */
+    var_AUTO(bin, meta->recycle_bin);
+    // Enqueue all the allocated buffers in the meta allocator for re-use.
     for (iter = allocator->old_list, iter_next = OLD_LIST_NEXT(iter); iter_next;
          iter = iter_next, iter_next = OLD_LIST_NEXT(iter))
     {
-        OLD_LIST_NEXT(iter) = meta->recycle_bin;
-        meta->recycle_bin = iter;
+        OLD_LIST_NEXT(iter) = bin;
+        bin = iter;
     }
 
-    OLD_LIST_NEXT(iter) = meta->recycle_bin;
+    OLD_LIST_NEXT(iter) = bin;
     meta->recycle_bin = iter;
 }
 
