@@ -16,6 +16,7 @@
 #include "instance.h"
 #include "move.h"
 
+#ifdef FCS_WITH_MOVES
 static inline fcs_move_stack_t *fc_solve_move_stack_compact_allocate(
     fc_solve_hard_thread_t *const hard_thread,
     fcs_move_stack_t *const old_move_stack_to_parent)
@@ -34,3 +35,4 @@ static inline fcs_move_stack_t *fc_solve_move_stack_compact_allocate(
         sizeof(fcs_move_t) * old_move_stack_to_parent->num_moves);
     return new_move_stack_to_parent;
 }
+#endif
