@@ -7,23 +7,17 @@
  *
  * Copyright (c) 2000 Shlomi Fish
  */
-/*
- * offloading_queue.h - header file for the offloading-to-hard-disk
- * queue.
- */
+// offloading_queue.h - header file for the offloading-to-hard-disk queue.
 #pragma once
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#include <string.h>
-#include <limits.h>
-#include <stdio.h>
-#include <unistd.h>
-
-#include "config.h"
-#include "state.h"
+#include "rinutils.h"
+#ifdef __unix__
+#include <fcntl.h>
+#endif
 
 typedef const unsigned char *fcs_offloading_queue_item_t;
 
@@ -78,11 +72,10 @@ static inline void fcs_offloading_queue__init(
 
     queue->queue_head = queue->queue_tail = queue->queue_recycle_bin = NULL;
     fcs_queue_stats_init(&queue->stats);
-
-    return;
 }
 
-static inline void fcs_offloading_queue__destroy(fcs_offloading_queue_t *queue)
+static inline void fcs_offloading_queue__destroy(
+    fcs_offloading_queue_t *const queue)
 {
     fc_solve_compact_allocator_finish(&(queue->queue_allocator));
 }
@@ -113,10 +106,10 @@ static inline fcs_bool_t fcs_offloading_queue__extract(
 }
 
 static inline void fcs_offloading_queue__insert(
-    fcs_offloading_queue_t *queue, const fcs_offloading_queue_item_t *datum)
+    fcs_offloading_queue_t *const queue,
+    const fcs_offloading_queue_item_t *const datum)
 {
     fcs_Q_item_wrapper_t *new_item;
-
     if (queue->queue_recycle_bin)
     {
         queue->queue_recycle_bin = (new_item = queue->queue_recycle_bin)->next;
@@ -138,60 +131,49 @@ static inline void fcs_offloading_queue__insert(
         queue->queue_head = queue->queue_tail = new_item;
     }
     q_stats_insert(&queue->stats);
-    return;
 }
 
-/* Implement the standard in-memory queue as a linked list. */
 #else
 
+static const size_t NUM_ITEMS_PER_PAGE = (128 * 1024);
 typedef struct
 {
-    size_t num_items_per_page;
     long page_index, queue_id;
     size_t write_to_idx;
     size_t read_from_idx;
     unsigned char *data;
-} fcs_offloading_queue_page_t;
+} off_q_page_t;
 
-static inline void fcs_offloading_queue_page__recycle(
-    fcs_offloading_queue_page_t *const page)
+static inline void fcs_offloading_queue_page__recycle(off_q_page_t *const page)
 {
     page->write_to_idx = 0;
     page->read_from_idx = 0;
 }
 
 static inline void fcs_offloading_queue_page__init(
-    fcs_offloading_queue_page_t *const page, const size_t num_items_per_page,
-    const long page_index, const long queue_id)
+    off_q_page_t *const page, const long page_index, const long queue_id)
 {
-    fcs_offloading_queue_page_t new_page = {
-        .num_items_per_page = num_items_per_page,
-        .page_index = page_index,
+    *page = (typeof(*page)){.page_index = page_index,
         .queue_id = queue_id,
         .data =
-            malloc(sizeof(fcs_offloading_queue_item_t) * num_items_per_page)};
-    *page = new_page;
+            malloc(sizeof(fcs_offloading_queue_item_t) * NUM_ITEMS_PER_PAGE)};
     fcs_offloading_queue_page__recycle(page);
-
-    return;
 }
 
-static inline void fcs_offloading_queue_page__destroy(
-    fcs_offloading_queue_page_t *const page)
+static inline void fcs_offloading_queue_page__destroy(off_q_page_t *const page)
 {
     free(page->data);
     page->data = NULL;
 }
 
 static inline fcs_bool_t fcs_offloading_queue_page__can_extract(
-    const fcs_offloading_queue_page_t *const page)
+    const off_q_page_t *const page)
 {
     return (page->read_from_idx < page->write_to_idx);
 }
 
 static inline void fcs_offloading_queue_page__extract(
-    fcs_offloading_queue_page_t *const page,
-    fcs_offloading_queue_item_t *const out_item)
+    off_q_page_t *const page, fcs_offloading_queue_item_t *const out_item)
 {
     memcpy(out_item,
         (page->data + sizeof(*out_item) * ((page->read_from_idx)++)),
@@ -199,82 +181,87 @@ static inline void fcs_offloading_queue_page__extract(
 }
 
 static inline fcs_bool_t fcs_offloading_queue_page__can_insert(
-    const fcs_offloading_queue_page_t *const page)
+    const off_q_page_t *const page)
 {
-    return (page->write_to_idx < page->num_items_per_page);
+    return (page->write_to_idx < NUM_ITEMS_PER_PAGE);
 }
 
 static inline void fcs_offloading_queue_page__insert(
-    fcs_offloading_queue_page_t *const page,
-    const fcs_offloading_queue_item_t *const in_item)
+    off_q_page_t *const page, const fcs_offloading_queue_item_t *const in_item)
 {
     memcpy(page->data + ((page->write_to_idx)++) * sizeof(*in_item), in_item,
         sizeof(*in_item));
 }
 
-static inline const char *fcs_offloading_queue_page__calc_filename(
-    fcs_offloading_queue_page_t *const page, char *const buffer,
+static inline void fcs_offloading_queue_page__calc_filename(
+    off_q_page_t *const page, char *const buffer,
     const char *const offload_dir_path)
 {
     sprintf(buffer, "%s/fcs_queue%lXq_%020lX.page", offload_dir_path,
         page->queue_id, page->page_index);
-
-    return buffer;
 }
 
 static inline void fcs_offloading_queue_page__start_after(
-    fcs_offloading_queue_page_t *const page,
-    const fcs_offloading_queue_page_t *const other_page)
+    off_q_page_t *const page, const off_q_page_t *const other_page)
 {
     page->page_index = other_page->page_index + 1;
     fcs_offloading_queue_page__recycle(page);
 }
 
-static inline void fcs_offloading_queue_page__bump(
-    fcs_offloading_queue_page_t *const page)
+static inline void fcs_offloading_queue_page__bump(off_q_page_t *const page)
 {
     fcs_offloading_queue_page__start_after(page, page);
 }
 
 static inline void fcs_offloading_queue_page__read_next_from_disk(
-    fcs_offloading_queue_page_t *const page, const char *const offload_dir_path)
+    off_q_page_t *const page, const char *const offload_dir_path)
 {
     fcs_offloading_queue_page__bump(page);
     char page_filename[PATH_MAX + 1];
     fcs_offloading_queue_page__calc_filename(
         page, page_filename, offload_dir_path);
-
+#ifdef __unix__
+    const int f = open(page_filename, O_RDONLY);
+    read(f, page->data,
+        sizeof(fcs_offloading_queue_item_t) * NUM_ITEMS_PER_PAGE);
+    close(f);
+#else
     FILE *const f = fopen(page_filename, "rb");
-    fread(page->data, sizeof(fcs_offloading_queue_item_t),
-        page->num_items_per_page, f);
+    fread(
+        page->data, sizeof(fcs_offloading_queue_item_t), NUM_ITEMS_PER_PAGE, f);
     fclose(f);
+#endif
 
     /* We need to set this limit because it's a read-only page that we
      * retrieve from the disk and otherwise ->can_extract() will return
      * false for most items.
      * */
-    page->write_to_idx = page->num_items_per_page;
+    page->write_to_idx = NUM_ITEMS_PER_PAGE;
 
     unlink(page_filename);
 }
 
 static inline void fcs_offloading_queue_page__offload(
-    fcs_offloading_queue_page_t *const page, const char *const offload_dir_path)
+    off_q_page_t *const page, const char *const offload_dir_path)
 {
     char page_filename[PATH_MAX + 1];
-
     fcs_offloading_queue_page__calc_filename(
         page, page_filename, offload_dir_path);
-
+#ifdef __unix__
+    const int f = creat(page_filename, 0644);
+    write(f, page->data,
+        sizeof(fcs_offloading_queue_item_t) * NUM_ITEMS_PER_PAGE);
+    close(f);
+#else
     FILE *const f = fopen(page_filename, "wb");
-    fwrite(page->data, sizeof(fcs_offloading_queue_item_t),
-        page->num_items_per_page, f);
+    fwrite(
+        page->data, sizeof(fcs_offloading_queue_item_t), NUM_ITEMS_PER_PAGE, f);
     fclose(f);
+#endif
 }
 
 typedef struct
 {
-    int num_items_per_page;
     const char *offload_dir_path;
     fcs_queue_stats_t stats;
     long id;
@@ -283,30 +270,28 @@ typedef struct
      * always point to the two "pages" below, but they can be swapped and
      * page_idx_for_backup may be NULL.
      */
-    int page_idx_to_write_to, page_idx_for_backup, page_idx_to_read_from;
-    fcs_offloading_queue_page_t pages[2];
+    int_fast32_t page_idx_to_write_to, page_idx_for_backup,
+        page_idx_to_read_from;
+    off_q_page_t pages[2];
 } fcs_offloading_queue_t;
 
-const size_t NUM_ITEMS_PER_PAGE = (128 * 1024);
 static inline void fcs_offloading_queue__init(
     fcs_offloading_queue_t *const queue, const char *const offload_dir_path,
     const long id)
 {
-    queue->num_items_per_page = NUM_ITEMS_PER_PAGE;
     queue->offload_dir_path = offload_dir_path;
     fcs_queue_stats_init(&queue->stats);
     queue->id = id;
 
-    fcs_offloading_queue_page__init(
-        &(queue->pages[0]), NUM_ITEMS_PER_PAGE, 0, queue->id);
-    fcs_offloading_queue_page__init(
-        &(queue->pages[1]), NUM_ITEMS_PER_PAGE, 0, queue->id);
+    fcs_offloading_queue_page__init(&(queue->pages[0]), 0, queue->id);
+    fcs_offloading_queue_page__init(&(queue->pages[1]), 0, queue->id);
 
     queue->page_idx_to_read_from = queue->page_idx_to_write_to = 0;
     queue->page_idx_for_backup = 1;
 }
 
-static inline void fcs_offloading_queue__destroy(fcs_offloading_queue_t *queue)
+static inline void fcs_offloading_queue__destroy(
+    fcs_offloading_queue_t *const queue)
 {
     fcs_offloading_queue_page__destroy(&(queue->pages[0]));
     fcs_offloading_queue_page__destroy(&(queue->pages[1]));
@@ -359,7 +344,7 @@ static inline fcs_bool_t fcs_offloading_queue__extract(
          *
          * if (queue->_page_idx_to_read_from->page_index ==
          *     queue->_page_idx_to_write_to->page_index)
-        */
+         */
         if (queue->pages[queue->page_idx_to_read_from].page_index + 1 ==
             queue->pages[queue->page_idx_to_write_to].page_index)
         {
