@@ -16,7 +16,6 @@
 extern "C" {
 #endif
 
-#include <stdio.h>
 #include "rinutils.h"
 #include "fcs_enums.h"
 #include "fcs_user.h"
@@ -34,7 +33,6 @@ typedef struct
     fcs_bool_t display_10_as_t;
 #endif
     fcs_bool_t display_parent_iter_num;
-    fcs_bool_t debug_iter_output_on;
     fcs_bool_t display_moves;
     fcs_bool_t display_states;
     fcs_bool_t show_exceeded_limits;
@@ -63,14 +61,53 @@ static inline void fc_solve_output_result_to_file(FILE *const output_fh,
     const_AUTO(display_context, (*dc_ptr));
     if (ret == FCS_STATE_WAS_SOLVED)
     {
-        fprintf(output_fh, "-=-=-=-=-=-=-=-=-=-=-=-\n\n");
+        fputs("-=-=-=-=-=-=-=-=-=-=-=-\n\n", output_fh);
 #ifdef FCS_WITH_MOVES
-        {
-            fcs_move_t move;
-            FILE *move_dump;
-            char state_as_string[1000];
+        fcs_move_t move;
+        char state_as_string[1000];
 
-            move_dump = output_fh;
+        if (display_context.display_states)
+        {
+            freecell_solver_user_current_state_stringify(instance,
+                state_as_string FC_SOLVE__PASS_PARSABLE(
+                    display_context.parseable_output),
+                display_context.canonized_order_output FC_SOLVE__PASS_T(
+                    display_context.display_10_as_t));
+
+            fputs(state_as_string, output_fh);
+            fputs("\n\n====================\n\n", output_fh);
+        }
+
+        int move_num = 0;
+        while (freecell_solver_user_get_next_move(instance, &move) == 0)
+        {
+            if (display_context.display_moves)
+            {
+                if (display_context.display_states &&
+                    display_context.standard_notation)
+                {
+                    fputs("Move: ", output_fh);
+                }
+
+                freecell_solver_user_stringify_move_w_state(instance,
+                    state_as_string, move, display_context.standard_notation);
+                fprintf(output_fh,
+                    (display_context.standard_notation ? "%s " : "%s\n"),
+                    state_as_string);
+                move_num++;
+                if (display_context.standard_notation)
+                {
+                    if ((move_num % 10 == 0) || display_context.display_states)
+                    {
+                        putc('\n', output_fh);
+                    }
+                }
+                if (display_context.display_states)
+                {
+                    putc('\n', output_fh);
+                }
+                fflush(output_fh);
+            }
 
             if (display_context.display_states)
             {
@@ -80,89 +117,41 @@ static inline void fc_solve_output_result_to_file(FILE *const output_fh,
                     display_context.canonized_order_output FC_SOLVE__PASS_T(
                         display_context.display_10_as_t));
 
-                fprintf(move_dump, "%s\n", state_as_string);
-                fprintf(move_dump, "%s", "\n====================\n\n");
+                fprintf(output_fh, "%s\n", state_as_string);
             }
 
-            int move_num = 0;
-            while (freecell_solver_user_get_next_move(instance, &move) == 0)
+            if (display_context.display_states ||
+                (!display_context.standard_notation))
             {
-                if (display_context.display_moves)
-                {
-                    if (display_context.display_states &&
-                        display_context.standard_notation)
-                    {
-                        fprintf(move_dump, "Move: ");
-                    }
-
-                    freecell_solver_user_stringify_move_w_state(instance,
-                        state_as_string, move,
-                        display_context.standard_notation);
-                    fprintf(move_dump,
-                        (display_context.standard_notation ? "%s " : "%s\n"),
-                        state_as_string);
-                    move_num++;
-                    if (display_context.standard_notation)
-                    {
-                        if ((move_num % 10 == 0) ||
-                            display_context.display_states)
-                        {
-                            fprintf(move_dump, "\n");
-                        }
-                    }
-                    if (display_context.display_states)
-                    {
-                        fprintf(move_dump, "\n");
-                    }
-                    fflush(move_dump);
-                }
-
-                if (display_context.display_states)
-                {
-                    freecell_solver_user_current_state_stringify(instance,
-                        state_as_string FC_SOLVE__PASS_PARSABLE(
-                            display_context.parseable_output),
-                        display_context.canonized_order_output FC_SOLVE__PASS_T(
-                            display_context.display_10_as_t));
-
-                    fprintf(move_dump, "%s\n", state_as_string);
-                }
-
-                if (display_context.display_states ||
-                    (!display_context.standard_notation))
-                {
-                    fprintf(move_dump, "%s", "\n====================\n\n");
-                }
+                fputs("\n====================\n\n", output_fh);
             }
+        }
 
-            if (display_context.standard_notation &&
-                (!display_context.display_states))
-            {
-                fprintf(move_dump, "\n\n");
-            }
+        if (display_context.standard_notation &&
+            (!display_context.display_states))
+        {
+            fputs("\n\n", output_fh);
         }
 #endif
 
-        fprintf(output_fh, "This game is solveable.\n");
+        fputs("This game is solveable.\n", output_fh);
     }
     else if (display_context.show_exceeded_limits &&
              (ret == FCS_STATE_SUSPEND_PROCESS))
     {
-        fprintf(output_fh, "Iterations count exceeded.\n");
+        fputs("Iterations count exceeded.\n", output_fh);
     }
     else
     {
-        fprintf(output_fh, "I could not solve this game.\n");
+        fputs("I could not solve this game.\n", output_fh);
     }
 
-    fprintf(output_fh, "Total number of states checked is %li.\n",
+    fprintf(output_fh, "Total number of states checked is %ld.\n",
         (long)freecell_solver_user_get_num_times_long(instance));
-#if 1
-    fprintf(output_fh, "This scan generated %li states.\n",
+#ifndef FCS_DISABLE_NUM_STORED_STATES
+    fprintf(output_fh, "This scan generated %ld states.\n",
         (long)freecell_solver_user_get_num_states_in_collection_long(instance));
 #endif
-
-    return;
 }
 
 #ifdef __cplusplus
