@@ -7,31 +7,23 @@
  *
  * Copyright (c) 2002 Tom Holroyd
  */
-/* This is a 32 bit FNV hash.  For more information, see
-http://www.isthe.com/chongo/tech/comp/fnv/index.html */
-
+// This is a 32 bit FNV hash.  For more information, see
+// http://www.isthe.com/chongo/tech/comp/fnv/index.html
 #pragma once
-
-#include <sys/types.h>
 #include "config.h"
 
-#include "portable_int32.h"
-
 #define FNV1_32_INIT 0x811C9DC5
 #define FNV_32_PRIME 0x01000193
 
-static inline uint32_t fnv_hash(const char x, const uint32_t hash)
+static inline uint32_t fnv_hash(const unsigned char x, const uint32_t hash)
 {
-    return ((hash * FNV_32_PRIME) ^ x);
+    return ((hash * FNV_32_PRIME) ^ (uint32_t)x);
 }
 
-/* Hash a 0 terminated string. */
-
-static inline uint32_t fnv_hash_str(const u_char *s)
+static inline uint32_t fnv_hash_str(const unsigned char *s)
 {
     uint32_t h = FNV1_32_INIT;
-
-    u_char c;
+    unsigned char c;
     while ((c = *(s++)))
     {
         h = fnv_hash(c, h);
