@@ -7,15 +7,11 @@
  *
  * Copyright (c) 2002 Tom Holroyd
  */
-/*
- * Deal Microsoft Freecell / FreeCell-Pro deals.
- */
+// Deal Microsoft Freecell / FreeCell-Pro deals.
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/types.h>
 #include <string.h>
 #include <stdint.h>
-#include "portable_int32.h"
 
 typedef unsigned long long LONG;
 typedef uint32_t UINT;
@@ -47,7 +43,7 @@ static const char Suit[] = "CDHS";
 int main(int argc, char **argv)
 {
     int j, c;
-    /* num_left_cards is the cards left to be chosen in shuffle */
+    // num_left_cards is the cards left to be chosen in shuffle
     int num_left_cards = NUM_CARDS;
     CARD deck[NUM_CARDS];
     CARD pos[10][10];
@@ -117,7 +113,7 @@ int main(int argc, char **argv)
         }
         putchar('\n');
     }
-    /* leftover cards to temp */
+    // leftover cards to temp
     c = -1;
     for (size_t i = 0; i < 4; i++)
     {
