@@ -7,16 +7,9 @@
  *
  * Copyright (c) 2002 Tom Holroyd
  */
-
-/* Solve Freecell and Seahaven type patience (solitaire) games. */
-
 #pragma once
-#include <stdio.h>
-#include <sys/types.h>
-
 #include "config.h"
 
-#include "portable_int32.h"
 #include "game_type_params.h"
 #include "fcs_enums.h"
 #include "tree.h"
@@ -28,18 +21,13 @@
 #include "alloc_wrap.h"
 #include "instance.h"
 
-/* A card is represented as (suit << 4) + rank. */
-
 #define FCS_PATS__COLOR 0x01 /* black if set */
 #define FCS_PATS__SUIT 0x03  /* mask both suit bits */
 
 #define FCS_PATS__KING 13
 
-/* Some macros used in get_possible_moves(). */
-
-/* The following macro implements
-   (Same_suit ? (suit(a) == suit(b)) : (color(a) != color(b)))
-*/
+/* The following implements
+   (Same_suit ? (suit(a) == suit(b)) : (color(a) != color(b))) */
 #ifdef FCS_FREECELL_ONLY
 static inline fcs_bool_t fcs_pats_is_suitable(
     const fcs_card_t a, const fcs_card_t b)
@@ -62,21 +50,18 @@ static inline fcs_bool_t fcs_pats_is_king_only(
     return (not_king_only || fcs_card_rank(card) == FCS_PATS__KING);
 }
 
-/* Represent a move. */
+// Represent a move.
 
 typedef struct
 {
     fcs_card_t card; /* the card we're moving */
-    u_char from;     /* from pile number */
-    u_char to;       /* to pile number */
-    u_char fromtype; /* O, T, or W */
-    u_char totype;
+    unsigned char from, to, fromtype, totype;
     fcs_card_t srccard;  /* card we're uncovering */
     fcs_card_t destcard; /* card we're moving to */
     signed char pri;     /* move priority (low priority == low value) */
 } fcs_pats__move_t;
 
-/* Pile types */
+// Pile types
 #define FCS_PATS__TYPE_FOUNDATION 1
 #define FCS_PATS__TYPE_FREECELL 2
 #define FCS_PATS__TYPE_WASTE 3
@@ -86,23 +71,21 @@ Temp cells are stored separately since they don't have to be compared.
 We also store the move that led to this position from the parent, as well
 as a pointers back to the parent, and the btree of all positions examined so
 far. */
-
 typedef struct fc_solve_pats__pos__struct
 {
     struct fc_solve_pats__pos__struct *queue; /* next position in the queue */
     struct fc_solve_pats__pos__struct
-        *parent;                   /* point back up the move stack */
-    fcs_pats__tree_t *node;        /* compact position rep.'s tree node */
-    fcs_pats__move_t move;         /* move that got us here from the parent */
-    unsigned short cluster;        /* the cluster this node is in */
-    short depth;                   /* number of moves so far */
-    u_char num_cards_in_freecells; /* number of cards in T */
-    u_char num_childs;             /* number of child nodes left */
+        *parent;            /* point back up the move stack */
+    fcs_pats__tree_t *node; /* compact position rep.'s tree node */
+    fcs_pats__move_t move;  /* move that got us here from the parent */
+    unsigned short cluster; /* the cluster this node is in */
+    short depth;            /* number of moves so far */
+    unsigned char num_cards_in_freecells; /* number of cards in T */
+    unsigned char num_childs;             /* number of child nodes left */
 } fcs_pats_position_t;
 
-/* Temp storage for possible moves. */
-
-/* > max # moves from any position */
+// Temp storage for possible moves.
+// > max # moves from any position
 #define FCS_PATS__MAX_NUM_MOVES 64
 
 typedef enum {
@@ -125,12 +108,11 @@ typedef enum {
 } fc_solve_pats__status_fail_reason_t;
 #endif
 
-/* Memory. */
-
+// Memory.
 typedef struct fcs_pats__block_struct
 {
-    u_char *block;
-    u_char *ptr;
+    unsigned char *block;
+    unsigned char *ptr;
     size_t remaining;
     struct fcs_pats__block_struct *next;
 } fcs_pats__block_t;
@@ -149,20 +131,19 @@ typedef struct fcs_pats__treelist_struct
 
 typedef struct fcs_pats__bucket_list_struct
 {
-    u_char *pile;  /* 0 terminated copy of the pile */
-    uint32_t hash; /* the pile's hash code */
-    int pilenum;   /* the unique id for this pile */
+    unsigned char *pile; /* 0 terminated copy of the pile */
+    uint32_t hash;       /* the pile's hash code */
+    int pilenum;         /* the unique id for this pile */
     struct fcs_pats__bucket_list_struct *next;
 } fcs_pats__bucket_list_t;
 
-/* Statistics. */
-
+// Statistics.
 #define FC_SOLVE_PATS__NUM_QUEUES 100
 
 #ifdef PATSOLVE_STANDALONE
 struct fc_solve_instance_struct
 {
-    /* game parameters */
+    // game parameters
     fcs_game_type_params_t game_params;
 #ifndef FCS_FREECELL_ONLY
     fcs_card_t game_variant_suit_mask;
@@ -173,15 +154,6 @@ struct fc_solve_instance_struct
 typedef struct fc_solve_instance_struct fc_solve_instance_t;
 #endif
 
-typedef struct
-{
-    fcs_pats_position_t *parent;
-    int nmoves;
-    fcs_pats__move_t *mp0, *mp_end, *mp;
-    fcs_bool_t q;
-    fcs_pats_position_t *pos;
-} fcs_pats__solve_depth_t;
-
 enum FC_SOLVE_PATS__MYDIR
 {
     FC_SOLVE_PATS__UP,
@@ -204,8 +176,7 @@ struct fc_solve__patsolve_thread_struct
 #endif
     fcs_pats_position_t *freed_positions; /* position freelist */
 
-    /* Work arrays. */
-
+    // Work arrays.
     struct
     {
         fcs_state_t s;
@@ -223,16 +194,12 @@ struct fc_solve__patsolve_thread_struct
     } current_pos;
 
     /* Temp storage for possible moves. */
-
     fcs_pats__move_t possible_moves[FCS_PATS__MAX_NUM_MOVES];
 
     /* Statistics. */
-
-    int num_checked_states, max_num_checked_states;
-    int num_states_in_collection;
-
+    long num_checked_states, max_num_checked_states;
+    long num_states_in_collection;
     fcs_pats_xy_param_t pats_solve_params;
-
     size_t position_size;
 
     fcs_pats__bucket_list_t *buckets_list[FC_SOLVE_BUCKETLIST_NBUCKETS];
@@ -248,7 +215,7 @@ struct fc_solve__patsolve_thread_struct
      * value.
      * Default should be FALSE.
      * */
-    int to_stack;
+    fcs_bool_t to_stack;
     /* Switch between depth- and breadth-first. Default is "1".*/
     int num_moves_to_cut_off;
     /* win, lose, or fail */
@@ -256,41 +223,43 @@ struct fc_solve__patsolve_thread_struct
 #ifdef FCS_PATSOLVE__WITH_FAIL_REASON
     fc_solve_pats__status_fail_reason_t fail_reason;
 #endif
-
 #define FCS_PATS__TREE_LIST_NUM_BUCKETS 499 /* a prime */
     fcs_pats__treelist_t *tree_list[FCS_PATS__TREE_LIST_NUM_BUCKETS];
     fcs_pats__block_t *my_block;
 
     ssize_t dequeue__minpos, dequeue__qpos;
     fcs_pats__move_t *moves_to_win;
-    int num_moves_to_win;
+    size_t num_moves_to_win;
 
 #define FCS_PATS__SOLVE_LEVEL_GROW_BY 16
     int curr_solve_depth, max_solve_depth;
-    fcs_pats__solve_depth_t *solve_stack;
+    struct
+    {
+        fcs_pats_position_t *parent;
+        int num_moves;
+        fcs_pats__move_t *moves_start, *moves_end, *move_ptr;
+        fcs_bool_t q;
+        fcs_pats_position_t *pos;
+    } * solve_stack;
     fcs_pats_position_t *curr_solve_pos;
     enum FC_SOLVE_PATS__MYDIR curr_solve_dir;
 };
 
 typedef struct fc_solve__patsolve_thread_struct fcs_pats_thread_t;
 
-/* Prototypes. */
-
 extern fcs_pats__insert_code_t fc_solve_pats__insert(
     fcs_pats_thread_t *soft_thread, int *cluster, int d,
     fcs_pats__tree_t **node);
 extern void fc_solve_pats__do_it(fcs_pats_thread_t *);
 extern fcs_pats__move_t *fc_solve_pats__get_moves(
     fcs_pats_thread_t *soft_thread, fcs_pats_position_t *, int *);
-extern u_char *fc_solve_pats__new_from_block(
+extern unsigned char *fc_solve_pats__new_from_block(
     fcs_pats_thread_t *soft_thread, size_t);
 extern void fc_solve_pats__sort_piles(fcs_pats_thread_t *soft_thread);
 
 extern fcs_pats__block_t *fc_solve_pats__new_block(
     fcs_pats_thread_t *const soft_thread);
 
-/* Clusters are also stored in a hashed array. */
-
 static inline void fc_solve_pats__init_clusters(
     fcs_pats_thread_t *const soft_thread)
 {
@@ -309,7 +278,6 @@ static inline size_t fc_solve_pats__align(const size_t i)
     return ((i & ALIGN_BITS) ? ((i | ALIGN_BITS) + 1) : i);
 }
 
-/* Initialize the hash buckets. */
 static inline void fc_solve_pats__init_buckets(
     fcs_pats_thread_t *const soft_thread)
 {
@@ -319,22 +287,20 @@ static inline void fc_solve_pats__init_buckets(
     const int stacks_num = INSTANCE_STACKS_NUM;
     const int freecells_num = INSTANCE_FREECELLS_NUM;
 
-    /* Packed positions need 3 bytes for every 2 piles. */
+    // Packed positions need 3 bytes for every 2 piles.
     soft_thread->bytes_per_pile =
-        (((stacks_num * 3) >> 1) + (stacks_num & 0x1));
+        (size_t)(((stacks_num * 3) >> 1) + (stacks_num & 0x1));
 
     memset(soft_thread->buckets_list, 0, sizeof(soft_thread->buckets_list));
     soft_thread->next_pile_idx = 0;
     soft_thread->bytes_per_tree_node = fc_solve_pats__align(
         sizeof(fcs_pats__tree_t) + soft_thread->bytes_per_pile);
-    soft_thread->position_size =
-        fc_solve_pats__align(sizeof(fcs_pats_position_t) + freecells_num);
+    soft_thread->position_size = fc_solve_pats__align(
+        sizeof(fcs_pats_position_t) + (size_t)freecells_num);
 }
 
-/* A function and some macros for allocating memory. */
-/* Allocate some space and return a pointer to it.  See fc_solve_pats__new() .
- * */
-
+// A function and some macros for allocating memory.
+// Allocate some space and return a pointer to it.  See fc_solve_pats__new()
 static inline void *fc_solve_pats__malloc(
     fcs_pats_thread_t *const soft_thread, const size_t s)
 {
@@ -347,7 +313,7 @@ static inline void *fc_solve_pats__malloc(
 
         while (soft_thread->freed_positions) {
             pos = soft_thread->freed_positions->queue;
-            fc_solve_pats__free_array(soft_thread->freed_positions, u_char, sizeof(*pos) + LOCAL_FREECELLS_NUM);
+            fc_solve_pats__free_array(soft_thread->freed_positions, unsigned char, sizeof(*pos) + LOCAL_FREECELLS_NUM);
             soft_thread->freed_positions = pos;
         }
         if (s > soft_thread->remaining_memory) {
@@ -379,7 +345,6 @@ static inline void fc_solve_pats__release(fcs_pats_thread_t *const soft_thread,
     void *const ptr, const size_t count_freed)
 {
     free(ptr);
-
     soft_thread->remaining_memory += count_freed;
 }
 
@@ -396,11 +361,11 @@ static inline void fc_solve_pats__free_buckets(
 {
     for (int i = 0; i < FC_SOLVE_BUCKETLIST_NBUCKETS; i++)
     {
-        typeof(soft_thread->buckets_list[i]) l = soft_thread->buckets_list[i];
+        var_AUTO(l, soft_thread->buckets_list[i]);
         while (l)
         {
-            typeof(l->next) n = l->next;
-            fc_solve_pats__free_array(soft_thread, l->pile, u_char,
+            var_AUTO(n, l->next);
+            fc_solve_pats__free_array(soft_thread, l->pile, unsigned char,
                 strlen((const char *)l->pile) + 1);
             fc_solve_pats__free_ptr(soft_thread, l, fcs_pats__bucket_list_t);
             l = n;
@@ -412,12 +377,12 @@ static inline void fc_solve_pats__free_buckets(
 static inline void fc_solve_pats__free_blocks(
     fcs_pats_thread_t *const soft_thread)
 {
-    typeof(soft_thread->my_block) b = soft_thread->my_block;
+    var_AUTO(b, soft_thread->my_block);
     while (b)
     {
-        typeof(b->next) next = b->next;
+        const_AUTO(next, b->next);
         fc_solve_pats__free_array(
-            soft_thread, b->block, u_char, FC_SOLVE__PATS__BLOCKSIZE);
+            soft_thread, b->block, unsigned char, FC_SOLVE__PATS__BLOCKSIZE);
         fc_solve_pats__free_ptr(soft_thread, b, fcs_pats__block_t);
         b = next;
     }
@@ -429,10 +394,10 @@ static inline void fc_solve_pats__free_clusters(
 {
     for (int i = 0; i < FCS_PATS__TREE_LIST_NUM_BUCKETS; i++)
     {
-        typeof(soft_thread->tree_list[i]) l = soft_thread->tree_list[i];
+        var_AUTO(l, soft_thread->tree_list[i]);
         while (l)
         {
-            typeof(l) n = l->next;
+            var_AUTO(n, l->next);
             fc_solve_pats__free_ptr(soft_thread, l, fcs_pats__treelist_t);
             l = n;
         }
@@ -491,7 +456,7 @@ static inline void fc_solve_pats__init_soft_thread(
 
     soft_thread->max_solve_depth = FCS_PATS__SOLVE_LEVEL_GROW_BY;
     soft_thread->solve_stack = (typeof(soft_thread->solve_stack))SMALLOC(
-        soft_thread->solve_stack, soft_thread->max_solve_depth);
+        soft_thread->solve_stack, (size_t)soft_thread->max_solve_depth);
 }
 
 static inline void fc_solve_pats__destroy_soft_thread(
@@ -503,17 +468,15 @@ static inline void fc_solve_pats__destroy_soft_thread(
     soft_thread->curr_solve_depth = -1;
 }
 
-/* Hash a pile. */
 static inline void fc_solve_pats__hashpile(
     fcs_pats_thread_t *const soft_thread, const int w)
 {
-    fcs_cards_column_t col = fcs_state_get_col(soft_thread->current_pos.s, w);
+    var_AUTO(col, fcs_state_get_col(soft_thread->current_pos.s, w));
     fcs_col_get_card(col, (int)fcs_col_len(col)) = '\0';
     soft_thread->current_pos.stack_hashes[w] =
-        fnv_hash_str((const u_char *)(col + 1));
-
-    /* Invalidate this pile's id.  We'll calculate it later. */
+        fnv_hash_str((const unsigned char *)(col + 1));
 
+    // Invalidate this pile's id.  We'll calculate it later.
     soft_thread->current_pos.stack_ids[w] = -1;
 }
 
@@ -544,8 +507,7 @@ static inline void fc_solve_pats__hash_layout(
 static inline void fc_solve_pats__initialize_solving_process(
     fcs_pats_thread_t *const soft_thread)
 {
-    /* Init the queues. */
-
+    // Init the queues.
     for (int i = 0; i < FC_SOLVE_PATS__NUM_QUEUES; i++)
     {
         soft_thread->queue_head[i] = NULL;
@@ -558,13 +520,12 @@ static inline void fc_solve_pats__initialize_solving_process(
         sizeof(soft_thread->num_positions_in_queue));
 #endif
 
-    /* Queue the initial position to get started. */
-
+    // Queue the initial position to get started.
     fc_solve_pats__hash_layout(soft_thread);
     fc_solve_pats__sort_piles(soft_thread);
     fcs_pats__move_t m;
     m.card = fc_solve_empty_card;
-    fcs_pats_position_t *pos =
+    fcs_pats_position_t *const pos =
         fc_solve_pats__new_position(soft_thread, NULL, &m);
     if (pos == NULL)
     {
