@@ -14,9 +14,6 @@ priority to positions with more cards out, so the solution found is not
 guaranteed to be the shortest, but it'll be better than with a depth-first
 search. */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <math.h>
 #include "instance.h"
 #include "fnv.h"
@@ -97,10 +94,6 @@ static inline void unpack_position(
     }
 
     {
-        u_char c;
-        u_char *p;
-        fcs_pats__bucket_list_t *l;
-
         /* Unpack bytes p into pile numbers j.
            p         p         p
            +--------+----:----+--------+
@@ -108,9 +101,9 @@ static inline void unpack_position(
            +--------+----:----+--------+
            j             j
            */
-
-        c = 0;
-        p = (u_char *)(pos->node) + sizeof(fcs_pats__tree_t);
+        unsigned char c = 0;
+        unsigned char *p =
+            (unsigned char *)(pos->node) + sizeof(fcs_pats__tree_t);
         fcs_bool_t k = FALSE;
         for (int w = 0; w < LOCAL_STACKS_NUM; w++)
         {
@@ -128,9 +121,8 @@ static inline void unpack_position(
             }
             k = !k;
             soft_thread->current_pos.stack_ids[w] = i;
-            l = soft_thread->bucket_from_pile_lookup[i];
-            fcs_cards_column_t w_col =
-                fcs_state_get_col(soft_thread->current_pos.s, w);
+            var_AUTO(l, soft_thread->bucket_from_pile_lookup[i]);
+            var_AUTO(w_col, fcs_state_get_col(soft_thread->current_pos.s, w));
             i = strecpy(w_col + 1, (char *)(l->pile));
             fcs_col_len(w_col) = i;
             soft_thread->current_pos.stack_hashes[w] = l->hash;
@@ -140,7 +132,8 @@ static inline void unpack_position(
     /* soft_thread->current_pos.freecells cells. */
 
     {
-        u_char *p = (((u_char *)pos) + sizeof(fcs_pats_position_t));
+        unsigned char *p =
+            (((unsigned char *)pos) + sizeof(fcs_pats_position_t));
         for (int i = 0; i < LOCAL_FREECELLS_NUM; i++)
         {
             fcs_freecell_card(soft_thread->current_pos.s, i) = *(p++);
@@ -158,25 +151,23 @@ static inline fcs_pats_position_t *dequeue_position(
     but we still get lots of low priority action (instead of
     ignoring it completely). */
 
-    fcs_bool_t last = FALSE;
+    fcs_bool_t is_last = FALSE;
     do
     {
-        soft_thread->dequeue__qpos--;
-        if (soft_thread->dequeue__qpos < soft_thread->dequeue__minpos)
+        if ((--soft_thread->dequeue__qpos) < soft_thread->dequeue__minpos)
         {
-            if (last)
+            if (is_last)
             {
                 return NULL;
             }
             soft_thread->dequeue__qpos = soft_thread->max_queue_idx;
-            soft_thread->dequeue__minpos--;
-            if (soft_thread->dequeue__minpos < 0)
+            if ((--soft_thread->dequeue__minpos) < 0)
             {
                 soft_thread->dequeue__minpos = soft_thread->max_queue_idx;
             }
             if (soft_thread->dequeue__minpos == 0)
             {
-                last = TRUE;
+                is_last = TRUE;
             }
         }
     } while (soft_thread->queue_head[soft_thread->dequeue__qpos] == NULL);
@@ -185,29 +176,26 @@ static inline fcs_pats_position_t *dequeue_position(
         soft_thread->queue_head[soft_thread->dequeue__qpos];
     soft_thread->queue_head[soft_thread->dequeue__qpos] = pos->queue;
 #ifdef DEBUG
-    soft_thread->num_positions_in_queue[soft_thread->dequeue__qpos]--;
+    --soft_thread->num_positions_in_queue[soft_thread->dequeue__qpos];
 #endif
 
     /* Decrease soft_thread->max_queue_idx if that queue emptied. */
-
     while (soft_thread->queue_head[soft_thread->dequeue__qpos] == NULL &&
            soft_thread->dequeue__qpos == soft_thread->max_queue_idx &&
            soft_thread->max_queue_idx > 0)
     {
-        soft_thread->max_queue_idx--;
-        soft_thread->dequeue__qpos--;
-        if (soft_thread->dequeue__qpos < soft_thread->dequeue__minpos)
+        --soft_thread->max_queue_idx;
+        if ((--soft_thread->dequeue__qpos) < soft_thread->dequeue__minpos)
         {
             soft_thread->dequeue__minpos = soft_thread->dequeue__qpos;
         }
     }
 
     /* Unpack the position into the work arrays. */
-
     unpack_position(soft_thread, pos);
 
 #ifdef DEBUG
-    soft_thread->num_positions_in_clusters[pos->cluster]--;
+    --soft_thread->num_positions_in_clusters[pos->cluster];
 #endif
     return pos;
 }
@@ -217,9 +205,8 @@ fcs_pats_position_t *fc_solve_pats__new_position(
     const fcs_pats__move_t *const m)
 {
     DECLARE_STACKS();
-    int t, cluster;
-    u_char *p;
-    fcs_pats_position_t *pos;
+    int cluster;
+    unsigned char *p;
 
     /* Search the list of stored positions.  If this position is found,
     then ignore it and return (unless this position is better). */
@@ -238,14 +225,12 @@ fcs_pats_position_t *fc_solve_pats__new_position(
     }
 
     /* A new or better position.  fc_solve_pats__insert() already stashed it in
-    the
-    tree, we just have to wrap a fcs_pats_position_t struct around it, and link
-    it
-    into the move stack.  Store the temp cells after the fcs_pats_position_t. */
-
+    the tree, we just have to wrap a fcs_pats_position_t struct around it, and
+    link it into the move stack.  Store the temp cells after the
+    fcs_pats_position_t. */
     if (soft_thread->freed_positions)
     {
-        p = (u_char *)soft_thread->freed_positions;
+        p = (unsigned char *)soft_thread->freed_positions;
         soft_thread->freed_positions = soft_thread->freed_positions->queue;
     }
     else
@@ -258,7 +243,7 @@ fcs_pats_position_t *fc_solve_pats__new_position(
         }
     }
 
-    pos = (fcs_pats_position_t *)p;
+    var_AUTO(pos, (fcs_pats_position_t *)p);
     pos->queue = NULL;
     pos->parent = parent;
     pos->node = node;
@@ -269,13 +254,12 @@ fcs_pats_position_t *fc_solve_pats__new_position(
 
     p += sizeof(fcs_pats_position_t);
     int i = 0;
-    for (t = 0; t < LOCAL_FREECELLS_NUM; t++)
+    for (int t = 0; t < LOCAL_FREECELLS_NUM; t++)
     {
         *p++ = fcs_freecell_card(soft_thread->current_pos.s, t);
-        if (fcs_freecell_card(soft_thread->current_pos.s, t) !=
-            fc_solve_empty_card)
+        if (!fcs_freecell_is_empty(soft_thread->current_pos.s, t))
         {
-            i++;
+            ++i;
         }
     }
     pos->num_cards_in_freecells = i;
@@ -283,7 +267,7 @@ fcs_pats_position_t *fc_solve_pats__new_position(
     return pos;
 }
 
-/* Hash the whole layout.  This is called once, at the start. */
+// Hash the whole layout.  This is called once, at the start.
 static inline fcs_bool_t check_for_exceeded(
     fcs_pats_thread_t *const soft_thread)
 {
@@ -307,12 +291,10 @@ static inline void freecell_solver_pats__make_move(
     fcs_pats_thread_t *const soft_thread, const fcs_pats__move_t *const m)
 {
     fcs_card_t card;
+    const_SLOT(from, m);
+    const_SLOT(to, m);
 
-    const int from = m->from;
-    const int to = m->to;
-
-    /* Remove from pile. */
-
+    // Remove from pile.
     if (m->fromtype == FCS_PATS__TYPE_FREECELL)
     {
         card = fcs_freecell_card(soft_thread->current_pos.s, from);
@@ -320,60 +302,52 @@ static inline void freecell_solver_pats__make_move(
     }
     else
     {
-        fcs_cards_column_t from_col =
-            fcs_state_get_col(soft_thread->current_pos.s, from);
+        var_AUTO(from_col, fcs_state_get_col(soft_thread->current_pos.s, from));
         fcs_col_pop_card(from_col, card);
         fc_solve_pats__hashpile(soft_thread, from);
     }
 
-    /* Add to pile. */
+    // Add to pile.
 
-    if (m->totype == FCS_PATS__TYPE_FREECELL)
+    switch (m->totype)
     {
+    case FCS_PATS__TYPE_FREECELL:
         fcs_freecell_card(soft_thread->current_pos.s, to) = card;
-    }
-    else if (m->totype == FCS_PATS__TYPE_WASTE)
-    {
-        fcs_cards_column_t to_col =
-            fcs_state_get_col(soft_thread->current_pos.s, to);
-        fcs_col_push_card(to_col, card);
+        break;
+    case FCS_PATS__TYPE_WASTE:
+        fcs_state_push(&soft_thread->current_pos.s, to, card);
         fc_solve_pats__hashpile(soft_thread, to);
-    }
-    else
-    {
+        break;
+    default:
         fcs_increment_foundation(soft_thread->current_pos.s, to);
+        break;
     }
 }
 
 static inline void fc_solve_pats__undo_move(
     fcs_pats_thread_t *const soft_thread, const fcs_pats__move_t *const m)
 {
-    const typeof(m->from) from = m->from;
-    const typeof(m->to) to = m->to;
-
-    /* Remove from 'to' pile. */
-
+    const_SLOT(from, m);
+    const_SLOT(to, m);
+    // Remove from 'to' pile.
     fcs_card_t card;
-    if (m->totype == FCS_PATS__TYPE_FREECELL)
+    switch (m->totype)
     {
+    case FCS_PATS__TYPE_FREECELL:
         card = fcs_freecell_card(soft_thread->current_pos.s, to);
         fcs_empty_freecell(soft_thread->current_pos.s, to);
-    }
-    else if (m->totype == FCS_PATS__TYPE_WASTE)
-    {
-        fcs_cards_column_t to_col =
-            fcs_state_get_col(soft_thread->current_pos.s, to);
-        fcs_col_pop_card(to_col, card);
+        break;
+    case FCS_PATS__TYPE_WASTE:
+        card = fcs_state_pop_col_card(&soft_thread->current_pos.s, to);
         fc_solve_pats__hashpile(soft_thread, to);
-    }
-    else
-    {
+        break;
+    default:
         card = fcs_make_card(
             fcs_foundation_value(soft_thread->current_pos.s, to), to);
-        fcs_foundation_value(soft_thread->current_pos.s, to)--;
+        --fcs_foundation_value(soft_thread->current_pos.s, to);
+        break;
     }
-
-    /* Add to 'from' pile. */
+    // Add to 'from' pile.
 
     if (m->fromtype == FCS_PATS__TYPE_FREECELL)
     {
@@ -381,9 +355,7 @@ static inline void fc_solve_pats__undo_move(
     }
     else
     {
-        fcs_cards_column_t from_col =
-            fcs_state_get_col(soft_thread->current_pos.s, from);
-        fcs_col_push_card(from_col, card);
+        fcs_state_push(&soft_thread->current_pos.s, from, card);
         fc_solve_pats__hashpile(soft_thread, from);
     }
 }
@@ -394,19 +366,18 @@ static inline int solve(
 #define DEPTH (soft_thread->curr_solve_depth)
 #define LEVEL (soft_thread->solve_stack[DEPTH])
 #define UP_LEVEL (soft_thread->solve_stack[DEPTH + 1])
-    typeof(soft_thread->curr_solve_dir) mydir = soft_thread->curr_solve_dir;
+    var_AUTO(mydir, soft_thread->curr_solve_dir);
 
     *is_finished = FALSE;
     while (DEPTH >= 0)
     {
-
         if (check_for_exceeded(soft_thread))
         {
             wrap_up_solve(soft_thread, mydir);
             return TRUE;
         }
 
-        typeof(LEVEL.parent) parent = LEVEL.parent;
+        var_S_SLOT(parent, LEVEL);
         /* If we've won already (or failed), we just go through the motions
         but always return FALSE from any position.  This enables the cleanup
         of the move stack and eventual destruction of the position store. */
@@ -415,34 +386,35 @@ static inline int solve(
             (parent->node->depth < parent->depth))
         {
             LEVEL.q = FALSE;
-            DEPTH--;
+            --DEPTH;
             mydir = FC_SOLVE_PATS__DOWN;
             continue;
         }
 
-        /* Generate an array of all the moves we can make. */
-        int nmoves;
-        if (!LEVEL.mp0)
+        // Generate an array of all the moves we can make.
+        int num_moves;
+        if (!LEVEL.moves_start)
         {
-            LEVEL.mp0 = fc_solve_pats__get_moves(soft_thread, parent, &nmoves);
-            if (!LEVEL.mp0)
+            LEVEL.moves_start =
+                fc_solve_pats__get_moves(soft_thread, parent, &num_moves);
+            if (!LEVEL.moves_start)
             {
                 LEVEL.q = FALSE;
-                DEPTH--;
+                --DEPTH;
                 mydir = FC_SOLVE_PATS__DOWN;
                 continue;
             }
-            LEVEL.mp_end = LEVEL.mp0 + (parent->num_childs = nmoves);
-            LEVEL.mp = LEVEL.mp0;
+            LEVEL.moves_end =
+                LEVEL.moves_start + (parent->num_childs = num_moves);
+            LEVEL.move_ptr = LEVEL.moves_start;
             LEVEL.q = FALSE;
         }
         else
         {
-            nmoves = LEVEL.mp_end - LEVEL.mp0;
+            num_moves = LEVEL.moves_end - LEVEL.moves_start;
         }
 
-        /* Make each move and either solve or queue the result. */
-
+        // Make each move and either solve or queue the result.
         if (mydir == FC_SOLVE_PATS__DOWN)
         {
             if (UP_LEVEL.q)
@@ -453,35 +425,35 @@ static inline int solve(
             {
                 free_position_non_recursive(soft_thread, LEVEL.pos);
             }
-            fc_solve_pats__undo_move(soft_thread, LEVEL.mp);
+            fc_solve_pats__undo_move(soft_thread, LEVEL.move_ptr);
             mydir = FC_SOLVE_PATS__UP;
-            LEVEL.mp++;
+            LEVEL.move_ptr++;
         }
 
-        if (LEVEL.mp == LEVEL.mp_end)
+        if (LEVEL.move_ptr == LEVEL.moves_end)
         {
             fc_solve_pats__free_array(
-                soft_thread, LEVEL.mp0, fcs_pats__move_t, nmoves);
-            LEVEL.mp0 = NULL;
-            DEPTH--;
+                soft_thread, LEVEL.moves_start, fcs_pats__move_t, num_moves);
+            LEVEL.moves_start = NULL;
+            --DEPTH;
             mydir = FC_SOLVE_PATS__DOWN;
             continue;
         }
         else
         {
-            freecell_solver_pats__make_move(soft_thread, LEVEL.mp);
+            freecell_solver_pats__make_move(soft_thread, LEVEL.move_ptr);
 
-            /* Calculate indices for the new piles. */
+            // Calculate indices for the new piles.
             fc_solve_pats__sort_piles(soft_thread);
 
-            /* See if this is a new position. */
-            LEVEL.pos =
-                fc_solve_pats__new_position(soft_thread, parent, LEVEL.mp);
+            // See if this is a new position.
+            LEVEL.pos = fc_solve_pats__new_position(
+                soft_thread, parent, LEVEL.move_ptr);
             if (!LEVEL.pos)
             {
                 parent->num_childs--;
-                fc_solve_pats__undo_move(soft_thread, LEVEL.mp);
-                LEVEL.mp++;
+                fc_solve_pats__undo_move(soft_thread, LEVEL.move_ptr);
+                LEVEL.move_ptr++;
                 mydir = FC_SOLVE_PATS__UP;
             }
             else
@@ -493,7 +465,7 @@ static inline int solve(
                    later. */
 
                 if (LEVEL.pos->cluster != parent->cluster ||
-                    nmoves < soft_thread->num_moves_to_cut_off)
+                    num_moves < soft_thread->num_moves_to_cut_off)
                 {
                     if (DEPTH + 1 >= soft_thread->max_solve_depth)
                     {
@@ -504,28 +476,25 @@ static inline int solve(
                                 soft_thread->max_solve_depth);
                     }
                     UP_LEVEL.parent = LEVEL.pos;
-                    UP_LEVEL.mp0 = NULL;
-                    DEPTH++;
+                    UP_LEVEL.moves_start = NULL;
+                    ++DEPTH;
                     mydir = FC_SOLVE_PATS__UP;
                 }
                 else
                 {
                     fc_solve_pats__queue_position(
-                        soft_thread, LEVEL.pos, (LEVEL.mp)->pri);
-                    fc_solve_pats__undo_move(soft_thread, LEVEL.mp);
+                        soft_thread, LEVEL.pos, (LEVEL.move_ptr)->pri);
+                    fc_solve_pats__undo_move(soft_thread, LEVEL.move_ptr);
                     LEVEL.q = TRUE;
-                    LEVEL.mp++;
+                    LEVEL.move_ptr++;
                     mydir = FC_SOLVE_PATS__UP;
                 }
             }
         }
-
-        /* Return true if this position needs to be kept around. */
     }
     *is_finished = TRUE;
     wrap_up_solve(soft_thread, mydir);
-    typeof(UP_LEVEL.q) ret = UP_LEVEL.q;
-    return ret;
+    return UP_LEVEL.q;
 #undef LEVEL
 #undef UP_LEVEL
 #undef DEPTH
@@ -533,8 +502,6 @@ static inline int solve(
 
 DLLEXPORT void fc_solve_pats__do_it(fcs_pats_thread_t *const soft_thread)
 {
-    /* Solve it. */
-
     while (1)
     {
         if (!soft_thread->curr_solve_pos)
@@ -546,7 +513,7 @@ DLLEXPORT void fc_solve_pats__do_it(fcs_pats_thread_t *const soft_thread)
             }
             soft_thread->solve_stack[0].parent = soft_thread->curr_solve_pos =
                 pos;
-            soft_thread->solve_stack[0].mp0 = NULL;
+            soft_thread->solve_stack[0].moves_start = NULL;
             soft_thread->curr_solve_depth = 0;
             soft_thread->curr_solve_dir = FC_SOLVE_PATS__UP;
         }
@@ -578,12 +545,9 @@ having separate queues). */
 void fc_solve_pats__queue_position(fcs_pats_thread_t *const soft_thread,
     fcs_pats_position_t *const pos, int pri)
 {
-    double x;
-
     /* In addition to the priority of a move, a position gets an
     additional priority depending on the number of cards out.  We use a
     "queue squashing function" to map num_cards_out to priority.  */
-
     const int num_cards_out =
         fcs_foundation_value(soft_thread->current_pos.s, 0) +
         fcs_foundation_value(soft_thread->current_pos.s, 1) +
@@ -591,20 +555,16 @@ void fc_solve_pats__queue_position(fcs_pats_thread_t *const soft_thread,
         fcs_foundation_value(soft_thread->current_pos.s, 3);
 
     /* y_param[0] * nout^2 + y_param[1] * nout + y_param[2] */
-
-    const typeof(soft_thread->pats_solve_params.y[0]) *const y_param =
-        soft_thread->pats_solve_params.y;
-
-    x = (y_param[0] * num_cards_out + y_param[1]) * num_cards_out + y_param[2];
-    {
-        /*
-         * GCC gives a warning with some flags if we cast the result
-         * of floor to an int directly. As a result, we need to use
-         * an intermediate variable.
-         * */
-        const double rounded_x = (floor(x + .5));
-        pri += (int)rounded_x;
-    }
+    const_PTR(y_param, soft_thread->pats_solve_params.y);
+    const double x =
+        (y_param[0] * num_cards_out + y_param[1]) * num_cards_out + y_param[2];
+    /*
+     * GCC gives a warning with some flags if we cast the result
+     * of floor to an int directly. As a result, we need to use
+     * an intermediate variable.
+     * */
+    const double rounded_x = (floor(x + .5));
+    pri += (int)rounded_x;
 
     if (pri < 0)
     {
@@ -643,8 +603,8 @@ void fc_solve_pats__queue_position(fcs_pats_thread_t *const soft_thread,
         }
     }
 #ifdef DEBUG
-    soft_thread->num_positions_in_queue[pri]++;
-    soft_thread->num_positions_in_clusters[pos->cluster]++;
+    ++soft_thread->num_positions_in_queue[pri];
+    ++soft_thread->num_positions_in_clusters[pos->cluster];
 #endif
 }
 
