@@ -17,16 +17,14 @@
 extern "C" {
 #endif
 
-#include "config.h"
 #include <Judy.h>
-#include "state.h"
 #include "meta_alloc.h"
 #include "fcs_enums.h"
 #include "dbm_common.h"
 #include "delta_states.h"
 #include "dbm_calc_derived_iface.h"
 
-typedef fcs_state_t fcs_pdfs_key_t;
+typedef fcs_encoded_state_buffer_t fcs_pdfs_key_t;
 
 typedef struct fcs_pdfs_key_info_struct
 {
@@ -43,9 +41,7 @@ typedef struct
     Pvoid_t states_values_to_keys_map;
     fcs_compact_allocator_t states_values_to_keys_allocator;
     long count_elements_in_cache, max_num_elements_in_cache;
-
     fcs_pdfs_cache_key_info_t *lowest_pri, *highest_pri;
-
 #define RECYCLE_BIN_NEXT(item) ((item)->higher_pri)
     fcs_pdfs_cache_key_info_t *recycle_bin;
 } fcs_pseudo_dfs_lru_cache_t;
@@ -74,7 +70,7 @@ static inline void fcs_pdfs_cache_init(fcs_pseudo_dfs_lru_cache_t *const cache,
     cache->max_num_elements_in_cache = max_num_elements_in_cache;
 }
 
-static inline const fcs_bool_t fcs_pdfs_cache_does_key_exist(
+static inline fcs_bool_t fcs_pdfs_cache_does_key_exist(
     fcs_pseudo_dfs_lru_cache_t *const cache, fcs_pdfs_key_t *const key)
 {
     fcs_pdfs_cache_key_info_t *existing;
@@ -138,7 +134,7 @@ static inline fcs_pdfs_cache_key_info_t *fcs_pdfs_cache_insert(
     {
         cache_key = (fcs_pdfs_cache_key_info_t *)fcs_compact_alloc_ptr(
             &(cache->states_values_to_keys_allocator), sizeof(*cache_key));
-        cache->count_elements_in_cache++;
+        ++cache->count_elements_in_cache;
     }
 
     cache_key->key = *key;
