@@ -59,7 +59,12 @@ GetOptions(
     '--glob=s'       => \$tests_glob,
     '--prove!'       => \$use_prove,
     '--jobs|j=n'     => \$num_jobs,
-) or die "--glob='tests_glob'";
+) or die "Wrong opts - $!";
+
+sub myglob
+{
+    return glob( shift . "/$tests_glob" );
+}
 
 {
     my $fcs_path = Cwd::getcwd();
@@ -71,50 +76,30 @@ GetOptions(
         ( path( File::Spec->catdir( File::Spec->curdir(), "t", "TAGS.txt" ) )
             ->slurp_utf8 =~ /([a-zA-Z_0-9]+)/g );
 
-    my $testing_preset_rc;
+    my $preset_dest =
+        File::Spec->catdir( File::Spec->curdir(), "t", "Presets" );
+    my $preset_src = File::Spec->catfile( $abs_bindir, "Presets" );
+    my $pset_files_dest = File::Spec->catdir( $preset_dest, "presets" );
+    my $pset_files_src  = File::Spec->catdir( $preset_src,  "presets" );
+    mkpath($pset_files_dest);
+    foreach my $f ( path($pset_files_src)->children(qr{\.sh\z}) )
     {
-        my $preset_dest =
-            File::Spec->catdir( File::Spec->curdir(), "t", "Presets" );
-        my $preset_src = File::Spec->catfile( $abs_bindir, "Presets" );
-        my $pset_files_dest = File::Spec->catdir( $preset_dest, "presets" );
-        my $pset_files_src  = File::Spec->catdir( $preset_src,  "presets" );
-        mkpath($pset_files_dest);
-        opendir( my $d, $pset_files_src );
-        while ( defined( my $f = readdir($d) ) )
-        {
-
-            if ( $f =~ m{\.sh\z} )
-            {
-                copy(
-                    File::Spec->catfile( $pset_files_src,  $f ),
-                    File::Spec->catfile( $pset_files_dest, $f )
-                );
-            }
-        }
-        closedir($d);
-
-        my $files_dest_abs =
-            File::Spec->rel2abs(
-            File::Spec->catfile( $pset_files_dest, "STUB.TXT" ) );
-
-        $files_dest_abs =~ s{STUB\.TXT\z}{};
-
-        $testing_preset_rc =
-            File::Spec->rel2abs(
-            File::Spec->catfile( $preset_dest, "presetrc" ) );
-
-        open my $in, "<",
-            File::Spec->catfile( $fcs_path, "Presets", "presetrc" );
-        open my $out, ">", $testing_preset_rc;
-        while ( my $line = <$in> )
-        {
-            $line =~ s{\A(dir=)(.*)}{$1$files_dest_abs\n}ms;
-            print {$out} $line;
-        }
-        close($in);
-        close($out);
+        $f->copy( File::Spec->catfile( $pset_files_dest, $f->basename ) );
     }
 
+    my $files_dest_abs =
+        File::Spec->rel2abs(
+        File::Spec->catfile( $pset_files_dest, "STUB.TXT" ) );
+
+    $files_dest_abs =~ s{STUB\.TXT\z}{};
+
+    my $testing_preset_rc =
+        File::Spec->rel2abs( File::Spec->catfile( $preset_dest, "presetrc" ) );
+    path($testing_preset_rc)
+        ->spew_utf8(
+        path( File::Spec->catfile( $fcs_path, "Presets", "presetrc" ) )
+            ->slurp_utf8 =~ s{^(dir=)([^\n]*)}{$1$files_dest_abs}gmrs );
+
     local $ENV{FREECELL_SOLVER_PRESETRC} = $testing_preset_rc;
     local $ENV{FREECELL_SOLVER_QUIET}    = 1;
     Env::Path->PATH->Prepend(
@@ -185,15 +170,6 @@ GetOptions(
         }
     }
 
-# Cancelled because it is done by the build system.
-# if (! glob('t/valgrind--*.t'))
-# {
-# my $start = time();
-# system($^X, "$abs_bindir/scripts/generate-individual-valgrind-test-scripts.pl");
-# my $end = time();
-# print "StartGen=$start\nEndGen=$end\n";
-# }
-
     # Put the valgrind tests last, because they take a long time.
     my @tests =
         sort {
@@ -202,11 +178,11 @@ GetOptions(
             || ( basename($a) cmp basename($b) )
             || ( $a cmp $b )
         } (
-        glob("t/$tests_glob"),
-        glob("./$tests_glob"),
+        myglob('t'),
+        myglob('.'),
         (
               ( $fcs_path ne $abs_bindir )
-            ? ( glob("$abs_bindir/t/t/$tests_glob") )
+            ? ( myglob("$abs_bindir/t/t") )
             : ()
         ),
         );
@@ -216,6 +192,7 @@ GetOptions(
         my $re = qr/$exclude_re_s/ms;
         @tests = grep { basename($_) !~ $re } @tests;
     }
+    @tests = grep { basename($_) !~ /\A(?:lextab|yacctab)\.py\z/ } @tests;
 
     if ( !$ENV{FCS_TEST_BUILD} )
     {
@@ -227,22 +204,20 @@ GetOptions(
         @tests = grep { !/valgrind/ } @tests;
     }
 
+    print STDERR "FCS_PATH = $ENV{FCS_PATH}\n";
+    print STDERR "FCS_SRC_PATH = $ENV{FCS_SRC_PATH}\n";
+    print STDERR "FCS_TEST_TAGS = <$ENV{FCS_TEST_TAGS}>\n";
+    if ( $ENV{FCS_TEST_SHELL} )
     {
-        print STDERR "FCS_PATH = $ENV{FCS_PATH}\n";
-        print STDERR "FCS_SRC_PATH = $ENV{FCS_SRC_PATH}\n";
-        print STDERR "FCS_TEST_TAGS = <$ENV{FCS_TEST_TAGS}>\n";
-        if ( $ENV{FCS_TEST_SHELL} )
-        {
-            system("bash");
-        }
-        elsif (@execute)
-        {
-            system(@execute);
-        }
-        else
-        {
-            run_tests( \@tests );
-        }
+        system("bash");
+    }
+    elsif (@execute)
+    {
+        system(@execute);
+    }
+    else
+    {
+        run_tests( \@tests );
     }
 }
 
