@@ -12,30 +12,12 @@
  * Currently Hard DFS, Soft-DFS, Random-DFS, BeFS and BFS are implemented.
  */
 
-#include "dll_thunk.h"
-#include <stdlib.h>
-#include <string.h>
-#include <limits.h>
-#include <stdio.h>
-#include <assert.h>
-
-#include "config.h"
-#include "state.h"
 #include "scans.h"
 #include "meta_alloc.h"
-#include "instance.h"
 #include "move_stack_compact_alloc.h"
-#include "freecell.h"
-#include "simpsim.h"
-#include "move_funcs_maps.h"
+#include "instance.h"
 
-#if 0
-#define DEBUG 1
-#endif
-
-#if 1
 #undef DEBUG
-#endif
 
 #include "scans_impl.h"
 
@@ -48,26 +30,23 @@ void fc_solve_increase_dfs_max_depth(fc_solve_soft_thread_t *const soft_thread)
 {
     const_AUTO(new_dfs_max_depth,
         DFS_VAR(soft_thread, dfs_max_depth) + SOFT_DFS_DEPTH_GROW_BY);
-
     DFS_VAR(soft_thread, soft_dfs_info) =
         SREALLOC(DFS_VAR(soft_thread, soft_dfs_info), new_dfs_max_depth);
+    var_AUTO(soft_dfs_info, DFS_VAR(soft_thread, soft_dfs_info) +
+                                DFS_VAR(soft_thread, dfs_max_depth));
+    const_AUTO(end_soft_dfs_info, soft_dfs_info + SOFT_DFS_DEPTH_GROW_BY);
 
-    fcs_soft_dfs_stack_item_t *soft_dfs_info =
-        DFS_VAR(soft_thread, soft_dfs_info) +
-        DFS_VAR(soft_thread, dfs_max_depth);
-
-    fcs_soft_dfs_stack_item_t *const end_soft_dfs_info =
-        soft_dfs_info + SOFT_DFS_DEPTH_GROW_BY;
     for (; soft_dfs_info < end_soft_dfs_info; soft_dfs_info++)
     {
         *soft_dfs_info = (fcs_soft_dfs_stack_item_t){
             .state = NULL,
-            .tests_list_index = 0,
+            .move_func_list_idx = 0,
             .move_func_idx = 0,
             .current_state_index = 0,
             .derived_states_list =
                 {
-                    .num_states = 0, .states = NULL,
+                    .num_states = 0,
+                    .states = NULL,
                 },
             .derived_states_random_indexes = NULL,
             .derived_states_random_indexes_max_size = 0,
@@ -78,7 +57,6 @@ void fc_solve_increase_dfs_max_depth(fc_solve_soft_thread_t *const soft_thread)
 }
 
 #ifdef FCS_RCS_STATES
-
 /* TODO : Unit-test this function as it had had a bug beforehand
  * because fcs_lru_side_t had been an unsigned long.
  * */
@@ -89,8 +67,7 @@ extern int fc_solve_compare_lru_cache_keys(const void *const void_a,
 {
 #define GET_PARAM(p)                                                           \
     ((fcs_lru_side_t)(((const fcs_cache_key_info_t *)(p))->val_ptr))
-    fcs_lru_side_t a = GET_PARAM(void_a);
-    fcs_lru_side_t b = GET_PARAM(void_b);
+    const fcs_lru_side_t a = GET_PARAM(void_a), b = GET_PARAM(void_b);
 
     return ((a > b) ? 1 : (a < b) ? (-1) : 0);
 #undef GET_PARAM
@@ -99,15 +76,12 @@ extern int fc_solve_compare_lru_cache_keys(const void *const void_a,
 #define NEXT_CACHE_STATE(s) ((s)->lower_pri)
 fcs_state_t *fc_solve_lookup_state_key_from_val(
     fc_solve_instance_t *const instance,
-    fcs_collectible_state_t *const orig_ptr_state_val)
+    const fcs_collectible_state_t *const orig_ptr_state_val)
 {
 #if (FCS_RCS_CACHE_STORAGE == FCS_RCS_CACHE_STORAGE_JUDY)
     PWord_t PValue;
 #endif
-#if ((!defined(HARD_CODED_NUM_FREECELLS)) || (!defined(HARD_CODED_NUM_STACKS)))
-    SET_GAME_PARAMS();
-#endif
-
+    FC__STACKS__SET_PARAMS();
     fcs_lru_cache_t *cache = &(instance->rcs_states_cache);
 
     ssize_t parents_stack_len = 1;
@@ -116,7 +90,7 @@ fcs_state_t *fc_solve_lookup_state_key_from_val(
     struct
     {
         fcs_cache_key_info_t *new_cache_state;
-        fcs_collectible_state_t *state_val;
+        const fcs_collectible_state_t *state_val;
     } *parents_stack = SMALLOC(parents_stack, parents_stack_max_len);
 
     parents_stack[0].state_val = orig_ptr_state_val;
@@ -149,34 +123,32 @@ fcs_state_t *fc_solve_lookup_state_key_from_val(
             }
         }
 #else
+        if (cache->recycle_bin)
         {
-            if (cache->recycle_bin)
-            {
-                new_cache_state = cache->recycle_bin;
-                cache->recycle_bin = NEXT_CACHE_STATE(new_cache_state);
-            }
-            else
-            {
-                new_cache_state = fcs_compact_alloc_ptr(
-                    &(cache->states_values_to_keys_allocator),
+            new_cache_state = cache->recycle_bin;
+            cache->recycle_bin = NEXT_CACHE_STATE(new_cache_state);
+        }
+        else
+        {
+            new_cache_state =
+                fcs_compact_alloc_ptr(&(cache->states_values_to_keys_allocator),
                     sizeof(*new_cache_state));
-            }
+        }
 
-            new_cache_state->val_ptr =
-                parents_stack[parents_stack_len - 1].state_val;
-            fcs_cache_key_info_t *const existing_cache_state =
-                (fcs_cache_key_info_t *)fc_solve_kaz_tree_alloc_insert(
-                    cache->kaz_tree, new_cache_state);
+        new_cache_state->val_ptr =
+            parents_stack[parents_stack_len - 1].state_val;
+        fcs_cache_key_info_t *const existing_cache_state =
+            (fcs_cache_key_info_t *)fc_solve_kaz_tree_alloc_insert(
+                cache->kaz_tree, new_cache_state);
 
-            if (existing_cache_state)
-            {
-                NEXT_CACHE_STATE(new_cache_state) = cache->recycle_bin;
-                cache->recycle_bin = new_cache_state;
+        if (existing_cache_state)
+        {
+            NEXT_CACHE_STATE(new_cache_state) = cache->recycle_bin;
+            cache->recycle_bin = new_cache_state;
 
-                parents_stack[parents_stack_len - 1].new_cache_state =
-                    new_cache_state = existing_cache_state;
-                break;
-            }
+            parents_stack[parents_stack_len - 1].new_cache_state =
+                new_cache_state = existing_cache_state;
+            break;
         }
 #endif
 
@@ -188,14 +160,12 @@ fcs_state_t *fc_solve_lookup_state_key_from_val(
         new_cache_state->val_ptr =
             parents_stack[parents_stack_len - 1].state_val;
 #endif
-
         new_cache_state->lower_pri = new_cache_state->higher_pri = NULL;
-
-        cache->count_elements_in_cache++;
+        ++cache->count_elements_in_cache;
 
         if (!FCS_S_PARENT(parents_stack[parents_stack_len - 1].state_val))
         {
-            new_cache_state->key = instance->state_copy_ptr->s;
+            new_cache_state->key = instance->state_copy.s;
             break;
         }
         else
@@ -210,9 +180,8 @@ fcs_state_t *fc_solve_lookup_state_key_from_val(
         }
     }
 
-    for (parents_stack_len--; parents_stack_len > 0; parents_stack_len--)
+    for (--parents_stack_len; parents_stack_len > 0; --parents_stack_len)
     {
-
         new_cache_state = parents_stack[parents_stack_len - 1].new_cache_state;
 
         fcs_state_t *const pass_key = &(new_cache_state->key);
@@ -227,7 +196,6 @@ fcs_state_t *fc_solve_lookup_state_key_from_val(
 
         for (; next_move < moves_end; next_move++)
         {
-
             fc_solve_apply_move(pass_key, NULL,
                 (*next_move)PASS_FREECELLS(LOCAL_FREECELLS_NUM)
                     PASS_STACKS(LOCAL_STACKS_NUM));
@@ -276,54 +244,44 @@ fcs_state_t *fc_solve_lookup_state_key_from_val(
 
     free(parents_stack);
 
-    if (cache->count_elements_in_cache > cache->max_num_elements_in_cache)
-    {
-        long count = cache->count_elements_in_cache;
-        long limit = cache->max_num_elements_in_cache;
+    var_AUTO(count, cache->count_elements_in_cache);
+    const_AUTO(limit, cache->max_num_elements_in_cache);
 
-        while (count > limit)
-        {
-            fcs_cache_key_info_t *lowest_pri = cache->lowest_pri;
+    while (count > limit)
+    {
+        fcs_cache_key_info_t *lowest_pri = cache->lowest_pri;
 #if (FCS_RCS_CACHE_STORAGE == FCS_RCS_CACHE_STORAGE_JUDY)
-            int rc_int;
-            JLD(rc_int, cache->states_values_to_keys_map,
-                (Word_t)(lowest_pri->val_ptr));
+        int rc_int;
+        JLD(rc_int, cache->states_values_to_keys_map,
+            (Word_t)(lowest_pri->val_ptr));
 #else
-            fc_solve_kaz_tree_delete_free(cache->kaz_tree,
-                fc_solve_kaz_tree_lookup(cache->kaz_tree, lowest_pri));
+        fc_solve_kaz_tree_delete_free(cache->kaz_tree,
+            fc_solve_kaz_tree_lookup(cache->kaz_tree, lowest_pri));
 #endif
 
-            cache->lowest_pri = lowest_pri->higher_pri;
-            cache->lowest_pri->lower_pri = NULL;
+        cache->lowest_pri = lowest_pri->higher_pri;
+        cache->lowest_pri->lower_pri = NULL;
 
-            NEXT_CACHE_STATE(lowest_pri) = cache->recycle_bin;
+        NEXT_CACHE_STATE(lowest_pri) = cache->recycle_bin;
 
-            cache->recycle_bin = lowest_pri;
-            count--;
-        }
-
-        cache->count_elements_in_cache = count;
+        cache->recycle_bin = lowest_pri;
+        --count;
     }
 
+    cache->count_elements_in_cache = count;
+
     return &(new_cache_state->key);
 }
 
 #undef NEXT_CACHE_STATE
-
 #endif
 
 #ifdef DEBUG
-
 #define TRACE0(message)                                                        \
     fcs_trace("BestFS - %s ; Iters=%ld.\n", message,                           \
-        (long)(*instance_num_checked_states_ptr));
-
+        (long)(*instance_num_checked_states_ptr))
 #else
-
-#define TRACE0(no_use)                                                         \
-    {                                                                          \
-    }
-
+#define TRACE0(no_use)
 #endif
 
 #define my_brfs_queue (BRFS_VAR(soft_thread, bfs_queue))
@@ -343,21 +301,14 @@ static inline void fc_solve_initialize_bfs_queue(
     /* Initialize the BFS queue. We have one dummy element at the beginning
        in order to make operations simpler. */
     my_brfs_queue = NEW_BRFS_QUEUE_ITEM();
-
     my_brfs_queue_last_item = my_brfs_queue->next = NEW_BRFS_QUEUE_ITEM();
-
     my_brfs_queue_last_item->next = NULL;
-
     my_brfs_recycle_bin = NULL;
-
-    return;
 }
 
 void fc_solve_soft_thread_init_befs_or_bfs(
     fc_solve_soft_thread_t *const soft_thread)
 {
-    fc_solve_soft_thread_update_initial_cards_val(soft_thread);
-
     if (soft_thread->is_befs)
     {
 #define WEIGHTING(soft_thread) (&(BEFS_VAR(soft_thread, weighting)))
@@ -370,30 +321,30 @@ void fc_solve_soft_thread_init_befs_or_bfs(
         fc_solve_initialize_bfs_queue(soft_thread);
     }
 
-    if (!BEFS_M_VAR(soft_thread, tests_list))
+    if (!BEFS_M_VAR(soft_thread, moves_list))
     {
         size_t num = 0;
-        fc_solve_solve_for_state_move_func_t *tests_list = NULL;
+        fcs_move_func *moves_list = NULL;
 
         for (size_t group_idx = 0;
-             group_idx < soft_thread->by_depth_tests_order.by_depth_tests[0]
-                             .tests_order.num_groups;
-             group_idx++)
+             group_idx < soft_thread->by_depth_moves_order.by_depth_moves[0]
+                             .moves_order.num;
+             ++group_idx)
         {
-            add_to_move_funcs_list(&tests_list, &num,
-                soft_thread->by_depth_tests_order.by_depth_tests[0]
-                    .tests_order.groups[group_idx]
-                    .order_group_tests,
-                soft_thread->by_depth_tests_order.by_depth_tests[0]
-                    .tests_order.groups[group_idx]
+            add_to_move_funcs_list(&moves_list, &num,
+                soft_thread->by_depth_moves_order.by_depth_moves[0]
+                    .moves_order.groups[group_idx]
+                    .move_funcs,
+                soft_thread->by_depth_moves_order.by_depth_moves[0]
+                    .moves_order.groups[group_idx]
                     .num);
         }
-        BEFS_M_VAR(soft_thread, tests_list) = tests_list;
-        BEFS_M_VAR(soft_thread, tests_list_end) = tests_list + num;
+        BEFS_M_VAR(soft_thread, moves_list) = moves_list;
+        BEFS_M_VAR(soft_thread, moves_list_end) = moves_list + num;
     }
     BEFS_M_VAR(soft_thread, first_state_to_check) =
         FCS_STATE_keyval_pair_to_collectible(
-            HT_INSTANCE(soft_thread->hard_thread)->state_copy_ptr);
+            &fcs_st_instance(soft_thread)->state_copy);
 }
 
 /*
@@ -404,8 +355,9 @@ void fc_solve_soft_thread_init_befs_or_bfs(
  *
  *  It goes on in this fashion until the final state was reached or
  *  there are no more states in the queue.
-*/
-int fc_solve_befs_or_bfs_do_solve(fc_solve_soft_thread_t *const soft_thread)
+ */
+fc_solve_solve_process_ret_t fc_solve_befs_or_bfs_do_solve(
+    fc_solve_soft_thread_t *const soft_thread)
 {
     fc_solve_hard_thread_t *const hard_thread = soft_thread->hard_thread;
     fc_solve_instance_t *const instance = HT_INSTANCE(hard_thread);
@@ -424,28 +376,26 @@ int fc_solve_befs_or_bfs_do_solve(fc_solve_soft_thread_t *const soft_thread)
     const_AUTO(soft_thread_id, soft_thread->id);
     const fcs_bool_t is_a_complete_scan =
         STRUCT_QUERY_FLAG(soft_thread, FCS_SOFT_THREAD_IS_A_COMPLETE_SCAN);
+#ifndef FCS_DISABLE_NUM_STORED_STATES
     const_SLOT(effective_max_num_states_in_collection, instance);
+#endif
 
     fcs_states_linked_list_item_t *queue = NULL;
     fcs_states_linked_list_item_t *queue_last_item = NULL;
     pri_queue_t *pqueue = NULL;
+    fc_solve_solve_process_ret_t error_code;
+    fcs_derived_states_list_t derived = {.num_states = 0, .states = NULL};
 
-    int error_code;
-
-    fcs_int_limit_t hard_thread_max_num_checked_states;
-
-    fcs_derived_states_list_t derived;
-    derived.num_states = 0;
-    derived.states = NULL;
-
-    const fc_solve_solve_for_state_move_func_t *const tests_list =
-        BEFS_M_VAR(soft_thread, tests_list);
-    const fc_solve_solve_for_state_move_func_t *const tests_list_end =
-        BEFS_M_VAR(soft_thread, tests_list_end);
+    const fcs_move_func *const moves_list = BEFS_M_VAR(soft_thread, moves_list);
+    const fcs_move_func *const moves_list_end =
+        BEFS_M_VAR(soft_thread, moves_list_end);
 
     DECLARE_STATE();
-    ASSIGN_ptr_state(BEFS_M_VAR(soft_thread, first_state_to_check));
+    PTR_STATE = BEFS_M_VAR(soft_thread, first_state_to_check);
+    FCS_ASSIGN_STATE_KEY();
+#ifndef FCS_ENABLE_PRUNE__R_TF__UNCOND
     const fcs_bool_t enable_pruning = soft_thread->enable_pruning;
+#endif
 
     fcs_int_limit_t *const instance_num_checked_states_ptr =
         &(instance->i__num_checked_states);
@@ -467,18 +417,11 @@ int fc_solve_befs_or_bfs_do_solve(fc_solve_soft_thread_t *const soft_thread)
         queue = my_brfs_queue;
         queue_last_item = my_brfs_queue_last_item;
     }
-
-#if ((!defined(HARD_CODED_NUM_FREECELLS)) || (!defined(HARD_CODED_NUM_STACKS)))
-    SET_GAME_PARAMS();
-#endif
-
-    CALC_HARD_THREAD_MAX_NUM_CHECKED_STATES();
-
+    FC__STACKS__SET_PARAMS();
+    const_AUTO(max_num_states, calc_ht_max_num_states(instance, hard_thread));
 #ifndef FCS_WITHOUT_ITER_HANDLER
-    const fcs_instance_debug_iter_output_func_t debug_iter_output_func =
-        instance->debug_iter_output_func;
-    const fcs_instance_debug_iter_output_context_t debug_iter_output_context =
-        instance->debug_iter_output_context;
+    const_SLOT(debug_iter_output_func, instance);
+    const_SLOT(debug_iter_output_context, instance);
 #endif
 
     /* Continue as long as there are states in the queue or
@@ -501,7 +444,7 @@ int fc_solve_befs_or_bfs_do_solve(fc_solve_soft_thread_t *const soft_thread)
         if (fcs__should_state_be_pruned(enable_pruning, PTR_STATE))
         {
             fcs_collectible_state_t *const after_pruning_state =
-                fc_solve_sfs_raymond_prune(soft_thread, &pass);
+                fc_solve_sfs_raymond_prune(soft_thread, pass);
             if (after_pruning_state)
             {
                 ASSIGN_ptr_state(after_pruning_state);
@@ -538,7 +481,6 @@ int fc_solve_befs_or_bfs_do_solve(fc_solve_soft_thread_t *const soft_thread)
         if (check_if_limits_exceeded())
         {
             BEFS_M_VAR(soft_thread, first_state_to_check) = PTR_STATE;
-
             TRACE0("error_code - FCS_STATE_SUSPEND_PROCESS");
             error_code = FCS_STATE_SUSPEND_PROCESS;
             goto my_return_label;
@@ -558,7 +500,7 @@ int fc_solve_befs_or_bfs_do_solve(fc_solve_soft_thread_t *const soft_thread)
                         ? 0
                         : FCS_S_VISITED_ITER(FCS_S_PARENT(PTR_STATE)))
 #endif
-                );
+            );
         }
 #endif
 
@@ -570,10 +512,10 @@ int fc_solve_befs_or_bfs_do_solve(fc_solve_soft_thread_t *const soft_thread)
         if ((num_vacant_stacks == LOCAL_STACKS_NUM) &&
             (num_vacant_freecells == LOCAL_FREECELLS_NUM))
         {
+#ifdef FCS_WITH_MOVES
             instance->final_state = PTR_STATE;
-
+#endif
             BUMP_NUM_CHECKED_STATES();
-
             error_code = FCS_STATE_WAS_SOLVED;
             goto my_return_label;
         }
@@ -588,20 +530,18 @@ int fc_solve_befs_or_bfs_do_solve(fc_solve_soft_thread_t *const soft_thread)
             ,
             is_simple_simon
 #endif
-            );
+        );
 
         TRACE0("perform_tests");
-
         /*
          * Do all the tests at one go, because that is the way it should be
          * done for BFS and BeFS.
-        */
+         */
         derived.num_states = 0;
-        for (const fc_solve_solve_for_state_move_func_t *move_func_ptr =
-                 tests_list;
-             move_func_ptr < tests_list_end; move_func_ptr++)
+        for (const fcs_move_func *move_func_ptr = moves_list;
+             move_func_ptr < moves_list_end; move_func_ptr++)
         {
-            (*move_func_ptr)(soft_thread, &pass, &derived);
+            move_func_ptr->f(soft_thread, pass, &derived);
         }
 
         if (is_a_complete_scan)
@@ -612,7 +552,6 @@ int fc_solve_befs_or_bfs_do_solve(fc_solve_soft_thread_t *const soft_thread)
         /* Increase the number of iterations by one .
          * */
         BUMP_NUM_CHECKED_STATES();
-
         TRACE0("Insert all states");
         /* Insert all the derived states into the PQ or Queue */
         fcs_derived_states_list_item_t *derived_iter;
@@ -737,7 +676,7 @@ my_return_label:
  * */
 int fc_solve_sfs_check_state_begin(fc_solve_hard_thread_t *const hard_thread,
     fcs_kv_state_t *const out_new_state_out,
-    fcs_kv_state_t *const raw_ptr_state_raw SFS__PASS_MOVE_STACK(
+    fcs_kv_state_t raw_state_raw SFS__PASS_MOVE_STACK(
         fcs_move_stack_t *const moves))
 {
     fcs_collectible_state_t *raw_ptr_new_state;
@@ -757,7 +696,7 @@ int fc_solve_sfs_check_state_begin(fc_solve_hard_thread_t *const hard_thread,
     }
 
     FCS_STATE_collectible_to_kv(out_new_state_out, raw_ptr_new_state);
-    fcs_duplicate_kv_state(out_new_state_out, raw_ptr_state_raw);
+    fcs_duplicate_kv_state(out_new_state_out, &raw_state_raw);
 #ifdef FCS_RCS_STATES
 #define INFO_STATE_PTR(kv_ptr) ((kv_ptr)->val)
 #else
@@ -767,7 +706,7 @@ int fc_solve_sfs_check_state_begin(fc_solve_hard_thread_t *const hard_thread,
     /* Some BeFS and BFS parameters that need to be initialized in
      * the derived state.
      * */
-    FCS_S_PARENT(raw_ptr_new_state) = INFO_STATE_PTR(raw_ptr_state_raw);
+    FCS_S_PARENT(raw_ptr_new_state) = INFO_STATE_PTR(&raw_state_raw);
 #ifdef FCS_WITH_MOVES
     FCS_S_MOVES_TO_PARENT(raw_ptr_new_state) = moves;
 #endif
@@ -788,12 +727,10 @@ int fc_solve_sfs_check_state_begin(fc_solve_hard_thread_t *const hard_thread,
     return 0;
 }
 
-extern void fc_solve_sfs_check_state_end(
-    fc_solve_soft_thread_t *const soft_thread,
-    fcs_kv_state_t *const raw_ptr_state_raw,
-    fcs_kv_state_t *const raw_ptr_new_state_raw, const int state_context_value,
-    fcs_move_stack_t *const moves,
-    fcs_derived_states_list_t *const derived_states_list)
+extern fcs_collectible_state_t *fc_solve_sfs_check_state_end(
+    fc_solve_soft_thread_t *const soft_thread, fcs_kv_state_t raw_state_raw,
+    fcs_kv_state_t *const raw_ptr_new_state_raw FCS__pass_moves(
+        fcs_move_stack_t *const moves))
 {
     const_SLOT(hard_thread, soft_thread);
     const_AUTO(instance, HT_INSTANCE(hard_thread));
@@ -809,12 +746,10 @@ extern void fc_solve_sfs_check_state_end(
     fcs_kv_state_t existing_state;
 
 #define ptr_new_state_foo (raw_ptr_new_state_raw->val)
-#define ptr_state (raw_ptr_state_raw->val)
 
     if (!fc_solve_check_and_add_state(
             hard_thread, raw_ptr_new_state_raw, &existing_state))
     {
-#define existing_state_val (existing_state.val)
         if (HT_FIELD(hard_thread, allocated_from_list))
         {
             ptr_new_state_foo->parent = instance->list_of_vacant_states;
@@ -838,39 +773,35 @@ extern void fc_solve_sfs_check_state_end(
  * already have, then re-assign its parent to this state.
  * */
 #ifndef FCS_HARD_CODE_REPARENT_STATES_AS_FALSE
+#define ptr_state (raw_state_raw.val)
         if (STRUCT_QUERY_FLAG(instance, FCS_RUNTIME_TO_REPARENT_STATES_REAL) &&
             (kv_calc_depth(&existing_state) >
-                kv_calc_depth(raw_ptr_state_raw) + 1))
+                kv_calc_depth(&raw_state_raw) + 1))
         {
 #ifdef FCS_WITH_MOVES
             /* Make a copy of "moves" because "moves" will be destroyed */
-            existing_state_val->moves_to_parent =
+            existing_state.val->moves_to_parent =
                 fc_solve_move_stack_compact_allocate(hard_thread, moves);
 #endif
-            if (!(existing_state_val->visited & FCS_VISITED_DEAD_END))
+            if (!(existing_state.val->visited & FCS_VISITED_DEAD_END))
             {
                 if ((--(FCS_S_NUM_ACTIVE_CHILDREN(
-                        existing_state_val->parent))) == 0)
+                        existing_state.val->parent))) == 0)
                 {
-                    MARK_AS_DEAD_END(existing_state_val->parent);
+                    MARK_AS_DEAD_END(existing_state.val->parent);
                 }
                 ptr_state->num_active_children++;
             }
-            existing_state_val->parent = INFO_STATE_PTR(raw_ptr_state_raw);
+            existing_state.val->parent = INFO_STATE_PTR(&raw_state_raw);
 #ifndef FCS_WITHOUT_DEPTH_FIELD
-            existing_state_val->depth = ptr_state->depth + 1;
+            existing_state.val->depth = ptr_state->depth + 1;
 #endif
         }
 #endif
-
-        fc_solve_derived_states_list_add_state(derived_states_list,
-            FCS_STATE_kv_to_collectible(&existing_state), state_context_value);
+        return FCS_STATE_kv_to_collectible(&existing_state);
     }
     else
     {
-        fc_solve_derived_states_list_add_state(derived_states_list,
-            INFO_STATE_PTR(raw_ptr_new_state_raw), state_context_value);
+        return INFO_STATE_PTR(raw_ptr_new_state_raw);
     }
-
-    return;
 }
