@@ -17,18 +17,21 @@
 extern "C" {
 #endif
 
-#include "config.h"
 #include "instance.h"
 
 static inline void fc_solve__assign_dest_stack_and_col_ptr(
     int8_t *const positions_by_rank, const int8_t dest_stack,
     const int8_t dest_col, const fcs_card_t dest_card)
 {
+#ifdef FCS_FREECELL_ONLY
+    int8_t *ptr = &positions_by_rank[(dest_card - 4) << 1];
+#else
     int8_t *ptr = &positions_by_rank[(FCS_POS_BY_RANK_WIDTH *
                                          (fcs_card_rank(dest_card) - 1)) +
                                      (fcs_card_suit(dest_card) << 1)];
+#endif
 
-#if (!defined(HARD_CODED_NUM_DECKS) || (HARD_CODED_NUM_DECKS == 1))
+#if ((!defined(HARD_CODED_NUM_DECKS)) || (HARD_CODED_NUM_DECKS > 1))
     for (; (*ptr) != -1; ptr += (4 << 1))
     {
     }
@@ -46,11 +49,10 @@ static inline void fc_solve__calc_positions_by_rank_data(
     ,
     const fcs_bool_t is_simple_simon
 #endif
-    )
+)
 {
-#if (!(defined(HARD_CODED_NUM_STACKS) && defined(HARD_CODED_NUM_DECKS) &&      \
-         defined(FCS_DISABLE_SIMPLE_SIMON)))
-    fc_solve_instance_t *const instance = HT_INSTANCE(soft_thread->hard_thread);
+#ifndef HARD_CODED_ALL
+    var_AUTO(instance, fcs_st_instance(soft_thread));
     SET_GAME_PARAMS();
 #endif
 
@@ -59,15 +61,12 @@ static inline void fc_solve__calc_positions_by_rank_data(
 #ifndef FCS_DISABLE_SIMPLE_SIMON
     if (is_simple_simon)
     {
-#define FCS_SS_POS_BY_RANK_SIZE                                                \
-    (sizeof(fcs_pos_by_rank_t) * FCS_SS_POS_BY_RANK_LEN)
 #define FCS_POS_IDX(rank, suit) ((suit)*FCS_SS_POS_BY_RANK_WIDTH + (rank))
         fcs_pos_by_rank_t *const p_by_r =
             (fcs_pos_by_rank_t *)positions_by_rank;
         for (int ds = 0; ds < LOCAL_STACKS_NUM; ds++)
         {
-            const fcs_const_cards_column_t dest_col =
-                fcs_state_get_col(*ptr_state_key, ds);
+            const_AUTO(dest_col, fcs_state_get_col(*ptr_state_key, ds));
             const int dest_cards_num = fcs_col_len(dest_col);
 
             for (int dc = 0; dc < dest_cards_num; dc++)
@@ -103,8 +102,7 @@ static inline void fc_solve__calc_positions_by_rank_data(
 
         for (int ds = 0; ds < LOCAL_STACKS_NUM; ds++)
         {
-            const fcs_const_cards_column_t dest_col =
-                fcs_state_get_col(state_key, ds);
+            const_AUTO(dest_col, fcs_state_get_col(state_key, ds));
             int top_card_idx = fcs_col_len(dest_col);
 
             if (unlikely((top_card_idx--) == 0))
@@ -150,30 +148,28 @@ static inline const int8_t *fc_solve_calc_positions_by_rank_location(
 }
 
 static inline void add_to_move_funcs_list(
-    fc_solve_solve_for_state_move_func_t **const out_move_funcs_list,
-    size_t *const num_so_far, const size_t *const indexes,
-    const size_t count_to_add)
+    fcs_move_func **const out_move_funcs_list, size_t *const num_so_far,
+    const fcs_move_func *const indexes, const size_t count_to_add)
 {
-    const size_t num = *num_so_far;
-    fc_solve_solve_for_state_move_func_t *const move_funcs_list =
+    size_t num = *num_so_far;
+    fcs_move_func *const move_funcs_list =
         SREALLOC(*out_move_funcs_list, num + count_to_add);
-    size_t next_i = num;
-    for (size_t i = 0; i < count_to_add; i++)
+    for (size_t i = 0; i < count_to_add; ++i)
     {
-        move_funcs_list[next_i++] = fc_solve_sfs_move_funcs[indexes[i]];
+        move_funcs_list[num++].f = fc_solve_sfs_move_funcs[indexes[i].idx];
     }
 
     *out_move_funcs_list = move_funcs_list;
-    *num_so_far = next_i;
+    *num_so_far = num;
 }
 
 extern int fc_solve_sfs_check_state_begin(fc_solve_hard_thread_t *const,
     fcs_kv_state_t *const,
-    fcs_kv_state_t *const SFS__PASS_MOVE_STACK(fcs_move_stack_t *const));
+    fcs_kv_state_t SFS__PASS_MOVE_STACK(fcs_move_stack_t *const));
 
-extern void fc_solve_sfs_check_state_end(fc_solve_soft_thread_t *const,
-    fcs_kv_state_t *const, fcs_kv_state_t *const, const int,
-    fcs_move_stack_t *const, fcs_derived_states_list_t *const);
+extern fcs_collectible_state_t *fc_solve_sfs_check_state_end(
+    fc_solve_soft_thread_t *, fcs_kv_state_t,
+    fcs_kv_state_t *FCS__pass_moves(fcs_move_stack_t *));
 
 #ifdef __cplusplus
 }
