@@ -16,14 +16,13 @@
 extern "C" {
 #endif
 
-#include <stdlib.h>
 #include "rinutils.h"
 #include "fcs_user.h"
 
 static inline void fc_solve_set_weights(freecell_solver_str_t start_num,
     const freecell_solver_str_t string_end, double *const befs_weights)
 {
-    for (int i = 0; i < FCS_NUM_BEFS_WEIGHTS; i++)
+    for (int i = 0; i < FCS_NUM_BEFS_WEIGHTS; ++i)
     {
         while (*start_num == ',')
         {
@@ -43,7 +42,8 @@ static inline void fc_solve_set_weights(freecell_solver_str_t start_num,
             return;
         }
         char *end_num;
-        befs_weights[i] = strtod(start_num, &end_num);
+        const_AUTO(val, strtod(start_num, &end_num));
+        befs_weights[i] = max(val, 0.0);
         start_num = end_num;
     }
 }
