@@ -7,13 +7,8 @@
  *
  * Copyright (c) 2000 Shlomi Fish
  */
-/*
- * simpsim.c - a module that contains Simple Simon Moves.
- */
+// simpsim.c - a module that contains Simple Simon moves.
 
-#include "dll_thunk.h"
-#include <stdio.h>
-#include "config.h"
 #include "simpsim.h"
 #include "scans.h"
 
@@ -21,13 +16,9 @@
  * FCS_DISABLE_SIMPLE_SIMON.
  * */
 #ifdef FCS_DISABLE_SIMPLE_SIMON
-
 char fc_solve_simple_simon_nothing;
-
 #else
 
-#include "instance.h"
-
 #include "meta_move_funcs_helpers.h"
 
 static inline fcs_bool_t fcs_is_ss_false_parent(
@@ -52,7 +43,7 @@ static inline fcs_bool_t fcs_is_ss_true_parent(
 #define STACK_SOURCE_LOOP_START(min_num_cards)                                 \
     for (int stack_idx = 0; stack_idx < LOCAL_STACKS_NUM; stack_idx++)         \
     {                                                                          \
-        const fcs_cards_column_t col = fcs_state_get_col(state, stack_idx);    \
+        const_AUTO(col, fcs_state_get_col(state, stack_idx));                  \
         const int cards_num = fcs_col_len(col);                                \
         if (cards_num < min_num_cards)                                         \
         {                                                                      \
@@ -69,7 +60,7 @@ static inline fcs_bool_t fcs_is_ss_true_parent(
             continue;                                                          \
         }                                                                      \
                                                                                \
-        const fcs_cards_column_t dest_col = fcs_state_get_col(state, ds);      \
+        const_AUTO(dest_col, fcs_state_get_col(state, ds));                    \
         const int dest_cards_num = fcs_col_len(dest_col);                      \
                                                                                \
         if (dest_cards_num < min_num_cards)                                    \
@@ -100,10 +91,6 @@ static inline fcs_bool_t fcs_is_ss_true_parent(
 
 #define STACKS_MAP_LEN MAX_NUM_STACKS
 
-#ifndef FCS_WITH_MOVES
-#define moves NULL
-#endif
-
 static inline void init_stacks_map(
     fcs_bool_t *const stacks_map, const int stack_idx, const int ds)
 {
@@ -154,20 +141,18 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_founds)
     if (a > FCS_MAX_RANK)
     {
         /* We can move this sequence up there */
-
         sfs_check_state_begin();
 
         my_copy_stack(stack_idx);
 
-        fcs_cards_column_t new_src_col =
-            fcs_state_get_col(new_state, stack_idx);
+        var_AUTO(new_src_col, fcs_state_get_col(new_state_key, stack_idx));
         for (a = 0; a < FCS_MAX_RANK; a++)
         {
             fcs_col_pop_top(new_src_col);
         }
 
         const int suit = fcs_card_suit(card);
-        fcs_set_foundation(new_state, suit, FCS_MAX_RANK);
+        fcs_set_foundation(new_state_key, suit, FCS_MAX_RANK);
 
         fcs_move_stack_non_seq_push(
             moves, FCS_MOVE_TYPE_SEQ_TO_FOUNDATION, stack_idx, suit);
@@ -175,12 +160,10 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_founds)
         sfs_check_state_end();
     }
     STACK_SOURCE_LOOP_END()
-
-    return;
 }
 
 #define LOOK_FOR_TRUE_PARENT_with_ds_dc__START(card)                           \
-    if (fcs_card_rank(card) < 13)                                              \
+    if (!fcs_card_is_king(card))                                               \
     {                                                                          \
         const fcs_pos_by_rank_t pos = positions_by_rank[FCS_POS_IDX(           \
             fcs_card_rank(card) + 1, fcs_card_suit(card))];                    \
@@ -189,7 +172,7 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_founds)
         if (ds != stack_idx)                                                   \
         {                                                                      \
             const int dc = pos.height;                                         \
-            const fcs_cards_column_t dest_col = fcs_state_get_col(state, ds);  \
+            const_AUTO(dest_col, fcs_state_get_col(state, ds));                \
             const int dest_cards_num = fcs_col_len(dest_col);
 
 #define LOOK_FOR_TRUE_PARENT_with_ds_dc__END()                                 \
@@ -199,7 +182,6 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_founds)
 #define LOOK_FOR_TRUE_PARENT_AT_TOP__START(card)                               \
     LOOK_FOR_TRUE_PARENT_with_ds_dc__START(card) if (dc == dest_cards_num - 1) \
     {
-
 #define LOOK_FOR_TRUE_PARENT_AT_TOP__END()                                     \
     }                                                                          \
     LOOK_FOR_TRUE_PARENT_with_ds_dc__END()
@@ -247,10 +229,7 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_true_parent)
         /* We can do it - so let's move */
 
         sfs_check_state_begin();
-
-        my_copy_stack(stack_idx);
-        my_copy_stack(ds);
-
+        copy_two_stacks(stack_idx, ds);
         fcs_move_sequence(ds, stack_idx, cards_num - h - 1);
         sfs_check_state_end();
         LOOK_FOR_TRUE_PARENT_AT_TOP__END()
@@ -269,7 +248,7 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_true_parent)
         }
         if (!fcs_is_ss_suit_true(card, prev_card))
         {
-            num_true_seqs++;
+            ++num_true_seqs;
             /* We can no longer perform the move so go to the next
              * stack. */
             if (calc_max_simple_simon_seq_move(num_vacant_stacks) <
@@ -303,7 +282,7 @@ static inline int get_seq_h(
 
         if (!fcs_is_ss_suit_true(next_card, card))
         {
-            num_true_seqs++;
+            ++num_true_seqs;
         }
 
         card = next_card;
@@ -360,18 +339,12 @@ DECLARE_MOVE_FUNCTION(
     /* We can do it - so let's move */
 
     sfs_check_state_begin();
-
-    my_copy_stack(stack_idx);
-    my_copy_stack(ds);
-
+    copy_two_stacks(stack_idx, ds);
     fcs_move_sequence(ds, stack_idx, cards_num - h);
-
     sfs_check_state_end();
     STACK_DEST_LOOP_END()
 #undef h
     STACK_SOURCE_LOOP_END()
-
-    return;
 }
 
 /*
@@ -416,7 +389,7 @@ static inline void populate_seq_points(const fcs_cards_column_t dest_col,
 }
 
 static inline fcs_bool_t generic_false_seq_index_loop(const int stacks_num,
-    fcs_kv_state_t *const raw_ptr_state_raw, const int num_vacant_stacks,
+    fcs_kv_state_t raw_state_raw, const int num_vacant_stacks,
     const fcs_cards_column_t col, sequences_analysis_t *const seqs,
     const int stack_idx, const int ds, const fcs_bool_t behavior_flag,
     const fcs_bool_t should_src_col, const fcs_card_t src_card,
@@ -441,8 +414,9 @@ static inline fcs_bool_t generic_false_seq_index_loop(const int stacks_num,
 
         /* Find a suitable place to put it */
         const fcs_card_t the_card =
-            is_ultimate_iter ? src_card : fcs_col_get_card(col,
-                                              seqs->seq_points[false_seq_idx]);
+            is_ultimate_iter
+                ? src_card
+                : fcs_col_get_card(col, seqs->seq_points[false_seq_idx]);
 
         const size_t the_num_true_seqs =
             is_ultimate_iter ? num_src_junk_true_seqs
@@ -451,7 +425,7 @@ static inline fcs_bool_t generic_false_seq_index_loop(const int stacks_num,
         /* Let's try to find a suitable parent on top one of the stacks */
         int clear_junk_dest_stack;
         for (clear_junk_dest_stack = 0; clear_junk_dest_stack < stacks_num;
-             clear_junk_dest_stack++)
+             ++clear_junk_dest_stack)
         {
             const fcs_const_cards_column_t clear_junk_dest_col =
                 fcs_state_get_col(state, clear_junk_dest_stack);
@@ -470,38 +444,33 @@ static inline fcs_bool_t generic_false_seq_index_loop(const int stacks_num,
                 if (calc_max_simple_simon_seq_move(after_junk_num_freestacks) >=
                     the_num_true_seqs)
                 {
-                    stacks_map[clear_junk_dest_stack] = TRUE;
-                    break;
+                    goto found;
                 }
             }
         }
 
-        if (clear_junk_dest_stack == stacks_num)
+        /* Check if there is a vacant stack */
+        if (behavior_flag ||
+            (!((num_vacant_stacks > 0) &&
+                (calc_max_simple_simon_seq_move(
+                     after_junk_num_freestacks - 1) >= the_num_true_seqs))))
         {
-            /* Check if there is a vacant stack */
-            if (behavior_flag ||
-                (!((num_vacant_stacks > 0) &&
-                    (calc_max_simple_simon_seq_move(
-                         after_junk_num_freestacks - 1) >= the_num_true_seqs))))
+            break;
+        }
+        --after_junk_num_freestacks;
+        /* Find an empty stack and designate it as the destination for the
+         * junk */
+        for (clear_junk_dest_stack = 0;; ++clear_junk_dest_stack)
+        {
+            if (fcs_state_col_is_empty(state, clear_junk_dest_stack) &&
+                (!stacks_map[clear_junk_dest_stack]))
             {
                 break;
             }
-            /* Find an empty stack and designate it as the destination for the
-             * junk */
-            for (clear_junk_dest_stack = 0; clear_junk_dest_stack < stacks_num;
-                 clear_junk_dest_stack++)
-            {
-                if ((fcs_col_len(fcs_state_get_col(
-                         state, clear_junk_dest_stack)) == 0) &&
-                    (!stacks_map[clear_junk_dest_stack]))
-                {
-                    stacks_map[clear_junk_dest_stack] = TRUE;
-                    break;
-                }
-            }
-            after_junk_num_freestacks--;
         }
 
+    found:
+        stacks_map[clear_junk_dest_stack] = TRUE;
         seqs->junk_move_to_stacks[false_seq_idx] = clear_junk_dest_stack;
     }
 
@@ -510,19 +479,19 @@ static inline fcs_bool_t generic_false_seq_index_loop(const int stacks_num,
 }
 
 static inline fcs_bool_t false_seq_index_loop(const int stacks_num,
-    fcs_kv_state_t *const raw_ptr_state_raw, const int num_vacant_stacks,
+    fcs_kv_state_t raw_state_raw, const int num_vacant_stacks,
     const fcs_cards_column_t col, sequences_analysis_t *const seqs,
     const int stack_idx, const int ds, const fcs_bool_t behavior_flag)
 {
-    return generic_false_seq_index_loop(stacks_num, raw_ptr_state_raw,
+    return generic_false_seq_index_loop(stacks_num, raw_state_raw,
         num_vacant_stacks, col, seqs, stack_idx, ds, behavior_flag,
         /* Params that should be ignored in this case. */
         FALSE, fc_solve_empty_card, 0);
 }
 
 #define IS_false_seq_index_loop(col, behavior_flag, stack_idx, ds)             \
-    false_seq_index_loop(LOCAL_STACKS_NUM, raw_ptr_state_raw,                  \
-        num_vacant_stacks, col, &seqs, stack_idx, ds, behavior_flag)
+    false_seq_index_loop(LOCAL_STACKS_NUM, raw_state_raw, num_vacant_stacks,   \
+        col, &seqs, stack_idx, ds, behavior_flag)
 
 #define POPULATE_AND_CHECK_IF_FALSE_SEQ(                                       \
     col, height, stack_idx, ds, behavior_flag)                                 \
@@ -628,11 +597,8 @@ DECLARE_MOVE_FUNCTION(
                  * */
 
                 sfs_check_state_begin();
-
-                my_copy_stack(stack_idx);
-                my_copy_stack(ds);
-
-                /* Move the junk cards to their place */
+                copy_two_stacks(stack_idx, ds);
+                // Move the junk cards to their place
                 move_sequences_analysis_seqs_loop(
                     &pass_new_state SFS__PASS_MOVE_STACK(moves), &seqs, ds,
                     dest_cards_num PASS_IND_BUF_T(indirect_stacks_buffer));
@@ -651,12 +617,10 @@ DECLARE_MOVE_FUNCTION(
         }
         if (should_increment_num_true_seqs)
         {
-            num_true_seqs++;
+            ++num_true_seqs;
         }
     }
     STACK_SOURCE_LOOP_END()
-
-    return;
 }
 
 DECLARE_MOVE_FUNCTION(
@@ -692,13 +656,9 @@ DECLARE_MOVE_FUNCTION(
                         (calc_max_simple_simon_seq_move(
                              seqs.after_junk_num_freestacks) > num_true_seqs)))
                 {
-
-                    sfs_check_state_begin();
-
-                    my_copy_stack(stack_idx);
-                    my_copy_stack(ds);
-
                     /* Let's boogie - we can move everything */
+                    sfs_check_state_begin();
+                    copy_two_stacks(stack_idx, ds);
 
                     /* Move the junk cards to their place */
                     move_sequences_analysis_seqs_loop(
@@ -716,14 +676,12 @@ DECLARE_MOVE_FUNCTION(
             }
             if (!fcs_is_ss_suit_true(card, above_card))
             {
-                num_true_seqs++;
+                ++num_true_seqs;
             }
             card = above_card;
         }
     }
     STACK_SOURCE_LOOP_END()
-
-    return;
 }
 
 /*
@@ -809,7 +767,7 @@ DECLARE_MOVE_FUNCTION(
         }
         if (!fcs_is_ss_suit_true(next_card, card))
         {
-            num_true_seqs++;
+            ++num_true_seqs;
         }
         card = next_card;
     }
@@ -825,7 +783,7 @@ DECLARE_MOVE_FUNCTION(
 
         populate_seq_points(dest_col, dc, &seqs);
 
-        if (generic_false_seq_index_loop(LOCAL_STACKS_NUM, raw_ptr_state_raw,
+        if (generic_false_seq_index_loop(LOCAL_STACKS_NUM, raw_state_raw,
                 num_vacant_stacks, dest_col, &seqs, stack_idx, ds, FALSE, TRUE,
                 fcs_col_get_card(col, after_end_of_junk),
                 num_src_junk_true_seqs) &&
@@ -833,12 +791,8 @@ DECLARE_MOVE_FUNCTION(
                 num_true_seqs))
         {
             /* We can do it - so let's move everything */
-
             sfs_check_state_begin();
-
-            my_copy_stack(stack_idx);
-            my_copy_stack(ds);
-
+            copy_two_stacks(stack_idx, ds);
             /* Move the junk cards to their place */
 
             for (int seq_index = 0;
@@ -847,12 +801,8 @@ DECLARE_MOVE_FUNCTION(
                 const s_e_src_t s_e = calc_start_end_src_stack(seq_index, &seqs,
                     after_end_of_junk, cards_num, stack_idx, ds,
                     dest_cards_num);
-
-                my_copy_stack(s_e.src_stack);
-
                 const_AUTO(dest_col_i, seqs.junk_move_to_stacks[seq_index]);
-                my_copy_stack(dest_col_i);
-
+                copy_two_stacks(s_e.src_stack, dest_col_i);
                 fcs_move_sequence(dest_col_i, s_e.src_stack, s_e.cards_num);
             }
 
@@ -872,9 +822,9 @@ typedef fcs_pos_by_rank_t ds_dc_t;
 static inline void sort_ds_dcs(ds_dc_t *const ds_dcs, const int len)
 {
 #define start ds_dcs
-    ds_dc_t *const limit = start + len;
+    ds_dc_t *const end = start + len;
 
-    for (ds_dc_t *b = start + 1; b < limit; b++)
+    for (ds_dc_t *b = start + 1; b < end; b++)
     {
         for (ds_dc_t *c = b;
              (c > start) &&
@@ -938,7 +888,7 @@ DECLARE_MOVE_FUNCTION(
 #define h 0
     const fcs_card_t card = fcs_col_get_card(col, h);
 
-    if (fcs_card_rank(card) == 13)
+    if (fcs_card_is_king(card))
     {
         continue;
     }
@@ -964,7 +914,7 @@ DECLARE_MOVE_FUNCTION(
     {
         const size_t ds = ds_dcs[i].col;
         const size_t dc = ds_dcs[i].height;
-        const fcs_cards_column_t dest_col = fcs_state_get_col(state, ds);
+        const_AUTO(dest_col, fcs_state_get_col(state, ds));
         const int dest_cards_num = fcs_col_len(dest_col);
 
         /* This is a suitable parent - let's check if there's a sequence above
@@ -974,12 +924,8 @@ DECLARE_MOVE_FUNCTION(
         if (POPULATE_AND_CHECK_IF_FALSE_SEQ(dest_col, dc, stack_idx, ds, TRUE))
         {
             /* We can do it - so let's move */
-
             sfs_check_state_begin();
-
-            my_copy_stack(stack_idx);
-            my_copy_stack(ds);
-
+            copy_two_stacks(stack_idx, ds);
             /* Move the junk cards to their place */
             move_sequences_analysis_seqs_loop(
                 &pass_new_state SFS__PASS_MOVE_STACK(moves), &seqs, ds,
@@ -992,8 +938,6 @@ DECLARE_MOVE_FUNCTION(
         }
     }
     STACK_SOURCE_LOOP_END()
-
-    return;
 }
 
 DECLARE_MOVE_FUNCTION(
@@ -1022,16 +966,9 @@ DECLARE_MOVE_FUNCTION(
             {
                 continue;
             }
-/* We have a matching parent and child cards */
-#if 0
-                printf("Stack %i, Parent %i, Child %i\n", stack_idx, parent_card_height, child_card_height);
-                fflush(stdout);
-#endif
-
-            /*
-             * Now let's try to find stacks to place the cards above
-             * the child card.
-             * */
+            // We have a matching parent and child cards
+            // Now let's try to find stacks to place the cards above the child
+            // card.
 
             int end_of_child_seq = child_card_height;
             size_t child_num_true_seqs = 1;
@@ -1043,7 +980,7 @@ DECLARE_MOVE_FUNCTION(
                 child_num_true_seqs += (!fcs_is_ss_true_parent(
                     fcs_col_get_card(col, end_of_child_seq),
                     fcs_col_get_card(col, end_of_child_seq + 1)));
-                end_of_child_seq++;
+                ++end_of_child_seq;
             }
 
             sequences_analysis_t seqs;
@@ -1070,7 +1007,6 @@ DECLARE_MOVE_FUNCTION(
                 continue;
             }
             /* We can do it - so let's move everything */
-
             sfs_check_state_begin();
 
             /* Move the junk cards to their place */
@@ -1079,21 +1015,15 @@ DECLARE_MOVE_FUNCTION(
             move_sequences_analysis_seqs_loop(
                 &pass_new_state SFS__PASS_MOVE_STACK(moves), &seqs, stack_idx,
                 cards_num PASS_IND_BUF_T(indirect_stacks_buffer));
-
-            {
-                const_AUTO(
-                    src_col_i, seqs.junk_move_to_stacks[child_seq_index]);
-                my_copy_stack(src_col_i);
-                fcs_move_sequence(stack_idx, src_col_i,
-                    end_of_child_seq - child_card_height + 1);
-            }
+            const_AUTO(src_col_i, seqs.junk_move_to_stacks[child_seq_index]);
+            my_copy_stack(src_col_i);
+            fcs_move_sequence(
+                stack_idx, src_col_i, end_of_child_seq - child_card_height + 1);
 
             sfs_check_state_end();
         }
     }
     STACK_SOURCE_LOOP_END()
-
-    return;
 }
 
 DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_false_parent)
@@ -1136,19 +1066,13 @@ DECLARE_MOVE_FUNCTION(fc_solve_sfs_simple_simon_move_sequence_to_false_parent)
 
     /* We can do it - so let's perform the move */
     sfs_check_state_begin();
-
-    my_copy_stack(stack_idx);
-    my_copy_stack(ds);
-
+    copy_two_stacks(stack_idx, ds);
     fcs_move_sequence(ds, stack_idx, cards_num - h);
     sfs_check_state_end();
     STACK_DEST_LOOP_END()
     STACK_SOURCE_LOOP_END()
-
-    return;
 }
 
 #undef state
-#undef new_state
 
 #endif /* #ifdef FCS_DISABLE_SIMPLE_SIMON */
