@@ -13,10 +13,6 @@
  * line arguments from files.
  */
 
-#include "dll_thunk.h"
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include "split_cmd_line.h"
 
 typedef struct
@@ -53,16 +49,12 @@ static inline void add_to_last_arg(
         manager->last_arg_end += new_last_arg - manager->last_arg + 1024;
         manager->last_arg = new_last_arg;
     }
-
-    return;
 }
 
 static inline void push_args_last_arg(args_man_wrapper_t *const manager)
 {
     const int len = manager->last_arg_ptr - manager->last_arg;
-
     char *const new_arg = SMALLOC(new_arg, len + 1);
-
     strncpy(new_arg, manager->last_arg, len);
     new_arg[len] = '\0';
 
@@ -76,8 +68,6 @@ static inline void push_args_last_arg(args_man_wrapper_t *const manager)
 
     /* Reset last_arg_ptr so we will have an entirely new argument */
     manager->last_arg_ptr = manager->last_arg;
-
-    return;
 }
 
 static inline fcs_bool_t is_whitespace(const char c)
@@ -97,7 +87,6 @@ args_man_t fc_solve_args_man_chop(const char *const string)
     const char *s = string;
 
     args_man_wrapper_t manager = {.args_man = fc_solve_args_man_alloc()};
-
     manager.last_arg_ptr = manager.last_arg = SMALLOC(manager.last_arg, 1024);
     manager.last_arg_end = manager.last_arg + 1023;
 
@@ -123,121 +112,117 @@ args_man_t fc_solve_args_man_chop(const char *const string)
             continue;
         }
 
+        fcs_bool_t should_still_loop = TRUE;
+        fcs_bool_t in_arg = FALSE;
+        while (should_still_loop)
         {
-            fcs_bool_t should_still_loop = TRUE;
-            fcs_bool_t in_arg = FALSE;
-            while (should_still_loop)
+            switch (*s)
             {
-                switch (*s)
-                {
-                case ' ':
-                case '\t':
-                case '\n':
-                case '\0':
-                case '\r':
+            case ' ':
+            case '\t':
+            case '\n':
+            case '\0':
+            case '\r':
 
-                    push_next_arg_flag = TRUE;
-                    should_still_loop = FALSE;
+                push_next_arg_flag = TRUE;
+                should_still_loop = FALSE;
 
-                    break;
-
-                case '\\':
-                {
-                    const_AUTO(next_char, *(++s));
-                    s++;
-                    switch (next_char)
-                    {
-                    case '\0':
-                        s--;
-                        push_next_arg_flag = TRUE;
-                        should_still_loop = FALSE;
-                        break;
-                    case '\r':
-                    case '\n':
-                        /* Skip to the next line. */
-                        if (!in_arg)
-                        {
-                            should_still_loop = FALSE;
-                        }
-                        break;
-                    default:
-                        add_to_last_arg(&manager, next_char);
-                        break;
-                    }
-                }
                 break;
 
-                case '\"':
-
-                    in_arg = TRUE;
-                    while (TRUE)
-                    {
-                        const_AUTO(c, *(++s));
-                        switch (c)
-                        {
-                        case '\"':
-                            ++s;
-                        case '\0':
-                            goto after_quote;
-
-                        case '\\':
-                        {
-                            const_AUTO(next_char, *(++s));
-
-                            switch (next_char)
-                            {
-                            case '\0':
-                                push_args_last_arg(&manager);
-                                goto END_OF_LOOP;
-
-                            case '\n':
-                            case '\r':
-                                /* Do nothing */
-                                break;
-
-                            case '\\':
-                            case '\"':
-                                add_to_last_arg(&manager, next_char);
-                                break;
-
-                            default:
-                                add_to_last_arg(&manager, '\\');
-                                add_to_last_arg(&manager, next_char);
-                                break;
-                            }
-                        }
-                        break;
-                        default:
-                        {
-                            add_to_last_arg(&manager, c);
-                        }
-                        break;
-                        }
-                    }
-                after_quote:
-                    break;
-
-                case '#':
-
-                    in_arg = FALSE;
-                    /* Skip to the next line */
-                    while ((*s != '\0') && (*s != '\n'))
-                    {
-                        s++;
-                    }
+            case '\\':
+            {
+                const_AUTO(next_char, *(++s));
+                s++;
+                switch (next_char)
+                {
+                case '\0':
+                    s--;
                     push_next_arg_flag = TRUE;
                     should_still_loop = FALSE;
                     break;
-
+                case '\r':
+                case '\n':
+                    /* Skip to the next line. */
+                    if (!in_arg)
+                    {
+                        should_still_loop = FALSE;
+                    }
+                    break;
                 default:
-
-                    in_arg = TRUE;
-                    add_to_last_arg(&manager, *s);
-                    s++;
-
+                    add_to_last_arg(&manager, next_char);
                     break;
                 }
             }
+            break;
+
+            case '\"':
+
+                in_arg = TRUE;
+                while (TRUE)
+                {
+                    const_AUTO(c, *(++s));
+                    switch (c)
+                    {
+                    case '\"':
+                        ++s;
+                    case '\0':
+                        goto after_quote;
+
+                    case '\\':
+                    {
+                        const_AUTO(next_char, *(++s));
+
+                        switch (next_char)
+                        {
+                        case '\0':
+                            push_args_last_arg(&manager);
+                            goto END_OF_LOOP;
+
+                        case '\n':
+                        case '\r':
+                            /* Do nothing */
+                            break;
+
+                        case '\\':
+                        case '\"':
+                            add_to_last_arg(&manager, next_char);
+                            break;
+
+                        default:
+                            add_to_last_arg(&manager, '\\');
+                            add_to_last_arg(&manager, next_char);
+                            break;
+                        }
+                    }
+                    break;
+                    default:
+                    {
+                        add_to_last_arg(&manager, c);
+                    }
+                    break;
+                    }
+                }
+            after_quote:
+                break;
+
+            case '#':
+
+                in_arg = FALSE;
+                /* Skip to the next line */
+                while ((*s != '\0') && (*s != '\n'))
+                {
+                    s++;
+                }
+                push_next_arg_flag = TRUE;
+                should_still_loop = FALSE;
+                break;
+
+            default:
+                in_arg = TRUE;
+                add_to_last_arg(&manager, *s);
+                s++;
+                break;
+            }
         }
 
         if (push_next_arg_flag)
