@@ -59,7 +59,7 @@ static inline int FccEntryPointNode_compare(
 }
 
 RB_HEAD(FccEntryPointList, FccEntryPointNode);
-const FccEntryPointList FccEntryPointList_init =
+static const FccEntryPointList FccEntryPointList_init =
     RB_INITIALIZER(&(instance->fcc_entry_points));
 
 typedef struct
@@ -101,14 +101,13 @@ typedef struct
 
 #include "dbm_procs.h"
 #include "fcs_base64.h"
-#define __unused GCC_UNUSED
 RB_GENERATE_STATIC(
     FccEntryPointList, FccEntryPointNode, entry_, FccEntryPointNode_compare);
 
 static inline void instance_init(fcs_dbm_solver_instance_t *const instance,
     const fcs_dbm_common_input_t *const inp,
     fcs_which_moves_bitmask_t *fingerprint_which_irreversible_moves_bitmask,
-    FILE *out_fh)
+    FILE *const out_fh)
 {
     fc_solve_meta_compact_allocator_init(&(instance->fcc_meta_alloc));
     instance->fcc_entry_points = FccEntryPointList_init;
@@ -159,20 +158,16 @@ static inline void instance_init(fcs_dbm_solver_instance_t *const instance,
     }
 }
 
-static inline void instance_destroy(fcs_dbm_solver_instance_t *instance)
+static inline void instance_destroy(fcs_dbm_solver_instance_t *const instance)
 {
-    fcs_dbm_collection_by_depth_t *coll;
-
     fc_solve_compact_allocator_finish(&(instance->fcc_entry_points_allocator));
     fc_solve_meta_compact_allocator_finish(&(instance->fcc_meta_alloc));
-    {
-        coll = &(instance->coll);
-        fcs_depth_multi_queue__destroy(&(coll->depth_queue));
+    fcs_dbm_collection_by_depth_t *coll = &(instance->coll);
+    fcs_depth_multi_queue__destroy(&(coll->depth_queue));
 #ifndef FCS_DBM_USE_OFFLOADING_QUEUE
-        fc_solve_meta_compact_allocator_finish(&(coll->queue_meta_alloc));
+    fc_solve_meta_compact_allocator_finish(&(coll->queue_meta_alloc));
 #endif
-        DESTROY_CACHE(coll);
-    }
+    DESTROY_CACHE(coll);
     fcs_lock_destroy(&instance->common.storage_lock);
     fcs_lock_destroy(&instance->global_lock);
     fcs_lock_destroy(&instance->fcc_entry_points_lock);
@@ -188,16 +183,11 @@ struct fcs_dbm_solver_thread_struct
     int state_depth;
 };
 
-typedef struct
-{
-    fcs_dbm_solver_thread_t *thread;
-} thread_arg_t;
-
 static void *instance_run_solver_thread(void *const void_arg)
 {
     fcs_dbm_queue_item_t physical_item;
     fcs_dbm_record_t *token = NULL;
-    fcs_derived_state_t *derived_list, *derived_list_recycle_bin, *derived_iter;
+    fcs_derived_state_t *derived_list = NULL, *derived_list_recycle_bin = NULL;
     fcs_compact_allocator_t derived_list_allocator;
     fcs_state_keyval_pair_t state;
     char *base64_encoding_buffer = NULL;
@@ -210,18 +200,15 @@ static void *instance_run_solver_thread(void *const void_arg)
     fc_solve_init_locs(&locs);
 #endif
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
-
     const_AUTO(thread, ((thread_arg_t *)void_arg)->thread);
     const_SLOT(instance, thread);
     const_AUTO(delta_stater, &(thread->delta_stater));
     const_AUTO(local_variant, instance->common.variant);
 
     fcs_dbm_queue_item_t *item = NULL, *prev_item = NULL;
-    int queue_num_extracted_and_processed = 0;
+    long queue_num_extracted_and_processed = 0;
     fc_solve_compact_allocator_init(
         &(derived_list_allocator), &(thread->thread_meta_alloc));
-    derived_list_recycle_bin = NULL;
-    derived_list = NULL;
     FILE *const out_fh = instance->common.out_fh;
     TRACE("%s\n", "instance_run_solver_thread start");
     const_AUTO(coll, &(instance->coll));
@@ -235,7 +222,7 @@ static void *instance_run_solver_thread(void *const void_arg)
 
         if (prev_item)
         {
-            instance->common.queue_num_extracted_and_processed--;
+            --instance->common.queue_num_extracted_and_processed;
         }
 
         if (instance->common.should_terminate == DONT_TERMINATE)
@@ -268,7 +255,9 @@ static void *instance_run_solver_thread(void *const void_arg)
         {
             /* Sleep until more items become available in the
              * queue. */
-            usleep(5000);
+            const struct timespec ts = {.tv_sec = 0, .tv_nsec = 5000000};
+            struct timespec no_use;
+            nanosleep(&ts, &no_use);
         }
         else
         {
@@ -373,7 +362,7 @@ static void *instance_run_solver_thread(void *const void_arg)
             }
 
             /* Encode all the states. */
-            for (derived_iter = derived_list; derived_iter;
+            for (var_AUTO(derived_iter, derived_list); derived_iter;
                  derived_iter = derived_iter->next)
             {
                 fcs_init_and_encode_state(delta_stater, local_variant,
@@ -382,22 +371,14 @@ static void *instance_run_solver_thread(void *const void_arg)
 
             instance_check_multiple_keys(thread, instance, &(coll->cache_store),
                 &(coll->queue_meta_alloc), &derived_list, 1
-#ifdef FCS_DBM_CACHE_ONLY
+#ifndef FCS_DBM_WITHOUT_CACHES
                 ,
                 item->moves_to_key
 #endif
-                );
+            );
 
-            /* Now recycle the derived_list */
-            while (derived_list)
-            {
-#define derived_list_next derived_iter
-                derived_list_next = derived_list->next;
-                derived_list->next = derived_list_recycle_bin;
-                derived_list_recycle_bin = derived_list;
-                derived_list = derived_list_next;
-#undef derived_list_next
-            }
+            fcs_derived_state_list__recycle(
+                &derived_list_recycle_bin, &derived_list);
             /* End handle item. */
         }
         /* End of main thread loop */
@@ -405,11 +386,8 @@ static void *instance_run_solver_thread(void *const void_arg)
     }
 
     free(base64_encoding_buffer);
-
     fc_solve_compact_allocator_finish(&(derived_list_allocator));
-
     TRACE("%s\n", "instance_run_solver_thread end");
-
     return NULL;
 }
 
@@ -431,11 +409,11 @@ static inline void instance_check_key(fcs_dbm_solver_thread_t *const thread,
     fcs_encoded_state_buffer_t *const key, fcs_dbm_record_t *const parent,
     const unsigned char move GCC_UNUSED,
     const fcs_which_moves_bitmask_t *const which_irreversible_moves_bitmask
-#ifdef FCS_DBM_CACHE_ONLY
+#ifndef FCS_DBM_WITHOUT_CACHES
     ,
     const fcs_fcc_move_t *moves_to_parent
 #endif
-    )
+)
 {
 #ifdef DEBUG_OUT
     fcs_state_locs_struct_t locs;
@@ -447,8 +425,8 @@ static inline void instance_check_key(fcs_dbm_solver_thread_t *const thread,
     if ((token = cache_store__has_key(&coll->cache_store, key, parent)))
     {
 #ifndef FCS_DBM_WITHOUT_CACHES
-        fcs_cache_key_info_t *cache_key = cache_store__insert_key(
-            &(coll->cache_store.pre_cache), key, parent, moves_to_parent, move);
+        cache_store__insert_key(
+            &(coll->cache_store), key, parent, moves_to_parent, move);
 #endif
 
         if (key_depth == instance->curr_depth)
@@ -456,17 +434,12 @@ static inline void instance_check_key(fcs_dbm_solver_thread_t *const thread,
             /* Now insert it into the queue. */
             fcs_lock_lock(&instance->global_lock);
 
-            fcs_depth_multi_queue__insert(
-                &(coll->depth_queue), thread->state_depth + 1,
-#ifdef FCS_DBM_WITHOUT_CACHES
-                (const fcs_offloading_queue_item_t *)(&token)
-#else
-                key
-#endif
-                    );
+            fcs_depth_multi_queue__insert(&(coll->depth_queue),
+                thread->state_depth + 1,
+                (const fcs_offloading_queue_item_t *)(&token));
 
-            instance->common.count_of_items_in_queue++;
-            instance->common.num_states_in_collection++;
+            ++instance->common.count_of_items_in_queue;
+            ++instance->common.num_states_in_collection;
 
             instance_debug_out_state(instance, &(token->key));
 
@@ -569,7 +542,7 @@ static inline void instance_check_key(fcs_dbm_solver_thread_t *const thread,
                 DECLARE_IND_BUF_T(indirect_stacks_buffer)
                 fc_solve_delta_stater_decode_into_state(&(thread->delta_stater),
                     key->s, &state, indirect_stacks_buffer);
-                char state_str[2000];
+                fcs_render_state_str_t state_str;
                 FCS__RENDER_STATE(state_str, &(state.s), &locs);
                 fprintf(stderr, "Check Key: <<<\n%s\n>>>\n\n[%s %s %ld %s]\n\n",
                     state_str, fingerprint_base64, state_base64,
@@ -578,17 +551,15 @@ static inline void instance_check_key(fcs_dbm_solver_thread_t *const thread,
             }
 #endif
             fflush(instance->fcc_exit_points_out_fh);
-
             fcs_lock_unlock(&instance->fcc_exit_points_output_lock);
-
             free(trace);
         }
     }
 }
 
-static void instance_run_all_threads(fcs_dbm_solver_instance_t *instance,
-    fcs_state_keyval_pair_t *init_state, FccEntryPointNode *key_ptr,
-    size_t num_threads)
+static void instance_run_all_threads(fcs_dbm_solver_instance_t *const instance,
+    fcs_state_keyval_pair_t *const init_state, FccEntryPointNode *key_ptr,
+    const size_t num_threads)
 {
     const_AUTO(threads,
         dbm__calc_threads(instance, init_state, num_threads, init_thread));
@@ -600,14 +571,12 @@ static void instance_run_all_threads(fcs_dbm_solver_instance_t *instance,
     instance->start_key_moves_count = (key_ptr->kv.val.depth);
 #endif
 
+    dbm__spawn_threads(instance, num_threads, threads);
+    if (!instance->common.queue_solution_was_found)
     {
-        dbm__spawn_threads(instance, num_threads, threads);
-        if (!instance->common.queue_solution_was_found)
-        {
-            mark_and_sweep_old_states(instance,
-                fc_solve_dbm_store_get_dict(instance->coll.cache_store.store),
-                instance->curr_depth);
-        }
+        mark_and_sweep_old_states(instance,
+            fc_solve_dbm_store_get_dict(instance->coll.cache_store.store),
+            instance->curr_depth);
     }
     dbm__free_threads(instance, num_threads, threads, free_thread);
 }
@@ -701,61 +670,59 @@ int main(int argc, char *argv[])
         ,
         CALC_SEQUENCES_ARE_BUILT_BY()
 #endif
-            );
+    );
 
+    fcs_dbm_solver_instance_t instance;
+    FccEntryPointNode *key_ptr = NULL;
+    fcs_encoded_state_buffer_t parent_state_enc;
+
+    instance_init(
+        &instance, &inp, &fingerprint_which_irreversible_moves_bitmask, stdout);
+
+    FILE *fingerprint_fh = fopen(fingerprint_input_location_path, "rt");
+    if (!fingerprint_fh)
     {
-        fcs_dbm_solver_instance_t instance;
-        FccEntryPointNode *key_ptr = NULL;
-        fcs_encoded_state_buffer_t parent_state_enc;
-
-        instance_init(&instance, &inp,
-            &fingerprint_which_irreversible_moves_bitmask, stdout);
-
-        FILE *fingerprint_fh = fopen(fingerprint_input_location_path, "rt");
-
-        if (!fingerprint_fh)
-        {
-            fc_solve_err("Cannot open '%s' for reading. Exiting.",
-                fingerprint_input_location_path);
-        }
+        fc_solve_err("Cannot open '%s' for reading. Exiting.",
+            fingerprint_input_location_path);
+    }
 
 #ifdef HAVE_GETLINE
-        instance.fingerprint_line = NULL;
-        instance.fingerprint_line_size = 0;
+    instance.fingerprint_line = NULL;
+    instance.fingerprint_line_size = 0;
 #else
-        instance.fingerprint_line_size = 16000;
-        instance.fingerprint_line =
-            SMALLOC(instance.fingerprint_line, instance.fingerprint_line_size);
+    instance.fingerprint_line_size = 16000;
+    instance.fingerprint_line =
+        SMALLOC(instance.fingerprint_line, instance.fingerprint_line_size);
 #endif
 
-        long location_in_file = 0;
-        fcs_bool_t was_init = FALSE;
+    long location_in_file = 0;
+    fcs_bool_t was_init = FALSE;
 #ifdef HAVE_GETLINE
-        while (getline(&instance.fingerprint_line,
-                   &instance.fingerprint_line_size, fingerprint_fh) != -1)
+    while (getline(&instance.fingerprint_line, &instance.fingerprint_line_size,
+               fingerprint_fh) != -1)
 #else
-        while (fgets(instance.fingerprint_line,
-            instance.fingerprint_line_size - 1, fingerprint_fh))
+    while (fgets(instance.fingerprint_line, instance.fingerprint_line_size - 1,
+        fingerprint_fh))
 #endif
-        {
-            FccEntryPointNode *const entry_point = fcs_compact_alloc_ptr(
-                &(instance.fcc_entry_points_allocator), sizeof(*entry_point));
-            fcs_dbm_record_set_parent_ptr(&(entry_point->kv.key), NULL);
-            char state_base64[100];
-            int state_depth;
-            sscanf(instance.fingerprint_line, "%99s %d", state_base64,
-                &state_depth);
-            size_t unused_size;
-            base64_decode(state_base64, strlen(state_base64),
-                ((unsigned char *)&(entry_point->kv.key.key)), &(unused_size));
+    {
+        FccEntryPointNode *const entry_point = fcs_compact_alloc_ptr(
+            &(instance.fcc_entry_points_allocator), sizeof(*entry_point));
+        fcs_dbm_record_set_parent_ptr(&(entry_point->kv.key), NULL);
+        char state_base64[100];
+        int state_depth;
+        sscanf(
+            instance.fingerprint_line, "%99s %d", state_base64, &state_depth);
+        size_t unused_size;
+        base64_decode(state_base64, strlen(state_base64),
+            ((unsigned char *)&(entry_point->kv.key.key)), &(unused_size));
 
-            assert(unused_size == sizeof(entry_point->kv.key.key));
+        assert(unused_size == sizeof(entry_point->kv.key.key));
 
-            /* TODO : Should traverse starting from key. */
-            key_ptr = entry_point;
-            /* The NULL parent_state_enc and move for indicating this is the
-             * initial state. */
-            fcs_init_encoded_state(&(parent_state_enc));
+        /* TODO : Should traverse starting from key. */
+        key_ptr = entry_point;
+        /* The NULL parent_state_enc and move for indicating this is the
+         * initial state. */
+        fcs_init_encoded_state(&(parent_state_enc));
 
 /*
  * These states should be traversed - not blocked, so don't
@@ -763,82 +730,66 @@ int main(int argc, char *argv[])
  * */
 #if 0
 #ifndef FCS_DBM_WITHOUT_CACHES
-            cache_store__insert_key(&(instance.cache_store), &(key_ptr->kv.key.key), &parent_state_enc, NULL, '\0');
+        cache_store__insert_key(&(instance.cache_store), &(key_ptr->kv.key.key), &parent_state_enc, NULL, '\0');
 #else
-            token = fc_solve_dbm_store_insert_key_value(instance.coll.store, &(key_ptr->kv.key.key), NULL, TRUE);
+        token = fc_solve_dbm_store_insert_key_value(instance.coll.store, &(key_ptr->kv.key.key), NULL, TRUE);
 #endif
 
 #if 0
-            entry_point->kv.val.was_consumed =
-                entry_point->kv.val.is_reachable =
-                FALSE;
+        entry_point->kv.val.was_consumed =
+            entry_point->kv.val.is_reachable =
+            FALSE;
 #endif
 #endif
-            entry_point->kv.val.location_in_file = location_in_file;
+        entry_point->kv.val.location_in_file = location_in_file;
 
-            RB_INSERT(
-                FccEntryPointList, &(instance.fcc_entry_points), entry_point);
+        RB_INSERT(FccEntryPointList, &(instance.fcc_entry_points), entry_point);
 
+        {
+            const fcs_offloading_queue_item_t token =
+                ((const fcs_offloading_queue_item_t)(&(entry_point->kv.key)));
+            if (was_init)
             {
-                const fcs_offloading_queue_item_t token = ((
-                    const fcs_offloading_queue_item_t)(&(entry_point->kv.key)));
-                if (was_init)
-                {
-                    fcs_depth_multi_queue__insert(
-                        &(instance.coll.depth_queue), state_depth, &(token));
-                }
-                else
-                {
-#ifdef FCS_DBM_USE_OFFLOADING_QUEUE
-                    fcs_depth_multi_queue__init(&(instance.coll.depth_queue),
-                        instance.offload_dir_path, state_depth, &(token));
-#else
-                    fc_solve_meta_compact_allocator_init(
-                        &(instance.coll.queue_meta_alloc));
-                    fcs_offloading_queue__init(
-                        &(instance.coll.cache_store.queue),
-                        &(instance.coll.queue_meta_alloc));
-                    fcs_depth_multi_queue__insert(
-                        &(instance.coll.queue), state_depth, &(token));
-#endif
-
-                    was_init = TRUE;
-                }
+                fcs_depth_multi_queue__insert(
+                    &(instance.coll.depth_queue), state_depth, &(token));
+            }
+            else
+            {
+                fcs_depth_multi_queue__init(&(instance.coll.depth_queue),
+                    instance.offload_dir_path, state_depth, &(token));
+                was_init = TRUE;
             }
-            instance.common.num_states_in_collection++;
-            instance.common.count_of_items_in_queue++;
-            /* Valid for the next iteration: */
-            location_in_file = ftell(fingerprint_fh);
         }
-
-        fseek(fingerprint_fh, 0, SEEK_SET);
-        instance.fingerprint_fh = fingerprint_fh;
-
-        long count_of_instance_runs = 0;
-        instance.moves_to_state = NULL;
-        instance.max_moves_to_state_len = instance.moves_to_state_len = 0;
-
-        count_of_instance_runs++;
-
-        char fcc_exit_points_out_fn[PATH_MAX + 1],
-            fcc_exit_points_out_fn_temp[PATH_MAX + 1];
-        sprintf(fcc_exit_points_out_fn, "%s/exits.%ld", path_to_output_dir,
-            count_of_instance_runs);
-        sprintf(fcc_exit_points_out_fn_temp, "%s.temp", fcc_exit_points_out_fn);
-        instance.fcc_exit_points_out_fh =
-            fopen(fcc_exit_points_out_fn_temp, "wt");
-
-        instance_run_all_threads(
-            &instance, &init_state, key_ptr, NUM_THREADS());
-        fclose(instance.fcc_exit_points_out_fh);
-        rename(fcc_exit_points_out_fn_temp, fcc_exit_points_out_fn);
-        fclose(fingerprint_fh);
-        handle_and_destroy_instance_solution(&instance, &delta);
-        free(instance.moves_to_state);
-        free(instance.moves_base64_encoding_buffer);
-        free(instance.fingerprint_line);
+        ++instance.common.num_states_in_collection;
+        ++instance.common.count_of_items_in_queue;
+        /* Valid for the next iteration: */
+        location_in_file = ftell(fingerprint_fh);
     }
 
+    fseek(fingerprint_fh, 0, SEEK_SET);
+    instance.fingerprint_fh = fingerprint_fh;
+
+    long count_of_instance_runs = 0;
+    instance.moves_to_state = NULL;
+    instance.max_moves_to_state_len = instance.moves_to_state_len = 0;
+
+    count_of_instance_runs++;
+
+    char fcc_exit_points_out_fn[PATH_MAX + 1],
+        fcc_exit_points_out_fn_temp[PATH_MAX + 1];
+    sprintf(fcc_exit_points_out_fn, "%s/exits.%ld", path_to_output_dir,
+        count_of_instance_runs);
+    sprintf(fcc_exit_points_out_fn_temp, "%s.temp", fcc_exit_points_out_fn);
+    instance.fcc_exit_points_out_fh = fopen(fcc_exit_points_out_fn_temp, "wt");
+
+    instance_run_all_threads(&instance, &init_state, key_ptr, NUM_THREADS());
+    fclose(instance.fcc_exit_points_out_fh);
+    rename(fcc_exit_points_out_fn_temp, fcc_exit_points_out_fn);
+    fclose(fingerprint_fh);
+    handle_and_destroy_instance_solution(&instance, &delta);
+    free(instance.moves_to_state);
+    free(instance.moves_base64_encoding_buffer);
+    free(instance.fingerprint_line);
     fc_solve_delta_stater_release(&delta);
 
     return 0;
