@@ -11,18 +11,9 @@
  * state.c - state functions module for Freecell Solver
  */
 
-#include "dll_thunk.h"
-
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-
-#include "config.h"
 #include "state.h"
 #include "fcs_enums.h"
 
-#include "p2u_rank.h"
-
 #ifdef COMPACT_STATES
 #define GET_CARD(s) (((const fcs_card_t *const)(s))[1])
 static inline int fcs_stack_compare(const void *s1, const void *s2)
@@ -34,11 +25,10 @@ static inline int fcs_stack_compare(const void *s1, const void *s2)
 
 #ifdef COMPACT_STATES
 
-#define DECLARE_TEMP_STACK() char temp_stack[(MAX_NUM_CARDS_IN_A_STACK + 1)]
+#define DECLARE_TEMP_STACK() char temp_stack[MAX_NUM_CARDS_IN_A_STACK + 1]
 #define STACK_COMPARE(a, b) (fcs_stack_compare((a), (b)))
 #define GET_STACK(c) (state_key->data + (c) * (MAX_NUM_CARDS_IN_A_STACK + 1))
 #define COPY_STACK(d, s) (memcpy(d, s, (MAX_NUM_CARDS_IN_A_STACK + 1)))
-#define GET_FREECELL(c) (state_key->data[FCS_FREECELLS_OFFSET + (c)])
 
 #elif defined(INDIRECT_STACK_STATES)
 
@@ -46,14 +36,13 @@ static inline int fcs_stack_compare(const void *s1, const void *s2)
 #define STACK_COMPARE(a, b) (fc_solve_stack_compare_for_comparison(a, b))
 #define GET_STACK(c) (state_key->columns[c])
 #define COPY_STACK(d, s) (d = s)
-#define GET_FREECELL(c) (state_key->freecells[(c)])
 
 #endif
+#define GET_FREECELL(c) (fcs_freecell_card(*state_key, (c)))
 
 void fc_solve_canonize_state(
     fcs_state_t *const ptr_state_key FREECELLS_AND_STACKS_ARGS())
 {
-
 #define state_key (ptr_state_key)
     /* Insertion-sort the columns */
 
@@ -69,7 +58,7 @@ void fc_solve_canonize_state(
                 COPY_STACK(GET_STACK(c - 1), temp_stack);
             }
 
-            c--;
+            --c;
         }
     }
 
@@ -80,18 +69,19 @@ void fc_solve_canonize_state(
         size_t c = b;
 
         while ((c > 0) && ((fc_solve_card_compare(
-                               (GET_FREECELL(c)), (GET_FREECELL(c - 1)))) < 0))
+                               GET_FREECELL(c), GET_FREECELL(c - 1))) < 0))
         {
             const fcs_card_t temp_freecell = GET_FREECELL(c);
             GET_FREECELL(c) = GET_FREECELL(c - 1);
             GET_FREECELL(c - 1) = temp_freecell;
 
-            c--;
+            --c;
         }
     }
 }
 #undef state_key
 
+#ifdef FCS_WITH_MOVES
 void fc_solve_canonize_state_with_locs(fcs_state_t *const ptr_state_key,
 #define state_key (ptr_state_key)
     fcs_state_locs_struct_t *const locs FREECELLS_AND_STACKS_ARGS())
@@ -113,7 +103,7 @@ void fc_solve_canonize_state_with_locs(fcs_state_t *const ptr_state_key,
             locs->stack_locs[c] = locs->stack_locs[c - 1];
             locs->stack_locs[c - 1] = swap_loc;
 
-            c--;
+            --c;
         }
     }
 
@@ -124,7 +114,7 @@ void fc_solve_canonize_state_with_locs(fcs_state_t *const ptr_state_key,
         size_t c = b;
 
         while ((c > 0) && ((fc_solve_card_compare(
-                               (GET_FREECELL(c)), (GET_FREECELL(c - 1)))) < 0))
+                               GET_FREECELL(c), GET_FREECELL(c - 1))) < 0))
         {
             const fcs_card_t temp_freecell = GET_FREECELL(c);
             GET_FREECELL(c) = GET_FREECELL(c - 1);
@@ -134,11 +124,12 @@ void fc_solve_canonize_state_with_locs(fcs_state_t *const ptr_state_key,
             locs->fc_locs[c] = locs->fc_locs[c - 1];
             locs->fc_locs[c - 1] = swap_loc;
 
-            c--;
+            --c;
         }
     }
 }
 #undef state_key
+#endif
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_HASH)
 int fc_solve_state_compare_equal(const void *const s1, const void *const s2)
@@ -147,12 +138,14 @@ int fc_solve_state_compare_equal(const void *const s1, const void *const s2)
 }
 #endif
 
-int fc_solve_state_compare_with_context(
-    const void *s1, const void *s2, fcs_compare_context_t context GCC_UNUSED)
+int fc_solve_state_compare_with_context(const void *const s1,
+    const void *const s2, fcs_compare_context_t context GCC_UNUSED)
 {
     return memcmp(s1, s2, sizeof(fcs_state_t));
 }
 
+#ifdef FCS_WITH_MOVES
+#include "rank2str.h"
 static inline void render_freecell_card(const fcs_card_t card,
     char *const freecell PASS_T(const fcs_bool_t display_10_as_t))
 {
@@ -174,7 +167,7 @@ void fc_solve_state_as_string(char *output_s, const fcs_state_t *const state,
         const fcs_bool_t display_10_as_t))
 {
     typedef char one_card_buffer[4];
-    char decks[MAX_NUM_DECKS * 4][10];
+    char founds[MAX_NUM_DECKS * 4][10];
 
 #define fc_solve_append_string_sprintf(...)                                    \
     output_s += sprintf(output_s, __VA_ARGS__)
@@ -208,29 +201,26 @@ void fc_solve_state_as_string(char *output_s, const fcs_state_t *const state,
 
     for (int i = 0; i < (DECKS_NUM__VAL << 2); i++)
     {
-        fc_solve_p2u_rank(
-            fcs_foundation_value(*state, i), decks[i] PASS_T(display_10_as_t));
-        if (decks[i][0] == ' ')
-            decks[i][0] = '0';
+        rank2str(
+            fcs_foundation_value(*state, i), founds[i] PASS_T(display_10_as_t));
+        if (founds[i][0] == ' ')
+            founds[i][0] = '0';
     }
 
-#if 0
-    fc_solve_append_string_init(&app_str_struct);
-#endif
-
 #ifndef FC_SOLVE_IMPLICIT_PARSABLE_OUTPUT
     if (!parseable_output)
     {
         int i;
         for (i = 0; i < ((FREECELLS_NUM__VAL / 4) +
-                            ((FREECELLS_NUM__VAL % 4 == 0) ? 0 : 1));
+                            ((FREECELLS_NUM__VAL % 4 == (0)) ? 0 : 1));
              i++)
         {
             char dashes_s[128];
             char *dashes_ptr = dashes_s;
             char freecells_s[128];
             char *freecells_s_end = freecells_s;
-            for (int b = 0; b < min(FREECELLS_NUM__VAL - i * 4, 4); b++)
+            const_AUTO(lim, min(FREECELLS_NUM__VAL - i * 4, 4));
+            for (int b = 0; b < lim; b++)
             {
                 one_card_buffer freecell;
                 render_freecell_card(
@@ -244,8 +234,8 @@ void fc_solve_state_as_string(char *output_s, const fcs_state_t *const state,
             {
                 fc_solve_append_string_sprintf(
                     "%-16s        H-%1s C-%1s D-%1s S-%1s\n", freecells_s,
-                    decks[i * 4], decks[i * 4 + 1], decks[i * 4 + 2],
-                    decks[i * 4 + 3]);
+                    founds[i * 4], founds[i * 4 + 1], founds[i * 4 + 2],
+                    founds[i * 4 + 3]);
             }
             else
             {
@@ -256,10 +246,11 @@ void fc_solve_state_as_string(char *output_s, const fcs_state_t *const state,
         for (; i < DECKS_NUM__VAL; i++)
         {
             fc_solve_append_string_sprintf(
-                "%-16s        H-%1s C-%1s D-%1s S-%1s\n", "", decks[i * 4],
-                decks[i * 4 + 1], decks[i * 4 + 2], decks[i * 4 + 3]);
+                "%-16s        H-%1s C-%1s D-%1s S-%1s\n", "", founds[i * 4],
+                founds[i * 4 + 1], founds[i * 4 + 2], founds[i * 4 + 3]);
         }
-        fc_solve_append_string_sprintf("%s", "\n\n");
+        append_char('\n');
+        append_char('\n');
 
         for (int s = 0; s < STACKS_NUM__VAL; s++)
         {
@@ -267,28 +258,25 @@ void fc_solve_state_as_string(char *output_s, const fcs_state_t *const state,
         }
         append_char('\n');
 
-        int max_num_cards = 0;
-        for (int s = 0; s < STACKS_NUM__VAL; s++)
+        size_t max_num_cards = 0;
+        for (size_t s = 0; s < STACKS_NUM__VAL; ++s)
         {
-            fcs_const_cards_column_t col =
-                fcs_state_get_col(*state, stack_locs[s]);
-            const int col_len = fcs_col_len(col);
+            const size_t col_len = fcs_state_col_len(*state, stack_locs[s]);
             if (col_len > max_num_cards)
             {
                 max_num_cards = col_len;
             }
         }
 
-        for (int card_idx = 0; card_idx < max_num_cards; card_idx++)
+        for (size_t card_idx = 0; card_idx < max_num_cards; ++card_idx)
         {
             for (int s = 0; s < STACKS_NUM__VAL; s++)
             {
-                fcs_const_cards_column_t col =
-                    fcs_state_get_col(*state, stack_locs[s]);
+                const_AUTO(col, fcs_state_get_col(*state, stack_locs[s]));
                 const int col_len = fcs_col_len(col);
                 if (card_idx >= col_len)
                 {
-                    fc_solve_append_string_sprintf("    ");
+                    fc_solve_append_string_sprintf("%s", "    ");
                 }
                 else
                 {
@@ -298,7 +286,7 @@ void fc_solve_state_as_string(char *output_s, const fcs_state_t *const state,
                     fc_solve_append_string_sprintf("%3s ", stack_card_str);
                 }
             }
-            fc_solve_append_string_sprintf("%s", "\n");
+            append_char('\n');
         }
     }
     else
@@ -307,8 +295,8 @@ void fc_solve_state_as_string(char *output_s, const fcs_state_t *const state,
         fc_solve_append_string_sprintf("%s", "Foundations:");
         for (int i = 0; i < (DECKS_NUM__VAL << 2); i += 4)
         {
-            fc_solve_append_string_sprintf(" H-%s C-%s D-%s S-%s", decks[i],
-                decks[i + 1], decks[i + 2], decks[i + 3]);
+            fc_solve_append_string_sprintf(" H-%s C-%s D-%s S-%s", founds[i],
+                founds[i + 1], founds[i + 2], founds[i + 3]);
         }
 #ifndef FC_SOLVE__REMOVE_TRAILING_WHITESPACE_IN_OUTPUT
         append_char(' ');
@@ -347,14 +335,13 @@ void fc_solve_state_as_string(char *output_s, const fcs_state_t *const state,
         }
         append_char('\n');
 
-        for (int s = 0; s < STACKS_NUM__VAL; s++)
+        for (int s = 0; s < STACKS_NUM__VAL; ++s)
         {
-            fcs_const_cards_column_t col =
-                fcs_state_get_col(*state, stack_locs[s]);
-            const int col_len = fcs_col_len(col);
+            const_AUTO(col, fcs_state_get_col(*state, stack_locs[s]));
+            const size_t col_len = fcs_col_len(col);
             append_char(':');
 
-            for (int card_idx = 0; card_idx < col_len; card_idx++)
+            for (size_t card_idx = 0; card_idx < col_len; ++card_idx)
             {
                 one_card_buffer stack_card_str;
                 fc_solve_card_stringify(fcs_col_get_card(col, card_idx),
@@ -373,3 +360,4 @@ void fc_solve_state_as_string(char *output_s, const fcs_state_t *const state,
     }
     *(output_s) = '\0';
 }
+#endif
