@@ -7,9 +7,7 @@
  *
  * Copyright (c) 2000 Shlomi Fish
  */
-/*
- * state.h - header file for state functions and macros for Freecell Solver
- */
+// state.h - header file for state functions and macros for Freecell Solver
 #pragma once
 
 #ifdef __cplusplus
@@ -17,11 +15,13 @@ extern "C" {
 #endif
 
 #include <ctype.h>
-#include <string.h>
 
 #include "config.h"
 
+#ifdef FCS_WITH_MOVES
 #include "fcs_move.h"
+#endif
+
 #include "fcs_enums.h"
 
 #include "rinutils.h"
@@ -52,12 +52,12 @@ extern "C" {
     ((FCS_S_SCAN_VISITED(ptr_state))[(scan_id) >> FCS_CHAR_BIT_SIZE_LOG2] &    \
         (1 << ((scan_id) & ((1 << (FCS_CHAR_BIT_SIZE_LOG2)) - 1))))
 
-#ifdef COMPACT_STATES
-
 typedef char fcs_card_t;
 typedef fcs_card_t *fcs_cards_column_t;
 typedef const fcs_card_t *fcs_const_cards_column_t;
 typedef fcs_card_t fcs_state_foundation_t;
+
+#ifdef COMPACT_STATES
 /*
  * Card:
  * Bits 0-3 - Card Number
@@ -95,24 +95,14 @@ typedef char fcs_locs_t;
 
 #define fcs_state_get_col(state, col_idx)                                      \
     ((state).data + ((col_idx) * (MAX_NUM_CARDS_IN_A_STACK + 1)))
-
-#define FCS_FREECELLS_OFFSET ((MAX_NUM_STACKS) * (MAX_NUM_CARDS_IN_A_STACK + 1))
-
-#define fcs_freecell_card(state, f) ((state).data[FCS_FREECELLS_OFFSET + (f)])
-
-#define FCS_FOUNDATIONS_OFFSET                                                 \
-    (((MAX_NUM_STACKS) * (MAX_NUM_CARDS_IN_A_STACK + 1)) + (MAX_NUM_FREECELLS))
-
+#define fcs_freecell_card(state, f)                                            \
+    ((state).data[(MAX_NUM_STACKS * (MAX_NUM_CARDS_IN_A_STACK + 1)) + (f)])
 #define fcs_foundation_value(state, d)                                         \
-    ((state).data[FCS_FOUNDATIONS_OFFSET + (d)])
-
-#elif defined(INDIRECT_STACK_STATES) /* #ifdef COMPACT_STATES */
-
-typedef char fcs_card_t;
-typedef fcs_card_t *fcs_cards_column_t;
-typedef const fcs_card_t *fcs_const_cards_column_t;
-typedef char fcs_state_foundation_t;
+    ((state).data[((MAX_NUM_STACKS * (MAX_NUM_CARDS_IN_A_STACK + 1)) +         \
+                      MAX_NUM_FREECELLS) +                                     \
+                  (d)])
 
+#elif defined(INDIRECT_STACK_STATES) // #ifdef COMPACT_STATES
 typedef struct
 {
     fcs_cards_column_t columns[MAX_NUM_STACKS];
@@ -143,54 +133,36 @@ typedef struct
         (dest_info).stacks_copy_on_write_flags = 0;                            \
     }
 
-typedef char fcs_locs_t;
+typedef uint8_t fcs_locs_t;
 
 #else
-#error Neither COMPACT_STATES nor INDIRECT_STACK_STATES are not defined.
+#error Neither COMPACT_STATES nor INDIRECT_STACK_STATES are defined.
 #endif /* #if defined COMPACT_STATES -                                         \
           #elif defined INDIRECT_STACK_STATES                                  \
         */
 
-/* These are macros or functions that are common to all three _STATES types. */
-
-#define fcs_freecell_card_suit(state, f)                                       \
-    (fcs_card_suit(fcs_freecell_card((state), (f))))
-
+// These are macros or functions that are common to all the _STATES types.
 #define fcs_increment_foundation(state, foundation_idx)                        \
-    ((fcs_foundation_value((state), (foundation_idx)))++)
-
+    (++(fcs_foundation_value((state), (foundation_idx))))
 #define fcs_set_foundation(state, foundation_idx, value)                       \
     ((fcs_foundation_value((state), (foundation_idx))) =                       \
             (fcs_state_foundation_t)(value))
-
 #define fcs_col_pop_top(col)                                                   \
-    {                                                                          \
-        fcs_col_get_card((col), (--fcs_col_len(col))) = fc_solve_empty_card;   \
-    }
-
+    (fcs_col_get_card((col), (--fcs_col_len(col))) = fc_solve_empty_card)
 #define fcs_col_pop_card(col, into)                                            \
     {                                                                          \
         into = fcs_col_get_card((col), (fcs_col_len(col) - 1));                \
         fcs_col_pop_top(col);                                                  \
     }
-
 #define fcs_col_push_card(col, from)                                           \
-    {                                                                          \
-        fcs_col_get_card((col), ((fcs_col_len(col))++)) = (from);              \
-    }
-
+    fcs_col_get_card((col), ((fcs_col_len(col))++)) = (from)
 #define fcs_freecell_is_empty(state, idx)                                      \
     (fcs_card_is_empty(fcs_freecell_card(state, idx)))
 
 #if defined(COMPACT_STATES)
 
-#define fcs_duplicate_state_extra(dest_info)                                   \
-    {                                                                          \
-    }
-
-#define fcs_copy_stack(state_key, state_val, idx, buffer)                      \
-    {                                                                          \
-    }
+#define fcs_duplicate_state_extra(dest_info)
+#define fcs_copy_stack(state_key, state_val, idx, buffer)
 
 #endif
 
@@ -212,11 +184,11 @@ static inline fcs_card_t fcs_make_card(const int rank, const int suit)
 #define fcs_char2card(c) (c)
 
 #define fcs_col_len(col) (((col)[0]))
-
 #define fcs_col_get_card(col, card_idx) ((col)[(card_idx) + 1])
+#define fcs_state_col_len(s, i) fcs_col_len(fcs_state_get_col((s), (i)))
+#define fcs_state_col_is_empty(s, i) (fcs_state_col_len((s), (i)) == 0)
 
 #define fcs_card_rank(card) ((card) >> 2)
-
 #define fcs_card_suit(card) ((card)&0x03)
 
 static inline fcs_card_t fcs_col_get_rank(
@@ -225,11 +197,16 @@ static inline fcs_card_t fcs_col_get_rank(
     return fcs_card_rank(fcs_col_get_card(col, card_idx));
 }
 #define FCS_RANK_KING 13
+#include "is_king.h"
+static inline fcs_bool_t fcs_card_is_king(const fcs_card_t card)
+{
+    return fc_solve_is_king_buf[(size_t)card];
+}
 
 static inline fcs_bool_t fcs_col_is_king(
     const fcs_const_cards_column_t col, const int card_idx)
 {
-    return (fcs_col_get_rank(col, card_idx) == FCS_RANK_KING);
+    return fcs_card_is_king(fcs_col_get_card(col, card_idx));
 }
 
 struct fcs_state_keyval_pair_struct;
@@ -321,13 +298,13 @@ typedef struct
     fcs_locs_t fc_locs[MAX_NUM_FREECELLS];
 } fcs_state_locs_struct_t;
 
-static inline void fc_solve_init_locs(fcs_state_locs_struct_t *locs)
+static inline void fc_solve_init_locs(fcs_state_locs_struct_t *const locs)
 {
-    for (int i = 0; i < MAX_NUM_STACKS; i++)
+    for (int i = 0; i < MAX_NUM_STACKS; ++i)
     {
         locs->stack_locs[i] = (fcs_locs_t)i;
     }
-    for (int i = 0; i < MAX_NUM_FREECELLS; i++)
+    for (int i = 0; i < MAX_NUM_FREECELLS; ++i)
     {
         locs->fc_locs[i] = (fcs_locs_t)i;
     }
@@ -378,7 +355,6 @@ static inline fcs_kv_state_t FCS_STATE_keyval_pair_to_kv(
  *
  * */
 #ifdef FCS_RCS_STATES
-
 typedef fcs_state_extra_info_t fcs_collectible_state_t;
 #define FCS_S_ACCESSOR(s, field) ((s)->field)
 
@@ -467,15 +443,9 @@ static inline void FCS_STATE_collectible_to_kv(
 #define DECKS_NUM__VAL decks_num
 #endif
 
-#define DECKS_NUM__ARG PASS_DECKS(const int decks_num)
-
 #define FREECELLS_AND_STACKS_ARGS() FREECELLS_NUM__ARG STACKS_NUM__ARG
 #define FREECELLS_STACKS_DECKS__ARGS()                                         \
-    FREECELLS_AND_STACKS_ARGS() DECKS_NUM__ARG
-
-#ifdef FC_SOLVE_IMPLICIT_T_RANK
-#define FCS_IMPLICIT_T_RANK
-#endif
+    FREECELLS_AND_STACKS_ARGS() PASS_DECKS(const int decks_num)
 
 #define PASS_T(arg) FC_SOLVE__PASS_T(arg)
 
@@ -491,16 +461,17 @@ typedef void *fcs_compare_context_t;
 typedef const void *fcs_compare_context_t;
 #endif
 
-static inline int fc_solve_state_compare(const void *s1, const void *s2)
+static inline int fc_solve_state_compare(
+    const void *const s1, const void *const s2)
 {
     return memcmp(s1, s2, sizeof(fcs_state_t));
 }
 
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_HASH)
-extern int fc_solve_state_compare_equal(const void *s1, const void *s2);
+extern int fc_solve_state_compare_equal(const void *, const void *);
 #endif
 extern int fc_solve_state_compare_with_context(
-    const void *s1, const void *s2, fcs_compare_context_t context);
+    const void *, const void *, fcs_compare_context_t);
 
 /*
  * Convert an entire card to its user representation.
@@ -522,7 +493,7 @@ extern void fc_solve_card_stringify(
  * The suit letter may come somewhat after the beginning of the string.
  *
  * */
-static inline int fc_solve_u2p_suit(const char *suit)
+static inline int fcs_str2suit(const char *suit)
 {
     while (TRUE)
     {
@@ -539,7 +510,7 @@ static inline int fc_solve_u2p_suit(const char *suit)
         case 'S':
             return 3;
         default:
-            suit++;
+            ++suit;
         }
     }
 }
@@ -550,7 +521,7 @@ static inline int fc_solve_u2p_suit(const char *suit)
  * the program.
  * */
 
-static inline int fc_solve_u2p_rank(const char *string)
+static inline int fcs_str2rank(const char *string)
 {
     while (1)
     {
@@ -596,7 +567,7 @@ static inline int fc_solve_u2p_rank(const char *string)
         case '9':
             return 9;
         default:
-            string++;
+            ++string;
         }
     }
 }
@@ -606,7 +577,7 @@ static inline int fc_solve_u2p_rank(const char *string)
  * */
 static inline fcs_card_t fc_solve_card_parse_str(const char *const str)
 {
-    return fcs_make_card(fc_solve_u2p_rank(str), fc_solve_u2p_suit(str));
+    return fcs_make_card(fcs_str2rank(str), fcs_str2suit(str));
 }
 
 #ifdef INDIRECT_STACK_STATES
@@ -623,19 +594,16 @@ static inline void fc_solve_state_init_proto(
         IND_BUF_T_PARAM(indirect_stacks_buffer))
 {
     memset(&(state->s), 0, sizeof(state->s));
-
 #ifdef INDIRECT_STACK_STATES
+    int i;
+    for (i = 0; i < STACKS_NUM__VAL; ++i)
     {
-        int i;
-        for (i = 0; i < STACKS_NUM__VAL; i++)
-        {
-            memset(state->s.columns[i] = &indirect_stacks_buffer[i << 7], '\0',
-                MAX_NUM_DECKS * 52 + 1);
-        }
-        for (; i < MAX_NUM_STACKS; i++)
-        {
-            state->s.columns[i] = NULL;
-        }
+        memset(state->s.columns[i] = &indirect_stacks_buffer[i << 7], '\0',
+            MAX_NUM_DECKS * 52 + 1);
+    }
+    for (; i < MAX_NUM_STACKS; ++i)
+    {
+        state->s.columns[i] = NULL;
     }
 #endif
     state->info.parent = NULL;
@@ -656,11 +624,29 @@ static inline void fc_solve_state_init_proto(
 #endif
 }
 
+#ifdef FCS_BREAK_BACKWARD_COMPAT_1
+#define FCS_PARSE_try_prefix(str, p, p_s) try_str_prefix(str, p)
+#else
 static const char *const fc_solve_freecells_prefixes[] = {
     "FC:", "Freecells:", "Freecell:", NULL};
 
 static const char *const fc_solve_foundations_prefixes[] = {"Decks:", "Deck:",
     "Founds:", "Foundations:", "Foundation:", "Found:", NULL};
+
+static inline const char *fc_solve__try_prefixes(
+    const char *const str, const char *const *const prefixes)
+{
+    for (const char *const *prefix = prefixes; (*prefix); ++prefix)
+    {
+        if (!strncasecmp(str, (*prefix), strlen(*prefix)))
+        {
+            return str + strlen(*prefix);
+        }
+    }
+    return NULL;
+}
+#define FCS_PARSE_try_prefix(str, p, p_s) fc_solve__try_prefixes(str, p_s)
+#endif
 #if defined(WIN32) && (!defined(HAVE_STRNCASECMP))
 #ifndef strncasecmp
 #define strncasecmp(a, b, c) (strnicmp((a), (b), (c)))
@@ -668,8 +654,7 @@ static const char *const fc_solve_foundations_prefixes[] = {"Decks:", "Deck:",
 #endif
 
 #define fc_solve_initial_user_state_to_c(string, out_state, freecells_num,     \
-                                         stacks_num, decks_num,                \
-                                         indirect_stacks_buffer)               \
+    stacks_num, decks_num, indirect_stacks_buffer)                             \
     fc_solve_initial_user_state_to_c_proto(string,                             \
         out_state PASS_FREECELLS(freecells_num) PASS_STACKS(stacks_num)        \
             PASS_DECKS(decks_num) PASS_IND_BUF_T(indirect_stacks_buffer))
@@ -679,14 +664,12 @@ static inline fcs_bool_t fc_solve_initial_user_state_to_c_proto(
     fcs_state_keyval_pair_t *const out_state FREECELLS_STACKS_DECKS__ARGS()
         IND_BUF_T_PARAM(indirect_stacks_buffer))
 {
-    const char *const *prefix;
-
     fc_solve_state_init(out_state, STACKS_NUM__VAL, indirect_stacks_buffer);
     const char *str = string;
 
     fcs_bool_t first_line = TRUE;
 
-#define ret (out_state->s)
+#define out (out_state->s)
 /* Handle the end of string - shouldn't happen */
 #define HANDLE_EOS()                                                           \
     {                                                                          \
@@ -704,29 +687,23 @@ static inline fcs_bool_t fc_solve_initial_user_state_to_c_proto(
             while ((*str) != '\n')
             {
                 HANDLE_EOS();
-                str++;
+                ++str;
             }
-            str++;
+            ++str;
         }
 
         first_line = FALSE;
 
-        for (prefix = fc_solve_freecells_prefixes; (*prefix); prefix++)
+        const_AUTO(new_str, FCS_PARSE_try_prefix(str,
+                                "Freecells:", fc_solve_freecells_prefixes));
+        if (new_str)
         {
-            if (!strncasecmp(str, (*prefix), strlen(*prefix)))
+            str = new_str;
+            for (size_t c = 0; c < FREECELLS_NUM__VAL; ++c)
             {
-                str += strlen(*prefix);
-                break;
+                fcs_empty_freecell(out, c);
             }
-        }
-
-        if (*prefix)
-        {
-            for (int c = 0; c < FREECELLS_NUM__VAL; c++)
-            {
-                fcs_empty_freecell(ret, c);
-            }
-            for (int c = 0; c < FREECELLS_NUM__VAL; c++)
+            for (size_t c = 0; c < FREECELLS_NUM__VAL; ++c)
             {
                 if (c != 0)
                 {
@@ -734,80 +711,74 @@ static inline fcs_bool_t fc_solve_initial_user_state_to_c_proto(
                            ((*str) != '\n') && ((*str) != '\r'))
                     {
                         HANDLE_EOS();
-                        str++;
+                        ++str;
                     }
                     if ((*str == '\n') || (*str == '\r'))
                     {
                         break;
                     }
-                    str++;
+                    ++str;
                 }
 
                 while ((*str == ' ') || (*str == '\t'))
                 {
-                    str++;
+                    ++str;
                 }
                 if ((*str == '\r') || (*str == '\n'))
                     break;
 
-                fcs_put_card_in_freecell(ret, c,
+                fcs_put_card_in_freecell(out, c,
                     ((*str == '*') || (*str == '-')) ? fc_solve_empty_card : ({
-                        const char rank = fc_solve_u2p_rank(str);
+                        const char rank = fcs_str2rank(str);
                         if (!rank)
                         {
                             return FALSE;
                         }
-                        fcs_make_card(rank, fc_solve_u2p_suit(str));
+                        fcs_make_card(rank, fcs_str2suit(str));
                     }));
             }
 
             while (*str != '\n')
             {
                 HANDLE_EOS();
-                str++;
+                ++str;
             }
-            s--;
+            --s;
             continue;
         }
 
-        for (prefix = fc_solve_foundations_prefixes; (*prefix); prefix++)
-        {
-            if (!strncasecmp(str, (*prefix), strlen(*prefix)))
-            {
-                str += strlen(*prefix);
-                break;
-            }
-        }
-
-        if (*prefix)
+        const_AUTO(new_str2, FCS_PARSE_try_prefix(str, "Foundations:",
+                                 fc_solve_foundations_prefixes));
+        if (new_str2)
         {
+            str = new_str2;
             for (int f_idx = 0; f_idx < (DECKS_NUM__VAL << 2); f_idx++)
             {
-                fcs_set_foundation(ret, f_idx, 0);
+                fcs_set_foundation(out, f_idx, 0);
             }
 
             int decks_index[4] = {0, 0, 0, 0};
             while (1)
             {
                 while ((*str == ' ') || (*str == '\t'))
-                    str++;
+                    ++str;
                 if ((*str == '\n') || (*str == '\r'))
                     break;
-                const int f_idx = fc_solve_u2p_suit(str);
-                str++;
+                const int f_idx = fcs_str2suit(str);
+                ++str;
                 while (*str == '-')
-                    str++;
-                /* Workaround for fc_solve_u2p_rank's willingness
+                    ++str;
+                /* Workaround for fcs_str2rank's willingness
                  * to designate the string '0' as 10. */
-                const int c = ((str[0] == '0') ? 0 : fc_solve_u2p_rank(str));
+                const int c = ((str[0] == '0') ? 0 : fcs_str2rank(str));
                 while ((*str != ' ') && (*str != '\t') && (*str != '\n') &&
                        (*str != '\r'))
                 {
                     HANDLE_EOS();
-                    str++;
+                    ++str;
                 }
 
-                fcs_set_foundation(ret, ((decks_index[f_idx] << 2) + f_idx), c);
+                fcs_set_foundation(out, ((decks_index[f_idx] << 2) + f_idx), c);
                 if ((++decks_index[f_idx]) >= DECKS_NUM__VAL)
                 {
                     decks_index[f_idx] = 0;
@@ -821,10 +792,10 @@ static inline fcs_bool_t fc_solve_initial_user_state_to_c_proto(
          * input states from -p -t mid-play. */
         if ((*str) == ':')
         {
-            str++;
+            ++str;
         }
 
-        fcs_cards_column_t col = fcs_state_get_col(ret, s);
+        var_AUTO(col, fcs_state_get_col(out, s));
         for (int c = 0; c < MAX_NUM_CARDS_IN_A_STACK; c++)
         {
             /* Move to the next card */
@@ -834,7 +805,7 @@ static inline fcs_bool_t fc_solve_initial_user_state_to_c_proto(
                        ((*str) != '\n') && ((*str) != '\r'))
                 {
                     HANDLE_EOS();
-                    str++;
+                    ++str;
                 }
                 if ((*str == '\n') || (*str == '\r'))
                 {
@@ -844,7 +815,7 @@ static inline fcs_bool_t fc_solve_initial_user_state_to_c_proto(
 
             while ((*str == ' ') || (*str == '\t'))
             {
-                str++;
+                ++str;
             }
             HANDLE_EOS();
             if ((*str == '\n') || (*str == '\r'))
@@ -863,7 +834,7 @@ static inline fcs_bool_t fc_solve_initial_user_state_to_c_proto(
     return TRUE;
 }
 
-#undef ret
+#undef out
 #undef HANDLE_EOS
 
 extern void fc_solve_state_as_string(char *output_s,
@@ -923,7 +894,7 @@ static inline fcs_state_validity_ret_t fc_solve_check_state_validity(
     /* Mark the card_counts in the columns */
     for (int s = 0; s < STACKS_NUM__VAL; s++)
     {
-        const fcs_const_cards_column_t col = fcs_state_get_col(*state, s);
+        const_AUTO(col, fcs_state_get_col(*state, s));
         const int col_len = fcs_col_len(col);
         for (int c = 0; c < col_len; c++)
         {
@@ -972,15 +943,13 @@ enum
     FCS_VISITED_GENERATED_BY_PRUNING = 0x10,
 };
 
-#define FCS_WITH_CARD_COMPARE_LOOKUP
-
 static inline int fc_solve_card_compare(
     const fcs_card_t c1, const fcs_card_t c2)
 {
     return (c1) - (c2);
 }
 
-#if defined(INDIRECT_STACK_STATES)
+#ifdef INDIRECT_STACK_STATES
 static inline int fc_solve_stack_compare_for_comparison(
     const void *const v_s1, const void *const v_s2)
 {
@@ -991,7 +960,7 @@ static inline int fc_solve_stack_compare_for_comparison(
         const int min_len = min(s1[0], s2[0]);
         for (int a = 1; a <= min_len; a++)
         {
-            int ret = fc_solve_card_compare(s1[a], s2[a]);
+            const int ret = fc_solve_card_compare(s1[a], s2[a]);
             if (ret != 0)
             {
                 return ret;
@@ -1018,7 +987,6 @@ static inline int fc_solve_stack_compare_for_comparison(
         return 0;
     }
 }
-
 #endif
 
 static inline void set_scan_visited(
@@ -1028,24 +996,22 @@ static inline void set_scan_visited(
         (1 << ((scan_id) & ((1 << (FCS_CHAR_BIT_SIZE_LOG2)) - 1)));
 }
 
-/*
- * This macro determines if child can be placed above parent.
- *
- * The variable sequences_are_built_by has to be initialized to
- * the sequences_are_built_by member of the instance.
- *
- * */
+    /*
+     * This macro determines if child can be placed above parent.
+     *
+     * The variable sequences_are_built_by has to be initialized to
+     * the sequences_are_built_by member of the instance.
+     *
+     * */
 
 #ifdef FCS_FREECELL_ONLY
 
-static inline fcs_bool_t fcs_is_parent_card__helper(
+#include "is_parent.h"
+static inline fcs_bool_t fcs_is_parent_card(
     const fcs_card_t child, const fcs_card_t parent)
 {
-    return ((fcs_card_rank(child) + 1 == fcs_card_rank(parent)) &&
-            ((fcs_card_suit(child) & 0x1) != (fcs_card_suit(parent) & 0x1)));
+    return fc_solve_is_parent_buf[(size_t)parent][(size_t)child];
 }
-#define fcs_is_parent_card(child, parent)                                      \
-    fcs_is_parent_card__helper(child, parent)
 
 #else
 
@@ -1086,7 +1052,7 @@ static inline void fcs_col_transfer_cards(fcs_cards_column_t dest_col,
 static inline fcs_card_t fcs_state_pop_col_card(
     fcs_state_t *const state, const int col_idx)
 {
-    const fcs_cards_column_t col = fcs_state_get_col(*state, col_idx);
+    var_AUTO(col, fcs_state_get_col(*state, col_idx));
     fcs_card_t ret;
     fcs_col_pop_card(col, ret);
     return ret;
@@ -1095,10 +1061,17 @@ static inline fcs_card_t fcs_state_pop_col_card(
 static inline void fcs_state_pop_col_top(
     fcs_state_t *const state, const int col_idx)
 {
-    const fcs_cards_column_t col = fcs_state_get_col(*state, col_idx);
+    var_AUTO(col, fcs_state_get_col(*state, col_idx));
     fcs_col_pop_top(col);
 }
 
+static inline void fcs_state_push(
+    fcs_state_t *const state, const int col_idx, const fcs_card_t card)
+{
+    var_AUTO(col, fcs_state_get_col(*state, col_idx));
+    fcs_col_push_card(col, card);
+}
+
 #ifdef FCS_FREECELL_ONLY
 #define FCS__SEQS_ARE_BUILT_BY_RANK() FALSE
 #else
