@@ -7,20 +7,12 @@
  *
  * Copyright (c) 2000 Shlomi Fish
  */
-/*
- * summarizing_solver.c - solves several indices of deals and prints a summary
- * of the solutions of each one.
- * */
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "fcs_user.h"
+// summarizing_solver.c - solves several indices of deals and prints a summary
+// of the solutions of each one.
 #include "fcs_cl.h"
-
-#include "rinutils.h"
 #include "range_solvers_gen_ms_boards.h"
 #include "handle_parsing.h"
+#include "try_param.h"
 
 static void __attribute__((noreturn)) print_help(void)
 {
@@ -34,16 +26,21 @@ static void __attribute__((noreturn)) print_help(void)
     exit(-1);
 }
 
-static long deals[32000];
-static size_t num_deals = 0;
+static long *mydeals = NULL;
+static size_t num_deals = 0, max_num_deals = 0;
 
 static inline void append(long idx)
 {
-    if (num_deals == COUNT(deals))
+    if (num_deals == max_num_deals)
     {
-        fc_solve_err("Number of deals exceeded %ld!\n", (long)(COUNT(deals)));
+        mydeals = SREALLOC(mydeals, max_num_deals += 1000);
+        if (!mydeals)
+        {
+            fc_solve_err(
+                "Number of deals exceeded %ld!\n", (long)max_num_deals);
+        }
     }
-    deals[num_deals++] = idx;
+    mydeals[num_deals++] = idx;
 }
 
 int main(int argc, char *argv[])
@@ -77,6 +74,35 @@ int main(int argc, char *argv[])
                 fc_solve_err("seq without args!\n");
             }
         }
+        if (!strcmp(argv[arg], "slurp"))
+        {
+            arg++;
+            if (arg < argc)
+            {
+                const_AUTO(fn, argv[arg++]);
+                FILE *f = fopen(fn, "rt");
+                if (f)
+                {
+                    while (!feof(f))
+                    {
+                        long deal;
+                        if (fscanf(f, "%ld", &deal) == 1)
+                        {
+                            append(deal);
+                        }
+                    }
+                    fclose(f);
+                }
+                else
+                {
+                    fc_solve_err("Cannot slurp file!\n");
+                }
+            }
+            else
+            {
+                fc_solve_err("slurp without arg!\n");
+            }
+        }
         else
         {
             append(atol(argv[arg++]));
@@ -92,17 +118,10 @@ int main(int argc, char *argv[])
 
     for (; arg < argc; arg++)
     {
-        if (!strcmp(argv[arg], "--variant"))
+        const char *param;
+        if ((param = TRY_P("--variant")))
         {
-            arg++;
-            if (arg == argc)
-            {
-                fprintf(stderr, "--variant came without an argument!\n");
-                print_help();
-            }
-            variant = argv[arg];
-
-            if (strlen(variant) > 50)
+            if (strlen(variant = param) > 50)
             {
                 fprintf(stderr, "--variant's argument is too long!\n");
                 print_help();
@@ -114,17 +133,14 @@ int main(int argc, char *argv[])
         }
     }
 
-    void *const instance = simple_alloc_and_parse(argc, argv, &arg);
+    void *const instance = simple_alloc_and_parse(argc, argv, arg);
 
     const fcs_bool_t variant_is_freecell = (!strcmp(variant, "freecell"));
-#ifndef FCS_FREECELL_ONLY
-    freecell_solver_user_apply_preset(instance, variant);
-#endif
     char buffer[2000];
 
     for (size_t deal_idx = 0; deal_idx < num_deals; deal_idx++)
     {
-        const_AUTO(board_num, deals[deal_idx]);
+        const_AUTO(board_num, mydeals[deal_idx]);
         if (variant_is_freecell)
         {
             get_board_l(board_num, buffer);
@@ -140,7 +156,7 @@ int main(int argc, char *argv[])
                 buffer, sizeof(buffer[0]), COUNT(buffer) - 1, from_make_pysol);
             pclose(from_make_pysol);
         }
-        buffer[COUNT(buffer) - 1] = '\0';
+        LAST(buffer) = '\0';
 
         long num_moves;
         const char *verdict;
@@ -174,6 +190,7 @@ int main(int argc, char *argv[])
     }
 
     freecell_solver_user_free(instance);
+    free(mydeals);
 
     return 0;
 }
