@@ -1,26 +1,13 @@
-/* Copyright (c) 2011 Shlomi Fish
+/*
+ * This file is part of Freecell Solver. It is subject to the license terms in
+ * the COPYING.txt file found in the top-level directory of this distribution
+ * and at http://fc-solve.shlomifish.org/docs/distro/COPYING.html . No part of
+ * Freecell Solver, including this file, may be copied, modified, propagated,
+ * or distributed except according to the terms contained in the COPYING file.
  *
- * Permission is hereby granted, free of charge, to any person
- * obtaining a copy of this software and associated documentation
- * files (the "Software"), to deal in the Software without
- * restriction, including without limitation the rights to use,
- * copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following
- * conditions:
- *
- * The above copyright notice and this permission notice shall be
- * included in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
- * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
- * OTHER DEALINGS IN THE SOFTWARE.
+ * Copyright (c) 2011 Shlomi Fish
  */
+
 /*
  * A test for the DBM kaztree compare records.
  */
@@ -78,7 +65,7 @@ static int main_tests(void)
     return 0;
 }
 
-int main(int argc, char * argv[])
+int main(void)
 {
     plan_tests(1);
     main_tests();
