@@ -7,13 +7,9 @@
  *
  * Copyright (c) 2011 Shlomi Fish
  */
-
-/*
- * A test for the delta states routines.
- */
+// A test for the delta states routines.
 
 #include <string.h>
-#include <stdio.h>
 
 #include <tap.h>
 
@@ -24,6 +20,9 @@
 #include "../indirect_buffer.h"
 #include "../trim_trailing_whitespace.h"
 #include "../render_state.h"
+#ifdef FCS_FREECELL_ONLY
+#include "is_parent.c"
+#endif
 
 static fcs_card_t make_card(int rank, int suit)
 {
@@ -202,10 +201,7 @@ static int main_tests(void)
              * */
             ok (enc.bit_in_char_idx == 7, "Only 7 bits (2).");
         }
-
-#define SUIT_HC 0
 #define SUIT_DS 1
-
         {
             fc_solve_column_encoding_composite_t enc;
             fc_solve_get_column_encoding_composite(&delta, 1, &enc);
