@@ -2,12 +2,13 @@ package FC_Solve::Paths;
 
 use strict;
 use warnings;
+use Socket qw(:crlf);
 use String::ShellQuote qw/shell_quote/;
 
 use parent 'Exporter';
 
 our @EXPORT_OK =
-    qw($FC_SOLVE_EXE $FC_SOLVE__RAW $IS_WIN $MAKE_PYSOL bin_board bin_exe_raw bin_file data_file dll_file is_freecell_only is_without_dbm is_without_flares is_without_patsolve is_without_valgrind samp_board samp_preset samp_sol);
+    qw($FC_SOLVE_EXE $FC_SOLVE__RAW $FIND_DEAL_INDEX $IS_WIN $MAKE_PYSOL bin_board bin_exe_raw bin_file data_file dll_file is_break is_freecell_only is_without_dbm is_without_flares is_without_patsolve is_without_valgrind normalize_lf samp_board samp_preset samp_sol);
 
 use File::Spec ();
 
@@ -32,7 +33,8 @@ my $FC_SOLVE__RAW__RAW = "$FCS_PATH/fc-solve";
 our $FC_SOLVE__RAW = _correct_path($FC_SOLVE__RAW__RAW) . $EXE_SUF;
 our $FC_SOLVE_EXE  = _correct_path( shell_quote($FC_SOLVE__RAW__RAW) );
 my $PY3 = ( $IS_WIN ? 'python3 ' : '' );
-our $MAKE_PYSOL = "${PY3}../board_gen/make_pysol_freecell_board.py";
+our $MAKE_PYSOL      = "${PY3}../board_gen/make_pysol_freecell_board.py";
+our $FIND_DEAL_INDEX = "${PY3}../board_gen/find-freecell-deal-index.py";
 
 sub _is_tag
 {
@@ -40,6 +42,7 @@ sub _is_tag
 
     return ( ( $ENV{FCS_TEST_TAGS} // '' ) =~ /\b\Q$tag\E\b/ );
 }
+my $BREAK_TAG   = _is_tag('break_backcompat');
 my $FC_ONLY     = _is_tag('fc_only');
 my $NO_FLARES   = _is_tag('no_flares');
 my $NO_PATSOLVE = _is_tag('no_pats');
@@ -74,6 +77,11 @@ sub data_file
     return File::Spec->catfile( $DATA_DIR, @{ shift @_ } );
 }
 
+sub is_break
+{
+    return $BREAK_TAG;
+}
+
 sub is_freecell_only
 {
     return $FC_ONLY;
@@ -105,7 +113,6 @@ sub samp_board
     return File::Spec->catfile( $BOARDS_DIR, shift );
 }
 
-# Returns a board from the sample-boards directory.
 sub samp_sol
 {
     return File::Spec->catfile( $SOLS_DIR, shift );
@@ -116,4 +123,11 @@ sub samp_preset
     return File::Spec->catfile( $PRESETS_DIR, shift );
 }
 
+sub normalize_lf
+{
+    my ($s) = @_;
+    $s =~ s#$CRLF#$LF#g;
+    return $s;
+}
+
 1;
