@@ -4,15 +4,11 @@ use strict;
 use warnings;
 
 use parent 'Exporter';
-
 our @EXPORT_OK = qw(trim_trail_ws);
 
 sub trim_trail_ws
 {
-    my $trim_trail_ws = shift;
-    $trim_trail_ws =~ s/[ \t]+$//gms;
-    return $trim_trail_ws;
+    return shift =~ s/[ \t]+$//gmrs;
 }
 
 1;
-
