@@ -7,26 +7,15 @@
  *
  * Copyright (c) 2012 Shlomi Fish
  */
-/*
- * typeof_wrap.h - convenient wrappers for GCC's typeof
- */
+// typeof_wrap.h - convenient wrappers for GCC's typeof
 #pragma once
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-/*
- * These emulate the C++ auto keyword.
- * */
+// These emulate the C++ auto keyword.
 #define const_AUTO(myvar, expr) const typeof(expr) myvar = (expr)
 #define var_AUTO(myvar, expr) typeof(expr) myvar = (expr)
+#define var_PTR(myvar, expr) typeof((expr)[0]) *myvar = (expr)
+#define const_PTR(myvar, expr) typeof((expr)[0]) *const myvar = (expr)
 
-/*
- * Common macros for object slots.
- * */
+// Common macros for object slots.
 #define const_SLOT(myslot, obj) const_AUTO(myslot, (obj)->myslot)
-
-#ifdef __cplusplus
-};
-#endif
+#define var_S_SLOT(myslot, obj) const_AUTO(myslot, (obj).myslot)
