@@ -7,13 +7,10 @@
  *
  * Copyright (c) 2012 Shlomi Fish
  */
-/*
- * var_base_reader.h - read from a packed integer using digits of variable
- * bases. Also see var_base_writer.h which complements it.
- */
+// var_base_reader.h - read from a packed integer using digits of variable
+// bases. Also see var_base_writer.h which complements it.
 #pragma once
 
-#include <assert.h>
 #include "var_base_int.h"
 
 typedef struct
